/*  Sesame - Storage and Querying architecture for RDF and RDF Schema
 *  Copyright (C) 2001-2005 Aduna
 *
 *  Contact: 
 *  	Aduna
 *  	Prinses Julianaplein 14 b
 *  	3817 CS Amersfoort
 *  	The Netherlands
 *  	tel. +33 (0)33 465 99 87
 *  	fax. +33 (0)33 465 99 87
 *
 *  	http://aduna.biz/
 *  	http://www.openrdf.org/
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.openrdf.model.impl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import org.openrdf.model.Graph;
import org.openrdf.model.Resource;
import org.openrdf.model.Statement;
import org.openrdf.model.URI;
import org.openrdf.model.Value;
import org.openrdf.model.ValueFactory;

import org.openrdf.sesame.sail.RdfRepository;
import org.openrdf.sesame.sail.SailInitializationException;
import org.openrdf.sesame.sail.SailUpdateException;
import org.openrdf.sesame.sail.StatementIterator;

/**
 * GraphImpl - an implementation of the graph interface that uses an in-memory
 * RdfRepository for storage.
 * 
 * @author Jeen Broekstra
 * @author Arjohn Kampman
 * @version $Revision: 1.9 $
 */
public class GraphImpl implements Graph {

/*----------+
| Variables |
+----------*/

	private RdfRepository _repository;

/*-------------+
| Constructors |
+-------------*/

	/**
	 * Creates a new GraphImpl. The GraphImpl uses an in-memory RdfRepository as
	 * storage backend.
	 **/
	public GraphImpl() {
		this(new org.openrdf.sesame.sailimpl.memory.RdfRepository());

		try {
			((org.openrdf.sesame.sailimpl.memory.RdfRepository)_repository).initialize();
		}
		catch (SailInitializationException e) {
			// This can only happen if there is an error in the supplied config
			// parameters. Since we do not supply any, it never should occur.
			throw new RuntimeException(e);
		}
	}

	/**
	 * Creates a new GraphImpl that uses the supplied RdfRepository as storage
	 * backend. The GraphImpl assumes that the RdfRepository is initialized.
	 * 
	 * @param repository The RdfRepository to be used as backend.
	 **/
	public GraphImpl(RdfRepository repository) {
		_repository = repository;
	}

/*--------+
| Methods |
+--------*/

	// implements Graph.add(Resource, URI, Value)
	public void add(Resource subject, URI predicate, Value object) {
		_repository.startTransaction();
		try {
			_repository.addStatement(subject, predicate, object);
		}
		catch (SailUpdateException e) {
			// only occurs if transaction is not started.
			throw new RuntimeException(e);
		}
		finally {
			_repository.commitTransaction();
		}
	}

	// implements Graph.add(Statement)
	public void add(Statement st) {
		_repository.startTransaction();
		try {
			_repository.addStatement(st.getSubject(), st.getPredicate(), st.getObject());
		}
		catch (SailUpdateException e) {
			// only occurs if transaction is not started.
			throw new RuntimeException(e);
		}
		finally {
			_repository.commitTransaction();
		}
	}

	// implements Graph.add(StatementIterator)
	public void add(StatementIterator iterator) {
		_repository.startTransaction();
		try {
			while (iterator.hasNext()) {
				Statement st = iterator.next();
				_repository.addStatement(st.getSubject(), st.getPredicate(), st.getObject());
			}
		}
		catch (SailUpdateException e) {
			// only occurs if transaction is not started.
			throw new RuntimeException(e);
		}
		finally {
			_repository.commitTransaction();
		}
	}

	// implements Graph.add(Collection)
	public void add(Collection statements) {
		Iterator iter = statements.iterator();

		_repository.startTransaction();
		try {
			while (iter.hasNext()) {
				Object o = iter.next();
				if (o instanceof Statement) {
					Statement st = (Statement)o;
					_repository.addStatement(st.getSubject(), st.getPredicate(), st.getObject());
				}
			}
		}
		catch (SailUpdateException e) {
			// only occurs if transaction is not started.
			throw new RuntimeException(e);
		}
		finally {
			_repository.commitTransaction();
		}
	}

	// implements Graph.add(Graph)
	public void add(Graph graph) {
		this.add(graph.getStatements());
	}

	// implements Graph.contains(Resource, URI, Value)
	public boolean contains(Resource subject, URI predicate, Value object) {
		return _repository.hasStatement(subject, predicate, object);
	}

	// implements Graph.contains(Statement)
	public boolean contains(Statement st) {
		return contains(st.getSubject(), st.getPredicate(), st.getObject());
	}

	// implements Graph.getStatements()
	public StatementIterator getStatements() {
		return _repository.getStatements(null, null, null);
	}

	// implements Graph.getStatements(Resource, URI, Value)
	public StatementIterator getStatements(Resource subject, URI predicate, Value object) {
		return _repository.getStatements(subject, predicate, object);
	}

	// implements Graph.remove(Resource, URI, Value)
	public int remove(Resource subject, URI predicate, Value object) {
		int result = 0;
		_repository.startTransaction();
		try {
			result = _repository.removeStatements(subject, predicate, object);
		}
		catch (SailUpdateException e) {
			// only occurs if transaction is not started.
			throw new RuntimeException(e);
		}
		finally {
			_repository.commitTransaction();
		}
		return result;
	}

	// implements Graph.remove(Statement)
	public int remove(Statement st) {
		return remove(st.getSubject(), st.getPredicate(), st.getObject());
	}

	// implements Graph.remove(StatementIterator)
	public int remove(StatementIterator iterator) {
		int result = 0;
		_repository.startTransaction();
		try {
			while (iterator.hasNext()) {
				Statement st = iterator.next();
				result += _repository.removeStatements(st.getSubject(), st.getPredicate(), st.getObject());
			}
		}
		catch (SailUpdateException e) {
			// only occurs if transaction is not started.
			throw new RuntimeException(e);
		}
		finally {
			_repository.commitTransaction();
		}
		return result;
	}

	// implements Graph.clear()
	public void clear() {
		_repository.startTransaction();
		try {
			_repository.clearRepository();
		}
		catch (SailUpdateException e) {
			// only occurs if transaction is not started.
			throw new RuntimeException(e);
		}
		finally {
			_repository.commitTransaction();
		}
	}

	// implements Graph.getStatementCollection(Resource, URI, Value)
	public Collection getStatementCollection(Resource subject, URI predicate, Value object) {
		Collection collection = new ArrayList();

		StatementIterator iterator = getStatements(subject, predicate, object);
		while (iterator.hasNext()) {
			collection.add(iterator.next());
		}
		iterator.close();
		return collection;
	}

	// implements Graph.remove(Graph)
	public int remove(Graph graph) {
		return remove(graph.getStatements());
	}

	// implements Graph.getValueFactory()
	public ValueFactory getValueFactory() {
		return _repository.getValueFactory();
	}
}