/*  Sesame - Storage and Querying architecture for RDF and RDF Schema
 *  Copyright (C) 2001-2005 Aduna
 *
 *  Contact:
 *  	Aduna
 *  	Prinses Julianaplein 14 b
 *  	3817 CS Amersfoort
 *  	The Netherlands
 *  	tel. +33 (0)33 465 99 87
 *  	fax. +33 (0)33 465 99 87
 *
 *  	http://aduna.biz/
 *  	http://www.openrdf.org/
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.openrdf.sesame.config;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * An object representation of the configuration parameters for a
 * Sesame repository. Configuration options for repositories include
 * read-write options and the configuration of the Sail stack.
 *
 * @author Jeen Broekstra, Arjohn Kampman
 * @version $Revision: 1.7 $
 */
public class RepositoryConfig implements RepositoryInfo, Cloneable {

/*-----------------------------+
| Constants                    |
+-----------------------------*/

	private static final Class RDF_SOURCE_CLASS;
	private static final Class RDF_REPOSITORY_CLASS;
	private static final Class RDF_SCHEMA_SOURCE_CLASS;
	private static final Class RDF_SCHEMA_REPOSITORY_CLASS;

	static {
		try {
			RDF_SOURCE_CLASS = Class.forName("org.openrdf.sesame.sail.RdfSource");
			RDF_REPOSITORY_CLASS = Class.forName("org.openrdf.sesame.sail.RdfRepository");
			RDF_SCHEMA_SOURCE_CLASS = Class.forName("org.openrdf.sesame.sail.RdfSchemaSource");
			RDF_SCHEMA_REPOSITORY_CLASS = Class.forName("org.openrdf.sesame.sail.RdfSchemaRepository");
		}
		catch (ClassNotFoundException e) {
			throw new RuntimeException(e);
		}
	}

/*-----------------------------+
| Variables                    |
+-----------------------------*/

	private String _id;
	private String _title;
	private boolean _worldReadable;
	private boolean _worldWriteable;

	private List _sailList;

/*-----------------------------+
| Constructors                 |
+-----------------------------*/

	/**
	 * Creates a new RepositoryConfig with the supplied repository id
	 *
	 * @param id the repository id
	 */
	public RepositoryConfig(String id) {
		_id = id;
		_sailList = new ArrayList();
	}

	/**
	 * Creates a new RepositoryConfig with the supplied repository id and title
	 *
	 * @param id Repository id
	 * @param title Repository title
	 */
	public RepositoryConfig(String id, String title) {
		_id = id;
		_title = title;
		_sailList = new ArrayList();
	}

/*-----------------------------+
| Methods                      |
+-----------------------------*/

	/**
	 * Gets the repository id
	 *
	 * @return the repository id
	 */
	public String getRepositoryId() {
		return _id;
	}

	/**
	 * Sets the repository id to the given id value
	 *
	 * @param id the new repository id
	 */
	public void setRepositoryId(String id) {
		_id = id;
	}

	/**
	 * Gets the title of the repository.
	 *
	 * @return the repository title
	 */
	public String getTitle() {
		return _title;
	}

	/**
	 * gets the title of the repository.
	 *
	 * @param title the repository title
	 */
	public void setTitle(String title) {
		_title = title;
	}

	public boolean isWorldReadable() {
		return _worldReadable;
	}

	public void setWorldReadable(boolean worldReadable) {
		_worldReadable = worldReadable;
	}

	public boolean isWorldWriteable() {
		return _worldWriteable;
	}

	public void setWorldWriteable(boolean worldWriteable) {
		_worldWriteable = worldWriteable;
	}

	/**
	 * Checks if this repository is readable.
	 *
	 * @return <tt>true</tt> if the Sail on top of the Sail stack
	 * implements <tt>org.openrdf.sesame.sail.RdfSource</tt>,
	 * <tt>false</tt> otherwise.
	 **/
	public boolean isReadable() {
		if (_sailList.size() > 0) {
			SailConfig topSail = (SailConfig)_sailList.get(0);

			try {
				Class sailClass = Class.forName(topSail.getSailClass());
				return RDF_SOURCE_CLASS.isAssignableFrom(sailClass);
			}
			catch (ClassNotFoundException ignore) {}
		}

		return false;
	}

	/**
	 * Checks if this repository is writeable.
	 *
	 * @return <tt>true</tt> if the Sail on top of the Sail stack
	 * implements <tt>org.openrdf.sesame.sail.RdfRepository</tt>,
	 * <tt>false</tt> otherwise.
	 **/
	public boolean isWriteable() {
		if (_sailList.size() > 0) {
			SailConfig topSail = (SailConfig)_sailList.get(0);

			try {
				Class sailClass = Class.forName(topSail.getSailClass());
				return RDF_REPOSITORY_CLASS.isAssignableFrom(sailClass);
			}
			catch (ClassNotFoundException ignore) {}
		}

		return false;
	}
	
	/**
	 * Adds the sail with the supplied class to the sail stack. Begin with
	 * adding the top-most Sail and work down towards the base Sail.
	 * 
	 * @param sailConfig SailConfig class
	 */
	public void addSail(SailConfig sailConfig) {
		_sailList.add(sailConfig);
	}

	/**
	 * Adds the sail represented by the supplied SailConfig to the sail
	 * stack. The sail is added in the stack at the position indicated by
	 * the supplied index.
	 *
	 * @param sailConfig the SailConfig class
	 * @param index the position in the sail stack for this Sail. The
	 * top-most sail is at index 0.
	 */
	public void addSail(SailConfig sailConfig, int index) {
		_sailList.add(index, sailConfig);
	}
	
	/**
	 * Removes the sail with the given class from the sail stack
	 *
	 * @param sailConfig SailConfig class
	 */
	public void removeSail(SailConfig sailConfig) {
	 	_sailList.remove(sailConfig);
	}

	/**
	 * Retrieves a list representation of the Sail stack. The first Sail
	 * in the list is the top-most Sail, the last one is the base sail.
	 *
	 * @return a list of SailConfig objects.
	 */
	public List getSailList() {
		return _sailList;
	}

	/**
	 * Sets the Sail stack.
	 *
	 * @param sailList a list of SailConfig objects.
	 */
	public void setSailList(List sailList) {
		_sailList = sailList;
	}

	/**
	 * Checks if any Sails are specified in this RepositoryConfig.
	 */
	public boolean hasASail() {
		return !_sailList.isEmpty();
	}

	/**
	 * Checks if the supplied sail is present in the sail stack.
	 *
	 * @param sailClass the class name of the sail to be searched for.
	 */
	public boolean hasSail(String sailClass) {
		return getSail(sailClass) != null;
	}

	/**
	 * Retrieves the SailConfig object for the supplied sail class.
	 * 
	 * @param sailClass the class name of the sail to be retrieved.
	 * @return a SailConfig object.
	 */
	public SailConfig getSail(String sailClass) {
		Iterator iter = _sailList.iterator();

		while (iter.hasNext()) {
			SailConfig sailConfig = (SailConfig)iter.next();

			if (sailConfig.getSailClass().equals(sailClass)) {
				return sailConfig;
			}
		}

		return null;
	}

	/**
	 * Retrieves the interface implemented by the top-most Sail in the
	 * stack. The returned interface is one of the four basic Sail
	 * interfaces: RdfSource, RdfRepository, RdfSchemaSource, and
	 * RdfSchemaRepository.
	 *
	 * @return the interface name
	 */
	public String getSailType() {
		String sailType = null;
		SailConfig topSailConfig = (SailConfig)_sailList.get(0);
		String sailClassName = topSailConfig.getSailClass();

		try {
			Class sailClass = Class.forName(sailClassName);

			if (RDF_SCHEMA_REPOSITORY_CLASS.isAssignableFrom(sailClass)) {
				sailType = "org.openrdf.sesame.sail.RdfSchemaRepository";
			}
			else if (RDF_REPOSITORY_CLASS.isAssignableFrom(sailClass)) {
				sailType = "org.openrdf.sesame.sail.RdfRepository";
			}
			else if (RDF_SCHEMA_SOURCE_CLASS.isAssignableFrom(sailClass)) {
				sailType = "org.openrdf.sesame.sail.RdfSchemaSource";
			}
			else if (RDF_SOURCE_CLASS.isAssignableFrom(sailClass)) {
				sailType = "org.openrdf.sesame.sail.RdfSource";
			}
		}
		catch (ClassNotFoundException ignore) {
		}

		return sailType;
	}

	/**
	 * Moves the sail with the supplied class one level up in the sail stack.
	 *
	 * @param sailClass Sail class
	 */
	public void sailUp(String sailClass) {
		int idx = 0;
		SailConfig sailConfig = null;

		while (idx < _sailList.size()) {
			sailConfig = (SailConfig)_sailList.get(idx);

			if (sailConfig.getSailClass().equals(sailClass)) {
				break;
			}

			idx++;
		}

		if (idx > 0 && idx < _sailList.size()) {
			_sailList.remove(idx);
			_sailList.add(idx - 1, sailConfig);
		}
	}

	/**
	 * Moves the sail with the supplied class one level down in the sail stack.
	 *
	 * @param sailClass Sail class
	 */
	public void sailDown(String sailClass) {
		int idx = 0;
		SailConfig sailConfig = null;

		while (idx < _sailList.size()) {
			sailConfig = (SailConfig)_sailList.get(idx);

			if (sailConfig.getSailClass().equals(sailClass)) {
				break;
			}

			idx++;
		}

		if (idx < _sailList.size() - 1) {
			_sailList.remove(idx);
			_sailList.add(idx + 1, sailConfig);
		}
	}
	
	// Overrides Object.clone()
	public Object clone() {
		try {
			RepositoryConfig clone = (RepositoryConfig)super.clone();

			// Create a deep-copy of the Sail list
			clone._sailList = new ArrayList(_sailList.size());
			Iterator iter = _sailList.iterator();
			while (iter.hasNext()) {
				SailConfig sailConfig = (SailConfig)iter.next();
				clone.addSail( (SailConfig)sailConfig.clone() );
			}

			return clone;
		}
		catch (CloneNotSupportedException e) {
			throw new RuntimeException(e);
		}
	}
}
