/*  Sesame - Storage and Querying architecture for RDF and RDF Schema
 *  Copyright (C) 2001-2005 Aduna
 *
 *  Contact: 
 *  	Aduna
 *  	Prinses Julianaplein 14 b
 *  	3817 CS Amersfoort
 *  	The Netherlands
 *  	tel. +33 (0)33 465 99 87
 *  	fax. +33 (0)33 465 99 87
 *
 *  	http://aduna.biz/
 *  	http://www.openrdf.org/
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.openrdf.sesame.query.rdql.model;

import org.openrdf.model.Literal;
import org.openrdf.model.Value;
import org.openrdf.model.impl.LiteralImpl;

import org.openrdf.sesame.query.QueryEvaluationException;

/**
 * Comparison operators on numeric values.
 **/
public class MathExpr implements Expression {

/*-------------------------------------------+
| Constants								  |
+-------------------------------------------*/

	/** Calculate the sum of the arguments **/
	public static final int PLUS = 0;

	/** Calculate the difference of the arguments **/
	public static final int MINUS = 1;

	/** Calculate the product of the arguments **/
	public static final int MULTIPLY = 2;

	/** Calculate the division of the arguments **/
	public static final int DIVIDE = 3;

	/** Calculate the remainder of the arguments **/
	public static final int REMAINDER = 4;

/*-------------------------------------------+
| Variables								  |
+-------------------------------------------*/

	protected Expression _arg1;
	protected Expression _arg2;
	protected int _operator;

/*-------------------------------------------+
| Constructors							   |
+-------------------------------------------*/

	public MathExpr(Expression arg1, int op, Expression arg2) {
		_arg1 = arg1;
		_arg2 = arg2;

		if (op < PLUS || op > REMAINDER) {
			throw new RuntimeException("Illegal operator: " + op);
		}
		_operator = op;
	}

/*-------------------------------------------+
| Methods									|
+-------------------------------------------*/

	public Expression getArg1() {
		return _arg1;
	}

	public Expression getArg2() {
		return _arg2;
	}

	public int getOperator() {
		return _operator;
	}

	public Value getValue() throws QueryEvaluationException {
		// Do the math
		Value val1 = _arg1.getValue();
		Value val2 = _arg2.getValue();

		if (val1 instanceof Literal && val2 instanceof Literal) {
			Literal lit1 = (Literal)val1;
			Literal lit2 = (Literal)val2;

			// try coercing both arguments to integer
			try {
				int result = _calculateInteger(lit1, lit2);
				return new LiteralImpl(String.valueOf(result));
			}
			catch (NumberFormatException notAnInteger) {
			}

			// try coercing both arguments to floating point numbers
			try {
				float result = _calculateReal(lit1, lit2);
				return new LiteralImpl(String.valueOf(result));
			}
			catch (NumberFormatException notAReal) {
				// arguments cannot be calculated, ignore (?)

				//throw new QueryEvaluationException(
						//"numeric operands expected: " + toString());
			}
		}

		throw new QueryEvaluationException(
				"numeric operands expected: " + toString());
	}

	protected int _calculateInteger(Literal arg1, Literal arg2)
		throws NumberFormatException
	{
		int intVal1 = Integer.parseInt(arg1.getLabel());
		int intVal2 = Integer.parseInt(arg2.getLabel());

		switch (_operator) {
			case PLUS: return intVal1 + intVal2;
			case MINUS: return intVal1 - intVal2;
			case MULTIPLY: return intVal1 * intVal2;
			case DIVIDE: return intVal1 / intVal2;
			case REMAINDER: return intVal1 % intVal2;
		}

		// should never be called:
		return 0;
	}

	protected float _calculateReal(Literal arg1, Literal arg2)
		throws NumberFormatException
	{
		float floatVal1 = Float.parseFloat(arg1.getLabel());
		float floatVal2 = Float.parseFloat(arg2.getLabel());

		switch (_operator) {
			case PLUS: return floatVal1 + floatVal2;
			case MINUS: return floatVal1 - floatVal2;
			case MULTIPLY: return floatVal1 * floatVal2;
			case DIVIDE: return floatVal1 / floatVal2;
			case REMAINDER: return floatVal1 % floatVal2;
		}

		// should never be called:
		return 0;
	}

	public String getQuery() {
		return this.toString();
	}

	public String toString() {
		String op = null;
		switch (_operator) {
			case PLUS: op = " + "; break;
			case MINUS: op = " - "; break;
			case MULTIPLY: op = " * "; break;
			case DIVIDE: op = " / "; break;
			case REMAINDER: op = " % "; break;
		}
		return _arg1.toString() + op + _arg2.toString();
	}
}
