/*  Sesame - Storage and Querying architecture for RDF and RDF Schema
 *  Copyright (C) 2001-2005 Aduna
 *
 *  Contact: 
 *  	Aduna
 *  	Prinses Julianaplein 14 b
 *  	3817 CS Amersfoort
 *  	The Netherlands
 *  	tel. +33 (0)33 465 99 87
 *  	fax. +33 (0)33 465 99 87
 *
 *  	http://aduna.biz/
 *  	http://www.openrdf.org/
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.openrdf.sesame.query.rdql.model;

import org.openrdf.model.Literal;
import org.openrdf.model.Value;

import org.openrdf.sesame.query.QueryEvaluationException;

/**
 * Comparison operators on numeric values.
 **/
public class NumberCompare extends BooleanExpr {

/*-------------------------------------------+
| Constants								  |
+-------------------------------------------*/

	/** Lower Than **/
	public static final int LT = 0;

	/** Lower or Equal **/
	public static final int LE = 1;

	/** EQual **/
	public static final int EQ = 2;

	/** Larger or Equal **/
	public static final int GE = 3;

	/** Larger Than **/
	public static final int GT = 4;

	/** Not Equal **/
	public static final int NE = 5;

	
/*-------------------------------------------+
| Variables								  |
+-------------------------------------------*/

	protected Expression _arg1;
	protected Expression _arg2;
	protected int _operator;

/*-------------------------------------------+
| Constructors							   |
+-------------------------------------------*/

	public NumberCompare(Expression arg1, int op, Expression arg2) {
		_arg1 = arg1;
		_arg2 = arg2;

		if (op < LT || op > NE) {
			throw new RuntimeException("Illegal operator: " + op);
		}
		_operator = op;
	}

/*-------------------------------------------+
| Methods									|
+-------------------------------------------*/

	public Expression getArg1() {
		return _arg1;
	}

	public Expression getArg2() {
		return _arg2;
	}

	public int getOperator() {
		return _operator;
	}

	/**
	 * Determines what the boolean value of the comparison operator is.
	 * Possible comparisons are &lt;, &lt;=, =, &gt;, &gt;=, !=.  When
	 * incompatible datatypes are compared, all comparisons (except !=)
	 * evaluate to <code>false</code>.
	 *
	 * @exception QueryEvaluationException is thrown when the comparison is
	 * between a set and a single element, or when a variable has no
	 * assigned value.
	 *
	 * @return <code>true</code> when the selected comparison operator
	 * evaluates to <code>true</code>, <code>false</code> otherwise.
	 **/
	public boolean isTrue() 
		throws QueryEvaluationException 
	{
		// Element comparison
		Value val1 = _arg1.getValue();
		Value val2 = _arg2.getValue();

		if (val1 instanceof Literal && val2 instanceof Literal) {
			Literal lit1 = (Literal)val1;
			Literal lit2 = (Literal)val2;

			// try coercing both arguments to integer
			try {
				return _compareIntegers(lit1, lit2);
			}
			catch (NumberFormatException notAnInteger) {
			}

			// try coercing both arguments to floating point numbers
			try {
				return _compareReals(lit1, lit2);
			}
			catch (NumberFormatException notAReal) {
				// arguments cannot be compared, ignore (?)

				//throw new QueryEvaluationException(
						//"numeric operands expected: " + toString());
			}
		}

		return false;
	}

	protected boolean _compareIntegers(Literal arg1, Literal arg2)
		throws NumberFormatException
	{
		int intVal1 = Integer.parseInt(arg1.getLabel());
		int intVal2 = Integer.parseInt(arg2.getLabel());

		switch (_operator) {
			case LT: return intVal1 <  intVal2;
			case LE: return intVal1 <= intVal2;
			case EQ: return intVal1 == intVal2;
			case GE: return intVal1 >= intVal2;
			case GT: return intVal1 >  intVal2;
			case NE: return intVal1 != intVal2;
		}
		return false;
	}

	protected boolean _compareReals(Literal arg1, Literal arg2)
		throws NumberFormatException
	{
		float floatVal1 = Float.parseFloat(arg1.getLabel());
		float floatVal2 = Float.parseFloat(arg2.getLabel());

		switch (_operator) {
			case LT: return floatVal1 <  floatVal2;
			case LE: return floatVal1 <= floatVal2;
			case EQ: return floatVal1 == floatVal2;
			case GE: return floatVal1 >= floatVal2;
			case GT: return floatVal1 >  floatVal2;
			case NE: return floatVal1 != floatVal2;
		}
		return false;
	}

	public String getQuery() {
		return this.toString();
	}

	public String toString() {
		String op = null;
		switch (_operator) {
			case LT: op = " < "; break;
			case LE: op = " <= "; break;
			case EQ: op = " = "; break;
			case GE: op = " >= "; break;
			case GT: op = " > "; break;
			case NE: op = " != "; break;
		}
		return _arg1.toString() + op + _arg2.toString();
	}
}
