/*  Sesame - Storage and Querying architecture for RDF and RDF Schema
 *  Copyright (C) 2001-2005 Aduna
 *
 *  Contact: 
 *  	Aduna
 *  	Prinses Julianaplein 14 b
 *  	3817 CS Amersfoort
 *  	The Netherlands
 *  	tel. +33 (0)33 465 99 87
 *  	fax. +33 (0)33 465 99 87
 *
 *  	http://aduna.biz/
 *  	http://www.openrdf.org/
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.openrdf.sesame.query.rdql.model;

import java.io.IOException;
import java.util.List;

import org.openrdf.model.Value;

import org.openrdf.sesame.query.QueryEvaluationException;
import org.openrdf.sesame.query.TableQueryResultListener;
import org.openrdf.sesame.sail.RdfSource;

public class SfwQuery {

	protected List _varList;

	protected List _tripleSelectors;

	protected BooleanExpr _suchthat;


	public SfwQuery(List varList, List tripleSelectors, BooleanExpr suchthat) {
		_varList = varList;
		_tripleSelectors = tripleSelectors;
		_suchthat = suchthat;
	}


	public void evaluate(RdfSource rdfSource, TableQueryResultListener listener) throws QueryEvaluationException {
		try {
			int size = _varList.size();
			Var var = null;
			String columnHeaders[] = new String[size];

			for (int i = 0; i < size; i++) {
				var = (Var)_varList.get(i);
				columnHeaders[i] = var.getName();
			}

			listener.startTableQueryResult(columnHeaders);

			boolean instantiated = _findNext(rdfSource, true);

			while (instantiated) {
				if (_suchthat == null || _suchthat.isTrue()) {
					_exportResults(listener);
				}
				instantiated = _findNext(rdfSource, false);
			}

			listener.endTableQueryResult();
		}
		catch (IOException e) {
			// Clear all selectors
			for (int i = _tripleSelectors.size() - 1; i >= 0; i--) {
				TripleSelector selector = (TripleSelector)_tripleSelectors.get(i);
				selector.clear();
			}

			throw new QueryEvaluationException(e);
		}
	}

	/**
	 * Finds the next result. If 'firstCall' is true, then this method will
	 * start with the initialization of the first selector. Otherwise, this
	 * method will assume that all selectors have been initialization before
	 * and will start/continue with the last selector.
	 *
	 * @return true if a next results has been found, false otherwise.
	 **/
	private boolean _findNext(RdfSource rdfSource, boolean firstCall) {
		int lastIndex = _tripleSelectors.size() - 1;
		int index;
		TripleSelector selector;

		if (firstCall) {
			index = 0;
			selector = (TripleSelector)_tripleSelectors.get(index);
			selector.initialize(rdfSource);
		}
		else {
			index = lastIndex;
			selector = (TripleSelector)_tripleSelectors.get(index);
		}

		while (true) {
			if (selector.selectNext(rdfSource)) {
				// Selector instantiated

				if (index == lastIndex) {
					// Last selector instantiated
					return true;
				}
				else {
					// Instantiate next selector
					index++;
					selector = (TripleSelector)_tripleSelectors.get(index);
					selector.initialize(rdfSource);
				}
			}
			else {
				// No more results found, go back in recursion
				selector.clear();

				if (index == 0) {
					// No more results
					return false;
				}
				else {
					index--;
					selector = (TripleSelector)_tripleSelectors.get(index);
				}
			}
		}
	}

	private void _exportResults(TableQueryResultListener listener) throws QueryEvaluationException {
		try {
			listener.startTuple();

			int varCount = _varList.size();
			for (int i = 0; i < varCount; i++) {
				Var var = (Var)_varList.get(i);

                Value varValue = var.getValue();
				listener.tupleValue(varValue);
			}
	
			listener.endTuple();
		}
		catch (IOException e) {
			throw new QueryEvaluationException(e);
		}
	}

	public List getTripleSelectors() {
		return _tripleSelectors;
	}

	public void setTripleSelectors(List tripleSelectors) {
		_tripleSelectors = tripleSelectors;
	}

	public BooleanExpr getWherePart() {
		return _suchthat;
	}

	public void setWherePart(BooleanExpr newSuchthat) {
		_suchthat = newSuchthat;
	}
	
	public List getProjection() {
		return _varList;
	}

	public String getQuery() {
		return this.toString();
	}

	public String toString() {
		StringBuffer result = new StringBuffer();

		result.append("select ");
		int varCount = _varList.size();
		if (varCount == 0) {
			result.append("*");
		}
		else {
			for (int i = 0; i < varCount; i++) {
				Var var = (Var)_varList.get(i);
				result.append(var.toString());
				if (i < varCount - 1) {
					result.append(", ");
				}
			}
		}

		result.append("\nwhere\n");
		int nofSelectors = _tripleSelectors.size();
		for (int i = 0; i < nofSelectors; i++) {
			TripleSelector selector = (TripleSelector)_tripleSelectors.get(i);
			result.append("\t");
			result.append(selector.toString());
			if (i < nofSelectors - 1) {
				result.append(", ");
			}
			result.append("\n");
		}

		if (_suchthat != null) {
			result.append("suchthat ");
			result.append(_suchthat.toString());
		}

		return result.toString();
	}
}
