/*  Sesame - Storage and Querying architecture for RDF and RDF Schema
 *  Copyright (C) 2001-2005 Aduna
 *
 *  Contact: 
 *  	Aduna
 *  	Prinses Julianaplein 14 b
 *  	3817 CS Amersfoort
 *  	The Netherlands
 *  	tel. +33 (0)33 465 99 87
 *  	fax. +33 (0)33 465 99 87
 *
 *  	http://aduna.biz/
 *  	http://www.openrdf.org/
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.openrdf.sesame.sail.query;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.openrdf.model.Value;
import org.openrdf.model.ValueFactory;
import org.openrdf.model.impl.URIImpl;
import org.openrdf.model.impl.ValueFactoryImpl;

/**
 * An abstract representation of an RDF query. The query consists of a number
 * of path expressions, additional boolean constraints on these path
 * expressions, and a projection describing which variable values should be
 * returned.
 *
 * @author Arjohn Kampman
 */
public class CfwQuery extends QueryComponent {
	
/*-------------------------------+
| Constructors                   |
+-------------------------------*/

	public CfwQuery() {
		this(false, new ArrayList(), new PathExpressionList());
	}

	public CfwQuery(boolean distinct, List projection, PathExpressionList pathExpressionList) {
		this(distinct, 0, Integer.MAX_VALUE, projection, pathExpressionList);
	}

	public CfwQuery(boolean distinct, int offset, int limit,
			List projection, PathExpressionList pathExpressionList)
	{
		_distinct = distinct;
		_offset = offset;
		_limit = limit;
		_projection = projection;
		_pathExpressionList = pathExpressionList;
	}

/*-------------------------------+
| Methods                        |
+-------------------------------*/

	public String[] getColumnHeaders() {
		// Print column headers
		String columnHeaders[] = new String[_projection.size()];
	
		for (int i = 0; i < _projection.size(); i++) {
			ProjectionItem pi = (ProjectionItem)_projection.get(i);
			
			if (pi.getAlias() != null) {
				columnHeaders[i] = pi.getAlias();
			} else {
				columnHeaders[i] = pi.getValueExpr().toString();
			}
		}
		return columnHeaders;
	}
	
	protected void _reportQueryAnswer(QueryAnswerListener listener) 
		throws IOException
	{
		for (int i = 0; i < _projection.size(); i++) {
			PathExpression pe = (PathExpression)_projection.get(i);
			
			ValueExpr subject = null;
			ValueExpr predicate = null;
			ValueExpr object = null;
			
            if (pe instanceof TriplePattern) {
                TriplePattern tp = (TriplePattern)pe;
                subject = tp.getSubjectVar();
				predicate = tp.getPredicateVar();
				object = tp.getObjectVar();
            }
            else if (pe instanceof DirectSubClassOf) {
            	DirectSubClassOf dsco = (DirectSubClassOf)pe;
                subject = dsco.getSubjectVar();
				predicate = new ResourceExpr(URIImpl.RDFS_SUBCLASSOF);
				object = dsco.getObjectVar();
            }
            else if (pe instanceof DirectSubPropertyOf) {
            	DirectSubPropertyOf dspo = (DirectSubPropertyOf)pe;
                subject = dspo.getSubjectVar();
				predicate = new ResourceExpr(URIImpl.RDFS_SUBPROPERTYOF);
				object = dspo.getObjectVar();
            }
            else if (pe instanceof DirectType) {
            	DirectType dt = (DirectType)pe;
                subject = dt.getSubjectVar();
				predicate = new ResourceExpr(URIImpl.RDF_TYPE);
				object = dt.getObjectVar();
            }
            else {
                throw new RuntimeException(
                		"Unknown PathExpression in constructExpressions: " + pe.getClass());
            }
            
            ValueFactory valFactory = new ValueFactoryImpl();
            
            Value subjectVal = subject.getValue();
            Value predicateVal = predicate.getValue();
            Value objectVal = object.getValue();
            
    		// check if subject is an anonymous variable. In such cases a
    		// new bnode should be constructed.
    		if (subject instanceof Var) {
    			Var subjVar = (Var)subject;
    			
    			if (subjVar.isAnonymous() && !subjVar.hasValue()) {
    				subjectVal = valFactory.createBNode();
    			}
    		}
    		
    		// check if object is an anonymous variable. In such cases a
    		// new bnode should be constructed.		
    		if (object instanceof Var) {
    			Var objVar = (Var)object;
    			
    			if (objVar.isAnonymous() && !objVar.hasValue()) {
    				objectVal = valFactory.createBNode();
    			}
    		}

    		// we should skip any triples for which a value is null; these are 'partial'
    		// matches for an optional path and should not be returned.
    		if (subjectVal != null && predicateVal != null && objectVal != null) {
    			Value[] result = new Value[] {
            		subjectVal,
					predicateVal,
					objectVal
					};
            
    			listener.queryAnswer(result);
    		}
		}
	}
	
	public String toString() {
		StringBuffer result = new StringBuffer(256);
		result.append("construct ");

		if (_distinct) {
			result.append("distinct ");
		}

		int projSize = _projection.size();
		for (int i = 0; i < projSize; i++) {
			PathExpression pe = (PathExpression)_projection.get(i);

			result.append(pe.toString());

			if (i < projSize - 1) {
				result.append(", ");
			}
		}

		result.append("\nfrom\n").append( _pathExpressionList.toString() );

		if (_limit < Integer.MAX_VALUE) {
			result.append("\nlimit ").append(_limit);
		}

		if (_offset > 0) {
			result.append("\noffset ").append(_offset);
		}

		return result.toString();
	}
}
