/*  Sesame - Storage and Querying architecture for RDF and RDF Schema
 *  Copyright (C) 2001-2005 Aduna
 *
 *  Contact: 
 *  	Aduna
 *  	Prinses Julianaplein 14 b
 *  	3817 CS Amersfoort
 *  	The Netherlands
 *  	tel. +33 (0)33 465 99 87
 *  	fax. +33 (0)33 465 99 87
 *
 *  	http://aduna.biz/
 *  	http://www.openrdf.org/
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.openrdf.sesame.sail.query;

import java.util.Collection;

import org.openrdf.model.Resource;
import org.openrdf.model.Statement;
import org.openrdf.model.impl.URIImpl;

import org.openrdf.sesame.sail.RdfSchemaSource;
import org.openrdf.sesame.sail.RdfSource;
import org.openrdf.sesame.sail.StatementIterator;
import org.openrdf.sesame.sail.util.EmptyStatementIterator;

/**
 * @author Arjohn Kampman
 */
public class DirectSubClassOf implements PathExpression {

/*-------------------------------+
| Variables                      |
+-------------------------------*/

	protected Var _subjectVar;
	protected Var _objectVar;

	protected StatementIterator _statementIter;

	protected boolean _subjectVarLocked;
	protected boolean _objectVarLocked;

	protected boolean _subjAndObjAreEqual;

/*-------------------------------+
| Constructors                   |
+-------------------------------*/

	public DirectSubClassOf(Var subjectVar, Var objectVar) {
		_subjectVar = subjectVar;
		_objectVar = objectVar;
	}

/*-------------------------------+
| Methods                        |
+-------------------------------*/

	public Var getSubjectVar() {
		return _subjectVar;
	}

	public Var getObjectVar() {
		return _objectVar;
	}

	public void initialize(RdfSource rdfSource) 
		throws SailQueryException
	{
		_subjectVarLocked = _subjectVar.hasValue();
		_objectVarLocked = _objectVar.hasValue();

		// Special case: subjectVar and objectVar could be one and the
		// same variable, or they could have been implicitely joined.
		if (!_subjectVarLocked && !_objectVarLocked) {
			// Set dummy value on subject
			_subjectVar.setValue(URIImpl.RDF_TYPE);

			// Check value of object
			_subjAndObjAreEqual = (_objectVar.getValue() != null);

			// Unset dummy value
			_subjectVar.setValue(null);
		}

		Resource subjectValue;
		Resource objectValue;

		try {
			subjectValue = (Resource)_subjectVar.getValue();
			objectValue = (Resource)_objectVar.getValue();
		}
		catch (ClassCastException e) {
			// subject or object is not a Resource possibly due to a
			// wrong comparison in the where clause.
			_statementIter = new EmptyStatementIterator();
			return;
		}

		try {
			RdfSchemaSource rss = (RdfSchemaSource)rdfSource;
			_statementIter = rss.getDirectSubClassOf(subjectValue, objectValue);
		}
		catch (ClassCastException e) {
			// repository is not schema-enabled
			throw new SailQueryException("Cannot evaluate directSubClassOf on a non-inferencing repository");
		}
	}

	public boolean selectNext(RdfSource rdfSource) 
		throws SailQueryException 
	{
		while (_statementIter.hasNext()) {
			Statement st = _statementIter.next();

			if (_subjAndObjAreEqual) {
				// subject and object should also be equal
				if (!st.getSubject().equals(st.getObject())) {
					// They aren't, ignore this result
					continue;
				}
			}

			// object should be a resource
			if ( !(st.getObject() instanceof Resource) ) {
				continue;
			}

			// Set found values on variables
			if (!_subjectVarLocked) {
				_subjectVar.setValue(st.getSubject());
			}
			if (!_objectVarLocked) {
				_objectVar.setValue(st.getObject());
			}

			return true;
		}

		// No more results, release locks
		if (!_subjectVarLocked) {
			_subjectVar.setValue(null);
		}
		if (!_objectVarLocked) {
			_objectVar.setValue(null);
		}

		return false;
	}

	public void clear() {
		if (_statementIter != null) {
			// Close the statement iterator
			_statementIter.close();

			// Release locks on vars
			if (!_subjectVarLocked) {
				_subjectVar.setValue(null);
			}
			if (!_objectVarLocked) {
				_objectVar.setValue(null);
			}

			_statementIter = null;
		}
	}

	public void getVariables(Collection variables) {
		variables.add(_subjectVar);
		variables.add(_objectVar);
	}

	public String toString() {
		StringBuffer result = new StringBuffer(128);
		result.append("(");
		result.append(_subjectVar.toString());
		result.append(",directSubClassOf,");
		result.append(_objectVar.toString());
		result.append(")");
		return result.toString();
	}
}
