/*  Sesame - Storage and Querying architecture for RDF and RDF Schema
 *  Copyright (C) 2001-2005 Aduna
 *
 *  Contact: 
 *  	Aduna
 *  	Prinses Julianaplein 14 b
 *  	3817 CS Amersfoort
 *  	The Netherlands
 *  	tel. +33 (0)33 465 99 87
 *  	fax. +33 (0)33 465 99 87
 *
 *  	http://aduna.biz/
 *  	http://www.openrdf.org/
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.openrdf.sesame.sail.query;

import java.io.IOException;
import java.util.Collection;
import java.util.List;

import org.openrdf.sesame.sail.RdfSource;

/**
 * An abstract representation of an RDF query.
 *
 * @author Arjan Kleine
 * @author Jeen Broekstra
 */
public abstract class QueryComponent {

/*-------------------------------+
| Variables                      |
+-------------------------------*/

	/**
	 * Flag indicating whether any duplicates should be filtered
	 * from the results.
	 **/
	protected boolean _distinct;

	/**
	 * The offset (0-based) of the first query result to return. By default,
	 * all query results are returned (offset is 0). By setting the offset to
	 * N, the first N results will not be returned.
	 **/
	protected int _offset;

	/**
	 * The maximum number of results to return. By default, all query results
	 * are returned (limit is Integer.MAX_VALUE). By setting limit to
	 * N, the query engine will stop evaluating the query as soon as it has
	 * returned N results.
	 **/
	protected int _limit;

	/**
	 * A List of Projection items indicating which values to
	 * return in the query result.
	 **/
	protected List _projection;

	/**
	 * A List of PathExpression objects indicating the path in the
	 * RDF graph that are relevant for the query.
	 **/
	protected PathExpressionList _pathExpressionList;

/*-------------------------------+
| Methods                        |
+-------------------------------*/

	protected abstract void _reportQueryAnswer(QueryAnswerListener listener)
		throws IOException;
		
	/**
	 * Gets all variables that are used in this QueryComponent.
	 * @param variables A Collection to add the Var objects to.
	 **/
	public void getVariables(Collection variables) {
		_pathExpressionList.getVariables(variables);
	}
	
	public void setPathExpressionList(PathExpressionList pathExpressionList) {
		_pathExpressionList = pathExpressionList;
	}

	public PathExpressionList getPathExpressionList() {
		return _pathExpressionList;
	}

	public void evaluate(RdfSource rdfSource, QueryAnswerListener listener)
		throws SailQueryException
	{
		// Build relevant QueryAnswerListener chain from back to front
		QueryResultsCounter countListener = null;
		if (_limit < Integer.MAX_VALUE) {
			// LIMIT has been specified
			countListener = new QueryResultsCounter(listener);
			listener = countListener;
		}
		if (_offset > 0) {
			// OFFSET has been specified
			listener = new OffsetFilter(listener, _offset);
		}
		if (_distinct) {
			// DISTINCT has been specified
			listener = new DuplicatesFilter(listener);
		}
			
		try {
			
			// If _limit == 0 we don't have to do anything
			if (_limit > 0) {
				_pathExpressionList.initialize(rdfSource);
				boolean instantiated = _pathExpressionList.findFirst(rdfSource);
				
				while (instantiated) {
					_reportQueryAnswer(listener);
					
					if (countListener == null || countListener.getCount() < _limit) {
						// No limit is set, or limit has not yet been reached. Get next result.
						instantiated = _pathExpressionList.findNext(rdfSource);
					}
					else {
						// limit reached, don't return any more results
						instantiated = false;
					}
				}
				
				_pathExpressionList.clear();
			}
		}
		catch (IOException e) {
			_pathExpressionList.clear();
			throw new SailQueryException(e);
		}
	}
	
	public abstract String[] getColumnHeaders();
	
	public boolean getDistinctFlag() {
		return _distinct;
	}
	
	public void setDistinctFlag(boolean distinct) {
		_distinct = distinct;
	}
	
	public int getOffset() {
		return _offset;
	}
	
	public void setOffset(int offset) {
		_offset = offset;
	}
	
	public int getLimit() {
		return _limit;
	}
	
	public void setLimit(int limit) {
		_limit = limit;
	}
	
	/**
	 * Gets the projection of the query.
	 **/
	public List getProjection() {
		return _projection;
	}
}
