/*  Sesame - Storage and Querying architecture for RDF and RDF Schema
 *  Copyright (C) 2001-2005 Aduna
 *
 *  Contact: 
 *  	Aduna
 *  	Prinses Julianaplein 14 b
 *  	3817 CS Amersfoort
 *  	The Netherlands
 *  	tel. +33 (0)33 465 99 87
 *  	fax. +33 (0)33 465 99 87
 *
 *  	http://aduna.biz/
 *  	http://www.openrdf.org/
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.openrdf.sesame.sail.query;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.openrdf.model.Value;

/**
 * An abstract representation of an RDF query. The query consists of a number
 * of path expressions, additional boolean constraints on these path
 * expressions, and a projection describing which variable values should be
 * returned.
 *
 * @author Arjohn Kampman
 */
public class SfwQuery extends QueryComponent {

/*-------------------------------+
| Constructors                   |
+-------------------------------*/

	public SfwQuery() {
		this(false, new ArrayList(), new PathExpressionList());
	}

	public SfwQuery(boolean distinct,
			List projection, PathExpressionList pathExpressionList)
	{
		this(distinct, 0, Integer.MAX_VALUE, projection, pathExpressionList);
	}

	public SfwQuery(boolean distinct, int offset, int limit,
			List projection, PathExpressionList pathExpressionList)
	{
		_distinct = distinct;
		_offset = offset;
		_limit = limit;
		_projection = projection;
		_pathExpressionList = pathExpressionList;
	}

/*-------------------------------+
| Methods                        |
+-------------------------------*/

	/**
	 * Adds a variable to the projection of the query.
	 **/
	public void addProjectionVar(Var v) {
		_projection.add(v);
	}

	public String[] getColumnHeaders() {
		// Print column headers
		String columnHeaders[] = new String[_projection.size()];
	
		for (int i = 0; i < _projection.size(); i++) {
			ProjectionItem pi = (ProjectionItem)_projection.get(i);
			
			if (pi.getAlias() != null) {
				columnHeaders[i] = pi.getAlias();
			} else {
				columnHeaders[i] = pi.getValueExpr().toString();
			}
		}
		return columnHeaders;
	}

	protected void _reportQueryAnswer(QueryAnswerListener listener) 
		throws IOException
	{
		Value[] queryAnswer = new Value[_projection.size()];
		
		for (int i = 0; i < _projection.size(); i++) {
			ProjectionItem pi = (ProjectionItem)_projection.get(i);
			queryAnswer[i] = pi.getValueExpr().getValue(); 
		}

		listener.queryAnswer(queryAnswer);
	}
	
	public String toString() {
		StringBuffer result = new StringBuffer(256);
		result.append("select ");

		if (_distinct) {
			result.append("distinct ");
		}

		int varCount = _projection.size();
		for (int i = 0; i < varCount; i++) {
			Var var = (Var)_projection.get(i);

			result.append(var.toString());

			if (i < varCount - 1) {
				result.append(", ");
			}
		}

		result.append("\nfrom\n").append( _pathExpressionList.toString() );

		if (_limit < Integer.MAX_VALUE) {
			result.append("\nlimit ").append(_limit);
		}

		if (_offset > 0) {
			result.append("\noffset ").append(_offset);
		}

		return result.toString();
	}
}
