/*  Sesame - Storage and Querying architecture for RDF and RDF Schema
 *  Copyright (C) 2001-2005 Aduna
 *
 *  Contact: 
 *  	Aduna
 *  	Prinses Julianaplein 14 b
 *  	3817 CS Amersfoort
 *  	The Netherlands
 *  	tel. +33 (0)33 465 99 87
 *  	fax. +33 (0)33 465 99 87
 *
 *  	http://aduna.biz/
 *  	http://www.openrdf.org/
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.openrdf.sesame.sail.query;

import java.io.IOException;

import org.openrdf.model.BNode;
import org.openrdf.model.Resource;
import org.openrdf.model.URI;
import org.openrdf.model.Value;
import org.openrdf.model.ValueFactory;
import org.openrdf.model.impl.URIImpl;

import org.openrdf.sesame.query.GraphQueryResultListener;

public class TripleConstructor {

/*----------------+
| Variables       |
+----------------*/

	protected ValueExpr _subject;
	protected ValueExpr _predicate;
	protected ValueExpr _object;

	private ValueFactory _valFactory;
	
/*----------------+
| Constructors    |
+----------------*/

	/**
	 * Creates a new TripleConstructor.
	 **/
	public TripleConstructor(
		ValueExpr subject, ValueExpr predicate, ValueExpr object)
	{
		_subject = subject;
		_predicate = predicate;
		_object = object;
	}

	/**
	 * Creates a new TripleConstructor that inherits the subject,
	 * predicate and object of the supplied TriplePattern.
	 **/
	public TripleConstructor(TriplePattern triplePattern) {
		this(
			triplePattern.getSubjectVar(),
			triplePattern.getPredicateVar(),
			triplePattern.getObjectVar());
	}

	/**
	 * Creates a new TripleConstructor that inherits the subject and
	 * object of the supplied DirectSubClassOf. The predicate of the
	 * TripleConstructor will be <tt>rdfs:subClassOf</tt>.
	 **/
	public TripleConstructor(DirectSubClassOf directSubClassOf) {
		this(
			directSubClassOf.getSubjectVar(),
			new ResourceExpr(URIImpl.RDFS_SUBCLASSOF),
			directSubClassOf.getObjectVar());
	}

	/**
	 * Creates a new TripleConstructor that inherits the subject and
	 * object of the supplied DirectSubPropertyOf. The predicate of
	 * the TripleConstructor will be <tt>rdfs:subPropertyOf</tt>.
	 **/
	public TripleConstructor(DirectSubPropertyOf directSubPropertyOf) {
		this(
			directSubPropertyOf.getSubjectVar(),
			new ResourceExpr(URIImpl.RDFS_SUBPROPERTYOF),
			directSubPropertyOf.getObjectVar());
	}

	/**
	 * Creates a new TripleConstructor that inherits the subject and
	 * object of the supplied DirectType. The predicate of the
	 * TripleConstructor will be <tt>rdfs:subClassOf</tt>.
	 **/
	public TripleConstructor(DirectType directType) {
		this(
			directType.getSubjectVar(),
			new ResourceExpr(URIImpl.RDF_TYPE),
			directType.getObjectVar());
	}

/*----------------+
| Methods         |
+----------------*/

	public void setValueFactory(ValueFactory valFactory) {
		_valFactory = valFactory;
	}
	
	public ValueExpr getSubject() {
		return _subject;
	}

	public ValueExpr getPredicate() {
		return _predicate;
	}

	public ValueExpr getObject() {
		return _object;
	}
	
	/* Resets the values of subject and object variables to null if they are anonymous
	 * vars and the values are bnodes. this is to ensure unique bnode id's in query
	 * answers when anonymous nodes are used in the construct-clause.
	 */ 
	public void resetBNodes() {
		if (_subject instanceof Var) {
			Var subjVar = (Var)_subject;
			
			if (subjVar.isAnonymous() && subjVar.getValue() instanceof BNode) {
				subjVar.setValue(null);
			}
		}
		if (_object instanceof Var) {
			Var objVar = (Var)_object;
			
			if (objVar.isAnonymous() && objVar.getValue() instanceof BNode) {
				objVar.setValue(null);
			}
		}	
	}
	
	public void reportTriple(GraphQueryResultListener listener)
		throws IOException
	{
		// check if subject is an anonymous variable. In such cases a
		// new bnode should be constructed.
		if (_subject instanceof Var) {
			Var subjVar = (Var)_subject;
			
			if (subjVar.isAnonymous() && !subjVar.hasValue()) {
				subjVar.setValue(_valFactory.createBNode());
			}
		}
		
		// check if object is an anonymous variable. In such cases a
		// new bnode should be constructed.		
		if (_object instanceof Var) {
			Var objVar = (Var)_object;
			
			if (objVar.isAnonymous() && !objVar.hasValue()) {
				objVar.setValue(_valFactory.createBNode());
			}
		}		

		Value subj = _subject.getValue();
		Value pred = _predicate.getValue();
		Value obj  = _object.getValue();

		if (subj instanceof Resource && // includes check subj != null
			pred instanceof URI && // includes check pred != null
			obj != null)
		{
			listener.triple((Resource)subj, (URI)pred, obj);
		}
	}

	public String toString() {
		StringBuffer result = new StringBuffer();
		result.append("{");
		result.append(_subject.toString());
		result.append("} ");
		result.append(_predicate.toString());
		result.append(" {");
		result.append(_object.toString());
		result.append("}");
		return result.toString();
	}
}
