/*  Sesame - Storage and Querying architecture for RDF and RDF Schema
 *  Copyright (C) 2001-2005 Aduna
 *
 *  Contact: 
 *  	Aduna
 *  	Prinses Julianaplein 14 b
 *  	3817 CS Amersfoort
 *  	The Netherlands
 *  	tel. +33 (0)33 465 99 87
 *  	fax. +33 (0)33 465 99 87
 *
 *  	http://aduna.biz/
 *  	http://www.openrdf.org/
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.openrdf.sesame.sailimpl.nativerdf;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import org.openrdf.util.ByteArrayUtil;

import org.openrdf.sesame.sailimpl.nativerdf.btree.BTree;
import org.openrdf.sesame.sailimpl.nativerdf.btree.BTreeIterator;
import org.openrdf.sesame.sailimpl.nativerdf.btree.BTreeValueComparator;

/**
 **/
public class TripleStore {

/*-------------+
| Constants    |
+-------------*/

	private static final String FILENAME = "triples.dat";

	// 13 bytes are used to represent a triple
	static final int RECORD_LENGTH = 13;

	static final byte EXPLICIT_FLAG = (byte)0x1; // 0000 0001

/*-------------+
| Variables    |
+-------------*/

	private File _file;
	private BTree _btree;

/*-------------+
| Constructors |
+-------------*/

	public TripleStore(File dir)
		throws IOException
	{
		_file = new File(dir, FILENAME);
		_btree = new BTree(_file, 2048, 13, new TripleComparator());
	}

/*----------+
| Methods   |
+----------*/

	public void close()
		throws IOException
	{
		_btree.close();
		_btree = null;
		_file = null;
	}

	public BTreeIterator getTriples(int subj, int pred, int obj)
		throws IOException
	{
		byte[] searchKey = _getSearchKey(subj, pred, obj);
		byte[] searchMask = _getSearchMask(subj, pred, obj);

		if (subj == 0) {
			// Use sequential scan
			return _btree.iterateValues(searchKey, searchMask);
		}
		else {
			// Use ranged search
			byte[] minValue = _getMinValue(subj, pred, obj);
			byte[] maxValue = _getMaxValue(subj, pred, obj);
			return _btree.iterateValues(searchKey, searchMask, minValue, maxValue);
		}

	}

	public void startTransaction()
		throws IOException
	{
		_btree.startTransaction();
	}

	public void commitTransaction()
		throws IOException
	{
		_btree.commitTransaction();
	}

	public void rollbackTransaction()
		throws IOException
	{
		// FIXME: transaction rollback not implemented yet
		throw new IOException("transaction rollback not implemented yet");
	}

	public void clear()
		throws IOException
	{
		_btree.clear();
	}

	public void storeTriple(int subj, int pred, int obj)
		throws IOException
	{
		byte[] data = _getData(subj, pred, obj);
		_btree.insert(data);
	}

	public int removeTriples(int subj, int pred, int obj)
		throws IOException
	{
		// FIXME: naive implementation below
		ArrayList removeList = new ArrayList();

		BTreeIterator iter = getTriples(subj, pred, obj);
		byte[] value = iter.next();
		while (value != null) {
			removeList.add(value);
			value = iter.next();
		}

		for (int i = 0; i < removeList.size(); i++) {
			_btree.remove( (byte[])removeList.get(i) );
		}

		return removeList.size();
	}

	private byte[] _getData(int subj, int pred, int obj) {
		byte[] data = new byte[RECORD_LENGTH];

		ByteArrayUtil.putInt(subj, data, 0);
		ByteArrayUtil.putInt(pred, data, 4);
		ByteArrayUtil.putInt(obj, data, 8);
		data[12] = EXPLICIT_FLAG;

		return data;
	}

	private byte[] _getSearchKey(int subj, int pred, int obj) {
		byte[] searchKey = _getData(subj, pred, obj);
		searchKey[12] = 0;
		return searchKey;
	}

	private byte[] _getSearchMask(int subj, int pred, int obj) {
		byte[] mask = new byte[RECORD_LENGTH];

		if (subj != 0) {
			ByteArrayUtil.putInt(0xffffffff, mask, 0);
		}
		if (pred != 0) {
			ByteArrayUtil.putInt(0xffffffff, mask, 4);
		}
		if (obj != 0) {
			ByteArrayUtil.putInt(0xffffffff, mask, 8);
		}

		return mask;
	}

	private byte[] _getMinValue(int subj, int pred, int obj) {
		byte[] minValue = new byte[RECORD_LENGTH];

		// No need to check for values being equal to 0 as
		// this happens to be the minimum value anyway
		ByteArrayUtil.putInt(subj, minValue, 0);
		ByteArrayUtil.putInt(pred, minValue, 4);
		ByteArrayUtil.putInt(obj, minValue, 8);

		return minValue;
	}

	private byte[] _getMaxValue(int subj, int pred, int obj) {
		byte[] maxValue = new byte[RECORD_LENGTH];

		ByteArrayUtil.putInt( (subj == 0 ? 0xffffffff : subj), maxValue, 0);
		ByteArrayUtil.putInt( (pred == 0 ? 0xffffffff : pred), maxValue, 4);
		ByteArrayUtil.putInt( (obj  == 0 ? 0xffffffff : obj),  maxValue, 8);

		return maxValue;
	}

	private static final class TripleComparator implements BTreeValueComparator {

		// implements BTreeValueComparator.compareBTreeValues()
		public final int compareBTreeValues(byte[] key, byte[] data, int offset, int length) {
			int result = 0;
			for (int i = 0; result == 0 && i < RECORD_LENGTH - 1; i++) {
				result = (key[i] & 0xff) - (data[offset+i] & 0xff);
			}
			return result;
		}
	}
}
