/*  Sesame - Storage and Querying architecture for RDF and RDF Schema
 *  Copyright (C) 2001-2005 Aduna
 *
 *  Contact: 
 *  	Aduna
 *  	Prinses Julianaplein 14 b
 *  	3817 CS Amersfoort
 *  	The Netherlands
 *  	tel. +33 (0)33 465 99 87
 *  	fax. +33 (0)33 465 99 87
 *
 *  	http://aduna.biz/
 *  	http://www.openrdf.org/
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.openrdf.sesame.sailimpl.nativerdf;

import java.io.File;
import java.io.IOException;
import java.util.Iterator;
import java.util.LinkedList;

import org.openrdf.util.ByteArrayUtil;

import org.openrdf.model.BNode;
import org.openrdf.model.Literal;
import org.openrdf.model.Resource;
import org.openrdf.model.Statement;
import org.openrdf.model.URI;
import org.openrdf.model.Value;
import org.openrdf.model.ValueFactory;
import org.openrdf.model.impl.StatementImpl;

import org.openrdf.sesame.sailimpl.nativerdf.datastore.DataStore;
import org.openrdf.sesame.sailimpl.nativerdf.model.NativeBNode;
import org.openrdf.sesame.sailimpl.nativerdf.model.NativeLiteral;
import org.openrdf.sesame.sailimpl.nativerdf.model.NativeURI;
import org.openrdf.sesame.sailimpl.nativerdf.model.NativeValue;

/**
 * Class that provides indexed storage and retrieval of RDF values.
 **/
public class ValueStore implements ValueFactory {

/*-------------+
| Constants    |
+-------------*/

	private static final String FILENAME_PREFIX = "values";
	private static final int MRU_CACHE_SIZE = 16;

	private static final byte VALUE_TYPE_MASK = 0x3; // 0000 0011
	private static final byte URI_VALUE       = 0x1; // 0000 0001
	private static final byte BNODE_VALUE     = 0x2; // 0000 0010
	private static final byte LITERAL_VALUE   = 0x3; // 0000 0011

/*-------------+
| Variables    |
+-------------*/

	private DataStore _dataStore;
	private NamespaceStore _namespaceStore;
	private NativeRdfRepository _repository;

	/** A simple cache containing the [MRU_CACHE_SIZE] most-recently used values for getValue(int id). **/
	private LinkedList _mruCache;

	/** The prefix for any new bnode IDs. **/
	private String _bnodePrefix;

	/** The ID for the next bnode that is created. **/
	private int _nextBNodeID;

/*-------------+
| Constructors |
+-------------*/

	public ValueStore(File dataDir, NamespaceStore namespaceStore, NativeRdfRepository repository)
		throws IOException
	{
		_dataStore = new DataStore(dataDir, FILENAME_PREFIX);
		_namespaceStore = namespaceStore;
		_repository = repository;

		_mruCache = new LinkedList();
		
		_updateBNodePrefix();
		_nextBNodeID = 1;
	}

	private void _updateBNodePrefix() {
		// BNode prefix is based on currentTimeMillis(). Combined with a
		// sequential number per session, this gives a unique identifier.
		_bnodePrefix = "node" + Long.toString(System.currentTimeMillis(), 32) + "x";
	}

/*----------+
| Methods   |
+----------*/

	/**
	 * Gets the value for the specified ID.
	 *
	 * @param id A value ID.
	 * @return The value for the ID, or <tt>null</tt> no such value could be
	 * found.
	 * @exception IOException If an I/O error occurred.
	 **/
	public Value getValue(int id)
		throws IOException
	{
		NativeValue result = null;

		// Check MRU cache
		synchronized (_mruCache) {
			Iterator iter = _mruCache.iterator();
			while (iter.hasNext()) {
				NativeValue cacheValue = (NativeValue)iter.next();

				if (cacheValue.getInternalId() == id) {
					// Cache hit, move hit to most-recent position
					iter.remove();
					_mruCache.addFirst(cacheValue);

					result = cacheValue;
					break;
				}
			}
		}

		if (result == null) {
			// Value not in cache, fetch it from file
			byte[] data = _dataStore.getData(id);
			if (data != null) {
				result = _data2value(id, data);

				// Add value to MRU cache
				synchronized (_mruCache) {
					if (_mruCache.size() >= MRU_CACHE_SIZE) {
						// Cache full, remove least-recently used value
						_mruCache.removeLast();
					}
					_mruCache.addFirst(result);
				}
			}
		}

		return result;
	}

	/**
	 * Gets the ID for the specified value.
	 *
	 * @param value A value.
	 * @return The ID for the specified value, or <tt>0</tt> if no such ID could
	 * be found.
	 * @exception IOException If an I/O error occurred.
	 **/
	public int getID(Value value)
		throws IOException
	{
		return getID(value, false);
	}

	public int getID(Value value, boolean dirtyReads)
		throws IOException
	{
		boolean isOwnValue = _isOwnValue(value);
		if (isOwnValue) {
			int internalId = ((NativeValue)value).getInternalId();
			if (internalId != 0) {
				// internal ID has been set
				return internalId;
			}
		}

		int id = 0;
		byte[] data = _value2data(value, dirtyReads, false);

		if (data != null) {
			id = _dataStore.getID(data);

			if (isOwnValue) {
				// Store id in value for fast access in any consecutive calls
				((NativeValue)value).setInternalId(id);
			}
		}

		return id;
	}

	/**
	 * Starts a transaction. This prepares the ValueStore for upcoming updates
	 * to the stored data.
	 *
	 * @exception IOException If an I/O error occurred.
	 **/
	public void startTransaction(boolean isolateTransaction)
		throws IOException
	{
		_namespaceStore.startTransaction();
		_dataStore.startTransaction(isolateTransaction);
	}

	/**
	 * Commits a transaction, applying all updates that have been performed
	 * during the transaction.
	 *
	 * @exception IOException If an I/O error occurred.
	 **/
	public void commitTransaction()
		throws IOException
	{
		_dataStore.commitTransaction();
		_namespaceStore.commitTransaction();
	}

	/**
	 * Rolls back all updates that have been performed in the current
	 * transaction and closes it.
	 *
	 * @exception IOException If an I/O error occurred.
	 **/
	public void rollbackTransaction()
		throws IOException
	{
		_dataStore.rollbackTransaction();
		_namespaceStore.rollbackTransaction();
	}

	/**
	 * Stores the supplied value and returns the ID that has been assigned to
	 * it. In case the value was already present, the value will not be stored
	 * again and the ID of the existing value is returned. This method can only
	 * be called as part of a transaction. The supplied value will not be stored
	 * until the transaction has been committed.
	 *
	 * @param value The Value to store.
	 * @return The ID that has been assigned to the value.
	 * @exception IOException If an I/O error occurred.
	 * @see #startTransaction
	 * @see #commitTransaction
	 **/
	public int storeValue(Value value)
		throws IOException
	{
		int internalId = 0;

		// Try to get the internal ID from the value itself
		boolean isOwnValue = _isOwnValue(value);
		if (isOwnValue) {
			internalId = ((NativeValue)value).getInternalId();
		}

		if (internalId == 0) {
			// Try to get the internal ID from a value in the MRU Cache
			synchronized (_mruCache) {
				Iterator iter = _mruCache.iterator();
				while (iter.hasNext()) {
					NativeValue cacheValue = (NativeValue)iter.next();

					if (cacheValue.equals(value)) {
						// Cache hit, move hit to most-recent position
						iter.remove();
						_mruCache.addFirst(cacheValue);

						internalId = cacheValue.getInternalId();
						break;
					}
				}
			}
		}

		if (internalId != 0) {
			// value has already been stored, increase reference count only
			_dataStore.increaseReferenceCount(internalId);
			return internalId;
		}

		// Unable to get internal ID in a cheap way, just store it in the data
		// store which will handle duplicates
		byte[] valueData = _value2data(value, true, true);

		int id = _dataStore.storeData(valueData);

		if (isOwnValue) {
			((NativeValue)value).setInternalId(id);

			// Add value to MRU cache
			synchronized (_mruCache) {
				if (_mruCache.size() >= MRU_CACHE_SIZE) {
					// Cache full, remove least-recently used value
					_mruCache.removeLast();
				}
				_mruCache.addFirst(value);
			}
		}

		return id;
	}

	/**
	 * Removes the value for the specified ID. This method can only be called as
	 * part of a transaction. The specified value will not be removed until the
	 * transaction has been committed.
	 *
	 * @param id The ID of the value that should be removed.
	 * @exception IOException If an I/O error occurred.
	 * @see #startTransaction
	 * @see #commitTransaction
	 **/
	public void removeValue(int id)
		throws IOException
	{
		_dataStore.removeData(id);
	}

	/**
	 * Removes all values from the ValueStore. This method can only be called as
	 * part of a transaction. The ValueStore will not be cleared until the
	 * transaction is committed.
	 *
	 * @exception IOException If an I/O error occurred.
	 * @see #startTransaction
	 * @see #commitTransaction
	 **/
	public void clear()
		throws IOException
	{
		_dataStore.clear();
		_namespaceStore.clear();
		_nextBNodeID = 1;
	}

	/**
	 * Closes the ValueStore, releasing any file references, etc. In case a
	 * transaction is currently open, it will be rolled back. Once closed, the
	 * ValueStore can no longer be used.
	 *
	 * @exception IOException If an I/O error occurred.
	 **/
	public void close()
		throws IOException
	{
		_dataStore.close();
		_namespaceStore.close();
	}

	private byte[] _value2data(Value value, boolean dirtyReads, boolean create)
		throws IOException
	{
		if (value instanceof URI) {
			return _uri2data((URI)value, dirtyReads, create);
		}
		else if (value instanceof BNode) {
			return _bnode2data((BNode)value, dirtyReads, create);
		}
		else if (value instanceof Literal) {
			return _literal2data((Literal)value, dirtyReads, create);
		}
		else {
			throw new IllegalArgumentException("value parameter should be a URI, BNode or Literal");
		}
	}

	private byte[] _uri2data(URI uri, boolean dirtyReads, boolean create)
		throws IOException
	{
		// Get namespace ID
		int nsID = _namespaceStore.getID(uri.getNamespace(), dirtyReads);
		if (nsID == 0) {
			// Unknown namespace
			if (create) {
				// Add namespace
				nsID = _namespaceStore.storeNamespace(uri.getNamespace());
			}
			else {
				// Unknown namespace means unknown URI
				return null;
			}
		}

		// Get local name in UTF-8
		byte[] localNameData = uri.getLocalName().getBytes("UTF-8");

		// Combine parts in a single byte array
		byte[] uriData = new byte[5 + localNameData.length];
		uriData[0] = URI_VALUE;
		ByteArrayUtil.putInt(nsID, uriData, 1);
		ByteArrayUtil.put(localNameData, uriData, 5);

		return uriData;
	}

	private byte[] _bnode2data(BNode bNode, boolean dirtyReads, boolean create)
		throws IOException
	{
		byte[] idData = bNode.getID().getBytes("UTF-8");

		byte[] bNodeData = new byte[1 + idData.length];
		bNodeData[0] = BNODE_VALUE;
		ByteArrayUtil.put(idData, bNodeData, 1);

		return bNodeData;
	}

	private byte[] _literal2data(Literal literal, boolean dirtyReads, boolean create)
		throws IOException
	{
		// Get datatype ID
		int datatypeID = 0;
		if (literal.getDatatype() != null) {
			if (create) {
				datatypeID = storeValue(literal.getDatatype());
			}
			else {
				datatypeID = getID(literal.getDatatype(), dirtyReads);
				if (datatypeID == 0) {
					// Unknown datatype means unknown literal
					return null;
				}
			}
		}

		// Get language tag in UTF-8
		byte[] langData = null;
		int langDataLength = 0;
		if (literal.getLanguage() != null) {
			langData = literal.getLanguage().getBytes("UTF-8");
			langDataLength = langData.length;
		}

		// Get label in UTF-8
		byte[] labelData = literal.getLabel().getBytes("UTF-8");

		// Combine parts in a single byte array
		byte[] literalData = new byte[6 + langDataLength + labelData.length];
		literalData[0] = LITERAL_VALUE;
		ByteArrayUtil.putInt(datatypeID, literalData, 1);
		literalData[5] = (byte)langDataLength;
		if (langData != null) {
			ByteArrayUtil.put(langData, literalData, 6);
		}
		ByteArrayUtil.put(labelData, literalData, 6 + langDataLength);
	
		return literalData;
	}

	private NativeValue _data2value(int id, byte[] data)
		throws IOException
	{
		switch ( (data[0] & VALUE_TYPE_MASK) ) {
			case URI_VALUE: return _data2uri(id, data);
			case BNODE_VALUE: return _data2bnode(id, data);
			case LITERAL_VALUE: return _data2literal(id, data);
			default: throw new IllegalArgumentException("data does not specify a known value type");
		}
	}

	private NativeURI _data2uri(int id, byte[] data)
		throws IOException
	{
		int nsID = ByteArrayUtil.getInt(data, 1);
		String namespace = _namespaceStore.getNamespaceName(nsID);
		String localName = new String(data, 5, data.length-5, "UTF-8");
		return new NativeURI(_repository, namespace, localName, id);
	}

	private NativeBNode _data2bnode(int id, byte[] data)
		throws IOException
	{
		String nodeID = new String(data, 1, data.length-1, "UTF-8");
		return new NativeBNode(_repository, nodeID, id);
	}

	private NativeLiteral _data2literal(int id, byte[] data)
		throws IOException
	{
		// Get datatype
		int datatypeID = ByteArrayUtil.getInt(data, 1);
		URI datatype = null;
		if (datatypeID != 0) {
			datatype = (URI)getValue(datatypeID);
		}

		// Get language tag
		String lang = null;
		int langLength = data[5];
		if (langLength > 0) {
			lang = new String(data, 6, langLength, "UTF-8");
		}

		// Get label
		String label = new String(data, 6 + langLength, data.length - 6 - langLength, "UTF-8");

		if (datatype != null) {
			return new NativeLiteral(_repository, label, datatype, id);
		}
		else if (lang != null) {
			return new NativeLiteral(_repository, label, lang, id);
		}
		else {
			return new NativeLiteral(_repository, label, id);
		}
	}

	private boolean _isOwnValue(Value value) {
		return value instanceof NativeValue &&
			((NativeValue)value).getRepository() == _repository;
	}

/*------------------------------------+
| Methods from interface ValueFactory |
+------------------------------------*/

	// Implements ValueFactory.createURI(String)
	public URI createURI(String uri) {
		return new NativeURI(_repository, uri);
	}
	
	public URI createURI(String namespace, String localName) {
		return new NativeURI(_repository, namespace, localName);
	}

	// Implements ValueFactory.createBNode()
	public BNode createBNode() {
		if (_nextBNodeID == Integer.MAX_VALUE) {
			// Start with a new bnode prefix
			_updateBNodePrefix();
			_nextBNodeID = 1;
		}

		return new NativeBNode(_repository, _bnodePrefix + _nextBNodeID++);
	}
	
	// Implements ValueFactory.createBNode(String)
	public BNode createBNode(String nodeId) {
		return new NativeBNode(_repository, nodeId);
	}

	// Implements ValueFactory.createLiteral(String)
	public Literal createLiteral(String value) {
		return new NativeLiteral(_repository, value);
	}

	// Implements ValueFactory.createLiteral(String, String)
	public Literal createLiteral(String value, String language) {
		return new NativeLiteral(_repository, value, language);
	}

	// Implements ValueFactory.createLiteral(String, URI)
	public Literal createLiteral(String value, URI datatype) {
		return new NativeLiteral(_repository, value, datatype);
	}

	// Implements ValueFactory.createStatement(Resource, URI, Value)
	public Statement createStatement(Resource subject, URI predicate, Value object) {
		return new StatementImpl(subject, predicate, object);
	}

/*-------------------+
| Test/debug methods |
+-------------------*/

	public static void main(String[] args)
		throws Exception
	{
		File dataDir = new File(args[0]);
		NamespaceStore namespaceStore = new NamespaceStore(dataDir);
		ValueStore valueStore = new ValueStore(dataDir, namespaceStore, null);

		int maxID = valueStore._dataStore.getMaxID();
		for (int id = 1; id <= maxID; id++) {
			Value value = valueStore.getValue(id);
			System.out.println("ID " + id + " : " + value.toString());
		}
	}
}
