package com.mkdoc.jspider;

/*
    Created:    2005-03-15
    Issued:     2005-03-15
    Modified:   2005-07-14

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import net.javacoding.jspider.api.event.JSpiderEvent;

import net.javacoding.jspider.api.event.resource.ResourceFetchedEvent;

import net.javacoding.jspider.spi.Plugin;

import com.mkdoc.sax.RdfContentHandler;

import com.mkdoc.schema.ApplicationProfile;

import com.mkdoc.util.ApplicationContext;

import java.io.InputStream;

import java.net.URL;

/**
 *  A base class for plugins that process RDF statements.
 *  @version    0.6 2005-07-12
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public abstract class AbstractRdfPlugin extends AbstractMKDocPlugin
                                                  implements Plugin {

    /**
     *  A custom metadata application profile.
     *  @since  Version 0.1 2005-03-15
     */
    protected ApplicationProfile profile;

    /**
     *  Receive notification of a JSpider event.
     *  @param  event   A JSpider event.
     *  @since  Version 0.2 2005-04-27
     */
    public final void notify(final JSpiderEvent event) {

        if (event != null) {

            if (event instanceof ResourceFetchedEvent) {

                ResourceFetchedEvent rfe = (ResourceFetchedEvent) event;

                parse(rfe.getResource().getInputStream(),
                      rfe.getURL());

            }
        }
        else {

            throw new IllegalArgumentException(
                        "Null event reference not permitted.");
        }
    }

    /**
     *  Add a resource to the repository from an input stream.
     *  @param  inStream The input stream source for the metadata.
     *  @param  url The URL of the source document.
     *  @since  Version 0.2 2005-04-27
     */
    protected abstract void parse(final InputStream inStream,
                                  final URL url);

    /**
     *  Load a custom application profile if specified in a property set.
     *  @param  context The application context for the plugin, which may
     *          contain a custom <code>ApplicationProfile</code> class name.
     *  @return A status message for logging.
     *  @exception  ClassNotFoundException  If the custom profile cannot be
     *          located or instantiated.
     *  @exception  IllegalAccessException  If the classloader is not permitted
     *          to access the custom profile.
     *  @exception  InstantiationException  If the custom profile cannot be
     *          instantiated.
     *  @since  Version 0.4 2005-05-26
     */
    protected final String loadApplicationProfile(
                                              final ApplicationContext context)
        throws ClassNotFoundException, InstantiationException,
               IllegalAccessException {

        if (context != null) {

            Class profileClass = context.getClassParameter("profile");

            if (profileClass != null) {

                profile = (ApplicationProfile) profileClass.newInstance();

                return "Custom application profile " +
                       profile.getClass().getName() +
                       " loaded.";
            }
            else {

                return "No custom application profile configured.";
            }
        }
        else {

            throw new IllegalArgumentException(
                        "Null application context set not permitted.");
        }
    }

    /**
     *  Configure an RDF handler with a custom application profile.
     *  @param  handler The handler to configure.
     *  @since  Version 0.4 2005-05-26
     */
    protected final void configureHandler(final RdfContentHandler handler) {

        if (handler != null) {

            if (profile != null) {

                handler.setApplicationProfile(profile);
            }
        }
        else {

            throw new IllegalArgumentException(
                        "Null RDF content handler not permitted.");
        }
    }

}
