package com.mkdoc.jspider;

/*
    Created:    2005-04-11
    Issued:     2005-11-02
    Modified:   2005-11-02

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import net.javacoding.jspider.core.logging.LogFactory;

import net.javacoding.jspider.core.util.config.PropertySet;

import net.javacoding.jspider.spi.Plugin;

import com.mkdoc.store.StoreManager;
import com.mkdoc.store.StoreManagerFactory;

import com.mkdoc.util.ApplicationContext;

import java.io.InputStream;

import java.net.URL;

/**
 *  A plugin for adding native RDF documents to a repository.
 *  <dl>
 *    <dt>
 *      <code>manager</code>
 *    </dt>
 *    <dd>
 *      The fully qualified class name of a
 *      <code>com.mkdoc.store.StoreManager</code> implementation,
 *      <em>required</em>. May be
 *      <code>com.mkdoc.store.LocalStoreManager</code> or
 *      <code>com.mkdoc.store.DatabaseStoreManager</code>, for example.
 *    </dd>
 *    <dt>
 *      <code>output.absolute</code>
 *    </dt>
 *    <dd>
 *      Whether the output directory configuration specifies an absolute
 *      directory path. If not set, a relative path is assumed.
 *    </dd>
 *    <dt>
 *      <code>output.folder</code>
 *    </dt>
 *    <dd>
 *      The base file output directory, which may be relative to the JSpider
 *      installation directory. If not set, the default <code>output</code>
 *      directory beneath the JSpider installation directory is assumed.
 *    </dd>
 *    <dt>
 *      <code>profile</code>
 *    </dt>
 *    <dd>
 *      The fully qualified class name of a
 *      <code>com.mkdoc.schema.ApplicationProfile</code> implementation to
 *      override the default Dublin Core profile.
 *    </dd>
 *  </dl>
 *  @version    0.5 2005-11-02
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class RdfStoreWriterPlugin extends AbstractRdfPlugin implements Plugin {

    /**
     *  The plugin description.
     *  @since  Version 0.2 2005-05-31
     */
    private static final String DESCRIPTION =
    "Store RDF documents in a Sesame repository.";

    /**
     *  The plugin name.
     *  @since  Version 0.2 2005-05-31
     */
    private static final String NAME =
    "RDF document store plugin for JSpider";

    /**
     *  The plugin version.
     *  @since  Version 0.2 2005-05-31
     */
    private static final String VERSION = "v0.5";

    /**
     *  A reference to a <code>StoreManager</code> store interface.
     *  @since  Version 0.1 2005-05-31
     */
    protected StoreManager manager;

    /**
     *  Obtain the output configuration.
     *  @param  propertySet The configuration property set for
     *          this plugin.
     *  @since  Version 0.1 2005-05-31
     */
    public RdfStoreWriterPlugin(final PropertySet propertySet) {

        super();

        if (propertySet == null) {

            throw new IllegalArgumentException(
                        "Null property set not permitted.");
        }

        log = LogFactory.getLog(RdfStoreWriterPlugin.class);

        ApplicationContext context = getApplicationContext(propertySet);

        try {

            log.info(loadApplicationProfile(context));
        }
        catch (Exception e) {

            log.error("Could not load custom application profile. " +
                      e.getMessage());
        }
        try {

            manager = StoreManagerFactory.newStoreManager(context);
        }
        catch (Exception e) {

            log.error(e.getMessage());
        }
    }

    /**
     *  Add a resource to the repository from an input stream.
     *  @param  inStream The input stream source for the metadata.
     *  @param  url The URL of the source document.
     *  @since  Version 0.2 2005-04-27
     */
    protected final void parse(final InputStream inStream,
                               final URL url) {

        if ((inStream == null) ||
            (url == null)) {

            throw new IllegalArgumentException(
                        "Null arguments not permitted.");
        }

        try {

            log.info("Adding XML/RDF stream at " +
                     url.toExternalForm());

            manager.addXmlRdfStream(inStream, url);
        }
        catch (Exception e) {

            log.error("Could not add XML RDF data. " +
                      e.getMessage());
        }
    }

    /**
     *  Shut down the plugin.
     *  @since  Version 0.1 2005-05-31
     */
    public final void shutdown() {

        if (manager != null) {

            manager.shutdown();
        }
    }

    /**
     *  Get the plugin description.
     *  @return A short description of this plugin.
     *  @since  Version 0.2 2005-05-31
     */
    public final String getDescription() {

        return DESCRIPTION;
    }

    /**
     *  Get the plugin name.
     *  @return The name of this plugin.
     *  @since  Version 0.2 2005-05-31
     */
    public final String getName() {

        return NAME;
    }

    /**
     *  Get the plugin version.
     *  @return The version information for this plugin.
     *  @since  Version 0.2 2005-05-31
     */
    public final String getVersion() {

        return VERSION;
    }

}
