package com.mkdoc.jspider;

/*
    Created:    2005-03-14
    Issued:     2005-03-29
    Modified:   2005-11-02

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import net.javacoding.jspider.core.logging.LogFactory;

import net.javacoding.jspider.core.util.config.PropertySet;

import net.javacoding.jspider.spi.Plugin;

import com.mkdoc.sax.RdfContentHandler;
import com.mkdoc.sax.XhtmlStoreWriter;
import com.mkdoc.sax.XhtmlMetadataFilter;

import com.mkdoc.store.StoreManager;
import com.mkdoc.store.StoreManagerFactory;

import com.mkdoc.jtidy.TidyDriver;

import com.mkdoc.util.ApplicationContext;

import java.io.InputStream;

import java.net.URL;

import org.xml.sax.ContentHandler;
import org.xml.sax.InputSource;
import org.xml.sax.XMLFilter;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

/**
 *  A generic HTML metadata RDF store plugin.
 *  <dl>
 *    <dt>
 *      <code>manager</code>
 *    </dt>
 *    <dd>
 *      The fully qualified class name of a
 *      <code>com.mkdoc.store.StoreManager</code> implementation,
 *      <em>required</em>. May be
 *      <code>com.mkdoc.store.LocalStoreManager</code> or
 *      <code>com.mkdoc.store.DatabaseStoreManager</code>, for example.
 *    </dd>
 *    <dt>
 *      <code>output.absolute</code>
 *    </dt>
 *    <dd>
 *      Whether the output directory configuration specifies an absolute
 *      directory path. If not set, a relative path is assumed.
 *    </dd>
 *    <dt>
 *      <code>output.folder</code>
 *    </dt>
 *    <dd>
 *      The base file output directory, which may be relative to the JSpider
 *      installation directory. If not set, the default <code>output</code>
 *      directory beneath the JSpider installation directory is assumed.
 *    </dd>
 *    <dt>
 *      <code>profile</code>
 *    </dt>
 *    <dd>
 *      The fully qualified class name of a
 *      <code>com.mkdoc.schema.ApplicationProfile</code> implementation to
 *      override the default Dublin Core profile.
 *    </dd>
 *  </dl>
 *  @version    0.8 2005-11-01
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class XhtmlStoreWriterPlugin extends AbstractStoreManagerPlugin
                                                       implements Plugin {

    /**
     *  The plugin description.
     *  @since  Version 0.1 2005-03-14
     */
    private static final String DESCRIPTION =
    "Store RDF statements from (X)HTML documents in a Sesame repository.";

    /**
     *  The plugin name.
     *  @since  Version 0.1 2005-03-14
     */
    private static final String NAME =
    "XHTML metadata RDF store plugin for JSpider";

    /**
     *  The plugin version.
     *  @since  Version 0.1 2005-03-14
     */
    private static final String VERSION = "v0.8";

    /**
     *  Obtain the output configuration.
     *  @param  propertySet The configuration property set for
     *          this plugin.
     *  @since  Version 0.1 2005-03-15
     */
    public XhtmlStoreWriterPlugin(final PropertySet propertySet) {

        super();

        if (propertySet == null) {

            throw new IllegalArgumentException(
                        "Null property set not permitted.");
        }

        log = LogFactory.getLog(XhtmlStoreWriterPlugin.class);

        ApplicationContext context = getApplicationContext(propertySet);

        try {

            log.info(loadApplicationProfile(context));
        }
        catch (Exception e) {

            log.error("Could not load custom application profile. " +
                      e.getMessage());
        }
        try {

            manager = StoreManagerFactory.newStoreManager(context);
        }
        catch (Exception e) {

            log.error(e.getMessage());
        }
    }

    /**
     *  Parse an input stream.
     *  @param  inStream    The resource input stream.
     *  @param  url The resource URL.
     *  @since  Version 0.3 2005-04-27
     */
    public final void parse(final InputStream inStream,
                            final URL url) {

        if ((inStream == null) ||
            (url == null)) {

            throw new IllegalArgumentException(
                        "Null arguments not permitted.");
        }
        try {

            TidyDriver tidy = new TidyDriver(log);

            InputStream tidyInStream = null;

            tidyInStream = tidy.tidyInputStream(inStream);

            InputSource inSource = new InputSource(tidyInStream);


            XMLReader xmlReader = null;

            xmlReader = XMLReaderFactory.createXMLReader();

            XMLFilter xmlFilter = new XhtmlMetadataFilter();


            ContentHandler handler = null;

            handler = getContentHandler(manager, url);


            xmlFilter.setParent(xmlReader);

            xmlFilter.setContentHandler(handler);

            xmlFilter.parse(inSource);

        }
        catch (Exception e) {

            log.error("Error parsing input stream. " +
                      e.getMessage());
        }
    }

    /**
     *  Get a pre-configured content handler.
     *  @param  manager The subject manager interface to store the triples.
     *  @param  url An initial URI subject.
     *  @return An <code>RdfContentHandler</code> instance with custom
     *          <code>Schema</code> if configured.
     *  @since  Version 0.1 2005-03-14
     */
    protected final RdfContentHandler getContentHandler(
                                                  final StoreManager manager,
                                                  final URL url) {

        if ((manager == null) ||
            (url == null)) {

            throw new IllegalArgumentException(
                        "Null arguments not permitted.");
        }

        RdfContentHandler handler = new XhtmlStoreWriter(manager,
                                                         url);

        configureHandler(handler);

        return handler;
    }


    /**
     *  Get the plugin description.
     *  @return A short description of this plugin.
     *  @since  Version 0.1 2005-03-14
     */
    public final String getDescription() {

        return DESCRIPTION;
    }

    /**
     *  Get the plugin name.
     *  @return The name of this plugin.
     *  @since  Version 0.1 2005-03-14
     */
    public final String getName() {

        return NAME;
    }

    /**
     *  Get the plugin version.
     *  @return The version information for this plugin.
     *  @since  Version 0.1 2005-03-14
     */
    public final String getVersion() {

        return VERSION;
    }

}
