package com.mkdoc.jspider;

/*
    Created:    2005-02-28
    Issued:     2005-03-01
    Modified:   2005-11-02

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import net.javacoding.jspider.core.logging.LogFactory;

import net.javacoding.jspider.core.util.config.PropertySet;

import net.javacoding.jspider.spi.Plugin;

import com.mkdoc.jtidy.TidyDriver;

import com.mkdoc.sax.RdfContentHandler;
import com.mkdoc.sax.XhtmlMetadataFilter;
import com.mkdoc.sax.XhtmlTripleWriter;

import com.mkdoc.store.FileStoreManager;
import com.mkdoc.store.StoreManagerFactory;

import com.mkdoc.util.ApplicationContext;

import java.io.InputStream;

import java.net.URL;

import org.xml.sax.ContentHandler;
import org.xml.sax.InputSource;
import org.xml.sax.XMLFilter;
import org.xml.sax.XMLReader;

import org.xml.sax.helpers.XMLReaderFactory;

/**
 *  A generic HTML metadata triple writer plugin.
 *  <p>
 *    Creates a set of N-Triple documents that mirrors the HTML URL structure
 *    of a Web site. Current version does not escape high range characters per
 *    the N-Triple specification.
 *  </p>
 *  <dl>
 *    <dt>
 *      <code>manager</code>
 *    </dt>
 *    <dd>
 *      The fully qualified class name of a
 *      <code>com.mkdoc.store.FileStoreManager</code> implementation,
 *      <em>required</em>. Should normally be
 *      <code>com.mkdoc.store.TripleFileStoreManager</code>.
 *    </dd>
 *    <dt>
 *      <code>output.absolute</code>
 *    </dt>
 *    <dd>
 *      Whether the output directory configuration specifies an absolute
 *      directory path. If not set, a relative path is assumed.
 *    </dd>
 *    <dt>
 *      <code>output.folder</code>
 *    </dt>
 *    <dd>
 *      The base file output directory, which may be relative to the JSpider
 *      installation directory. If not set, the default <code>output</code>
 *      directory beneath the JSpider installation directory is assumed.
 *    </dd>
 *    <dt>
 *      <code>profile</code>
 *    </dt>
 *    <dd>
 *      The fully qualified class name of a
 *      <code>com.mkdoc.schema.ApplicationProfile</code> implementation to
 *      override the default Dublin Core profile.
 *    </dd>
 *  </dl>
 *  @version    0.7 2005-07-12
 *  @author     Philip Shaw <phil@mkdoc.com>
 *  @TODO   Default to TripleFileStoreManager if the manager is not configured
 *          in the application context.
 */
public class XhtmlTripleWriterPlugin extends AbstractFileStoreManagerPlugin
                                                            implements Plugin {

    /**
     *  The plugin description.
     *  @since  Version 0.1 2005-02-28
     */
    private static final String DESCRIPTION =
    "Writes N-Triple style RDF statements for (X)HTML documents.";

    /**
     *  The plugin name.
     *  @since  Version 0.1 2005-02-28
     */
    private static final String NAME =
    "XHTML metadata triple writer plugin for JSpider";

    /**
     *  The plugin version.
     *  @since  Version 0.1 2005-02-28
     */
    private static final String VERSION = "v0.7";

    /**
     *  Obtain the output configuration.
     *  @param  propertySet The configuration property set for
     *          this plugin.
     *  @since  Version 0.1 2005-03-01
     */
    public XhtmlTripleWriterPlugin(final PropertySet propertySet) {

        if (propertySet != null) {

            log = LogFactory.getLog(XhtmlTripleWriterPlugin.class);

            ApplicationContext context = getApplicationContext(propertySet);

            try {

                log.info(loadApplicationProfile(context));
            }
            catch (Exception e) {

                log.error("Could not load custom application profile. " +
                          e.getMessage());
            }

            try {

                manager = StoreManagerFactory.newFileStoreManager(context);
            }
            catch (Exception e) {

                log.error(e.getMessage());
            }
        }
        else {

            throw new IllegalArgumentException(
                        "Null property set not permitted.");
        }
    }

    /**
     *  Parse an input stream.
     *  @param  inStream    The resource input stream.
     *  @param  url The resource URL.
     *  @since  Version 0.3 2005-04-27
     */
    public final void parse(final InputStream inStream,
                            final URL url) {

        if (inStream != null) {

            if (url != null) {

                try {

                    TidyDriver tidy = new TidyDriver(log);

                    InputStream tidyInStream = null;

                    tidyInStream = tidy.tidyInputStream(inStream);

                    InputSource inSource = new InputSource(tidyInStream);


                    ContentHandler handler = getContentHandler(manager,
                                                               url);

                    XMLReader xmlReader = null;

                    xmlReader = XMLReaderFactory.createXMLReader();


                    XMLFilter xmlFilter = new XhtmlMetadataFilter();

                    xmlFilter.setParent(xmlReader);

                    xmlFilter.setContentHandler(handler);

                    xmlFilter.parse(inSource);

                }
                catch (Exception e) {

                    log.error("Could not write output. " +
                              e.getMessage());
                }
            }
            else {

                throw new IllegalArgumentException(
                            "Null resource URL not permitted.");
            }
        }
        else {

            throw new IllegalArgumentException(
                        "Null input stream not permitted.");
        }
    }

    /**
     *  Get a pre-configured content handler.
     *  @param  manager   A manager interface to write the triples to.
     *  @param  url An initial URI subject.
     *  @return An <code>RdfContentHandler</code> instance with custom
     *          <code>Schema</code> if configured.
     *  @since  Version 0.1 2005-03-01
     */
    protected final RdfContentHandler getContentHandler(
                                                final FileStoreManager manager,
                                                final URL url) {

        if (manager != null) {

            if (url != null) {


                RdfContentHandler handler = new XhtmlTripleWriter(manager,
                                                                  url);

                configureHandler(handler);

                return handler;
            }
            else {

                throw new IllegalArgumentException(
                            "Null subject URL not permitted.");
            }
        }
        else {

            throw new IllegalArgumentException(
                        "Null subject manager reference not permitted.");
        }
    }


    /**
     *  Get the plugin description.
     *  @return A short description of this plugin.
     *  @since  Version 0.1 2005-02-28
     */
    public final String getDescription() {

        return DESCRIPTION;
    }

    /**
     *  Get the plugin name.
     *  @return The name of this plugin.
     *  @since  Version 0.1 2005-02-28
     */
    public final String getName() {

        return NAME;
    }

    /**
     *  Get the plugin version.
     *  @return The version information for this plugin.
     *  @since  Version 0.1 2005-02-28
     */
    public final String getVersion() {

        return VERSION;
    }

}
