package com.mkdoc.query;

/*
    Created:    2005-07-14
    Issued:     2005-11=01
    Modified:   2005-11-01

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import java.io.IOException;

import javax.servlet.ServletOutputStream;

import javax.servlet.jsp.JspWriter;

/**
 *  A base class for result renderers.
 *  @version    0.8 2005-11-01
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public abstract class AbstractResultRenderer implements ResultRenderer {

    /**
     *  The query against which the results were returned.
     *  @since  Version 0.4 2005-07-26
     */
    protected Query query;

    /**
     *  Set the query for the results.
     *  @param  query   The query from which the results were obtained.
     *  @since  Version 0.3 2005-07-25
     */
    public final void setQuery(final Query query) {

        if (query == null) {

            throw new IllegalArgumentException(
                        "Null query URI not permitted.");
        }
        else {

            this.query = query;
        }
    }

    /**
     *  Get the query for the results.
     *  @return The query from which the results were obtained.
     *  @since  Version 0.3 2005-07-25
     */
    public final Query getQuery() {

        return query;
    }


    /**
     *  Render a query results table to a servlet output stream in
     *  <code>FULL</code> mode.
     *  @param  outputStream    The output stream to which to write the
     *          results.
     *  @param  queryResult The query result to render.
     *  @exception  IOException If there is a problem writing results to the
     *          output stream.
     *  @since  Version 0.6 2005-09-12
     */
    public final void renderQueryResult(final ServletOutputStream outputStream,
                                        final QueryResult queryResult)
        throws IOException {

        renderQueryResult(outputStream, queryResult, FULL);
    }

    /**
     *  Render a query results table to a servlet output stream.
     *  @param  outputStream    The output stream to which to write the
     *          results.
     *  @param  queryResult The query result to render.
     *  @param  mode    The output mode, either
     *          <code>ResultRenderer.FULL</code> or
     *          <code>ResultRenderer.INSERT</code>.
     *  @exception  IOException If there is a problem writing results to the
     *          output stream.
     *  @since  Version 0.1 2005-07-14
     */
    public final void renderQueryResult(final ServletOutputStream outputStream,
                                        final QueryResult queryResult,
                                        final int mode)
        throws IOException {

        if (outputStream == null) {

            throw new IllegalArgumentException(
                        "Null output stream not permitted.");
        }
        else {

            StringBuffer buffer = getQueryResult(queryResult, mode);

            outputStream.print(buffer.toString());
        }
    }

    /**
     *  Render a query result to a servlet output stream in <code>FULL</code>
     *  mode.
     *  <p>
     *    Implementations may render output according to a particular markup
     *    language or content type.
     *  </p>
     *  @param  jspWriter    The output stream to which to write the results.
     *  @param  queryResult The query result to render.
     *  @exception  IOException If there is a problem writing results to the
     *          output stream.
     *  @since  Version 0.6 2005-09-12
     */
    public final void renderQueryResult(final JspWriter jspWriter,
                                        final QueryResult queryResult)
        throws IOException {

        renderQueryResult(jspWriter, queryResult, FULL);
    }

    /**
     *  Render a query result to a JSP output stream.
     *  @param  jspWriter   The JSP output stream to which to write the
     *          results.
     *  @param  queryResult The query result to render.
     *  @param  mode    The output mode, either
     *          <code>ResultRenderer.FULL</code> or
     *          <code>ResultRenderer.INSERT</code>.
     *  @exception  IOException If there is a problem writing results to the
     *          output stream.
     *  @since  Version 0.2 2005-07-19
     */
    public final void renderQueryResult(final JspWriter jspWriter,
                                        final QueryResult queryResult,
                                        final int mode)
        throws IOException {

        if (jspWriter == null) {

            throw new IllegalArgumentException(
                        "Null JSP writer not permitted.");
        }
        else {

            StringBuffer buffer = getQueryResult(queryResult, mode);

            jspWriter.print(buffer.toString());
        }
    }

    /**
     *  Render a query result to a string.
     *  @param  queryResult The query results to render.
     *  @param  mode    The output mode, either
     *          <code>ResultRenderer.FULL</code> or
     *          <code>ResultRenderer.INSERT</code>.
     *  @return A text formatted tabular layout of the query result.
     *  @since  Version 0.1 2005-07-14
     */
    public final String renderQueryResult(final QueryResult queryResult,
                                          final int mode) {

        StringBuffer buffer = getQueryResult(queryResult, mode);

        return buffer.toString();
    }

    /**
     *  Template method for rendering query results to a string buffer.
     *  @param  queryResult The query results to render.
     *  @param  mode    The output mode, either
     *          <code>ResultRenderer.FULL</code> or
     *          <code>ResultRenderer.INSERT</code>.
     *  @return A text formatted tabular layout of the query results.
     *  @since  Version 0.1 2005-01-26
     */
    protected abstract StringBuffer getQueryResult(
                                                final QueryResult queryResult,
                                                final int mode);

    /**
     *  Get the start index for rendering results.
     *  @param  queryResult The query result for which to get the start index.
     *  @return The first results item index to render.
     *  @since  Version 0.5 2005-08-02
     */
    protected final int getStartIndex(final QueryResult queryResult) {

        if (queryResult == null) {

            throw new IllegalArgumentException(
                        "Null query result not permitted.");
        }
        else {

            int rows = queryResult.getRows();

            if (rows > 0) {

                int start = (query.getPageNumber() - 1) *
                             query.getResultsPerPage();

                if (start >= rows) {

                    return rows - query.getResultsPerPage();
                }
                else {

                    return start;
                }
            }
            else {

                return 0;
            }
        }
    }

    /**
     *  Get the end index for rendering results.
     *  @param  queryResult The query result for which to get the end index.
     *  @return The last results item index to render.
     *  @since  Version 0.5 2005-08-02
     */
    protected final int getEndIndex(final QueryResult queryResult) {

        if (queryResult == null) {

            throw new IllegalArgumentException(
                        "Null query result not permitted.");
        }
        else {

            int rows = queryResult.getRows();

            if (rows > 0) {

                int end = getStartIndex(queryResult) +
                          query.getResultsPerPage();

                if (end >= queryResult.getRows()) {

                    return queryResult.getRows();
                }
                else {

                    return end;
                }
            }
            else {

                return 0;
            }
        }
    }

}
