package com.mkdoc.query;

/*
    Created:    2005-07-05
    Issued:     2005-10-28
    Modified:   2005-10-28

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

/**
 *  A simple query implementation.
 *  @version    0.4 2005-08-02
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class SimpleQuery implements Query {

    /**
     *  A SeRQL query.
     *  @since  Version 0.1 2005-07-05
     */
    private final String query;

    /**
     *  A query-specific URI without type and page information.
     *  @since  Version 0.1 2005-07-05
     */
    private final String resultKey;

    /**
     *  The query context.
     *  @since  Version 0.4 2005-08-02
     */
    private final QueryContext context;

    /**
     *  Construct with a literal query and URI.
     *  @param  query   The literal query to wrap.
     *  @param  resultKey A distinct result key in the form of a query URI,
     *          without output type and paging information.
     *  @param  context The output context for the query.
     *  @since  Version 0.1 2005-07-05
     */
    public SimpleQuery(final String query,
                       final String resultKey,
                       final QueryContext context) {

        super();

        if ((query == null) ||
            (resultKey == null) ||
            (context == null)) {

            throw new IllegalArgumentException(
                        "Null parameters not permitted.");
        }

        this.query = query;
        this.resultKey = resultKey;
        this.context = context;
    }

    /**
     *  Get an SeRQL query statement.
     *  @return An SeRQL query statement.
     *  @since  Version 0.1 2005-07-05
     */
    public final String getSeRQLQuery() {

        return query;
    }

    /**
     *  Get the unique result key for the query.
     *  @return A unique key for the query result in the form of a URI without
     *          output type or paging information.
     *  @since  Version 0.4 2005-08-01
     */
    public final String getResultKey() {

        return resultKey;
    }

    /**
     *  Get a unique request URI for the query.
     *  @return A unique URI for the query.
     *  @since  Version 0.2 2005-07-26
     */
    public final String getQueryUri() {

        return getQueryUri(getPageNumber(),
                           getResultsPerPage());
    }

    /**
     *  Get a request URI for a result set by page number.
     *  @param  page    The page number for which to render a query URI.
     *  @return A unique URI for the query.
     *  @since  Version 0.2 2005-07-26
     */
    public final String getQueryUri(final int page) {

        if (page < 1) {

            throw new IllegalArgumentException(
                        "Negative page number not permitted.");
        }

        return getQueryUri(page,
                           getResultsPerPage());
    }

    /**
     *  Get a request URI for a result set by page number.
     *  @param  page    The page number for which to render a query URI.
     *  @param  limit   The number of results per page for the query.
     *  @return A unique URI for the query.
     *  @since  Version 0.2 2005-07-26
     */
    protected final String getQueryUri(final int page,
                                       final int limit) {

        if (page < 1) {

            throw new IllegalArgumentException(
                        "Negative page number not permitted.");
        }
        if (limit < 1) {

            throw new IllegalArgumentException(
                        "Must have at least one item per page.");
        }

        StringBuffer uri = new StringBuffer(getResultKey());

        uri.append("&amp;page=");
        uri.append(String.valueOf(page));
        uri.append("&amp;limit=");
        uri.append(String.valueOf(limit));
        uri.append("&amp;type=");
        uri.append(getOutputType());

        return uri.toString();
    }

    /**
     *  Get the requested page number for any results.
     *  @return The page number for any results
     *  @since  Version 0.3 2005-08-01
     */
    public final int getPageNumber() {

        return context.getPageNumber();
    }

    /**
     *  Get the maximum number of results to display per page.
     *  @return The maximum number of results to display per page.
     *  @since  Version 0.3 2005-08-01
     */
    public final int getResultsPerPage() {

        return context.getResultsPerPage();
    }

    /**
     *  Get the requested output format for the result.
     *  @return The output type for the result.
     *  @since  Version 0.4 2005-08-01
     */
    public final String getOutputType() {

        return context.getOutputType();
    }

}
