package com.mkdoc.query;

/*
    Created:    2005-02-02
    Issued:     2005-02-02
    Modified:   2005-10-12

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

/**
 *  Renders Sesame query results to XHTML.
 *  @version    0.7 2005-10-12
 *  @author     Philip Shaw <phil@mkdoc.com>
 *  @TODO       Optimise markup literal output.
 */
public class XhtmlResultRenderer extends AbstractResultRenderer
                                      implements ResultRenderer {

    /**
     *  The text content type.
     *  @since  Version 0.1 2005-02-02
     */
    public static final String CONTENT_TYPE = "text/html";

    /**
     *  An XHTML document header.
     *  @since  Version 0.1 2005-02-02
     */
    public static final String HEADER =
    "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" " +
    "\"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n" +
    "<html xmlns=\"http://www.w3.org/1999/xhtml\" lang=\"en-GB\" " +
    "dir=\"ltr\">\n" +
    "  <head  profile=\"http://dublincore.org/documents/dcq-html/\">\n" +
    "    <title>\n" +
    "      MKSearch: Query results\n" +
    "    </title>\n" +
    "  </head>\n" +
    "  <body>\n";

    /**
     *  An XHTML document footer.
     *  @since  Version 0.1 2005-02-02
     */
    public static final String FOOTER =
    "   </body>\n" +
    "</html>\n";

    /**
     *  Terminate an HTML attribute and tag.
     *  @since  Version 0.1 2005-10-12
     */
    private static final String CLOSE_ATTR_TAG = "\">";

    /**
     *  The maximum number of pages in the results navigation.
     *  @since  Version 0.1 2005-02-02
     */
    public static final int PAGE_LIMIT = 8;


    /**
     *  The trailing &quot;lag&quot; behind the current results page.
     *  @since  Version 0.1 2005-02-02
     */
    public static final int LAG = 3;

    /**
     *  Construct a renderer for a specific query.
     *  @param  query   The query against which the results were returned.
     *  @since  Version 0.5 2005-07-26
     */
    public XhtmlResultRenderer(final Query query) {

        super();

        if (query == null) {

            throw new IllegalArgumentException(
                        "Null query not permitted.");
        }
        else {

            this.query = query;
        }
    }

    /**
     *  Render a query result to a string buffer.
     *  @param  queryResult   The query results to render.
     *  @param  mode    The output mode, either
     *          <code>ResultRenderer.FULL</code> or
     *          <code>ResultRenderer.INSERT</code>.
     *  @return A text formatted tabular layout of the query results.
     *  @since  Version 0.1 2005-02-02
     */
    protected final StringBuffer getQueryResult(final QueryResult queryResult,
                                                final int mode) {

        StringBuffer buffer = new StringBuffer();

        if (queryResult == null) {

            throw new IllegalArgumentException(
                        "Null query result not permitted.");
        }
        else {

            if (mode == FULL) {

                buffer.append(HEADER);
            }

            int rows = queryResult.getRows();

            if (rows > 0) {

                int start = getStartIndex(queryResult);

                int end = getEndIndex(queryResult);

                buffer.append("<p class=\"Summary\">");

                if (rows > 1) {

                    buffer.append("Results ");
                }
                else {

                    buffer.append("Result ");
                }

                buffer.append("<strong>");
                buffer.append(start + 1);

                if (rows > 1) {

                    buffer.append("</strong> to <strong>");
                    buffer.append(end);
                }

                buffer.append("</strong> of <strong>");
                buffer.append(String.valueOf(rows));
                buffer.append("</strong>");
                buffer.append("</p>");

                buffer.append("<dl>\n");

                for (int row = start; row < end; row++) {

                    buffer.append("<dt>\n");
                    buffer.append("<strong>");
                    buffer.append(row + 1);
                    buffer.append(".</strong> <a href=\"");
                    buffer.append(queryResult.getItemUri(row));
                    buffer.append(CLOSE_ATTR_TAG);
                    buffer.append(queryResult.getItemTitle(row));
                    buffer.append("</a>\n");
                    buffer.append("</dt>\n");
                    buffer.append("<dd>\n");
                    buffer.append(queryResult.getItemSummary(row));
                    buffer.append("\n</dd>\n");
                }

                buffer.append("</dl>\n");

                appendNavigation(queryResult, buffer);
            }
            else {
                buffer.append("<p>\n");
                buffer.append("  <strong>No results for this query.</strong> ");
                buffer.append("Please check the help pages for advice about ");
                buffer.append("how to find what you are looking for.\n");
                buffer.append("</p>\n");
            }

            if (mode == FULL) {

                buffer.append(FOOTER);
            }

            return buffer;
        }
    }

    /**
     *  Append a result page navigation bar if necessary.
     *  @param  queryResult The query result to render.
     *  @param  buffer  The output buffer to which to append the navigation.
     */
    private void appendNavigation(final QueryResult queryResult,
                                  final StringBuffer buffer) {

        if (queryResult.getRows() > query.getResultsPerPage()) {

            buffer.append("<p class=\"ResultNavigation\">Result page: ");

            int totalPages =  queryResult.getRows() / query.getResultsPerPage();

            if (queryResult.getRows() % query.getResultsPerPage() > 0) {

                totalPages += 1;
            }

            int startPage = 1;

            int endPage = totalPages;

            if (totalPages > PAGE_LIMIT) {

                if (query.getPageNumber() > LAG) {

                    if ((query.getPageNumber() - LAG) +
                        PAGE_LIMIT < totalPages) {

                        startPage = query.getPageNumber() - LAG;
                    }
                    else {

                        startPage = totalPages - PAGE_LIMIT;
                    }
                }

                if (startPage + PAGE_LIMIT < totalPages) {

                    endPage = startPage + PAGE_LIMIT;
                }
            }

            appendPreviousNavigation(query, buffer);

            appendPageNavigation(query, buffer, startPage, endPage);

            appendNextNavigation(query, buffer, totalPages);

            buffer.append("</p>\n");
        }
    }

    /**
     *  Render the numbered results page navigation.
     *  @param  query   The query for which to append the page navigation.
     *  @param  buffer  The buffer to append the navigation to.
     *  @param  startPage   The result page to start at.
     *  @param  endPage The result page to end at.
     *  @since  Version 0.7 2005-10-12
     */
    private void appendPageNavigation(final Query query,
                                      final StringBuffer buffer,
                                      final int startPage,
                                      final int endPage) {

        for (int i = startPage; i <= endPage; i++) {

            if (i == query.getPageNumber()) {

                buffer.append("<strong>");
                buffer.append(String.valueOf(i));
                buffer.append("</strong> ");
            }
            else {

                buffer.append("<a href=\"");
                buffer.append(query.getQueryUri(i));
                buffer.append(CLOSE_ATTR_TAG);
                buffer.append(String.valueOf(i));
                buffer.append("</a> ");
            }
        }
    }


    /**
     *  Render any &quot;Previous&quot; navigation.
     *  @param  query   The query for which to append a previous navigation.
     *  @param  buffer  The buffer to append the navigation to.
     *  @since  Version 0.7 2005-10-12
     */
    private void appendPreviousNavigation(final Query query,
                                          final StringBuffer buffer) {

        if (query.getPageNumber() > 1) {

            buffer.append("<strong><a href=\"");
            buffer.append(query.getQueryUri(query.getPageNumber() - 1));
            buffer.append(CLOSE_ATTR_TAG);
            buffer.append("&lt;&lt; Previous");
            buffer.append("</a></strong> ");
        }
    }

    /**
     *  Render any &quot;Next&quot; navigation.
     *  @param  query   The query for which to append a next navigation.
     *  @param  buffer  The buffer to append the navigation to.
     *  @param  totalPages  The total number of pages in the result set.
     *  @since  Version 0.7 2005-10-12
     */
    private void appendNextNavigation(final Query query,
                                      final StringBuffer buffer,
                                      final int totalPages) {

        if (query.getPageNumber() < totalPages) {

            buffer.append("<strong><a href=\"");
            buffer.append(query.getQueryUri(query.getPageNumber() + 1));
            buffer.append(CLOSE_ATTR_TAG);
            buffer.append("Next &gt;&gt;");
            buffer.append("</a></strong>");
        }
    }

    /**
     *  Get the content type for the renderer.
     *  @return The MIME type <code>text/html</code>.
     *  @since  Version 0.1 2005-02-02
     */
    public final String getContentType() {

        return CONTENT_TYPE;
    }

}
