package com.mkdoc.sax;

/*
    Created:    2004-12-22
    Issued:     2004-12-22
    Modified:   2005-06-06

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2004 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import com.mkdoc.schema.ApplicationProfile;
import com.mkdoc.schema.DublinCoreProfile;

import java.util.List;
import java.util.StringTokenizer;
import java.util.Vector;

/**
 *  Base class for RDF content handlers.
 *  @version    1.6 2005-05-31
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public abstract class AbstractRdfContentHandler
                 extends AbstractContentHandler
                   implements RdfContentHandler {

    /**
     *  The XHTML 1.0 namespace URI.
     *  @since  Version 0.9 2005-02-15
     */
    public static final String NS_XHTML1 =
    "http://www.w3.org/1999/xhtml";

    /**
     *  The RDF type URI.
     *  @since  Version 0.3 2005-01-06
     */
    public static final String RDF_TYPE =
    "http://www.w3.org/1999/02/22-rdf-syntax-ns#type";

    /**
     *  The RDF value URI.
     *  @since  Version 0.3 2005-01-06
     */
    public static final String RDF_VALUE =
    "http://www.w3.org/1999/02/22-rdf-syntax-ns#value";

    /**
     *  The Dublin Core terms URI type.
     *  @since  Version 0.3 2005-01-06
     */
    public static final String DCTERMS_URI = "http://purl.org/dc/terms/URI";

    /**
     *  The Dublin Core metadata scheme prefix.
     *  @since  Version 0.3 2005-01-06
     */
    public static final String DC_PREFIX = "dc";

    /**
     *  A number of schemes that may be used by a URI.
     *  @since  Version 0.1 2004-12-22
     */
    protected static final String[] URI_SCHEMES = new String[] {"http:",
                                                                "mailto:",
                                                                "https:",
                                                                "ftp:"};

    /**
     *  A blank node prefix.
     *  @since  Version 0.1 2004-12-22
     */
    protected static final String NODE_PREFIX = "_:";

    /**
     *  A blank node suffix.
     *  @since  Version 0.5 2005-01-07
     */
    protected static final String NODE_SUFFIX = "mkdoc";

    /**
     *  A universally unique blank node index.
     *  @TODO   Actually only unique for the current
     *          classloader. Need a mechanism to make an
     *          always unique ID.
     *  @since  Version 0.3 2005-01-06
     */
    private static int nodeIndex = 0;

    /**
     *  The subject of the RDF statement.
     *  @since  Version 0.4 2005-01-07
     */
    protected String subject;

    /**
     *  An application profile to resolve metadata values.
     *  @since  Version 1.4 2005-05-25
     */
    protected ApplicationProfile profile = new DublinCoreProfile();

    /**
     *  Set the application profile for the handler.
     *  @param  profile The application profile to be used.
     *  @since  Version 1.5 2005-05-26
     */
    public final void setApplicationProfile(final ApplicationProfile profile) {

        if (profile != null) {

            this.profile = profile;
        }
        else {

            throw new IllegalArgumentException(
                        "Null application profile not permitted.");
        }
    }

    /**
     *  Set the subject of the RDF content handler.
     *  @param  uri The URI of the subject, such as the URL
     *          of an (X)HTML document.
     *  @since  Version 0.4 2005-01-07
     */
    public final void setSubject(final String uri) {

        if (uri != null) {

            this.subject = uri;
        }
        else {

            throw new IllegalArgumentException(
                        "Null URI subject not permitted.");
        }
    }

    /**
     *  Get the current subject URI for the RDF handler.
     *  @return The URI of the RDF statement, which may be
     *          the URL of an (X)HTML document.
     *  @since  Version 0.4 2005-01-07
     */
    public final String getSubject() {

        return subject;
    }

    /**
     *  Expand a schema encoded value to a URI.
     *  @param  predicate   The encoded triple predicate to
     *          expand in the form &quot;DC.Title&quot; for
     *          instance.
     *  @return The expanded URI reference for the triple
     *          predicate or the original string if it is
     *          not matched.
     *  @since  Version 0.1 2004-12-22
     */
    public final String expandPredicate(final String predicate) {

        return profile.expandPredicate(predicate);
    }

    /**
     *  Expand a space separated list of schema encoded values.
     *  <p>
     *    Intended to extract Dublin Core encoded values from HTML
     *    <code>link</code> elements' <code>rel</code> or <code>rev</code>
     *    attributes, which may be mixed with generic link types. Distinguishes
     *    properties that are <em>not</em> encoded metadata predicates and
     *    silently drops them.
     *  </p>
     *  @param  list    The encoded, space separated link list.
     *  @return An array of expanded URI references only.
     *  @since  Version 0.8 2005-02-14
     */
    public final String[] expandPredicates(final String list) {

        if (list != null) {

            String trimmed = list.trim();

            StringTokenizer tokens = new StringTokenizer(trimmed, " ");

            List tokenList = new Vector();

            String item = null;

            String uri = null;

            while (tokens.hasMoreTokens()) {

                item = tokens.nextToken();

                uri = expandPredicate(item);

                if (!item.equals(uri)) {

                    tokenList.add(uri);
                }
            }

            int size = tokenList.size();

            return (String[]) tokenList.toArray(new String[size]);
        }
        else {

            throw new IllegalArgumentException(
                        "Null link list not permitted.");
        }
    }

    /**
     *  Expand an encoding scheme to a URI.
     *  @param  scheme  The encoded triple component to
     *          expand in the form
     *          &quot;DCTERMS.W3CDTF&quot; for instance.
     *  @return The expanded URI reference for the triple
     *          component or the original string if it is
     *          not matched.
     *  @since  Version 0.3 2005-01-06
     */
    public final String expandScheme(final String scheme) {

        return profile.expandScheme(scheme);
    }

    /**
     *  Escape an N-Triple URI or literal.
     *  @param  component   An N-Triple component.
     *  @return An N-Triple escaped URI or literal.
     *  @since  Verison 0.1 2004-12-22
     */
    public static final String escape(final String component) {

        if (component != null) {

            if (isUri(component)) {

                return "<" + component + ">";
            }
            else {

                return "\"" + component + "\"";
            }
        }
        else {

            throw new IllegalArgumentException(
                        "Null component not permitted.");
        }
    }

    /**
     *  Check whether a component is an absolute URI.
     *  <p>
     *    Only checks for a number of common URI
     *    <em>scheme</em> prefixes, the full URI is not
     *    validated:
     *  </p>
     *  @param  component   The the string to check.
     *  @return <code>true</code> if the component appears
     *          to be a URI.
     *  @since  Version 0.1 2004-12-22
     */
    public static final boolean isUri(final String component) {

        boolean isURI = false;

        if (component != null) {

            String lower = component.toLowerCase();

            for (int i = 0; i < URI_SCHEMES.length; i++) {

                if (lower.startsWith(URI_SCHEMES[i])) {

                    isURI = true;
                    break;
                }
            }
        }
        return isURI;
    }

    /**
     *  Get a blank node.
     *  @return A unique blank node reference.
     *  @since  Version 0.3 2005-01-06
     */
    public final String getBNode() {

        return NODE_PREFIX + NODE_SUFFIX + (++nodeIndex);
    }

    /**
     *  Get a blank node identifier.
     *  @return A unique blank node identifier.
     *  @since  Version 0.6 2005-01-13
     */
    public final String getBNodeId() {

        return NODE_SUFFIX + (++nodeIndex);
    }

}
