package com.mkdoc.sax;

/*
    Created:    2005-02-14
    Issued:     2005-02-14
    Modified:   2005-11-01

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import org.openrdf.model.BNode;
import org.openrdf.model.Graph;
import org.openrdf.model.URI;
import org.openrdf.model.ValueFactory;

import org.openrdf.model.impl.GraphImpl;

import com.mkdoc.schema.ApplicationProfile;

import com.mkdoc.store.StoreManager;

import java.net.URL;

import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;

/**
 *  A Sesame repository writer for XHTML <code>link</code>
 *  content.
 *  <p>
 *    Generates RDF <code>Graph</code> objects and adds
 *    them to a Sesame repository. Supports Dublin Core
 *    element and qualified terms schemata.
 *  </p>
 *  @version    0.7 2005-11-01
 *  @author     Philip Shaw <phil@mkdoc.com>
 *  @TODO   Reduce the complexity of the addTriples method.
 */
public class LinkStoreWriter extends AbstractStoreWriter
                             implements ContentHandler, ElementRdfWriter {

    /**
     *  The local name of the <code>link</code> element.
     *  @since  Version 0.2 2005-02-15
     */
    public static final String LINK = "link";

    /**
     *  Package private default constructor for composition, default schemas.
     *  @since  Version 0.2 2005-02-15
     */
    LinkStoreWriter() {

        super();
    }

    /**
     *  Package private constructor for custom application profile.
     *  @param  profile A custom application profile for the parse.
     *  @since  Version 0.5 2005-05-25
     */
    LinkStoreWriter(final ApplicationProfile profile) {

        super();

        if (profile == null) {

            throw new IllegalArgumentException(
                        "Null application profile not permitted.");
        }
        else {

            this.profile = profile;
        }
    }


    /**
     *  Construct from a store manager and initial URI.
     *  @param  manager A subject manager interface for storing statements.
     *  @param  url An initial URI subject.
     *  @since  Version 0.1 2005-02-14
     */
    public LinkStoreWriter(final StoreManager manager,
                           final URL url) {

        super();

        if ((manager == null) ||
            (url == null)) {

            throw new IllegalArgumentException(
                        "Null arguments not permitted.");
        }

        this.manager = manager;

        graph = new GraphImpl();

        ValueFactory factory = graph.getValueFactory();

        subjectUri = factory.createURI(url.toExternalForm());
    }

    /**
     *  Report whether an instance accepts named element types.
     *  @param  uri The namespace URI for the document type.
     *  @param  localName   The local name of the element to check.
     *  @return <code>true</code> if this type is capable of handling the
     *          specified element.
     *  @since  Version 0.2 2005-02-15
     */
    public final boolean acceptElement(final String uri,
                                       final String localName) {

        if ((uri == null) ||
            (localName == null)) {

            throw new IllegalArgumentException(
                        "Null arguments not permitted.");
        }
        else {

            return ((uri.equals(NS_XHTML1)) &&
                    (localName.equals(LINK)));
        }
    }

    /**
     *  Extract <code>link</code> element content.
     *  @param  uri The namespace URI for the element.
     *  @param  localName   The local element name.
     *  @param  qName   The qualified element name.
     *  @param  attributes    A set of attributes.
     *  @exception  SAXException    If the element is not a <code>link</code>
     *          element or the instance is was not constructed with an output
     *          source and subject URI.
     *  @since  Version 0.1 2005-02-14
     */
    public final void startElement(final String uri,
                                   final String localName,
                                   final String qName,
                                   final Attributes attributes)
        throws SAXException {

        if (acceptElement(uri, localName)) {

            if ((subjectUri == null) ||
                (graph == null)) {

                throw new SAXException(
                            "Content handler not configured for this parse, " +
                            "has the start document event been called?");
            }
            else {

                addTriples(subjectUri, attributes, graph);
            }
        }
        else {

            throw new SAXException("Cannot process element '" +
                                   qName + "'.");
        }
    }

    /**
     *  Add statements to an RDF graph for a source URI.
     *  @param  sourceUri   The URI for the source document.
     *  @param  attributes  The element attribute set to process.
     *  @param  graph   An RDF graph to add the statements to.
     *  @since  Version 0.1 2005-02-14
     */
    public final void addTriples(final URI sourceUri,
                                 final Attributes attributes,
                                 final Graph graph) {

        if ((sourceUri == null) ||
            (attributes == null) ||
            (graph ==  null)) {

            throw new IllegalArgumentException(
                        "Null arguments not permitted.");
        }
        else {

            int length = attributes.getLength();

            String name = null;

            String[] predicates = null;

            String object = null;

            String value = null;

            // rel is a "forward" relation, rev is reversed
            boolean forward = false;

            for (int i = 0; i < length; i++) {

                name = attributes.getLocalName(i);

                if (name.equalsIgnoreCase("rel")) {

                    forward = true;

                    value = getAttributeValue(attributes, i);

                    predicates = expandPredicates(value);
                }
                else {

                    if (name.equalsIgnoreCase("rev")) {

                        forward = false;

                        value = getAttributeValue(attributes, i);

                        predicates = expandPredicates(value);
                    }
                }
                if (name.equalsIgnoreCase("href")) {

                    value = getAttributeValue(attributes, i);

                    object = value;
                }
            }
            if ((predicates != null) &&
                (predicates.length > 0) &&
                (object != null)) {

                ValueFactory factory = graph.getValueFactory();

                BNode bNode = null;

                int items = predicates.length;

                if (forward) {

                    for (int i = 0; i < items; i++) {

                        bNode = factory.createBNode(getBNodeId());

                        graph.add(sourceUri,
                                  factory.createURI(predicates[i]),
                                  bNode);

                        graph.add(bNode,
                                  factory.createURI(RDF_TYPE),
                                  factory.createURI(DCTERMS_URI));

                        graph.add(bNode,
                                  factory.createURI(RDF_VALUE),
                                  factory.createURI(object));
                    }
                }
                else {

                    for (int i = 0; i < items; i++) {

                        bNode = factory.createBNode(getBNodeId());

                        graph.add(factory.createURI(object),
                                  factory.createURI(predicates[i]),
                                  bNode);

                        graph.add(bNode,
                                  factory.createURI(RDF_TYPE),
                                  factory.createURI(DCTERMS_URI));

                        graph.add(bNode,
                                  factory.createURI(RDF_VALUE),
                                  sourceUri);
                    }
                }
            }
        }
    }

}
