package com.mkdoc.schema;

/*
    Created:    2004-12-23
    Issued:     2004-12-23
    Modified:   2005-06-16

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2004 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

/**
 *  A base implementation for metadata schemas.
 *  @version    0.7 2005-06-15
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public abstract class AbstractSchema implements Schema {

    /**
     *  Expand an encoded predicate to a URI.
     *  @param  predicate   The predicate to expand.
     *  @return The expanded URI reference for the
     *          predicate or the original string if it is
     *          not matched.
     *  @since  Version 0.1 2004-12-22
     */
    public final String expandPredicate(final String predicate) {

        return expandPredicate(predicate, predicate);
    }

    /**
     *  A recursive method to expand a predicate from a dot encoded value.
     *  @param  name    The initial or remaining part of the encoded predicate,
     *          in the form <code>DC.Date</code>,
     *          <code>DC.Date.modified</code>, or truncated to
     *          <code>Date</code> or <code>modified</code> respectively.
     *  @param  predicate   A copy of the original predicate value to fall back
     *          to.
     *  @return The expanded predicate URI, if matched, or the
     *          <code>predicate</code> argument.
     *  @since  Version 0.2 2005-03-10
     */
    protected abstract String expandPredicate(final String name,
                                              final String predicate);

    /**
     *  Expand an encoded scheme value to a URI.
     *  @param  scheme  The scheme to expand.
     *  @return The expanded URI reference for the scheme,
     *          or the original string if it is not
     *          matched.
     *  @since  Version 0.3 2005-05-24
     */
    public final String expandScheme(final String scheme) {

        return expandScheme(scheme, scheme);
    }

    /**
     *  Expand an encoded scheme value to a URI.
     *  <p>
     *    Allows recursive handling of schema prefixes.
     *  </p>
     *  @param  name    The encoding scheme name to expand. Initially, this may
     *          include a schema prefix (i.e. <code>DCTERMS.URI</code>), but
     *          may be processed recursively without its schema prefix.
     *  @param  scheme  The original scheme to fall back to if there is no
     *          match.
     *  @return The expanded URI reference for the scheme,
     *          or the original string if it is not
     *          matched.
     *  @since  Version 0.3 2005-05-24
     */
    protected abstract String expandScheme(final String name,
                                           final String scheme);

    /**
     *  Copy an array of schema prefixes.
     *  @param  prefixes    The master array of schema prefixes.
     *  @return An array of prefixes that may be used with a schema.
     *  @since  Version 0.5 2005-05-27
     */
    protected final String[] copyPrefixes(final String[] prefixes) {

        if (prefixes != null) {

            int elements = prefixes.length;

            if (elements > 0) {

                String[] copy = new String[elements];

                System.arraycopy(prefixes, 0, copy, 0, elements);

                return copy;
            }
            else {

                throw new IllegalArgumentException("Prefix array is empty");
            }
        }
        else {

            throw new IllegalArgumentException(
                        "Null prefix array not permitted.");
        }
    }

}

