package com.mkdoc.schema;

/*
    Created:    2004-12-22
    Issued:     2004-12-22
    Modified:   2005-09-15

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2004 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import java.util.Collection;
import java.util.Collections;
import java.util.Hashtable;
import java.util.Map;

/**
 *  The Dublin Core metadata elements schema.
 *  @see    http://dublincore.org/documents/dcmi-terms/
 *  @version    1.5 2005-09-15
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class DublinCoreElements extends AbstractSchema implements Schema {

    /**
     *  The common name of the schema.
     *  @since  Version 1.3 2005-06-17
     */
    public static final String NAME = "Dublin Core Elements";

    /**
     *  The base URI for the Dublin Core element set.
     *  @since  Version 0.1 2004-12-22
     */
    public static final String BASE = "http://purl.org/dc/elements/1.1/";

    /**
     *  The metadata prefixes recognised by this schema.
     *  @since  Version 0.5 2005-03-01
     */
    public static final String[] PREFIXES = new String[] {"DC"};

    /**
     *  The Title element URI suffix and name.
     *  @since  Version 0.1 2004-12-22
     */
    public static final String NAME_TITLE = "title";

    /**
     *  The Title element label.
     *  @since  Version 1.0 2005-06-15
     */
    public static final String LABL_TITLE = "Title";

    /**
     *  The Creator element URI suffix and name.
     *  @since  Version 0.1 2004-12-22
     */
    public static final String NAME_CREATR = "creator";

    /**
     *  The Creator element URI suffix and name.
     *  @since  Version 1.0 2005-06-15
     */
    public static final String LABL_CREATR = "Creator";

    /**
     *  The Subject element URI suffix.
     *  @since  Version 0.1 2004-12-22
     */
    public static final String NAME_SUBJCT = "subject";

    /**
     *  The Subject element label.
     *  @since  Version 1.0 2005-06-15
     */
    public static final String LABL_SUBJCT = "Subject";

    /**
     *  The Description element URI suffix.
     *  @since  Version 0.1 2004-12-22
     */
    public static final String NAME_DESCRN = "description";

    /**
     *  The Description element label.
     *  @since  Version 1.0 2005-06-15
     */
    public static final String LABL_DESCRN = "Description";

    /**
     *  The Publisher element URI suffix.
     *  @since  Version 0.1 2004-12-22
     */
    public static final String NAME_PUBSHR = "publisher";

    /**
     *  The Publisher element label.
     *  @since  Version 1.0 2005-06-15
     */
    public static final String LABL_PUBSHR = "Publisher";

    /**
     *  The Contributor element URI suffix.
     *  @since  Version 0.1 2004-12-22
     */
    public static final String NAME_CONTRB = "contributor";

    /**
     *  The Contributor element label.
     *  @since  Version 1.0 2005-06-15
     */
    public static final String LABL_CONTRB = "Contributor";

    /**
     *  The Date element URI suffix.
     *  @since  Version 0.1 2004-12-22
     */
    public static final String NAME_DATE = "date";

    /**
     *  The Date element label.
     *  @since  Version 1.0 2005-06-15
     */
    public static final String LABL_DATE = "Date";

    /**
     *  The Type element URI suffix.
     *  @since  Version 0.1 2004-12-22
     */
    public static final String NAME_TYPE = "type";

    /**
     *  The Type element label.
     *  @since  Version 1.0 2005-06-15
     */
    public static final String LABL_TYPE = "Type";

    /**
     *  The Format element URI suffix.
     *  @since  Version 0.1 2004-12-22
     */
    public static final String NAME_FORMAT = "format";

    /**
     *  The Format element label.
     *  @since  Version 1.0 2005-06-15
     */
    public static final String LABL_FORMAT = "Format";

    /**
     *  The Identifier element URI suffix.
     *  @since  Version 0.1 2004-12-22
     */
    public static final String NAME_IDNTFR = "identifier";

    /**
     *  The Identifier element label.
     *  @since  Version 1.0 2005-06-15
     */
    public static final String LABL_IDNTFR = "Identifier";

    /**
     *  The Source element URI suffix.
     *  @since  Version 0.1 2004-12-22
     */
    public static final String NAME_SOURCE = "source";

    /**
     *  The Source element label.
     *  @since  Version 1.0 2005-06-15
     */
    public static final String LABL_SOURCE = "Source";

    /**
     *  The Language element URI suffix.
     *  @since  Version 0.1 2004-12-22
     */
    public static final String NAME_LNGAGE = "language";

    /**
     *  The Language element URI suffix.
     *  @since  Version 1.0 2005-06-15
     */
    public static final String LABL_LNGAGE = "Language";

    /**
     *  The relation element URI suffix.
     *  @since  Version 0.1 2004-12-22
     */
    public static final String NAME_RELATN = "relation";

    /**
     *  The relation element label.
     *  @since  Version 1.0 2005-06-15
     */
    public static final String LABL_RELATN = "Relation";

    /**
     *  The Coverage element URI suffix.
     *  @since  Version 0.1 2004-12-22
     */
    public static final String NAME_COVRGE = "coverage";

    /**
     *  The Coverage element label.
     *  @since  Version 1.0 2005-06-16
     */
    public static final String LABL_COVRGE = "Coverage";

    /**
     *  The Rights element URI suffix.
     *  @since  Version 0.1 2004-12-22
     */
    public static final String NAME_RIGHTS = "rights";

    /**
     *  The Rights element label.
     *  @since  Version 1.0 2005-06-15
     */
    public static final String LABL_RIGHTS = "Rights";

    /**
     *  A mapping between encoded metadata names and their URI.
     *  <p>
     *    Encoded names are stored in lower case without their schema prefixes.
     *  </p>
     *  @since  Version 0.7 2005-05-24
     */
    private static final Map PREDICATES = new Hashtable();

    /**
     *  Populate the encoded metadata mapping.
     *  @since  Version 0.7 2005-02-24
     */
    static {

        // Elements only
        addPredicate(newSchemaProperty(BASE + NAME_TITLE,
                                       LABL_TITLE,
                                       NAME_TITLE,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_CREATR,
                                       LABL_CREATR,
                                       NAME_CREATR,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_SUBJCT,
                                       LABL_SUBJCT,
                                       NAME_SUBJCT,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_DESCRN,
                                       LABL_DESCRN,
                                       NAME_DESCRN,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_PUBSHR,
                                       LABL_PUBSHR,
                                       NAME_PUBSHR,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_CONTRB,
                                       LABL_CONTRB,
                                       NAME_CONTRB,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_DATE,
                                       LABL_DATE,
                                       NAME_DATE,
                                       true,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_TYPE,
                                       LABL_TYPE,
                                       NAME_TYPE,
                                       true,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_FORMAT,
                                       LABL_FORMAT,
                                       NAME_FORMAT,
                                       true,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_IDNTFR,
                                       LABL_IDNTFR,
                                       NAME_IDNTFR,
                                       true,
                                       true));

        addPredicate(newSchemaProperty(BASE + NAME_SOURCE,
                                       LABL_SOURCE,
                                       NAME_SOURCE,
                                       true,
                                       true));

        addPredicate(newSchemaProperty(BASE + NAME_LNGAGE,
                                       LABL_LNGAGE,
                                       NAME_LNGAGE,
                                       true,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_RELATN,
                                       LABL_RELATN,
                                       NAME_RELATN,
                                       true,
                                       true));

        addPredicate(newSchemaProperty(BASE + NAME_COVRGE,
                                       LABL_COVRGE,
                                       NAME_COVRGE,
                                       true,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_RIGHTS,
                                       LABL_RIGHTS,
                                       NAME_RIGHTS,
                                       false));

    }

    /**
     *  Get a new schema property without encoding schemes.
     *  @param  uri The resource URI.
     *  @param  label   The resource label.
     *  @param  name    The unique resource name.
     *  @return A new schema property constructed from the arguments.
     *  @since  Version 0.7 2005-06-15
     */
    protected static final SchemaProperty newSchemaProperty(final String uri,
                                                            final String label,
                                                            final String name) {

        return newSchemaProperty(uri, label, name, false, false);
    }

    /**
     *  Get a new schema property with optional encoding schemes.
     *  @param  uri The resource URI.
     *  @param  label   The resource label.
     *  @param  name    The unique resource name.
     *  @param  encoding    Whether the property has encoding schemes that may
     *          be applied to it.
     *  @return A new schema property constructed from the arguments.
     *  @since  Version 1.4 2005-09-14
     */
    protected static final SchemaProperty newSchemaProperty(
                                                      final String uri,
                                                      final String label,
                                                      final String name,
                                                      final boolean encoding) {

        return newSchemaProperty(uri, label, name, encoding, false);
    }

    /**
     *  Get a new schema property.
     *  @param  uri The resource URI.
     *  @param  label   The resource label.
     *  @param  name    The unique resource name.
     *  @param  encoding    Whether the property has encoding schemes that may
     *          be applied to it.
     *  @param  uriEncoding Whether the instance may have a URI encoding
     *          scheme.
     *  @return A new schema property constructed from the arguments.
     *  @since  Version 1.5 2005-09-15
     */
    protected static final SchemaProperty newSchemaProperty(
                                                   final String uri,
                                                   final String label,
                                                   final String name,
                                                   final boolean encoding,
                                                   final boolean uriEncoding) {


        return new DCElementProperty(uri, label, name, encoding, uriEncoding);
    }


    /**
     *  Add a predicate to the schema mapping.
     *  @param  predicate   The schema property to add to the mapping.
     *  @since  Version 1.0 2005-06-15
     */
    protected static final void addPredicate(final SchemaProperty predicate) {

        String name = predicate.getName();

        PREDICATES.put(name.toLowerCase(), predicate);
    }

    /**
     *  Expand an encoded predicate name to a URI.
     *  <p>
     *    Allows recursive handling of schema prefixes.
     *  </p>
     *  @param  name    The predicate name to expand. Initially, this may
     *          include a schema prefix (i.e. <code>DC.Identifier</code>), but
     *          may be processed recursively without its schema prefix to
     *          handle refined elements that use the dot separator scheme
     *          <code>DC.Date.modified</code> or
     *          <code>eGMS.Addressee.addresseeCopy</code>.
     *  @param  predicate   The original predicate to fall back to if there is
     *          no match.
     *  @return The expanded URI reference for the predicate name, or the
     *          original predicate if the name part is not matched.
     *  @since  Version 0.2 2004-12-23
     */
    public final String expandPredicate(final String name,
                                        final String predicate) {

        if ((name != null) &&
            (predicate != null)) {

            int dotIdx = name.indexOf(".");

            if (dotIdx > -1) {

                String element = name.substring(dotIdx + 1,
                                                name.length());

                if (hasPredicate(element)) {

                    return getPredicateUri(element);
                }
                // Fall through
            }
            // Fall through

            return predicate;
        }
        else {

            throw new IllegalArgumentException(
                        "Null arguments not permitted.");
        }
    }

    /**
     *  Encapsulated check whether the schema has the named predicate.
     *  @param  predicate   The predicate name without any prefix.
     *  @return <code>true</code> if the schema has a URI mapping for the
     *          predicate.
     *  @since  Vesion 0.8 2005-05-27
     */
    private final boolean hasPredicate(final String predicate) {

        return PREDICATES.containsKey(predicate.toLowerCase());
    }

    /**
     *  Get the URI mapped to the predicate.
     *  @param  predicate   The predicate name without any prefix.
     *  @return The URI mapped to the predicate or <code>null</code> if not
     *          found.
     *  @since  Version 0.8 2005-05-27
     */
    protected final String getPredicateUri(final String predicate) {

        if (predicate != null) {

            SchemaProperty property = getNamedPredicateProperty(predicate);

            if (property != null) {

                return property.getUri();
            }
            else {

                return null;
            }
        }
        else {

            throw new IllegalArgumentException(
                        "Null predicate not permitted.");
        }
    }

    /**
     *  Get the schema property for a given predicate.
     *  @param  name    The predicate name without prefix.
     *  @return The schema property for the predicate or <code>null</code> if no
     *          mapping exists.
     *  @since  Version 1.2 2005-06-16
     */
    public final SchemaProperty getNamedPredicateProperty(final String name) {

        return (SchemaProperty) PREDICATES.get(name.toLowerCase());
    }

    /**
     *  Expand an encoded scheme value to a URI.
     *  <p>
     *    Passes calls to a <code>DublinCoreTerms</code>
     *    instance.
     *  </p>
     *  @param  name    The encoding scheme name to expand. Initially, this may
     *          include a schema prefix (i.e. <code>DCTERMS.URI</code>), but
     *          may be processed recursively without its schema prefix.
     *  @param  scheme  The scheme to expand.
     *  @return The expanded URI reference for the scheme,
     *          or the original string if it is not
     *          matched.
     *  @since  Version 0.2 2004-12-23
     */
    protected final String expandScheme(final String name,
                                        final String scheme) {

        if (scheme != null) {

            return scheme;
        }
        else {

            throw new IllegalArgumentException(
                        "Null scheme not permitted.");
        }
    }

    /**
     *  Get the base URI for the metadata schema.
     *  @return The namespace URI for the metadata schema.
     *  @since  Version 0.2 2004-12-23
     */
    public final String getUri() {

        return BASE;
    }

    /**
     *  Get the default namespace prefix for the schema.
     *  @return The default namepace prefix for the schema.
     *  @since  Version 0.5 2005-03-01
     */
    public final String getDefaultPrefix() {

        return PREFIXES[0];
    }

    /**
     *  Get the HTML metadata prefixes recognised by this schema.
     *  @return An array of one or more schema prefixes in mixed case.
     *  @since  Version 0.8 2005-05-27
     */
    public final String[] getAllPrefixes() {

        return copyPrefixes(PREFIXES);
    }

    /**
     *  Signal whether the schema provides element qualifiers.
     *  @return <code>true</code> if the schema
     *  @since  Version 0.6 2005-03-10
     */
    public final boolean providesQualifiers() {

        return false;
    }

    /**
     *  Check whether the schema provides qualifiers for a specific schema.
     *  @param  uri The URI of another schema to check for extensions.
     *  @return <code>true</code> if the schema provieds qualifiers for the
     *        schema signified by the URI.
     *  @since  Version 0.6 2005-03-10
     */
    public final boolean providesQualifiersFor(final String uri) {

        return false;
    }

    /**
     *  Get a collection of schema property predicates for this schema.
     *  @return A collection of predicate names.
     *  @since  Version 0.9 2005-06-10
     */
    public final Collection getPredicates() {

        return Collections.unmodifiableCollection(PREDICATES.values());
    }

    /**
     *  Get the common name for the schema.
     *  @return The common name for the schema.
     *  @since  Version 1.3 2005-06-17
     */
    public final String getName() {

        return NAME;
    }

}
