package com.mkdoc.schema;

/*
    Created:    2004-12-23
    Issued:     2004-01-06
    Modified:   2005-09-15

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2004 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import java.util.Collection;
import java.util.Collections;
import java.util.Hashtable;
import java.util.Map;

/**
 *  The Dublin Core qualified terms schema.
 *  @see    http://dublincore.org/documents/dcmi-terms/
 *  @version    1.3. 2005-09-15
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class DublinCoreTerms extends AbstractSchema implements Schema {

    /**
     *  The common name of the schema.
     *  @since  Version 1.3 2005-06-17
     */
    public static final String NAME = "Dublin Core Qualifiers";

    /**
     *  The base URI for the Dublin Core qualifiers.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String BASE = "http://purl.org/dc/terms/";

    /**
     *  The default namespace prefix for the Dublin Core terms schema.
     *  @since  Version 0.3 2005-03-01
     */
    public static final String[] PREFIXES = new String[] {"DCTERMS",
                                                          "DC"};

    /**
     *  The Audience term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_AUDNCE = "audience";

    /**
     *  The Audience term label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_AUDNCE = "Audience";

    /**
     *  The Alternative term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_ALTIVE = "alternative";

    /**
     *  The Alternative term label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_ALTIVE = "Alternative";

    /**
     *  The Table Of Contents term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_TABCNT = "tableOfContents";

    /**
     *  The Table Of Contents term label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_TABCNT = "Table Of Contents";

    /**
     *  The Abstract term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_ABSRCT = "abstract";

    /**
     *  The Abstract term label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_ABSRCT = "Abstract";

    /**
     *  The Created term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_CREATD = "created";

    /**
     *  The Created term label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_CREATD = "Created";

    /**
     *  The Valid term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_VALID = "valid";

    /**
     *  The Valid term label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_VALID = "Valid";

    /**
     *  The Available term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_AVAIBL = "available";

    /**
     *  The Available term label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_AVAIBL = "Available";

    /**
     *  The Issued term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_ISSUED = "issued";

    /**
     *  The Issued term label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_ISSUED = "Issued";

    /**
     *  The Modified term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_MODFID = "modified";

    /**
     *  The Modified term label.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String LABL_MODFID = "Modified";

    /**
     *  The Extent term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_EXTENT = "extent";

    /**
     *  The Extent term label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_EXTENT = "Extent";

    /**
     *  The Medium term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_MEDIUM = "medium";

    /**
     *  The Medium term label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_MEDIUM = "Medium";

    /**
     *  The Is Version Of term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_VERNOF = "isVersionOf";

    /**
     *  The Is Version Of term label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_VERNOF = "Is Version Of";

    /**
     *  The Has Version URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_HASVER = "hasVersion";

    /**
     *  The Has Version label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_HASVER = "Has Version";

    /**
     *  The Is Replaced By term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_REPLBY = "isReplacedBy";

    /**
     *  The Is Replaced By term label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_REPLBY = "Is Replaced By";

    /**
     *  The Replaces term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_REPLCS = "replaces";

    /**
     *  The Replaces term label.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String LABL_REPLCS = "Replaces";

    /**
     *  The Is Required By term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_REQDBY = "isRequiredBy";

    /**
     *  The Is Required By term label.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String LABL_REQDBY = "Is Required By";

    /**
     *  The Requires term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_REQRES = "requires";

    /**
     *  The Requires term label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_REQRES = "Requires";

    /**
     *  The Is Part Of term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_PARTOF = "isPartOf";

    /**
     *  The Is Part Of term label.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String LABL_PARTOF = "Is Part Of";

    /**
     *  The Has Part term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_HASPRT = "hasPart";

    /**
     *  The Has Part term label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_HASPRT = "Has Part";

    /**
     *  The Is Referenced By term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_REFDBY = "isReferencedBy";

    /**
     *  The Is Referenced By term label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_REFDBY = "Is Referenced By";

    /**
     *  The References term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_REFRCS = "references";

    /**
     *  The References term label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_REFRCS = "References";

    /**
     *  The Is Format Of term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_FMATOF = "isFormatOf";

    /**
     *  The Is Format Of term label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_FMATOF = "Is Format Of";

    /**
     *  The Has Format term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_HASFMT = "hasFormat";

    /**
     *  The Has Format term label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_HASFMT = "Has Format";

    /**
     *  The Conforms To term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_CONFTO = "conformsTo";

    /**
     *  The Conforms To term label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_CONFTO = "Conforms To";

    /**
     *  The Spatial term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_SPTIAL = "spatial";

    /**
     *  The Spatial term label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_SPTIAL = "Spatial";

    /**
     *  The Temporal term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_TEMPRL = "temporal";

    /**
     *  The Temporal term label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_TEMPRL = "Temporal";

    /**
     *  The Mediator term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_MEDITR = "mediator";

    /**
     *  The Mediator term label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_MEDITR = "Mediator";

    /**
     *  The Date Accepted term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_ACCPTD = "dateAccepted";

    /**
     *  The Date Accepted term label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_ACCPTD = "Date Accepted";

    /**
     *  The Date Copyrighted URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_CPYRGT = "dateCopyrighted";

    /**
     *  The Date Copyrighted label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_CPYRGT = "Date Copyrighted";

    /**
     *  The Date Submitted URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_SUBMTD = "dateSubmitted";

    /**
     *  The Date Submitted label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_SUBMTD = "Date Submitted";

    /**
     *  The Education Level term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_EDULVL = "educationLevel";

    /**
     *  The Education Level term label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_EDULVL = "Education Level";

    /**
     *  The Access Rights term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_ACCRGT = "accessRights";

    /**
     *  The Access Rights term label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_ACCRGT = "Access Rights";

    /**
     *  The Bibliographic Citation term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_BIBCIT = "bibliographicCitation";

    /**
     *  The Bibliographic Citation term label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_BIBCIT = "Bibliographic Citation";

    /**
     *  The License term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_LICNSE = "license";

    /**
     *  The License term label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_LICNSE = "License";

    /**
     *  The Rights Holder term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_RGTHLD = "rightsHolder";

    /**
     *  The Rights Holder term label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_RGTHLD = "Rights Holder";

    /**
     *  The Provenance term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_PRVNCE = "provenance";

    /**
     *  The Provenance term label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_PRVNCE = "Provenance";

    /**
     *  The Instructional Method term URI suffix.
     *  @since  Version 1.1 2005-07-04
     */
    public static final String NAME_INSMTH = "instructionalMethod";

    /**
     *  The Instructional Method term label.
     *  @since  Version 1.1 2005-07-04
     */
    public static final String LABL_INSMTH = "Instructional Method";

    /**
     *  The Accural Policy term URI suffix.
     *  @since  Version 1.1 2005-07-04
     */
    public static final String NAME_ACCMTH = "accrualMethod";

    /**
     *  The Accrual Policy term label.
     *  @since  Version 1.1 2005-07-04
     */
    public static final String LABL_ACCMTH = "Accrual Method";

    /**
     *  The Accural Periodicity term URI suffix.
     *  @since  Version 1.1 2005-07-04
     */
    public static final String NAME_ACCPER = "accrualPeriodicity";

    /**
     *  The Accrual Periodicity term label.
     *  @since  Version 1.1 2005-07-04
     */
    public static final String LABL_ACCPER = "Accrual Periodicity";

    /**
     *  The Accural Periodicity term URI suffix.
     *  @since  Version 1.1 2005-07-04
     */
    public static final String NAME_ACCPOL = "accrualPolicy";

    /**
     *  The Accrual Periodicity term label.
     *  @since  Version 1.1 2005-07-04
     */
    public static final String LABL_ACCPOL = "Accrual Policy";

    /**
     *  The LCSH term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_LCSH = "LCSH";

    /**
     *  The LCSH term label.
     *  @since  Version 0.8 2005-06-16
     */
    public static final String LABL_LCSH = NAME_LCSH;

    /**
     *  The MeSH term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_MESH = "MESH";

    /**
     *  The MeSH term label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_MESH = NAME_MESH;

    /**
     *  The DDC term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_DDC = "DDC";

    /**
     *  The DDC term label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_DDC = NAME_DDC;

    /**
     *  The LCC term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_LCC = "LCC";

    /**
     *  The LCC term label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_LCC = NAME_LCC;

    /**
     *  The UDC term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_UDC = "UDC";

    /**
     *  The UDC term label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_UDC = NAME_UDC;

    /**
     *  The DCMI Type term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_DCTYPE = "DCMIType";

    /**
     *  The DCMI Type term label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_DCTYPE = "DCMI Type Vocabulary";

    /**
     *  The IMT term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_IMT = "IMT";

    /**
     *  The IMT term label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_IMT = NAME_IMT;

    /**
     *  The ISO639-2 term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_IS3692 = "ISO639-2";

    /**
     *  The ISO639-2 term URI suffix.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_IS3692 = "ISO 639-2";

    /**
     *  The RFC 1766 term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_RC1766 = "RFC1766";

    /**
     *  The RFC 1766 term label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_RC1766 = "RFC 1766";

    /**
     *  The URI term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_URI = "URI";

    /**
     *  The URI term label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_URI = NAME_URI;

    /**
     *  The DCMI Point term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_POINT = "Point";

    /**
     *  The DCMI Point term label.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String LABL_POINT = "DCMI Point";

    /**
     *  The ISO3166 term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_IS3166 = "ISO3166";

    /**
     *  The ISO3166 term label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_IS3166 = "ISO 3166";

    /**
     *  The DCMI Box term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_BOX = "Box";

    /**
     *  The DCMI Box term label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_BOX = "DCMI Box";

    /**
     *  The NAME_TGN term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_TGN = "TGN";

    /**
     *  The NAME_TGN term label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_TGN = NAME_TGN;

    /**
     *  The DCMI Period term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_PERIOD = "Period";

    /**
     *  The DCMI Period term label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_PERIOD = "DCMI Period";

    /**
     *  The W3C-DTF term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_W3CDTF = "W3CDTF";

    /**
     *  The W3C-DTF term label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_W3CDTF = "W3C-DTF";

    /**
     *  The RFC 3066 term URI suffix.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_RC3066 = "RFC3066";

    /**
     *  The RFC 3066 term label.
     *  @since  Version 0.8 2005-06-15
     */
    public static final String LABL_RC3066 = "RFC 3066";

    /**
     *  The NLM term URI suffix.
     *  @since  Version 1.1 2005-07-04
     */
    public static final String NAME_NLM = "NLM";

    /**
     *  The NLMterm label.
     *  @since  Version 1.1 2005-07-04
     */
    public static final String LABL_NLM = NAME_NLM;


    /**
     *  A mapping between encoded metadata names and their URI.
     *  <p>
     *    Encoded names are stored in lower case without their schema prefixes.
     *  </p>
     *  @since  Version 0.5 2005-05-24
     */
    private static final Map PREDICATES = new Hashtable();

    /**
     *  A mapping between encoded metadata SCHEMES and their URI.
     *  <p>
     *    Encoded names are stored in lower case without their schema prefixes.
     *  </p>
     *  @since  Version 0.5 2005-05-24
     */
    private static final Map SCHEMES = new Hashtable();

    /**
     *  Populate the encoded metadata mapping.
     *  @since  Version 0.7 2005-02-24
     */
    static {

        // Elements
        addPredicate(newSchemaProperty(BASE + NAME_AUDNCE,
                                       LABL_AUDNCE,
                                       NAME_AUDNCE,
                                       false));


        addPredicate(newSchemaProperty(BASE + NAME_ALTIVE,
                                       LABL_ALTIVE,
                                       NAME_ALTIVE,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_TABCNT,
                                       LABL_TABCNT,
                                       NAME_TABCNT,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_ABSRCT,
                                       LABL_ABSRCT,
                                       NAME_ABSRCT,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_CREATD,
                                       LABL_CREATD,
                                       NAME_CREATD,
                                       true,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_VALID,
                                       LABL_VALID,
                                       NAME_VALID,
                                       true,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_AVAIBL,
                                       LABL_AVAIBL,
                                       NAME_AVAIBL,
                                       true,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_ISSUED,
                                       LABL_ISSUED,
                                       NAME_ISSUED,
                                       true,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_MODFID,
                                       LABL_MODFID,
                                       NAME_MODFID,
                                       true,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_EXTENT,
                                       LABL_EXTENT,
                                       NAME_EXTENT,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_MEDIUM,
                                       LABL_MEDIUM,
                                       NAME_MEDIUM,
                                       true,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_VERNOF,
                                       LABL_VERNOF,
                                       NAME_VERNOF,
                                       true,
                                       true));

        addPredicate(newSchemaProperty(BASE + NAME_HASVER,
                                       LABL_HASVER,
                                       NAME_HASVER,
                                       true,
                                       true));

        addPredicate(newSchemaProperty(BASE + NAME_REPLBY,
                                       LABL_REPLBY,
                                       NAME_REPLBY,
                                       true,
                                       true));

        addPredicate(newSchemaProperty(BASE + NAME_REPLCS,
                                       LABL_REPLCS,
                                       NAME_REPLCS,
                                       true,
                                       true));

        addPredicate(newSchemaProperty(BASE + NAME_REQDBY,
                                       LABL_REQDBY,
                                       NAME_REQDBY,
                                       true,
                                       true));

        addPredicate(newSchemaProperty(BASE + NAME_REQRES,
                                       LABL_REQRES,
                                       NAME_REQRES,
                                       true,
                                       true));

        addPredicate(newSchemaProperty(BASE + NAME_PARTOF,
                                       LABL_PARTOF,
                                       NAME_PARTOF,
                                       true,
                                       true));

        addPredicate(newSchemaProperty(BASE + NAME_HASPRT,
                                       LABL_HASPRT,
                                       NAME_HASPRT,
                                       true,
                                       true));

        addPredicate(newSchemaProperty(BASE + NAME_REFDBY,
                                       LABL_REFDBY,
                                       NAME_REFDBY,
                                       true,
                                       true));

        addPredicate(newSchemaProperty(BASE + NAME_REFRCS,
                                       LABL_REFRCS,
                                       NAME_REFRCS,
                                       true,
                                       true));

        addPredicate(newSchemaProperty(BASE + NAME_FMATOF,
                                       LABL_FMATOF,
                                       NAME_FMATOF,
                                       true,
                                       true));

        addPredicate(newSchemaProperty(BASE + NAME_HASFMT,
                                       LABL_HASFMT,
                                       NAME_HASFMT,
                                       true));

        addPredicate(newSchemaProperty(BASE + NAME_CONFTO,
                                       LABL_CONFTO,
                                       NAME_CONFTO,
                                       true,
                                       true));

        addPredicate(newSchemaProperty(BASE + NAME_SPTIAL,
                                       LABL_SPTIAL,
                                       NAME_SPTIAL,
                                       true,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_TEMPRL,
                                       LABL_TEMPRL,
                                       NAME_TEMPRL,
                                       true,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_MEDITR,
                                       LABL_MEDITR,
                                       NAME_MEDITR,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_ACCPTD,
                                       LABL_ACCPTD,
                                       NAME_ACCPTD,
                                       true,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_CPYRGT,
                                       LABL_CPYRGT,
                                       NAME_CPYRGT,
                                       true,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_SUBMTD,
                                       LABL_SUBMTD,
                                       NAME_SUBMTD,
                                       true,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_EDULVL,
                                       LABL_EDULVL,
                                       NAME_EDULVL,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_ACCRGT,
                                       LABL_ACCRGT,
                                       NAME_ACCRGT,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_BIBCIT,
                                       LABL_BIBCIT,
                                       NAME_BIBCIT,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_LICNSE,
                                       LABL_LICNSE,
                                       NAME_LICNSE,
                                       true,
                                       true));

        addPredicate(newSchemaProperty(BASE + NAME_RGTHLD,
                                       LABL_RGTHLD,
                                       NAME_RGTHLD,
                                       true,
                                       true));

        addPredicate(newSchemaProperty(BASE + NAME_PRVNCE,
                                       LABL_PRVNCE,
                                       NAME_PRVNCE,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_INSMTH,
                                       LABL_INSMTH,
                                       NAME_INSMTH,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_ACCMTH,
                                       LABL_ACCMTH,
                                       NAME_ACCMTH,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_ACCPER,
                                       LABL_ACCPER,
                                       NAME_ACCPER,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_ACCPOL,
                                       LABL_ACCPOL,
                                       NAME_ACCPOL,
                                       false));

        // Schemes
        addScheme(newSchemaProperty(BASE + NAME_LCSH,
                                    LABL_LCSH,
                                    NAME_LCSH));

        addScheme(newSchemaProperty(BASE + NAME_MESH,
                                    LABL_MESH,
                                    NAME_MESH));

        addScheme(newSchemaProperty(BASE + NAME_DDC,
                                    LABL_DDC,
                                    NAME_DDC));

        addScheme(newSchemaProperty(BASE + NAME_LCC,
                                    LABL_LCC,
                                    NAME_LCC));

        addScheme(newSchemaProperty(BASE + NAME_UDC,
                                    LABL_UDC,
                                    NAME_UDC));

        addScheme(newSchemaProperty(BASE + NAME_DCTYPE,
                                    LABL_DCTYPE,
                                    NAME_DCTYPE));

        addScheme(newSchemaProperty(BASE + NAME_IMT,
                                    LABL_IMT,
                                    NAME_IMT));

        addScheme(newSchemaProperty(BASE + NAME_IS3692,
                                    LABL_IS3692,
                                    NAME_IS3692));

        addScheme(newSchemaProperty(BASE + NAME_RC1766,
                                    LABL_RC1766,
                                    NAME_RC1766));

        addScheme(newSchemaProperty(BASE + NAME_URI,
                                    LABL_URI,
                                    NAME_URI));

        addScheme(newSchemaProperty(BASE + NAME_POINT,
                                    LABL_POINT,
                                    NAME_POINT));

        addScheme(newSchemaProperty(BASE + NAME_IS3166,
                                    LABL_IS3166,
                                    NAME_IS3166));

        addScheme(newSchemaProperty(BASE + NAME_BOX,
                                    LABL_BOX,
                                    NAME_BOX));

        addScheme(newSchemaProperty(BASE + NAME_TGN,
                                    LABL_TGN,
                                    NAME_TGN));

        addScheme(newSchemaProperty(BASE + NAME_PERIOD,
                                    LABL_PERIOD,
                                    NAME_PERIOD));

        addScheme(newSchemaProperty(BASE + NAME_W3CDTF,
                                    LABL_W3CDTF,
                                    NAME_W3CDTF));

        addScheme(newSchemaProperty(BASE + NAME_RC3066,
                                    LABL_RC3066,
                                    NAME_RC3066));

        addScheme(newSchemaProperty(BASE + NAME_NLM,
                                    LABL_NLM,
                                    NAME_NLM));
    }

    /**
     *  Get a new schema property.
     *  @param  uri The resource URI.
     *  @param  label   The resource label.
     *  @param  name    The unique resource name.
     *  @return A new schema property constructed from the arguments.
     *  @since  Version 0.7 2005-06-15
     */
    protected static final SchemaProperty newSchemaProperty(
                                                           final String uri,
                                                           final String label,
                                                           final String name) {

        return newSchemaProperty(uri, label, name, false, false);
    }

    /**
     *  Get a new schema property with optional encoding schemes.
     *  @param  uri The resource URI.
     *  @param  label   The resource label.
     *  @param  name    The unique resource name.
     *  @param  encoding    Whether the property has scheme encodings that may
     *          be applied.
     *  @return A new schema property constructed from the arguments.
     *  @since  Version 1.2 2005-09-14
     */
    protected static final SchemaProperty newSchemaProperty(
                                                      final String uri,
                                                      final String label,
                                                      final String name,
                                                      final boolean encoding) {

        return newSchemaProperty(uri, label, name, encoding, false);
    }

    /**
     *  Get a new schema property.
     *  @param  uri The resource URI.
     *  @param  label   The resource label.
     *  @param  name    The unique resource name.
     *  @param  encoding    Whether the property has scheme encodings that may
     *          be applied.
     *  @param  uriEncoding Whether the instance may have a URI encoding
     *          scheme.
     *  @return A new schema property constructed from the arguments.
     *  @since  Version 1.3 2005-09-15
     */
    protected static final SchemaProperty newSchemaProperty(
                                                   final String uri,
                                                   final String label,
                                                   final String name,
                                                   final boolean encoding,
                                                   final boolean uriEncoding) {

        return new DCTermProperty(uri, label, name, encoding, uriEncoding);
    }

    /**
     *  Add a predicate to the schema mapping.
     *  @param  predicate   The schema property to add to the mapping.
     *  @since  Version 0.8 2005-06-15
     */
    protected static final void addPredicate(final SchemaProperty predicate) {

        String name = predicate.getName();

        PREDICATES.put(name.toLowerCase(), predicate);
    }

    /**
     *  Add an encoding scheme to the schema mapping.
     *  @param  scheme  The schema property to add to the mapping.
     *  @since  Version 0.8 2005-06-15
     */
    protected static final void addScheme(final SchemaProperty scheme) {

        String name = scheme.getName();

        SCHEMES.put(name.toLowerCase(), scheme);
    }

    /**
     *  Expand an encoded predicate name to a URI.
     *  <p>
     *    Allows recursive handling of schema prefixes.
     *  </p>
     *  @param  name    The predicate name to expand. Initially, this may
     *          include a schema prefix (i.e. <code>DC.Identifier</code>), but
     *          may be processed recursively without its schema prefix to
     *          handle refined elements that use the dot separator scheme
     *          <code>DC.Date.modified</code> or
     *          <code>eGMS.Addressee.addresseeCopy</code>.
     *  @param  predicate   The original predicate to fall back to if there is
     *          no match.
     *  @return The expanded URI reference for the predicate name, or the
     *          original predicate if the name part is not matched.
     *  @since  Verison 0.1 2004-12-23
     */
    public final String expandPredicate(final String name,
                                        final String predicate) {

        if ((name != null) &&
            (predicate != null)) {

            // name may be qualified "DCTERMS.modified"
            // or the tail end of an old "DC.Date.modified"
            // qualified format, i.e. "Date.modified"
            int dotIdx = name.indexOf(".");

            if (dotIdx > -1) {

                String term = name.substring(dotIdx + 1,
                                             name.length());

                if (term.indexOf(".") > -1) {

                    return expandPredicate(term, predicate);
                }
                else {

                    if (hasPredicate(term)) {

                        return getPredicateUri(term);
                    }
                }
            }
            return predicate;
        }
        else {

            throw new IllegalArgumentException(
                        "Null arguments not permitted.");
        }
    }

    /**
     *  Encapsulated check whether the schema has the named predicate.
     *  @param  predicate   The predicate name without any prefix.
     *  @return <code>true</code> if the schema has a URI mapping for the
     *          predicate.
     *  @since  Vesion 0.6 2005-05-27
     */
    private final boolean hasPredicate(final String predicate) {

        return PREDICATES.containsKey(predicate.toLowerCase());
    }

    /**
     *  Get the URI mapped to the predicate.
     *  @param  predicate   The predicate name without any prefix.
     *  @return The URI mapped to the predicate or <code>null</code> if not
     *          found.
     *  @since  Version 0.6 2005-05-27
     */
    protected final String getPredicateUri(final String predicate) {

        if (predicate != null) {

            SchemaProperty property = getNamedPredicateProperty(predicate);

            if (property != null) {

                return property.getUri();
            }
            else {

                return null;
            }
        }
        else {

            throw new IllegalArgumentException(
                        "Null predicate notpermitted.");
        }
    }

    /**
     *  Get the schema property for a given predicate.
     *  @param  name    The predicate name without prefix.
     *  @return The schema property for the predicate or <code>null</code> if no
     *          mapping exists.
     *  @since  Version 0.9 2005-06-16
     */
    public final SchemaProperty getNamedPredicateProperty(final String name) {

        return (SchemaProperty) PREDICATES.get(name.toLowerCase());
    }

    /**
     *  Expand an encoded scheme value to a URI.
     *  <p>
     *    Allows recursive handling of schema prefixes.
     *  </p>
     *  @param  name    The encoding scheme name to expand. Initially, this may
     *          include a schema prefix (i.e. <code>DCTERMS.URI</code>), but
     *          may be processed recursively without its schema prefix.
     *  @param  scheme  The original scheme to fall back to if there is no
     *          match.
     *  @return The expanded URI reference for the scheme,
     *          or the original string if it is not
     *          matched.
     *  @since  Version 0.1 2004-12-23
     */
    protected final String expandScheme(final String name,
                                        final String scheme) {

        if ((name != null) &&
            (scheme != null)) {

            int dotIdx = name.indexOf(".");

            if (dotIdx > -1) {

                String term = scheme.substring(dotIdx + 1,
                                               name.length());

                return expandScheme(term, scheme);
            }
            else {

                if (hasScheme(name)) {

                    return getSchemeUri(name);
                }
                // Fall through

                return scheme;
            }
        }
        else {

            throw new IllegalArgumentException(
                        "Null arguments not permitted.");
        }
    }

    /**
     *  Encapsulated check whether the schema has the named scheme.
     *  @param  scheme  The scheme name without any prefix.
     *  @return <code>true</code> if the schema has a URI mapping for the
     *          predicate.
     *  @since  Vesion 0.6 2005-05-27
     */
    private final boolean hasScheme(final String scheme) {

        return SCHEMES.containsKey(scheme.toLowerCase());
    }

    /**
     *  Get the URI mapped to the scheme.
     *  @param  scheme  The scheme name without any prefix.
     *  @return The URI mapped to the scheme or <code>null</code> if not
     *          found.
     *  @since  Version 0.6 2005-05-27
     */
    protected final String getSchemeUri(final String scheme) {

        if (scheme != null) {

            SchemaProperty property = null;

            property = (SchemaProperty) SCHEMES.get(scheme.toLowerCase());

            if (property != null) {

                return property.getUri();
            }
            else {

                return null;
            }
        }
        else {

            throw new IllegalArgumentException(
                        "Null scheme not permitted.");
        }
    }

    /**
     *  Get the base URI for the metadata schema.
     *  @return The namespace URI for the metadata schema.
     *  @since  Version 0.1 2004-12-23
     */
    public final String getUri() {

        return BASE;
    }

    /**
     *  Get the default namespace prefix for the schema.
     *  @return The default namepace prefix for the schema.
     *  @since  Version 0.3 2005-03-01
     */
    public final String getDefaultPrefix() {

        return PREFIXES[0];
    }

    /**
     *  Get the HTML metadata prefixes recognised by this schema.
     *  @return An array of one or more schema prefixes in mixed case.
     *  @since  Version 0.6 2005-05-27
     */
    public final String[] getAllPrefixes() {

        return copyPrefixes(PREFIXES);
    }

    /**
     *  Signal whether the schema provides element qualifiers.
     *  @return <code>true</code> if the schema
     *  @since  Version 0.4 2005-03-10
     */
    public final boolean providesQualifiers() {

        return true;
    }

    /**
     *  Check whether the schema provides qualifiers for a specific schema.
     *  @param  uri The URI of another schema to check for extensions.
     *  @return <code>true</code> if the schema provieds qualifiers for the
     *        schema signified by the URI.
     *  @since  Version 0.6 2005-03-10
     */
    public final boolean providesQualifiersFor(final String uri) {

        if (uri != null) {

            return uri.equals(DublinCoreElements.BASE);
        }
        else {

            throw new IllegalArgumentException(
                        "Null schema URI not permitted.");
        }
    }

    /**
     *  Get a collection of schema property predicates for this schema.
     *  @return A collection of predicate names.
     *  @since  Version 0.7 2005-06-10
     */
    public final Collection getPredicates() {

        return Collections.unmodifiableCollection(PREDICATES.values());
    }

    /**
     *  Get the common name for the schema.
     *  @return The common name for the schema.
     *  @since  Version 1.0 2005-06-17
     */
    public final String getName() {

        return NAME;
    }

}
