package com.mkdoc.schema;

/*
    Created:    2005-06-15
    Issued:     2005-06-15
    Modified:   2005-10-14

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import java.util.Locale;

/**
 *  Loosely modelled on an RDF Schema class.
 *  @version    0.6 2005-10-14
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public interface SchemaProperty {

    /**
     *  Get the resource URI of the class.
     *  @return The URI of the class.
     *  @since  Version 0.1 2005-06-15
     */
    String getUri();

    /**
     *  Get the human-readable label for the class.
     *  @return The human-readable label for the class.
     *  @since  Version 0.1 2005-06-15
     */
    String getLabel();

    /**
     *  Get a unique &quot;camel case&quot; name for the class.
     *  <p>
     *    This often, but not always, coincides with the URI suffix for the
     *    type.
     *  </p>
     *  @return A name that is unique within a given schema.
     */
    String getName();

    /**
     *  Get the qualified name of the class, with namespace prefix.
     *  @return The qualified name of the class with a namespace prefix.
     *  @since  Version 0.2 2005-06-16
     */
    String getQualifiedName();

    /**
     *  Get the namespace prefix for the class.
     *  @return The namespace prefix for the class.
     *  @since  Version 0.2 2005-06-16
     */
    String getNamespacePrefix();

    /**
     *  Get the namespace URI for the class.
     *  @return The namespace prefix for the class.
     *  @since  Version 0.2 2005-06-16
     */
    String getNamespaceUri();

    /**
     *  Get a unique name varible for query handling.
     *  @return A unique name variable for the class.
     *  @since  Version 0.2 2005-06-16
     */
    String getNameVariable();

    /**
     *  Get a unique varible name for extended bNode queries.
     *  @return A unique bNode variable name for the class.
     *  @since  Version 0.4 2005-09-13
     */
    String getBNodeVariable();

    /**
     *  Get a unique parameter name for HTML form handling.
     *  @return A unique parameter name for the class.
     *  @since  Version 0.3 2005-06-17
     */
    String getParameterName();

    /**
     *  Check whether a schema property may have an encoding scheme.
     *  @return <code>true</code> if the property may have an encoding scheme.
     *  @since  Version 0.4 2005-09-14
     */
    boolean hasEncodingScheme();

    /**
     *  Check whether a schema property may have a URI encoding scheme.
     *  @return <code>true</code> if the property may have a URI encoding
     *          scheme.
     *  @since  Version 0.5 2005-09-15
     */
    boolean hasUriEncodingScheme();

    /**
     *  Get the locale of the property for case conversions, etc.
     *  @return The locale for the schema property.
     *  @since  Version 0.6 2005-10-14
     */
    Locale getLocale();

}
