package com.mkdoc.store;

/*
    Created:    2005-11-01
    Issued:     2005-11-01
    Modified:   2005-11-01

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import org.openrdf.model.Graph;
import org.openrdf.model.URI;

import org.openrdf.sesame.admin.AdminListener;

import org.openrdf.sesame.constants.RDFFormat;
import org.openrdf.sesame.constants.QueryLanguage;

import org.openrdf.sesame.query.QueryResultsTable;

import org.openrdf.sesame.repository.SesameRepository;

import com.mkdoc.query.NullQueryResult;
import com.mkdoc.query.Query;
import com.mkdoc.query.QueryResult;

import java.io.InputStream;

import java.net.URL;

/**
 *  A base superclass for subject manager types.
 *  @version    0.1 2005-11-01
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public abstract class AbstractStoreManager {

    /**
     *  The Sesame repository.
     *  @since  Version 0.1 2005-11-01
     */
    protected SesameRepository repository;

    /**
     *  Is the repository active?
     *  @since  Version 0.1 2005-07-14
     */
    private boolean active = false;

    /**
     *  Add statements about a document by subject URI.
     *  @param  subjectUri  The URI of the subject document to add.
     *  @param  graph   The RDF graph of statements about this document.
     *  @exception  StoreException  If an error occurs with the triple store.
     *  @since  Version 0.1 2005-11-01
     */
    public final void addDocumentStatements(final URI subjectUri,
                                            final Graph graph)
        throws StoreException {


        if ((subjectUri == null) ||
            (graph == null)) {

                throw new IllegalArgumentException(
                            "Null arguments not permitted.");
        }

        if (repository == null) {

            throw new StoreException("Repository not initialized.");
        }

        try {

            repository.addGraph(graph);
        }
        catch (Exception e) {

            throw new StoreException(e.getMessage());
        }
    }

    /**
     *  Add an RDF data stream to the store.
     *  @param  inStream    The data stream to add to the store.
     *  @param  baseUri The URL base of the source.
     *  @exception  StoreException  If there is a problem reading the input
     *          stream or validating the RDF content.
     *  @since  Version 0.1 2005-11-01
     */
    public final void addXmlRdfStream(final InputStream inStream,
                                      final URL baseUri) throws StoreException {

        if ((inStream == null) ||
            (baseUri == null)) {

                throw new IllegalArgumentException(
                            "Null arguments not permitted.");
        }

        if (repository == null) {

            throw new StoreException("Repository not initialized.");
        }

        try {

            AdminListener listener = new ImportListener();

            repository.addData(inStream,
                               baseUri.toExternalForm(),
                               RDFFormat.RDFXML,
                               true,
                               listener);
        }
        catch (Exception e) {

            throw new StoreException(e.getMessage());
        }
    }

    /**
     *  Execute a query against the data store.
     *  @param  query   The query to apply to the store.
     *  @return A result set that fulfils the submitted query.
     *  @since  Version 0.1 2005-11-01
     */
    public final QueryResult executeQuery(final Query query) {

        if (query == null) {

            throw new IllegalArgumentException(
                        "Null query not permitted.");
        }

        QueryResultsTable resultsTable = null;

        try {

            resultsTable = repository.performTableQuery(QueryLanguage.SERQL,
                                                        query.getSeRQLQuery());

            return new StoreQueryResult(resultsTable);
        }
        catch (Exception e) {

            return new NullQueryResult();
        }
    }

    /**
     *  Check whether the store is active.
     *  @return <code>true</code> if the repository been
     *          initialized and not shut down.
     *  @since  Version 0.1 2005-11-01
     */
    protected final boolean isActive() {

        return active;
    }

    /**
     *  Set the store active.
     *  @param  setting Whether the store is active.
     *  @since  Version 0.1 2005-11-01
     */
    protected final void setActive(final boolean setting) {

        active = setting;
    }

}
