package com.mkdoc.store;

/*
    Created:    2005-11-01
    Issued:     2005-11=01
    Modified:   2005-11-01

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import org.openrdf.sesame.Sesame;

import org.openrdf.sesame.config.RepositoryConfig;
import org.openrdf.sesame.config.SailConfig;

import org.openrdf.sesame.repository.local.LocalRepository;
import org.openrdf.sesame.repository.local.LocalService;

import org.openrdf.sesame.sailimpl.rdbms.RdfSchemaRepositoryConfig;

import com.mkdoc.util.ApplicationContext;
import com.mkdoc.util.ApplicationContextException;

/**
 *  A store manager for a local Sesame repository.
 *  @version    0.1 2005-11-01
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class DatabaseStoreManager extends AbstractStoreManager
                                                      implements StoreManager {

    /**
     *  The database vendor key.
     *  @since  Version 0.1 2005-11-01
     */
    public static final String DB_VENDOR = "dbVendor";

    /**
     *  The database name key.
     *  @since  Version 0.1 2005-11-01
     */
    public static final String DB_NAME = "dbName";

    /**
     *  The database user key.
     *  @since  Version 0.1 2005-11-01
     */
    public static final String DB_USER = "dbUser";

    /**
     *  The database password key.
     *  @since  Version 0.1 2005-11-01
     */
    public static final String DB_PASS = "dbPass";

    /**
     *  The PostgreSQL database vendor name.
     *  @since  Version 0.1 2005-11-01
     */
    public static final String PGSQL = "pgsql";

    /**
     *  The PostgreSQL database driver class name.
     *  @since  Version 0.1 2005-11-01
     */
    private static final String PGSQL_DRIVER = "org.postgresql.Driver";

    /**
     *  The PostgreSQL database URL prefix.
     *  @since  Version 0.1 2005-11-01
     */
    private static final String PGSQL_PREFIX = "jdbc:postgresql:";

    /**
     *  The MySQL database vendor name.
     *  @since  Version 0.1 2005-11-01
     */
    public static final String MYSQL = "mysql";

    /**
     *  The MySQL database driver class name.
     *  @since  Version 0.1 2005-11-01
     */
    private static final String MYSQL_DRIVER = "com.mysql.jdbc.Driver";

    /**
     *  The MySQL database URL prefix.
     *  @since  Version 0.1 2005-11-01
     */
    private static final String MYSQL_PREFIX = "jdbc:mysql://localhost/";

    /**
     *  Initialize the manager.
     *  @param  context The application context for this manager.
     *  @since  Version 0.1 2005-07-14
     */
    public final void initialize(final ApplicationContext context) {

        configureRepository(context);

        setActive(true);
    }

    /**
     *  Shut down the manager.
     *  @since  Version 0.1 2005-07-14
     */
    public final void shutdown() {

        if (repository != null) {

            LocalRepository localRepo = (LocalRepository) repository;

            localRepo.shutDown();
        }

        setActive(false);
    }

    /**
     *  Configure the database from the plugin property set.
     *  @param  context The application context for the store.
     *  @return A status message giving the path of the output directory.
     *  @since  Version 0.1 2005-06-09
     */
    protected final String configureRepository(
                                            final ApplicationContext context) {

        if (context == null) {

            throw new IllegalArgumentException(
                        "Null property set not permitted.");
        }
        else {

            String className = this.getClass().getName();

            RepositoryConfig storeConfig = new RepositoryConfig(className);

            SailConfig syncConfig = new SailConfig(
            "org.openrdf.sesame.sailimpl.sync.SyncRdfSchemaRepository");

            String dbVendor = context.getStringParameter(DB_VENDOR);

            String dbName = context.getStringParameter(DB_NAME);

            String dbUser =  context.getStringParameter(DB_USER);

            String dbPass =  context.getStringParameter(DB_PASS);

            String dbUrl = null;

            String driver = null;

            if ((dbVendor != null) &&
                (dbName != null)) {

                if (dbVendor.equalsIgnoreCase(PGSQL)) {

                    dbUrl = PGSQL_PREFIX + dbName;

                    driver = PGSQL_DRIVER;
                }
                if (dbVendor.equalsIgnoreCase(MYSQL)) {

                    dbUrl = MYSQL_PREFIX + dbName;

                    driver = MYSQL_DRIVER;
                }
            }

            if (dbUrl == null) {

                throw new ApplicationContextException(
                            "Could not create database configuration.");
            }
            else {

                try {


                    SailConfig jdbcConfig = null;

                    jdbcConfig = new RdfSchemaRepositoryConfig(driver,
                                                               dbUrl,
                                                               dbUser,
                                                               dbPass);

                    storeConfig.addSail(syncConfig);
                    storeConfig.addSail(jdbcConfig);
                    storeConfig.setWorldReadable(true);
                    storeConfig.setWorldWriteable(true);

                    LocalService service = Sesame.getService();

                    // May throw ConfigurationException
                    repository = (LocalRepository) service.createRepository(
                                                                  storeConfig);

                    return "Using database repository" +
                           dbUrl;
                }
                catch (Exception e) {

                    throw new ApplicationContextException(
                                "Could not create repository. " +
                                 e.getMessage());
                }
            }
        }
    }

}
