package com.mkdoc.store;

/*
    Created:    2005-11-01
    Issued:     2005-11-01
    Modified:   2005-11-01

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import org.openrdf.sesame.Sesame;

import org.openrdf.sesame.config.RepositoryConfig;
import org.openrdf.sesame.config.SailConfig;

import org.openrdf.sesame.constants.RDFFormat;

import org.openrdf.sesame.repository.local.LocalRepository;
import org.openrdf.sesame.repository.local.LocalService;

import org.openrdf.sesame.sailimpl.memory.RdfSchemaRepositoryConfig;

import com.mkdoc.util.ApplicationContext;
import com.mkdoc.util.ApplicationContextException;

import java.io.File;
import java.io.IOException;

/**
 *  A store manager for a local Sesame repository.
 *  @version    0.1 2005-11-01
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class LocalStoreManager extends AbstractStoreManager
                                                      implements StoreManager {

    /**
     *  The current output directory.
     *  @since  Version 0.1 2005-11-01
     */
    protected File outputDir;

    /**
     *  Initialize the manager.
     *  @param  context The application context for this manager.
     *  @since  Version 0.1 2005-11-01
     */
    public final void initialize(final ApplicationContext context) {

        configureOutput(context);

        configureRepository(context);

        setActive(true);
    }

    /**
     *  Shut down the manager.
     *  @since  Version 0.1 2005-11-01
     */
    public final void shutdown() {

        if (repository != null) {

            LocalRepository localRepo = (LocalRepository) repository;

            localRepo.shutDown();
        }

        setActive(false);
    }

    /**
     *  Configure the output directory from the plugin property set.
     *  @param  context The plugin application context.
     *  @return A status message giving the path of the output directory.
     *  @since  Version 0.1 2005-11-01
     */
    protected final String configureOutput(final ApplicationContext context) {

        if (context == null) {

            throw new IllegalArgumentException(
                        "Null application context set not permitted.");
        }
        else {

            outputDir = context.getStoreDirectory();

            return "Writing to output folder: " + outputDir;
        }
    }

    /**
     *  Configure the output directory from the plugin property set.
     *  @param  context The plugin application context.
     *  @return A status message giving the path of the output directory.
     *  @since  Version 0.1 2005-11-01
     */
    protected final String configureRepository(
                                            final ApplicationContext context) {

        if (context == null) {

            throw new IllegalArgumentException(
                        "Null application context set not permitted.");
        }

        String className = this.getClass().getName();

        RepositoryConfig storeConfig = new RepositoryConfig(className);

        SailConfig syncConfig = new SailConfig(
        "org.openrdf.sesame.sailimpl.sync.SyncRdfSchemaRepository");

        String path = context.getStorePath();

        try {

            File file = null;

            if (path == null) {

                file = new File(outputDir,
                                (className + getExtension()));

                file.delete();

                // File may not have existed
                AbstractFileStoreManager.createDirectories(file);

                file.createNewFile();
            }
            else {

                file = new File(path);

                if (!file.exists()) {

                    throw new IOException(
                                "Repository file does not exist, '" +
                                file.getAbsolutePath() + "'.");
                }
            }

            SailConfig memConfig = new RdfSchemaRepositoryConfig(
                                                    file.getCanonicalPath(),
                                                    RDFFormat.RDFXML);

            storeConfig.addSail(syncConfig);
            storeConfig.addSail(memConfig);
            storeConfig.setWorldReadable(true);
            storeConfig.setWorldWriteable(true);

            LocalService service = Sesame.getService();

            // May throw ConfigurationException
            repository = (LocalRepository) service.createRepository(
                                                              storeConfig);

            return "Using repository file " +  file.getAbsolutePath();
        }
        catch (Exception e) {

            throw new ApplicationContextException(
                        "Could not create repository. " +
                        e.getMessage());
        }
    }

    /**
     *  Get the extension for storage files.
     *  @return An RDF file extension.
     *  @since  Version 0.1 2005-11-01
     */
    public final String getExtension() {

        return ".rdf";
    }

}
