package com.mkdoc.tag;

/*
    Created:    2005-07-19
    Issued:     2005-10-19
    Modified:   2005-11-01

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import com.mkdoc.query.Query;
import com.mkdoc.query.QueryBuilder;
import com.mkdoc.query.QueryFactory;
import com.mkdoc.query.QueryResult;
import com.mkdoc.query.ServletQueryBuilder;
import com.mkdoc.query.ResultRenderer;

import com.mkdoc.servlet.AbstractMKDocServlet;

import com.mkdoc.store.StoreManagerFactory;
import com.mkdoc.store.StoreManager;

import com.mkdoc.util.ApplicationContext;

import java.io.IOException;

import javax.servlet.ServletContext;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.JspException;

import javax.servlet.jsp.tagext.TagSupport;

/**
 *  Render query results to a JSP page.
 *  @version    0.8 2005-11-01
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class QueryResultTag extends TagSupport {

    /**
     *  The application context.
     *  @since  Version 0.4 2005-08-02
     */
    protected static ApplicationContext appContext;

    /**
     *  The query manager
     *  @since  Version 0.1 2005-07-19
     */
    protected static StoreManager manager;

    /**
     *  Whether extended bNode queries are supported.
     *  @since  Version 0.5 2005-09-13
     */
    private boolean extended = false;

    /**
     *  The tag output mode.
     *  @since  Version 0.1 2005-07-19
     */
    protected int mode;

    /**
     *  Get the configuration parameters for the tag.
     *  @exception  JspException    If there is an exception creating
     *          a store query manager to handle queries.
     *  @since  Version 0.1 2005-07-19
     */
    protected final void init() throws JspException {

        if (manager == null) {

            ServletContext context = pageContext.getServletContext();

            if (context == null) {

                throw new IllegalStateException(
                            "Servlet context has not been assigned.");
            }
            else {

                appContext = AbstractMKDocServlet.getApplicationContext(
                                                                      context);

                try {

                    manager = StoreManagerFactory.newStoreManager(appContext);
                }
                catch (Exception e) {

                    throw new JspException("Error in query result tag init.",
                                           e);
                }
            }
        }
        // Else done
    }

    /**
     *  Parse a query and output the results.
     *  <p>
     *    The appropriate content type should be set in the JSP template.
     *  </p>
     *  @exception  JspException    If there is a problem issuing a server
     *          error response.
     *  @return The skip body message.
     *  @since  Version 0.1 2005-07-19
     */
    public final int doStartTag() throws JspException {

        init();

        JspWriter out = pageContext.getOut();

        HttpServletResponse response = null;

        response = (HttpServletResponse) pageContext.getResponse();


        HttpServletRequest request = null;

        request = (HttpServletRequest) pageContext.getRequest();


        QueryBuilder builder = new ServletQueryBuilder(request, extended);

        builder.setDistinct(true);

        try {

            Query query = builder.getQuery();

            QueryResult result = manager.executeQuery(query);

            ResultRenderer renderer = null;

            renderer = QueryFactory.getResultRenderer(query);

            renderer.renderQueryResult(out, result, getMode());
        }
        catch (IllegalStateException ise) {

            try {

                response.sendError(response.SC_BAD_REQUEST,
                                   ise.getMessage());
            }
            catch (IOException ioe) {

                throw new JspException(ioe.getMessage(), ise);
            }
        }
        catch (Exception e) {

            try {

                response.sendError(response.SC_INTERNAL_SERVER_ERROR,
                                   e.getMessage());
            }
            catch (IOException ioe) {

                throw new JspException(ioe.getMessage(), e);
            }
        }
        return SKIP_BODY;
    }

    /**
     *  Get the output mode for the tag.
     *  @return The output mode, which is <code>QueryResultTag.FULL</code> or
     *          <code>QueryResultTag.INSERT</code>
     *  @since  Version 0.2 2005-07-22
     */
    public final int getMode() {

        return mode;
    }

    /**
     *  Set the output mode for the tag.
     *  @param  mode    The output mode for the tag, which should be either
     *          <code>ResultRenderer.FULL</code> or
     *          <code>ResultRenderer.INSERT</code>
     *  @since  Version 0.2 2005-07-22
     */
    public final void setMode(final int mode) {

        if ((mode == ResultRenderer.FULL) ||
            (mode == ResultRenderer.INSERT)) {

            this.mode = mode;
        }
        else {

            throw new IllegalArgumentException("Invalid mode , '" +
                                               mode + "'.");
        }
    }

    /**
     *  Set whether extended bNode queries are supported.
     *  @param  extended    Whether bNode queries are supported.
     *  @since  Version 0.5 2005-09-13
     */
    public final void setExtended(final boolean extended) {

        this.extended = extended;
    }

    /**
     *  Get extended bNode query status.
     *  @return Whether bNode queries are supported.
     *  @since  Version 0.7 2005-10-19
     */
    protected final boolean getExtended() {

        return extended;
    }

}
