package com.mkdoc.io;

/*
    Created:    2004-12-03
    Issued:     2004-12-03
    Modified:   2005-04-29

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2004 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import java.io.IOException;
import java.io.OutputStream;

/**
 *  Mock <code>OutputStream</code> for testing.
 *  <p>
 *    Includes a <code>setFail</code> method to simulate
 *    <code>IOException</code> on the output stream.
 *    Otherwise, stores all output to a
 *    <code>StringBuffer</code> for retieval. <em>Not to be
 *    used for significantly large output.</em>
 *  </p>
 *  @version    0.3 2005-04-29
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class MockOutputStream extends OutputStream {

    /**
     *  Whether throw exceptions with write operations.
     *  @since  Version 0.1 2004-12-03
     */
    private boolean failOnWrite = false;

    /**
     *  Whether throw exceptions when closing.
     *  @since  Version 0.1 2004-12-03
     */
    private boolean failOnClose = false;

    /**
     *  Exception message.
     *  @since  Version 0.1 2004-12-03
     */
    private String message = "MockOutputStream throws IOException.";

    /**
     *  Store the output.
     *  @since  Version 0.1 2004-12-03
     */
    private StringBuffer out = new StringBuffer();

    /**
     *  Write a single byte of data.
     *  @param  oneByte The byte to write to the output stream.
     *  @exception  IOException If the instance is set to
     *          fail.
     *  @since  Version 0.1 2004-12-03
     */
    public final void write(final int oneByte) throws IOException {

        if (failOnWrite) {

            throw new IOException(message);
        }
        else {

            out.append(oneByte);
        }
    }

    /**
     *  Write a byte array of data.
     *  @param  byteArray   The byte array to write to the
     *          output stream.
     *  @exception  IOException If the instance is set to
     *          fail.
     *  @since  Version 0.1 2004-12-03
     */
    public final void write(final byte[] byteArray) throws IOException {

        if (failOnWrite) {

            throw new IOException(message);
        }
        else {

            out.append(new String(byteArray));
        }
    }

    /**
     *  Write an extract of a byte array of data.
     *  @param  byteArray   The byte array to write to the
     *          output stream.
     *  @param  offset  The index from which to start
     *          writing.
     *  @param  length  The number of array items to write.
     *  @exception  IOException If the instance is set to
     *          fail.
     *  @since  Version 0.1 2004-12-03
     */
    public final void write(final byte[] byteArray,
                            final int offset,
                            final int length) throws IOException {

        if (failOnWrite) {

            throw new IOException(message);
        }
        else {

            out.append(new String(byteArray, offset, length));
        }
    }

    /**
     *  Flush the output stream.
     *  @exception  IOException If the instance is set to
     *          fail.
     *  @since  Version 0.1 2004-12-03
     */
    public final void flush() throws IOException {

        if (failOnWrite) {

            throw new IOException(message);
        }
        // Else do nothing
    }

    /**
     *  Close the output stream.
     *  @exception  IOException If the instance is set to
     *          fail.
     *  @since  Version 0.1 2004-12-03
     */
    public final void close() throws IOException {

        if (failOnClose) {

            throw new IOException(message);
        }
        // Else do nothing
    }

    /**
     *  Set the fail on write trigger.
     *  @param  setting Whether to fail on write actions.
     *  @since  Version 0.1 2004-12-03
     */
    public final void setFailOnWrite(final boolean setting) {

        failOnWrite = setting;
    }

    /**
     *  Set the fail on close trigger.
     *  @param  setting Whether to fail on close actions.
     *  @since  Version 0.1 2004-12-03
     */
    public final void setFailOnClose(final boolean setting) {

        failOnClose = setting;
    }

    /**
     *  Get the output.
     *  @return The content that has been written to the
     *          output.
     *  @since  Version 0.2 2004-12-16
     */
    public final String getOutput() {

        return out.toString();
    }

    /**
     *  Reset the output.
     *  @since  Version 0.3 2005-04-29
     */
    public final void reset() {

        out.setLength(0);
    }

}
