package com.mkdoc.io;

/*
    Created:    2004-12-17
    Issued:     2004-12-17
    Modified:   2005-04-29

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2004 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import java.io.IOException;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  Test suite for <code>MockOutputStream</code>.
 *  @version    0.2 2005-04-29
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class MockOutputStreamTest extends TestCase {

    /**
     *  A test string to write.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2004-12-17
     */
    protected static final String STRING = "MockOutputStreamTest string";

    /**
     *  An expected <code>IOException</code> message.
     *  @since  Versoin 0.2 2005-04-29
     */
    protected static final String EXPECTED_IOE =
    "Should throw an IOException.";

    /**
     *  A mock output stream for testing.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2004-12-17
     */
    protected MockOutputStream outStream;

    /**
     *  Named test instance.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2004-12-17
     */
    public MockOutputStreamTest(final String name) {

        super(name);
    }

    /**
     *  Set up a test scenario as required.
     *  @since  Version 0.1 2004-
     */
    protected final void setUp() {

        outStream = new MockOutputStream();
    }

    /**
     *  Written byte should be copied exactly.
     *  @exception  IOException If there is a problem
     *          writing the test value to the stream,
     *          indicates an error.
     *  @since  Version 0.1 2004-12-17
     */
    public final void testWriteByte() throws IOException {

        int testByte = 255;

        outStream.write(testByte);

        String testString = outStream.getOutput();

        int testInt = Integer.parseInt(testString);

        assertTrue(testInt == testByte);

    }

    /**
     *  Written byte should be copied exactly.
     *  @exception  IOException If there is a problem
     *          writing the test value to the stream,
     *          indicates an error.
     *  @since  Version 0.2 2005-04-29
     */
    public final void testReset() throws IOException {

        int testByte = 255;

        outStream.write(testByte);

        outStream.reset();

        String testString = outStream.getOutput();

        assertTrue(testString.equals(""));
    }

    /**
     *  Write should throw an <code>IOException</code>.
     *  @since  Version 0.1 2004-12-17
     */
    public final void testWriteByteFail() {

        int testByte = 101;

        outStream.setFailOnWrite(true);

        try {

            outStream.write(testByte);

            fail(EXPECTED_IOE);
        }
        catch (IOException ioe) { }

    }

    /**
     *  Written byte array should be copied exactly.
     *  @exception  IOException If there is a problem
     *          writing the test value to the stream,
     *          indicates an error.
     *  @since  Version 0.1 2004-12-17
     */
    public final void testWriteByteArray() throws IOException {

        outStream.write(STRING.getBytes());

        String testString = outStream.getOutput();

        assertTrue(testString.equals(STRING));

    }

    /**
     *  Write byte array should throw an
     *  <code>IOException</code>.
     *  @since  Version 0.1 2004-12-17
     */
    public final void testWriteByteArrayFail() {

        outStream.setFailOnWrite(true);

        try {

            outStream.write(STRING.getBytes());

            fail(EXPECTED_IOE);
        }
        catch (IOException ioe) { }

    }

    /**
     *  Written byte array segment should be copied
     *  exactly.
     *  @exception  IOException If there is a problem
     *          writing the test value to the stream,
     *          indicates an error.
     *  @since  Version 0.1 2004-12-17
     */
    public final void testWriteByteArraySegment() throws IOException {

        int testIdx = STRING.indexOf("Test");

        assertTrue(testIdx > -1);

        String expected = STRING.substring(testIdx, testIdx + 4);

        outStream.write(STRING.getBytes(), testIdx, 4);

        String testString = outStream.getOutput();

        assertTrue(testString.equals(expected));

    }

    /**
     *  Write byte array segment should throw an
     *  <code>IOExcption</code>.
     *  @since  Version 0.1 2004-12-17
     */
    public final void testWriteByteArraySegmentFail() {

        outStream.setFailOnWrite(true);

        try {

            outStream.write(STRING.getBytes(), 0, STRING.length());

            fail(EXPECTED_IOE);
        }
        catch (IOException ioe) { }

    }

    /**
     *  Pure coverage test, no side effect.
     *  @exception  IOException Should not be thrown by
     *          this test, indicates an error.
     *  @since  Version 0.1 2004-12-17
     */
    public final void testFlush() throws IOException {

        outStream.flush();

        // No assertion
    }

    /**
     *  Flush should throw an <code>IOException</code>.
     *  @since  Version 0.1 2004-12-17
     */
    public final void testFlushFail() {

        outStream.setFailOnWrite(true);

        try {

            outStream.flush();

            fail(EXPECTED_IOE);
        }
        catch (IOException ioe) { }
    }

    /**
     *  Pure coverage test, no side effect.
     *  @exception  IOException Should not be thrown by
     *          this test, indicates an error.
     *  @since  Version 0.1 2004-12-17
     */
    public final void testClose() throws IOException {

        outStream.close();

        // No assertion
    }

    /**
     *  Close should throw an <code>IOException</code>.
     *  @since  Version 0.1 2004-12-17
     */
    public final void testCloseFail() {

        outStream.setFailOnClose(true);

        try {

            outStream.close();

            fail(EXPECTED_IOE);
        }
        catch (IOException ioe) { }
    }


    /**
     *  Tear down a test scenario as required.
     *  @since  Version 0.1 2004-
    protected void tearDown() {

    }
     */

    /**
     *  Create a test suite for this test case.
     *  @return A test suite.
     *  @since  Version 0.1 2004-12-17
     */
    public static Test suite() {

        return new TestSuite(MockOutputStreamTest.class);
    }

    /**
     *  Run this test suite as a stand alone application.
     *  @param  args    Any command line arguments to pass
     *          to the test. None required or expected.
     *  @since  Version 0.1 2004-12-17
     */
    public static void main(final String[] args) {

        junit.textui.TestRunner.run(suite());
    }

}
