package com.mkdoc.jspider;

/*
    Created:    2004-12-16
    Issued:     2004-12-16
    Modified:   2005-07-14

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2004 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import net.javacoding.jspider.api.event.JSpiderEvent;

import net.javacoding.jspider.api.event.resource.ResourceFetchedEvent;

import net.javacoding.jspider.core.logging.Log;

import com.mkdoc.util.ApplicationContext;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  Test suite for <code>AbstractMKDocPlugin</code>.
 *  @version    0.7 2005-07-14
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class AbstractMKDocPluginTest extends TestCase {

    /**
     *  An expected <code>IllegalArgumentException</code> message.
     *  @since  Version 0.4 2005-03-29
     */
    public static final String EXPECTED_IAE =
    "Should throw an IllegalArgumentException.";

    /**
     *  A basic implementation for testing.
     *  @since  Version 0.1 2004-12-16
     */
    protected MockMKDocPlugin plugin;

    /**
     *  Named test instance.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2004-12-16
     */
    public AbstractMKDocPluginTest(final String name) {

        super(name);
    }

    /**
     *  Set up a test scenario as required.
     *  @since  Version 0.1 2004-12-16
     */
    protected final void setUp() {

        plugin = new MockMKDocPlugin();
    }

    /**
     *  Get an application context.
     *  @since  Version 0.7 2005-07-14
     */
    public final void testGetApplicationContextNullPropertySet() {

        try {

            plugin.getApplicationContext(null);

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Get an application context.
     *  @since  Version 0.7 2005-07-14
     */
    public final void testGetApplicationContext() {

        MockPropertySet propertySet = new MockPropertySet();

        String name = "test";

        String value = "success";


        propertySet.setString(name, value);


        ApplicationContext context = plugin.getApplicationContext(propertySet);

        String test = context.getStringParameter(name);

        assertTrue(test.equals(value));
    }

    /**
     *  Pure coverage test for initialize - empty.
     *  @since  Version 0.5 2005-04-29
     */
    public final void testInitialize() {

        plugin.initialize();

        // No assertion
    }

    /**
     *  Pure coverage test for shutdown - empty.
     *  @since  Version 0.5 2005-04-29
     */
    public final void testShutdown() {

        plugin.shutdown();

        // No assertion
    }

    /**
     *  Set a <code>null</code> log should throw an
     *  <code>IllegalArgumentException</code>.
     *  @since  Version 0.2 2005-01-14
     */
    public final void testSetLogNull() {

        try {

            plugin.setLog(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Set a <code>null</code> log should throw an
     *  <code>IllegalArgumentException</code>.
     *  @since  Version 0.2 2005-01-14
     */
    public final void testSetLog() {

        Log log1 = plugin.log;

        plugin.setLog(new MockLog());

        Log log2 = plugin.log;

        assertFalse(log2.equals(log1));

    }

    /**
     *  Set/get vendor, initially null.
     *  @since  Version 0.1 2004-12-16
     */
    public final void testGetVendor() {

        String vendor = "http://www.mkdoc.com";

        assertTrue(vendor.equals(plugin.getVendor()));

    }

    /**
     *  Set/get name, initially null.
     *  @since  Version 0.1 2004-12-16
     */
    public final void testSetGetName() {

        assertNull(plugin.getName());

        String name = "Plugin name";

        plugin.setName(name);

        assertTrue(name.equals(plugin.getName()));
    }

    /**
     *  Set/get description, initially null.
     *  @since  Version 0.1 2004-12-16
     */
    public final void testSetGetDescription() {

        assertNull(plugin.getDescription());

        String description = "Plugin description";

        plugin.setDescription(description);

        assertTrue(description.equals(plugin.getDescription()));
    }

    /**
     *  Set/get version, initially null.
     *  @since  Version 0.1 2004-12-16
     */
    public final void testSetGetVersion() {

        assertNull(plugin.getVersion());

        String version = "Version 0.1";

        plugin.setVersion(version);

        assertTrue(version.equals(plugin.getVersion()));
    }

    /**
     *  Notify and get JSpider event, initially null.
     *  @since  Version 0.1 2004-12-16
     */
    public final void testNotifyGetLast() {

        assertNull(plugin.getLastEvent());

        MockFetchedResource resource = new MockFetchedResource();

        JSpiderEvent event = new ResourceFetchedEvent(resource);

        plugin.notify(event);

        assertTrue(event.equals(plugin.getLastEvent()));
    }

    /**
     *  Tear down a test scenario as required.
     *  @exception  IOException For canonical file name
     *          check.
     *  @since  Version 0.1 2004-12-16
    protected final void tearDown() {

    }
     */

    /**
     *  Create a test suite for this test case.
     *  @return A test suite.
     *  @since  Version 0.1 2004-12-16
     */
    public static Test suite() {

        return new TestSuite(AbstractMKDocPluginTest.class);
    }

    /**
     *  Run this test suite as a stand alone application.
     *  @param  args    Any command line arguments to pass
     *          to the test. None required or expected.
     *  @since  Version 0.1 2004-12-16
     */
    public static void main(final String[] args) {

        junit.textui.TestRunner.run(suite());
    }

}
