package com.mkdoc.jspider;

/*
    Created:    2005-03-15
    Issued:     2005-03-15
    Modified:   2005-07-12

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import net.javacoding.jspider.api.event.resource.ResourceFetchedEvent;
import net.javacoding.jspider.api.event.resource.ResourceDiscoveredEvent;

import com.mkdoc.sax.RdfContentHandler;
import com.mkdoc.sax.MetaTripleWriter;

import com.mkdoc.schema.UKeGMSProfile;

import com.mkdoc.store.MockFileStoreManager;
import com.mkdoc.store.StoreException;

import com.mkdoc.util.ApplicationContext;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.IOException;

import java.net.URL;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  Test suite for <code>AbstractRdfPlugin</code>.
 *  @version    0.7 2005-07-12
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class AbstractRdfPluginTest extends TestCase {

    /**
     *  A mock plugin instance to test.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2005-03-15
     */
    protected MockRdfPlugin mockPlugin;

    /**
     *  A test domain.
     *  @since  Version 0.3 2005-05-10
     */
    protected static final String DOMAIN = System.getProperty("testDomain");

    /**
     *  The project source directory.
     *  @since  Version 0.3 2005-05-10
     */
    protected static final String HOME = System.getProperty("mkHome");

    /**
     *  The project test source directory.
     *  @since  Version 0.3 2005-05-10
     */
    protected static final String TEST = System.getProperty("testDir") +
                                         File.separator +
                                         "html";

    /**
     *  A test domain URL.
     *  @since  Version 0.3 2005-05-10
     */
    protected static final String URL_DOMAIN = "http://" + DOMAIN;

    /**
     *  The test source directory path.
     *  @since  Version 0.3 2005-05-10
     */
    protected static final String PATH_TEST = HOME +
                                              File.separator +
                                              TEST;

    /**
     *  An invalid source file for testing.
     *  @since  Version 0.3 2005-05-10
     */
    protected static final String INVALID = "invalid-Basic.html";

    /**
     *  A file reference for invalid source for testing.
     *  @since  Version 0.3 2005-05-10
     */
    protected static final File SRC_INVALID = new File(PATH_TEST +
                                                       File.separator +
                                                       INVALID);

    /**
     *  Named test instance.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2005-03-15
     */
    public AbstractRdfPluginTest(final String name) {

        super(name);
    }

    /**
     *  Set up a test scenario as required.
     *  @since  Version 0.1 2005-
     */
    protected final void setUp() {

        mockPlugin = new MockRdfPlugin();
    }

    /**
     *  Test load application profile with null property set should throw an
     *  <code>IllegalArgumentException</code>.
     *  @exception  ClassNotFoundException  Should not be thrown by this test.
     *  @exception  InstantiationException  Should not be thrown by this test.
     *  @exception  IllegalAccessException  Should not be thrown by this test.
     *  @since  Version 0.1 2005-03-15
     */
    public final void testLoadApplicationProfileNullPropertySet()
        throws ClassNotFoundException, InstantiationException,
        IllegalAccessException {

        try {

            mockPlugin.loadApplicationProfile(null);

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test load application profile none configured.
     *  @exception  ClassNotFoundException  Should not be thrown by this test.
     *  @exception  InstantiationException  Should not be thrown by this test.
     *  @exception  IllegalAccessException  Should not be thrown by this test.
     *  @since  Version 0.4 2005-05-26
     */
    public final void testLoadApplicationProfileNotConfigured()
        throws ClassNotFoundException, InstantiationException,
        IllegalAccessException {

        MockPropertySet propertySet = new MockPropertySet();

        ApplicationContext context = null;

        context = mockPlugin.getApplicationContext(propertySet);

        String message = mockPlugin.loadApplicationProfile(context);

        assertTrue(message.indexOf(
                    "No custom application profile configured") > -1);
    }

    /**
     *  Configure handler with no profile configured.
     *  @exception  ClassNotFoundException  Should not be thrown by this test.
     *  @exception  InstantiationException  Should not be thrown by this test.
     *  @exception  IllegalAccessException  Should not be thrown by this test.
     *  @exception  StoreException  Should not be thrown by this test.
     *  @exception  IOException  Should not be thrown by this test.
     *  @since  Version 0.4 2005-05-26
     */
    public final void testConfigureHandlerProfileNotConfigured()
        throws ClassNotFoundException, InstantiationException,
        IllegalAccessException, StoreException, IOException {

        MockPropertySet propertySet = new MockPropertySet();

        ApplicationContext context = null;

        context = mockPlugin.getApplicationContext(propertySet);

        mockPlugin.loadApplicationProfile(context);

        MockFileStoreManager manager = new MockFileStoreManager();

        URL url = new URL("http://www.remote.example/");

        RdfContentHandler handler = new MetaTripleWriter(manager,
                                                         url);


        mockPlugin.configureHandler(handler);


        String predicate = null;

        String expanded =  null;


        predicate = "eGMS.Addressee";

        expanded = handler.expandPredicate(predicate);

        assertTrue(expanded.equals(predicate));


        predicate = "dc.Identifier";

        expanded = handler.expandPredicate(predicate);

        String expected = "http://purl.org/dc/elements/1.1/identifier";

        assertTrue("The AbstractRdfContentHandler has a default " +
                   "Dublin Core Elements profile, so should expand " +
                   "this predicate to '" +
                   expected +
                   "', actual expanded value was '" +
                   expanded + "'.",
                   expanded.equals(expected));

    }

    /**
     *  Test load a custom profile.
     *  @exception  ClassNotFoundException  Should not be thrown by this test.
     *  @exception  InstantiationException  Should not be thrown by this test.
     *  @exception  IllegalAccessException  Should not be thrown by this test.
     *  @since  Version 0.4 2005-05-26
     */
    public final void testLoadApplicationProfileSuccess()
        throws ClassNotFoundException, InstantiationException,
        IllegalAccessException {

        String eGMS = "com.mkdoc.schema.UKeGMSProfile";

        MockPropertySet propertySet = new MockPropertySet();

        propertySet.setClass("profile", Class.forName(eGMS));

        ApplicationContext context = null;

        context = mockPlugin.getApplicationContext(propertySet);

        String message = mockPlugin.loadApplicationProfile(context);

        assertTrue(message.indexOf(eGMS) > -1);

        assertTrue(mockPlugin.profile instanceof UKeGMSProfile);
    }

    /**
     *  Test configure an RDF content handler with null handler.
     *  @since  Version 0.4 2005-05-26
     */
    public final void testConfigureHandlerNullHandler() {

        RdfContentHandler handler = null;

        try {

            mockPlugin.configureHandler(handler);

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }

    }


    /**
     *  Test configure an RDF content handler with custom profile.
     *  @exception  ClassNotFoundException  Should not be thrown by this test.
     *  @exception  InstantiationException  Should not be thrown by this test.
     *  @exception  IllegalAccessException  Should not be thrown by this test.
     *  @exception  StoreException  Should not be thrown by this test.
     *  @exception  IOException  Should not be thrown by this test.
     *  @since  Version 0.4 2005-05-26
     */
    public final void testConfigureHandlerProfileLoaded()
        throws ClassNotFoundException, InstantiationException,
        IllegalAccessException, StoreException, IOException  {

        String eGMS = "com.mkdoc.schema.UKeGMSProfile";

        MockPropertySet propertySet = new MockPropertySet();

        propertySet.setClass("profile", Class.forName(eGMS));

        ApplicationContext context = null;

        context = mockPlugin.getApplicationContext(propertySet);

        String message = mockPlugin.loadApplicationProfile(context);

        assertTrue(message.indexOf(eGMS) > -1);

        MockFileStoreManager manager = new MockFileStoreManager();

        URL url = new URL("http://www.remote.example/");


        RdfContentHandler handler = new MetaTripleWriter(manager, url);


        String predicate = null;

        String expanded =  null;


        predicate = "eGMS.Addressee";

        expanded = handler.expandPredicate(predicate);

        assertTrue(expanded.equals(predicate));


        mockPlugin.configureHandler(handler);


        expanded = handler.expandPredicate(predicate);

        String expected =
        "http://www.govtalk.gov.uk/terms/addressee";

        assertTrue("Should expand this predicate to '" +
                   expected +
                   "', actual expanded value was '" +
                   expanded + "'.",
                   expanded.equals(expected));

    }


    /**
     *  With null event should throw an <code>IllegalArgumentException</code>.
     *  @since  Version 0.3 2005-05-10
     */
    public final void testNotifyNullEvent() {

        try {

            mockPlugin.notify(null);

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  With wrong type should ignore.
     *  @since  Version 0.3 2005-05-10
     *  @exception  IOException Should not be thown by this test.
     */
    public final void testNotifyWrongType()
        throws IOException {

        MockFetchedResource resource = new MockFetchedResource();

        resource.setUrl(new URL(URL_DOMAIN));

        InputStream fileInStream = new FileInputStream(SRC_INVALID);

        resource.setInputStream(fileInStream);

        ResourceDiscoveredEvent rde = new ResourceDiscoveredEvent(resource);

        mockPlugin.notify(rde);

        assertNull("Should not parse source for this event type.",
                   mockPlugin.getParsedUrl());

    }

    /**
     *  Should parse the source.
     *  @exception  IOException In check for canonical
     *          path, or read write operations.
     *  @since  Version 0.3 2005-05-10
     */
    public final void testNotifyRightType()
        throws IOException {

        MockFetchedResource resource = new MockFetchedResource();

        URL url = new URL(URL_DOMAIN);

        resource.setUrl(url);

        InputStream fileInStream = new FileInputStream(SRC_INVALID);

        resource.setInputStream(fileInStream);

        ResourceFetchedEvent rfe = new ResourceFetchedEvent(resource);

        mockPlugin.notify(rfe);

        URL parsedUrl = mockPlugin.getParsedUrl();

        assertTrue("Should parse the source for this event type.",
                   (parsedUrl != null) &&
                   (parsedUrl.equals(url)));

    }

    /**
     *  Tear down a test scenario as required.
     *  @since  Version 0.1 2005-
    protected void tearDown() {

    }
     */

    /**
     *  Create a test suite for this test case.
     *  @return A test suite.
     *  @since  Version 0.1 2005-03-15
     */
    public static Test suite() {

        return new TestSuite(AbstractRdfPluginTest.class);
    }

    /**
     *  Run this test suite as a stand alone application.
     *  @param  args    Any command line arguments to pass
     *          to the test. None required or expected.
     *  @since  Version 0.1 2005-03-15
     */
    public static void main(final String[] args) {

        junit.textui.TestRunner.run(suite());
    }

}
