package com.mkdoc.jspider;

/*
    Created:    2005-10-04
    Issued:     2005-10-04
    Modified:   2005-10-28

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import com.mkdoc.util.ApplicationContext;

import java.io.File;

/**
 *  Test suite for <code>PluginApplicationContext</code>.
 *  @version    0.3 2005-10-28
 *  @author Philip Shaw <phil@mkdoc.com>
 */
public class PluginApplicationContextTest extends TestCase {


    /**
     *  An &quot;actual path was...&quot; test error report statement.
     *  @since  Version 0.2 2005-10-14
     */
    public static final String ACTUAL_PATH = "', actual path was: '";

    /**
     *  The home directory.
     *  @since  Version 0.2 2005-10-14
     */
    public static final String HOME = System.getProperty("mkHome");

    /**
     *  The output directory name.
     *  @since  Version 0.2 2005-10-14
     */
    public static final String OUT_DIR = System.getProperty("outputDir");

    /**
     *  The test directory name.
     *  @since  Version 0.2 2005-10-14
     */
    public static final String TEST_DIR = System.getProperty("testDir");

    /**
     *  The base path for the output directory.
     *  @since  Version 0.2 2005-10-14
     */
    public static final String BASE_PATH = HOME +
                                           File.separator +
                                           OUT_DIR;

    /**
     *  An alternative test directory path.
     *  @since  Version 0.2 2005-10-14
     */
    public static final String ALT_PATH = HOME +
                                          File.separator +
                                          TEST_DIR;

    /**
     *  The RDF store file name alone.
     *  @since  Version 0.2 2005-10-14
     */
    public static final String STORE_FILE = System.getProperty("storeFile");

    /**
     *  A relative store path.
     *  @since  Version 0.2 2005-10-14
     */
    public static final String STORE_PATH = File.separator +
                                            STORE_FILE;

    /**
     *  An absolute store path.
     *  @since  Version 0.2 2005-10-14
     */
    public static final String FULL_PATH = BASE_PATH + STORE_PATH;

    /**
     *  An alternative absolute store path.
     *  @since  Version 0.2 2005-10-14
     */
    public static final String FULL_ALT_PATH = ALT_PATH + STORE_PATH;

    /**
     *  An absolute test directory path.
     *  @since  Version 0.2 2005-10-14
     */
    public static final String TEST_OUTPUT_PATH = BASE_PATH +
                                                  File.separator +
                                                  TEST_DIR;

    /**
     *  A property set to configure.
     *  @since  Version 0.2 2005-10-14
     */
    protected MockPropertySet propertySet;

    /**
     *  An application context to configure.
     *  @since  Version 0.2 2005-10-14
     */
    protected ApplicationContext appContext;

    /**
     *  Named test instance.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2005-10-04
     */
    public PluginApplicationContextTest(final String name) {

        super(name);
    }

    /**
     *  Set up a test scenario as required.
     *  @since  Version 0.1 2005-10-04
     */
    protected final void setUp() {

        propertySet = new MockPropertySet();

        appContext = new PluginApplicationContext(propertySet);
    }

    /**
     *  Test get a class parameter.
     *  @since  Version 0.1 2005-10-05
     */
    public final void testGetClassParameter() {

        String key = "classKey";

        Class firstClass = this.getClass();

        propertySet.setClass(key, firstClass);

        Class testClass = appContext.getClassParameter(key);

        assertTrue(testClass.equals(firstClass));
    }

    /**
     *  Test get a class parameter.
     *  @since  Version 0.1 2005-10-05
     */
    public final void testGetClassParameterNull() {

        Class testClass = appContext.getClassParameter("classKey");

        assertNull(testClass);
    }

    /**
     *  Test constructor with null argument.
     *  @since  Version 0.1 2005-10-05
     */
    public final void testConstructorNullPropertySet() {

        try {

            new PluginApplicationContext(null);

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Test get store path.
     *  @since  Version 0.1 2005-10-04
     */
    public final void testGetStorePathNull() {

        String storePath = appContext.getStorePath();

        assertNull("Store path should be null.",
                   storePath);
    }

    /**
     *  Test get store directory, default.
     *  @since  Version 0.1 2005-10-04
     */
    public final void testGetStoreDirectoryDefault() {

        File dir = appContext.getStoreDirectory();

        String dirPath = dir.getAbsolutePath();

        File expectedFile = new File(BASE_PATH);

        String expected = expectedFile.getAbsolutePath();

        assertTrue("Directory path should equal: '" +
                   expected +
                   ACTUAL_PATH +
                   dirPath + "'.",
                   dirPath.equals(expected));
    }

    /**
     *  Test get store directory.
     *  @since  Version 0.1 2005-10-04
     */
    public final void testGetStoreDirectorySubdirectory() {

        propertySet.setString(ApplicationContext.DIRECTORY, TEST_DIR);

        File dir = appContext.getStoreDirectory();

        String dirPath = dir.getAbsolutePath();

        File expectedFile = new File(TEST_OUTPUT_PATH);

        String expected = expectedFile.getAbsolutePath();

        assertTrue("Directory path should equal: '" +
                   expected +
                   ACTUAL_PATH +
                   dirPath + "'.",
                   dirPath.equals(expected));
    }

    /**
     *  Test get store directory, default.
     *  @since  Version 0.1 2005-10-04
     */
    public final void testGetStoreDirectoryAbsoluteNoFolder() {

        propertySet.setBoolean(ApplicationContext.ABSOLUTE, true);

        File dir = appContext.getStoreDirectory();

        String dirPath = dir.getAbsolutePath();

        File expectedFile = new File(BASE_PATH);

        String expected = expectedFile.getAbsolutePath();

        assertTrue("Directory path should equal: '" +
                   expected +
                   ACTUAL_PATH +
                   dirPath + "'.",
                   dirPath.equals(expected));
    }



    /**
     *  Test get store path.
     *  @since  Version 0.1 2005-10-04
     */
    public final void testGetStorePathWithRepositoryParam() {

        propertySet.setString("repository", STORE_PATH);

        String storePath = appContext.getStorePath();

        File expectedFile = new File(FULL_PATH);

        String expected = expectedFile.getAbsolutePath();

        assertTrue("Store path should equal: '" +
                   expected +
                   ACTUAL_PATH +
                   storePath + "'.",
                   storePath.equals(expected));
    }

    /**
     *  Test get store path not under default output path.
     *  @since  Version 0.1 2005-10-04
     */
    public final void testGetStorePathWithAbsoluteFolder() {

        propertySet.setBoolean(ApplicationContext.ABSOLUTE, true);
        propertySet.setString(ApplicationContext.DIRECTORY, ALT_PATH);
        propertySet.setString("repository", STORE_PATH);

        String storePath = appContext.getStorePath();

        File expectedFile = new File(FULL_ALT_PATH);

        String expected = expectedFile.getAbsolutePath();

        assertTrue("Store path should equal: '" +
                   expected +
                   ACTUAL_PATH +
                   storePath + "'.",
                   storePath.equals(expected));
    }

    /**
     *  Tear down a test scenario as required.
     *  @since  Version 0.1 2005-10-04
    protected void tearDown() {

    }
     */

    /**
     *  Create a test suite for this test case.
     *  @return A test suite.
     *  @since  Version 0.1 2005-10-04
     */
    public static Test suite() {

        return new TestSuite(PluginApplicationContextTest.class);
    }

    /**
     *  Run this test suite as a stand alone application.
     *  @param  args    Any command line arguments to pass
     *          to the test. None required or expected.
     *  @since  Version 0.1 2005-10-04
     */
    public static void main(final String[] args) {

        junit.textui.TestRunner.run(suite());
    }

}
