package com.mkdoc.jspider;

/*
    Created:    2005-04-11
    Issued:     2005-04-11
    Modified:   2005-10-28

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import net.javacoding.jspider.api.model.Decision;

import java.io.IOException;

import java.net.URL;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  Test suite for <code>RdfMimeTypeOnlyRule</code>.
 *  @version    0.1 2005-10-28
 *  @author Philip Shaw <phil@mkdoc.com>
 */
public class RdfMimeTypeOnlyRuleTest extends TestCase {

    /**
     *  An RDF content type.
     *  @since  Version 0.1 2005-10-28
     */
    public static final String RDF_TYPE = "application/rdf+xml; charset=utf-8";

    /**
     *  An RSS content type.
     *  @since  Version 0.1 2005-10-28
     */
    public static final String RSS_TYPE = "application/rss+xml; charset=utf-8";

    /**
     *  An HTML content type.
     *  @since  Version 0.1 2005-10-28
     */
    public static final String HTML_TYPE = "text/html; charset=utf-8";

    /**
     *  A text content type.
     *  @since  Version 0.1 2005-10-28
     */
    public static final String TEXT_TYPE = "text/plain";

    /**
     *  A rule to test.
     *  @since  Version 0.1 2005-10-28
     */
    public static final RdfMimeTypeOnlyRule RULE =
    new RdfMimeTypeOnlyRule();

    /**
     *  A test domain.
     *  @since  Version 0.1 2005-10-28
     */
    public static final String DOMAIN = System.getProperty("testDomain");

    /**
     *  A base URL.
     *  @since  Version 0.1 2005-10-28
     */
    public static final String BASE_URL = "http://" +
                                          DOMAIN;

    /**
     *  A test URL.
     *  @since  Version 0.1 2005-10-28
     */
    public static final String RDF_URL = BASE_URL +
                                         "/RdfResource";


    /**
     *  A test URL.
     *  @since  Version 0.1 2005-10-28
     */
    public static final String RSS_URL = BASE_URL +
                                         "/RssResource";

    /**
     *  A test URL.
     *  @since  Version 0.1 2005-10-28
     */
    public static final String HTML_URL = BASE_URL +
                                         "/HtmlResource";

    /**
     *  A test URL.
     *  @since  Version 0.1 2005-10-28
     */
    public static final String TEXT_URL = BASE_URL +
                                          "/TextResource";


    /**
     *  A site to test.
     *  @since  Version 0.1 2005-10-28
     */
    protected MockSite site;

    /**
     *  A spider context for testing.
     *  @since  Version 0.1 2005-10-28
     */
    protected MockSpiderContext spiderContext;

    /**
     *  Named test instance.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2005-10-28
     */
    public RdfMimeTypeOnlyRuleTest(final String name) {

        super(name);
    }

    /**
     *  Set up a test scenario as required.
     *  @exception  IOException Should not be thrown by this method.
     *  @since  Version 0.1 2005-10-28
     */
    protected final void setUp() throws IOException {


        site = new MockSite(new URL(BASE_URL));

        MockResourceDAO resourceDAO = new MockResourceDAO();


        MockFetchedResource rdfResource = new MockFetchedResource();

        rdfResource.setMime(RDF_TYPE);

        resourceDAO.registerResource(rdfResource, new URL(RDF_URL));


        MockFetchedResource rssResource = new MockFetchedResource();

        rssResource.setMime(RSS_TYPE);

        resourceDAO.registerResource(rssResource, new URL(RSS_URL));


        MockFetchedResource htmlResource = new MockFetchedResource();

        htmlResource.setMime(HTML_TYPE);

        resourceDAO.registerResource(htmlResource, new URL(HTML_URL));


        MockFetchedResource textResource = new MockFetchedResource();

        textResource.setMime(TEXT_TYPE);

        resourceDAO.registerResource(textResource, new URL(TEXT_URL));


        MockStorage storage = new MockStorage();

        storage.setResourceDAO(resourceDAO);


        spiderContext = new MockSpiderContext();

        spiderContext.setStorage(storage);
    }


    /**
     *  Test the apply method.
     *  @exception  IOException Should not be thrown by this test.
     *  @since  Version 0.1 2005-10-28
     */
    public final void testApplyNullSpiderContext() throws IOException {

        try {

            RULE.apply(null, site, new URL(RDF_URL));

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }

    }


    /**
     *  Test the apply method.
     *  @exception  IOException Should not be thrown by this test.
     *  @since  Version 0.1 2005-10-28
     */
    public final void testApplyNullSite() throws IOException {

        try {

            RULE.apply(spiderContext, null, new URL(RDF_URL));

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Test the apply method.
     *  @exception  IOException Should not be thrown by this test.
     *  @since  Version 0.1 2005-10-28
     */
    public final void testApplyNullUrl() throws IOException {

        try {

            RULE.apply(spiderContext, site, null);

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }

    }



    /**
     *  Test the apply method.
     *  @exception  IOException Should not be thrown by this test.
     *  @since  Version 0.1 2005-10-28
     */
    public final void testApplyRdfUrl() throws IOException {

        Decision decision = RULE.apply(spiderContext, site, new URL(RDF_URL));

        assertTrue(decision.getDecision() == Decision.RULE_ACCEPT);
    }

    /**
     *  Test the apply method.
     *  @exception  IOException Should not be thrown by this test.
     *  @since  Version 0.1 2005-10-28
     */
    public final void testApplyRssUrl() throws IOException {

        Decision decision = RULE.apply(spiderContext, site, new URL(RSS_URL));

        assertTrue(decision.getDecision() == Decision.RULE_ACCEPT);
    }

    /**
     *  Test the apply method.
     *  @exception  IOException Should not be thrown by this test.
     *  @since  Version 0.1 2005-10-28
     */
    public final void testApplyHtmlUrl() throws IOException {

        Decision decision = RULE.apply(spiderContext, site, new URL(HTML_URL));

        assertTrue(decision.getDecision() == Decision.RULE_IGNORE);
    }

    /**
     *  Test the apply method.
     *  @exception  IOException Should not be thrown by this test.
     *  @since  Version 0.1 2005-10-28
     */
    public final void testApplyTextUrl() throws IOException {

        Decision decision = RULE.apply(spiderContext, site, new URL(TEXT_URL));

        assertTrue(decision.getDecision() == Decision.RULE_IGNORE);
    }

    /**
     *  Test the accept method.
     *  @since  Version 0.1 2005-10-28
     */
    public final void testAcceptNull() {

        assertFalse(RULE.accept(null));
    }

    /**
     *  Test the accept method.
     *  @since  Version 0.1 2005-10-28
     */
    public final void testAcceptRdf() {

        assertTrue(RULE.accept(RDF_TYPE));
    }

    /**
     *  Test the accept method.
     *  @since  Version 0.1 2005-10-28
     */
    public final void testAcceptRss() {

        assertTrue(RULE.accept(RSS_TYPE));
    }

    /**
     *  Test the accept method.
     *  @since  Version 0.1 2005-10-28
     */
    public final void testAcceptHtml() {

        assertFalse(RULE.accept(HTML_TYPE));
    }

    /**
     *  Test the accept method.
     *  @since  Version 0.1 2005-10-28
     */
    public final void testAcceptText() {

        assertFalse(RULE.accept(TEXT_TYPE));
    }


    /**
     *  Tear down a test scenario as required.
     *  @since  Version 0.1 2005-10-28
    protected void tearDown() {

    }
     */

    /**
     *  Create a test suite for this test case.
     *  @return A test suite.
     *  @since  Version 0.1 2005-10-28
     */
    public static Test suite() {

        return new TestSuite(RdfMimeTypeOnlyRuleTest.class);
    }

    /**
     *  Run this test suite as a stand alone application.
     *  @param  args    Any command line arguments to pass
     *          to the test. None required or expected.
     *  @since  Version 0.1 2005-10-28
     */
    public static void main(final String[] args) {

        junit.textui.TestRunner.run(suite());
    }

}
