package com.mkdoc.query;

/*
    Created:    2005-07-06
    Issued:     2005-07-06
    Modified:   2005-09-15

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import com.mkdoc.servlet.MockHttpServletRequest;

import com.mkdoc.schema.ApplicationProfile;
import com.mkdoc.schema.DublinCoreProfile;
import com.mkdoc.schema.SchemaProperty;
import com.mkdoc.schema.UKeGMSProfile;

import java.util.List;
import java.util.Vector;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  Test suite for <code>ServletQueryBuilder</code>.
 *  @version    0.8 2005-09-15
 *  @author Philip Shaw <phil@mkdoc.com>
 */
public class ServletQueryBuilderTest extends TestCase {

    /**
     *  An expected <code>IllegalArgumentException</code> message.
     *  @since  Version 0.1 2005-07-06
     */
    private static final String EXPECTED_IAE =
    "Should throw an IllegalArgumentException.";

    /**
     *  An expected <code>IllegalStateException</code> message.
     *  @since  Version 0.1 2005-07-06
     */
    private static final String EXPECTED_ISE =
    "Should throw an IllegalStateException.";

    /**
     *  A <code>dc:subject</code> parameter name.
     *  @since  Version 0.1 2005-07-06
     */
    private static final String PARAM_SUBJ = "dc.subject";

    /**
     *  A <code>dc:date</code> parameter name.
     *  @since  Version 0.7 2005-09-14
     */
    private static final String PARAM_DATE = "dc.date";

    /**
     *  A <code>dc:identifier</code> parameter name.
     *  @since  Version 0.8 2005-09-15
     */
    private static final String PARAM_ID = "dc.identifier";

    /**
     *  A URI identifier.
     *  @since  Version 0.8 2005-09-15
     */
    private static final String URI_ID = "http://remote.example/uri/test";

    /**
     *  A non-URI identifier
     *  @since  Version 0.8 2005-09-15
     */
    private static final String NON_URI_ID = "TestID:12345";

    /**
     *  A date parameter value.
     *  @since  Version 0.7 2005-09-14
     */
    private static final String DATE = "2005-09-14";

    /**
     *  A <code>dc:title</code> parameter name.
     *  @since  Version 0.1 2005-07-06
     */
    private static final String PARAM_TITLE = "dc.title";

    /**
     *  A single word subject query parameter.
     *  @since  Version 0.1 2005-07-06
     */
    private static final String SUBJ_BASIC = "Angiostatins";

    /**
     *  A quoted phrase subject query parameter.
     *  @since  Version 0.1 2005-07-06
     */
    private static final String SUBJ_QUOTED = "\"Test document\"";

    /**
     *  The expected 'WHERE' condition for the quoted subject.
     *  @since  Version 0.1 2005-07-06
     */
    private static final String WHERE_QUOTED =
    "WHERE label(dcSubject) LIKE \"Test document\"";

    /**
     *  The expected 'WHERE' condition for the extended, quoted subject.
     *  @since  Version 0.7 2005-09-13
     */
    private static final String X_DATE =
    "label(dcDate) LIKE \"2005-09-14\"";

    /**
     *  The expected 'WHERE' condition for a URI identifier.
     *  @since  Version 0.8 2005-09-15
     */
    private static final String X_URI =
    "dcIdentifier LIKE \"http://remote.example/uri/test\"";

    /**
     *  The expected 'WHERE' condition for a non-URI identifier.
     *  @since  Version 0.8 2005-09-15
     */
    private static final String X_NON_URI =
    "label(dcIdentifier) LIKE \"TestID:12345\"";


    /**
     *  A SeRQL SELECT statement.
     *  @since  Version 0.1 2005-07-06
     */
    private static final String SELECT = "SELECT ";

    /**
     *  The target URI variable.
     *  @since  Version 0.1 2005-07-06
     */
    public static final String TARGET_VAR = "targetUri";

    /**
     *  The subject variable.
     *  @since  Version 0.1 2005-07-06
     */
    public static final String SUBJECT_VAR = "dcSubject";

    /**
     *  The date variable.
     *  @since  Version 0.5 23005-09-14
     */
    public static final String DATE_VAR = "dcDate";

    /**
     *  The identifier variable.
     *  @since  Version 0.8 2005-09-15
     */
    public static final String ID_VAR = "dcIdentifier";

    /**
     *  The bNode subject variable.
     *  @since  Version 0.7 2005-09-13
     */
    public static final String SUBJECT_X = "dcSubjectBNode";

    /**
     *  A SeRQL FROM statement.
     *  @since  Version 0.1 2005-07-06
     */
    private static final String FROM = " FROM ";

    /**
     *  The subject path expression.
     *  @since  Version 0.1 2005-07-06
     */
    public static final String SUBJECT_EXPR =
    "{targetUri} dc:subject {dcSubject}";

    /**
     *  The extended bNode date path expression.
     *  @since  Version 0.1 2005-07-06
     */
    public static final String X_EXPR_DATE =
    "{targetUri} dc:date {} rdf:value {dcDate}";

    /**
     *  The extended bNode subject path expression.
     *  @since  Version 0.1 2005-07-06
     */
    public static final String X_EXPR_SUBJ =
    "{targetUri} dc:subject {} rdf:value {dcSubject}";

    /**
     *  The extended bNode identifier path expression.
     *  @since  Version 0.8 2005-09-15
     */
    public static final String X_EXPR_ID =
    "{targetUri} dc:identifier {} rdf:value {dcIdentifier}";

    /**
     *  A basic identifier path expression.
     *  @since  Version 0.8 2005-09-15
     */
    public static final String EXPR_ID =
    "{targetUri} dc:identifier {dcIdentifier}";

    /**
     *  The title path expression.
     *  @since  Version 0.1 2005-07-06
     */
    public static final String TITLE_EXPR =
    "{targetUri} dc:title {dcTitle}";

    /**
     *  The description path expression.
     *  @since  Version 0.1 2005-07-06
     */
    public static final String DESC_EXPR =
    "{targetUri} dc:description {dcDescription}";

    /**
     *  The <code>dc</code> namespace prefix.
     *  @since  Version 0.1 2005-07-06
     */
    public static final String DC_PREFIX = "dc";

    /**
     *  The <code>dc</code> namespace URI.
     *  @since  Version 0.1 2005-07-06
     */
    public static final String DC_URI = "http://purl.org/dc/elements/1.1/";

    /**
     *  An SeRQL query namespace declaration.
     *  @since  Version 0.1 2005-07-06
     */
    private static final String Q_NS_DC = " USING NAMESPACE " +
                                          DC_PREFIX +
                                          " = <" +
                                          DC_URI +
                                          ">";

    /**
     *  A single word title query parameter.
     *  @since  Version 0.1 2005-07-06
     */
    private static final String TITLE_BASIC = "Procurement";

    /**
     *  A multiple word title query parameter.
     *  @since  Version 0.4 2005-07-29
     */
    private static final String TITLE_MULTI = "Government Procurement ";

    /**
     *  The title variable.
     *  @since  Version 0.1 2005-07-06
     */
    public static final String TITLE_VAR = "dcTitle";

    /**
     *  A <code>dc:description</code> parameter name.
     *  @since  Version 0.1 2005-07-06
     */
    private static final String PARAM_DESC = "dc.description";

    /**
     *  A single word description query parameter.
     *  @since  Version 0.1 2005-07-06
     */
    private static final String DESC_BASIC = "Grammar";

    /**
     *  The description variable.
     *  @since  Version 0.1 2005-07-06
     */
    public static final String DESC_VAR = "dcDescription";

    /**
     *  An empty string query parameter.
     *  @since  Version 0.1 2005-07-06
     */
    private static final String EMPTY = "";

    /**
     *  The &quot;DISTINCT&quot; keyword.
     *  @since  Version 0.1 2005-07-06
     */
    private static final String DISTINCT = "DISTINCT ";

    /**
     *  A <code>dc:description</code> parameter name.
     *  @since  Version 0.1 2005-07-06
     */
    private static final String PARAM_BOGUS = "bogus.notRecognised";

    /**
     *  A single word description query parameter.
     *  @since  Version 0.1 2005-07-06
     */
    private static final String VALUE_BOGUS = "Test negative";

    /**
     *  A <code>dc:subject</code> predicate.
     *  @since  Version 0.2 2005-07-26
     */
    public static final String DC_SUBJECT = "dc:subject";

    /**
     *  A <code>dc:identifier</code> predicate.
     *  @since  Version 0.8 2005-09-15
     */
    public static final String DC_ID = "dc:identifier";


    /**
     *  A <code>dc:subject</code> predicate.
     *  @since  Version 0.7 2005-09-14
     */
    public static final String DC_DATE = "dc.date";

    /**
     *  A target subject term.
     *  @since  Version 0.2 2005-07-26
     */
    public static final String SUBJECT_1 = "Angiostatins";

    /**
     *  A variable name.
     *  @since  Version 0.2 2005-07-26
     */
    public static final String VAR_1 = "targetUri";

    /**
     *  A variable name.
     *  @since  Version 0.2 2005-07-26
     */
    public static final String VAR_2 = "dcSubject";

    /**
     *  A namespace declaration.
     *  @since  Version 0.2 2005-07-26
     */
    public static final String NS_1 = "dc = <http://purl.org/dc/elements/1.1/>";

    /**
     *  A string value.
     *  @since  Version 0.2 2005-07-26
     */
    public static final String VALUE_1 = "value1";

    /**
     *  A string value.
     *  @since  Version 0.2 2005-07-26
     */
    public static final String VALUE_2 = "value2";

    /**
     *  A string value.
     *  @since  Version 0.2 2005-07-26
     */
    public static final String VALUE_3 = "value3";

    /**
     *  An &quot; AND &quot; condition separator string.
     *  @since  Version 0.2 2005-07-26
     */
    public static final String AND = " AND ";

    /**
     *  An &quot; OR &quot; condition separator string.
     *  @since  Version 0.2 2005-07-26
     */
    public static final String OR_COND = " OR ";

    /**
     *  A &quot; LIKE &quot; query keyword.
     *  @since  Version 0.2 2005-07-26
     */
    public static final String LIKE = " LIKE ";

    /**
     *  A &quot; WHERE &quot; statement keyword.
     *  @since  Version 0.2 2005-07-26
     */
    public static final String WHERE = " WHERE ";

    /**
     *  A &quot; USING NAMESPACE &quot; clause.
     *  @since  Version 0.2 2005-07-26
     */
    public static final String USING = " USING NAMESPACE ";

    /**
     *  A space character.
     *  @since  Version 0.2 2005-07-26
     */
    public static final String SPACE = " ";

    /**
     *  An <code>CONSTRUCT</code> query keyword.
     *  @since  Version 0.2 2005-07-26
     */
    public static final String CONSTRUCT = "CONSTRUCT ";

    /**
     *  A path expression.
     *  @since  Version 0.2 2005-07-26
     */
    public static final String EXPR_1 = "{" +
                                        VAR_1 +
                                        "} " +
                                        DC_SUBJECT +
                                        " {" +
                                        VAR_2 +
                                        "}";

    /**
     *  A date path expression.
     *  @since  Version 0.2 2005-07-26
     */
    public static final String EXPR_DATE = "{" +
                                        VAR_1 +
                                        "} " +
                                        DC_DATE +
                                        " {" +
                                        DATE_VAR +
                                        "}";

    /**
     *  An extended identifier path expression.
     *  @since  Version 0.8 2005-09-15
     */
    public static final String EXPR_ID_X = "{" +
                                        VAR_1 +
                                        "} " +
                                        DC_ID +
                                        " {} rdf:value {" +
                                        ID_VAR +
                                        "}";


    /**
     *  A query condition.
     *  @since  Version 0.2 2005-07-26
     */
    public static final String COND_1 = "label(" +
                                        VAR_2 +
                                        ")" +
                                        LIKE +
                                        "\"" +
                                        SUBJECT_1 +
                                        "\"";

    /**
     *  A concatenated value list.
     *  @since  Version 0.2 2005-07-26
     */
    public static final String OR_LIST = VALUE_1 +
                                         OR_COND +
                                         VALUE_2 +
                                         OR_COND +
                                         VALUE_3;

    /**
     *  A minimal select query statement.
     *  @since  Version 0.2 2005-07-26
     */
    public static final String Q_MINIMAL = SELECT +
                                             VAR_1 +
                                             ", " +
                                             SUBJECT_VAR +
                                             FROM +
                                             EXPR_1 +
                                    " WHERE label(dcSubject) LIKE \"testing\"" +
                                      USING +
                                      NS_1;


    /**
     *  A distinct select query statement.
     *  @since  Version 0.2 2005-07-26
     */
    public static final String Q_DISTINCT = SELECT +
                                            DISTINCT +
                                             VAR_1 +
                                             ", " +
                                             SUBJECT_VAR +
                                             FROM +
                                             EXPR_1 +
                                    " WHERE label(dcSubject) LIKE \"testing\"" +
                                      USING +
                                      NS_1;

    /**
     *  A distinct select query statement.
     *  @since  Version 0.7 2005-09-14
     */
    public static final String Q_DIST_DATE = SELECT +
                                                 DISTINCT +
                                                 VAR_1 +
                                                 ", " +
                                                 DATE_VAR +
                                                 FROM +
                                                 EXPR_DATE +
                                    " WHERE label(dcDate) LIKE \"2005-09-14\"" +
                                      USING +
                                      NS_1;

    /**
     *  A URI identifier select query, basic form.
     *  @since  Version 0.8 2005-09-15
     */
    public static final String Q_ID_URI = SELECT +
                                          DISTINCT +
                                          VAR_1 +
                                          ", " +
                                          ID_VAR +
                                          FROM +
                                          EXPR_ID +
                                          " WHERE dcIdentifier LIKE \"" +
                                          URI_ID +
                                          "\"" +
                                          USING +
                                          NS_1;

    /**
     *  A URI identifier select query, extended form.
     *  @since  Version 0.8 2005-09-15
     */
    public static final String Q_ID_URI_X = SELECT +
                                            DISTINCT +
                                            VAR_1 +
                                            ", " +
                                            ID_VAR +
                                            FROM +
                                            EXPR_ID_X +
                                            " WHERE dcIdentifier LIKE \"" +
                                            URI_ID +
                                            "\"" +
                                            USING +
                                            NS_1;

    /**
     *  A non-URI identifier select query, basic form.
     *  @since  Version 0.8 2005-09-15
     */
    public static final String Q_ID_NON_URI = SELECT +
                                          DISTINCT +
                                          VAR_1 +
                                          ", " +
                                          ID_VAR +
                                          FROM +
                                          EXPR_ID +
                                          " WHERE label(dcIdentifier) LIKE \"" +
                                          NON_URI_ID +
                                          "\"" +
                                          USING +
                                          NS_1;

    /**
     *  A non-URI identifier select query, extended form.
     *  @since  Version 0.8 2005-09-15
     */
    public static final String Q_NON_URI_X = SELECT +
                                            DISTINCT +
                                            VAR_1 +
                                            ", " +
                                            ID_VAR +
                                            FROM +
                                            EXPR_ID_X +
                                         " WHERE label(dcIdentifier) LIKE \"" +
                                            NON_URI_ID +
                                            "\"" +
                                            USING +
                                            NS_1;


    /**
     *  A minimal select query with namespace declaration.
     *  @since  Version 0.2 2005-07-26
     */
    public static final String Q_NS = SELECT +
                                      VAR_1 +
                                      ", " +
                                      SUBJECT_VAR +
                                      FROM +
                                      EXPR_1 +
                                   " WHERE label(dcSubject) LIKE \"testing\"" +
                                      USING +
                                      NS_1;

    /**
     *  A minimal construct query statement.
     *  @since  Version 0.2 2005-07-26
     */
    public static final String Q_CONSTRUCT = CONSTRUCT +
                                             VAR_1 +
                                             ", " +
                                             SUBJECT_VAR +
                                             FROM +
                                             EXPR_1 +
                                   " WHERE label(dcSubject) LIKE \"testing\"" +
                                      USING +
                                      NS_1;

    /**
     *  A minimal select query statement with condition.
     *  @since  Version 0.2 2005-07-26
     */
    public static final String Q_CONDITION = SELECT +
                                             VAR_1 +
                                             FROM +
                                             EXPR_1 +
                                             WHERE +
                                             COND_1;

    /**
     *  A list of strings.
     *  @since  Version 0.2 2005-07-26
     */
    protected List listMultiple;

    /**
     *  A list of one string.
     *  @since  Version 0.2 2005-07-26
     */
    protected List listOne;

    /**
     *  A servlet request for testing.
     *  @since  Version 0.1 2005-07-06
     */
    protected MockHttpServletRequest msreq;

    /**
     *  A Dublin Core application profile for testing.
     *  @since  Version 0.1 2005-07-06
     */
    protected ApplicationProfile profile = new DublinCoreProfile();

    /**
     *  Named test instance.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2005-07-06
     */
    public ServletQueryBuilderTest(final String name) {

        super(name);
    }

    /**
     *  Set up a test scenario as required.
     *  @since  Version 0.1 2005-07-06
     */
    public final void setUp() {

        msreq = new MockHttpServletRequest();

        listMultiple = new Vector();

        listMultiple.add(VALUE_1);
        listMultiple.add(VALUE_2);
        listMultiple.add(VALUE_3);

        listOne = new Vector();

        listOne.add(VALUE_1);

    }

    /**
     *  Test appending a single term.
     *  @since  Version 0.6 2005-09-12
     */
    public final void testAppendTermSingle() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        StringBuffer buffer = new StringBuffer();

        String term = "Test";

        builder.appendTerm(term, buffer);

        String appended = buffer.toString();

        assertTrue("Appended terms should equal '" +
                   term + ", was actually '" +
                   appended + "'.",
                   appended.equals(term));
    }

    /**
     *  Test appending a multiple terms.
     *  @since  Version 0.6 2005-09-12
     */
    public final void testAppendTermPhrase() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        StringBuffer buffer = new StringBuffer();

        String term = "Test document";

        String expected = "%22Test+document%22";

        builder.appendTerm(term, buffer);

        String appended = buffer.toString();

        assertTrue("Appended term should equal '" +
                   expected + "', was actually '" +
                   appended + "'.",
                   appended.equals(expected));
    }



    /**
     *  Test append a list.
     *  @since  Version 0.2 2005-07-26
     */
/*
    public final void testAppendListNullList() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        StringBuffer buffer = new StringBuffer();

        try {

            builder.appendList(null, buffer, OR_COND);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }
*/

    /**
     *  Test append a list.
     *  @since  Version 0.2 2005-07-26
     */
/*
    public final void testAppendListNullBuffer() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        try {

            builder.appendList(listOne, null, OR_COND);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }
*/

    /**
     *  Test append a list.
     *  @since  Version 0.2 2005-07-26
     */
/*
    public final void testAppendListNullSeparator() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        StringBuffer buffer = new StringBuffer();

        try {

            builder.appendList(listOne, buffer, null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }
*/

    /**
     *  Test append a list.
     *  @since  Version 0.2 2005-07-26
     */
/*
    public final void testAppendListOneItem() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        StringBuffer buffer = new StringBuffer();

        builder.appendList(listOne, buffer, OR_COND);

        assertTrue(VALUE_1.equals(buffer.toString()));
    }
*/

    /**
     *  Test append a list.
     *  @since  Version 0.2 2005-07-26
     */
/*
    public final void testAppendListMultipleItems() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        StringBuffer buffer = new StringBuffer();

        builder.appendList(listMultiple, buffer, OR_COND);

        assertTrue(OR_LIST.equals(buffer.toString()));
    }
*/

    /**
     *  Test append a list.
     *  @since  Version 0.2 2005-07-26
     */
/*
    public final void testAppendListEmptyList() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        StringBuffer buffer = new StringBuffer();

        List emptyList = new Vector();

        try {

            builder.appendList(emptyList, buffer, OR_COND);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }
*/

    /**
     *  Test adding a null string variable.
     *  @since  Version 0.2 2005-07-26
     */
/* No longer used
    public final void testAddVariableNullString() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        String nullVariable = null;

        try {

            builder.addVariable(nullVariable);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }
*/

    /**
     *  Test get a query context.
     *  @since  Version 0.5 2005-08-02
     */
    public final void testGetQueryContext() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        QueryContext context = builder.getQueryContext();

        assertTrue(context instanceof XhtmlQueryContext);
    }

    /**
     *  Test get a query context.
     *  @since  Version 0.5 2005-08-02
     */
    public final void testGetQueryContextText() {

        msreq.setParameter("type", "text");

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        QueryContext context = builder.getQueryContext();

        assertTrue(context instanceof TextQueryContext);
    }

    /**
     *  Test get a query context.
     *  @since  Version 0.5 2005-08-02
     */
    public final void testGetQueryContextRss() {

        msreq.setParameter("type", "rss");

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        QueryContext context = builder.getQueryContext();

        assertTrue(context instanceof RssQueryContext);
    }

    /**
     *  Test get a query context.
     *  @since  Version 0.5 2005-08-02
     */
    public final void testGetQueryContextXhtml() {

        msreq.setParameter("type", "html");

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        QueryContext context = builder.getQueryContext();

        assertTrue(context instanceof XhtmlQueryContext);
    }

    /**
     *  Test adding a null schema property variable.
     *  @since  Version 0.2 2005-07-26
     */
    public final void testAddVariableNullSchemaProperty() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        SchemaProperty nullProperty = null;

        try {

            builder.addVariable(nullProperty);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test adding a null schema property expression.
     *  @since  Version 0.2 2005-07-26
     */
    public final void testAddExpressionNullSchemaProperty() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        SchemaProperty nullProperty = null;

        try {

            builder.addExpression(nullProperty);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test adding a null schema property expression.
     *  @since  Version 0.2 2005-07-26
     */
    public final void testAddSummaryExpressionNullSchemaProperty() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        SchemaProperty nullProperty = null;

        try {

            builder.addSummaryExpression(nullProperty);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test adding a null schema property expression.
     *  @since  Version 0.3 2005-07-27
     */
    public final void testAddSummaryExpression() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        ApplicationProfile eGMS = new UKeGMSProfile();

        SchemaProperty property = eGMS.getSummaryProperty();

        builder.addSummaryExpression(property);

        // No assertion
    }

    /**
     *  Test adding a null schema property expression.
     *  @since  Version 0.3 2005-07-27
     */
    public final void testAddTitleExpression() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        ApplicationProfile eGMS = new UKeGMSProfile();

        SchemaProperty property = eGMS.getTitleProperty();

        builder.addTitleExpression(property);

        // No assertion
    }


    /**
     *  Test adding a null schema property expression.
     *  @since  Version 0.2 2005-07-26
     */
    public final void testAddTitleExpressionNullSchemaProperty() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        SchemaProperty nullProperty = null;

        try {

            builder.addTitleExpression(nullProperty);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test adding a null schema property condition.
     *  @since  Version 0.2 2005-07-26
     */
    public final void testAddConditionNullSchemaProperty() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        SchemaProperty nullProperty = null;

        try {

            builder.addCondition(nullProperty, "test term");

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test adding a null schema property condition.
     *  @since  Version 0.2 2005-07-26
     */
    public final void testAddConditionNullTerm() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        ApplicationProfile eGMS = new UKeGMSProfile();

        SchemaProperty property = eGMS.getPrimarySearchProperty();

        try {

            builder.addCondition(property, null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test setting the query type.
     *  @since  Version 0.2 2005-07-26
     */
    public final void testSetQueryTypeNull() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        try {

            builder.setQueryType(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test setting the query type.
     *  @since  Version 0.2 2005-07-26
     */
    public final void testSetQueryTypeConstruct() {

        msreq.setParameter(PARAM_SUBJ, SUBJ_BASIC);

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        String type = "CONSTRUCT";

        builder.setQueryType(type);

        Query query = builder.getQuery();

        String serqlQuery = query.getSeRQLQuery();

        assertTrue(serqlQuery.indexOf(type) > -1);
    }

    /**
     *  Test to string, not ready.
     *  @since  Version 0.2 2005-07-26
     */
/*
    public final void testToStringIllegalStateException() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        try {

            builder.toString();

            fail(EXPECTED_ISE);
        }
        catch (IllegalStateException ise) { }
    }
*/

    /**
     *  Test to string, construct query.
     *  @since  Version 0.2 2005-07-26
     */
    public final void testGetSeRQLQueryConstruct() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        ApplicationProfile eGMS = new UKeGMSProfile();

        SchemaProperty property = eGMS.getPrimarySearchProperty();

        builder.setQueryType("CONSTRUCT");

        builder.addCondition(property, "testing");

        String query = builder.getSeRQLQuery(false);

        assertTrue("Query should equal:\n\n" + Q_CONSTRUCT + "'\n" +
                   "actual query was:\n\n" + query,
                   Q_CONSTRUCT.equals(query));
    }

    /**
     *  Test to string minimal.
     *  <p>
     *    Note this currently generates an invalid query because the
     *    <code>dc</code> namespace is not declared.
     *  </p>
     *  @since  Version 0.2 2005-07-26
     */
    public final void testGetSeRQLQueryMinimal() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        ApplicationProfile eGMS = new UKeGMSProfile();

        SchemaProperty property = eGMS.getPrimarySearchProperty();

        builder.addCondition(property, "testing");

        String query = builder.getSeRQLQuery(false);

        assertTrue("Query should equal:\n\n" + Q_MINIMAL + "'\n" +
                   "actual query was:\n\n" + query,
                   Q_MINIMAL.equals(query));
    }

    /**
     *  Test to string with distinct condition.
     *  <p>
     *    Note this currently generates an invalid query because the
     *    <code>dc</code> namespace is not declared.
     *  </p>
     *  @since  Version 0.2 2005-07-26
     */
    public final void testGetSeRQLQueryDistinct() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        ApplicationProfile eGMS = new UKeGMSProfile();

        SchemaProperty property = eGMS.getPrimarySearchProperty();

        builder.setDistinct(true);

        builder.addCondition(property, "testing");

        String query = builder.getSeRQLQuery(false);

        assertTrue("Query should equal:\n\n" + Q_DISTINCT + "'\n" +
                   "actual query was:\n\n" + query,
                   Q_DISTINCT.equals(query));
    }

    /**
     *  Test to string with distinct condition.
     *  <p>
     *    Note this currently generates an invalid query because the
     *    <code>dc</code> namespace is not declared.
     *  </p>
     *  @since  Version 0.7 2005-09-14
     */
    public final void testGetSeRQLQueryDistinctExtended() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        ApplicationProfile eGMS = new UKeGMSProfile();

        SchemaProperty property = eGMS.getPredicateProperty(PARAM_DATE);

        builder.setDistinct(true);

        builder.addCondition(property, DATE);

        String query = builder.getSeRQLQuery(true);

        assertTrue("Query should include:\n\n" +
                   X_DATE + "'\n" +
                   "actual query was:\n\n" + query,
                   query.indexOf(X_DATE) > -1);
    }


    /**
     *  Test adding a null schema property namespace declaration.
     *  @since  Version 0.2 2005-07-26
     */
    public final void testAddNamespaceDeclarationNullSchemaProperty() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        SchemaProperty nullProperty = null;

        try {

            builder.addNamespaceDeclaration(nullProperty);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test all delimiter characters.
     *  @since  Version 0.1 2005-07-06
     */
    public final void testIsDelimiterTrue() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        assertTrue(builder.isDelimiter(" "));
        assertTrue(builder.isDelimiter("\n"));
        assertTrue(builder.isDelimiter("\r"));
        assertTrue(builder.isDelimiter("\t"));
        assertTrue(builder.isDelimiter("\f"));
    }

    /**
     *  Test get query terms with null query.
     *  @since  Version 0.1 2005-07-06
     */
    public final void testGetQueryTermsNull() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        try {

            builder.getQueryTerms(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test get query terms with empty string.
     *  @since  Version 0.1 2005-07-06
     */
    public final void testGetQueryTermsEmpty() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        List list = builder.getQueryTerms("");

        assertTrue(list.size() == 0);

    }

    /**
     *  Test get query terms with un-terminated phrase.
     *  @since  Version 0.1 2005-07-06
     */
    public final void testGetQueryTermsUnTerminatedQuote() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        String quotation = "quotation, you see";

        String query = "un terminated \"" + quotation;

        List list = builder.getQueryTerms(query);

        assertTrue(list.contains(quotation));

    }

    /**
     *  Get a query, no parameters.
     *  @since  Version 0.1 2005-07-06
     */
    public final void testGetQueryNoParameters() {

        msreq = new MockHttpServletRequest();

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        try {

            builder.getQuery();

            fail(EXPECTED_ISE);
        }
        catch (IllegalStateException ise) { }

    }

    /**
     *  Get a query, no parameters.
     *  @since  Version 0.1 2005-07-06
     */
    public final void testGetQueryAddedVariableNoParameters() {

        msreq = new MockHttpServletRequest();

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        ApplicationProfile eGMS = new UKeGMSProfile();

        builder.addVariable(eGMS.getPrimarySearchProperty());

        try {

            builder.getQuery();

            fail(EXPECTED_ISE);
        }
        catch (IllegalStateException ise) { }

    }

    /**
     *  Get a query, empty parameter.
     *  @since  Version 0.1 2005-07-06
     */
    public final void testGetQueryEmptyParameter() {

        msreq = new MockHttpServletRequest();

        msreq.setParameter(PARAM_SUBJ, "");

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        try {

            builder.getQuery();

            fail(EXPECTED_ISE);
        }
        catch (IllegalStateException ise) { }

    }

    /**
     *  Build a subject query.
     *  @since  Version 0.1 2005-07-06
     */
    public final void testGetQuerySubjectBasic() {

        msreq = new MockHttpServletRequest();

        msreq.setParameter(PARAM_SUBJ, SUBJ_BASIC);

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        Query query = builder.getQuery();

        String serqlQuery = query.getSeRQLQuery();

        // The order of variables is not significant, nor controlled
        assertTrue("Query should include a select statement, " +
                   "target URI and title variables:\n\n" +
                   SELECT + ", " +
                   TARGET_VAR + ", " +
                   SUBJECT_VAR + "\n\n" +
                   "Actual query was:\n\n" +
                   serqlQuery,
               ((serqlQuery.indexOf(SELECT) > -1) &&
                ((serqlQuery.indexOf(TARGET_VAR + ", " + SUBJECT_VAR) > -1) ||
                 (serqlQuery.indexOf(SUBJECT_VAR + ", " + TARGET_VAR) > -1))));

        assertTrue("Query should include the subject path expression:\n\n" +
                   FROM + SUBJECT_EXPR +
                   "\n\nActual query was:\n\n" +
                   serqlQuery,
                   serqlQuery.indexOf(FROM + SUBJECT_EXPR) > -1);

        assertTrue("Query should include the DC namespace declaration:\n\n" +
                   Q_NS_DC +
                   "\n\nActual query was:\n\n" +
                   serqlQuery,
                   serqlQuery.indexOf(Q_NS_DC) > -1);

    }

    /**
     *  Build a subject query.
     *  @since  Version 0.1 2005-07-06
     */
    public final void testGetQuerySubjectDistinctFalse() {

        msreq = new MockHttpServletRequest();

        msreq.setParameter(PARAM_SUBJ, SUBJ_BASIC);

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        builder.setDistinct(false);

        Query query = builder.getQuery();

        String serqlQuery = query.getSeRQLQuery();

        // The order of variables is not significant, nor controlled
        assertTrue("Query should include a select statement, " +
                   "target URI and title variables:\n\n" +
                   SELECT + ", " +
                   TARGET_VAR + ", " +
                   SUBJECT_VAR + "\n\n" +
                   "Actual query was:\n\n" +
                   serqlQuery,
               ((serqlQuery.indexOf(SELECT) > -1) &&
                ((serqlQuery.indexOf(TARGET_VAR + ", " + SUBJECT_VAR) > -1) ||
                 (serqlQuery.indexOf(SUBJECT_VAR + ", " + TARGET_VAR) > -1))));

        assertTrue("Query should include the subject path expression:\n\n" +
                   FROM + SUBJECT_EXPR +
                   "\n\nActual query was:\n\n" +
                   serqlQuery,
                   serqlQuery.indexOf(FROM + SUBJECT_EXPR) > -1);

        assertTrue("Query should include the DC namespace declaration:\n\n" +
                   Q_NS_DC +
                   "\n\nActual query was:\n\n" +
                   serqlQuery,
                   serqlQuery.indexOf(Q_NS_DC) > -1);

        assertTrue("Query should not include the DISTINCT keyword, " +
                   "actual query was:\n\n" +
                   serqlQuery,
                   serqlQuery.indexOf(DISTINCT) == -1);

    }

    /**
     *  Build a subject query.
     *  @since  Version 0.1 2005-07-06
     */
    public final void testGetQuerySubjectDistinctTrue() {

        msreq = new MockHttpServletRequest();

        msreq.setParameter(PARAM_SUBJ, SUBJ_BASIC);

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        builder.setDistinct(true);

        Query query = builder.getQuery();

        String serqlQuery = query.getSeRQLQuery();

        // The order of variables is not significant, nor controlled
        assertTrue("Query should include a select statement, " +
                   "target URI and title variables:\n\n" +
                   SELECT + ", " +
                   TARGET_VAR + ", " +
                   SUBJECT_VAR + "\n\n" +
                   "Actual query was:\n\n" +
                   serqlQuery,
               ((serqlQuery.indexOf(SELECT) > -1) &&
                ((serqlQuery.indexOf(TARGET_VAR + ", " + SUBJECT_VAR) > -1) ||
                 (serqlQuery.indexOf(SUBJECT_VAR + ", " + TARGET_VAR) > -1))));

        assertTrue("Query should include the subject path expression:\n\n" +
                   FROM + SUBJECT_EXPR +
                   "\n\nActual query was:\n\n" +
                   serqlQuery,
                   serqlQuery.indexOf(FROM + SUBJECT_EXPR) > -1);

        assertTrue("Query should include the DC namespace declaration:\n\n" +
                   Q_NS_DC +
                   "\n\nActual query was:\n\n" +
                   serqlQuery,
                   serqlQuery.indexOf(Q_NS_DC) > -1);

        assertTrue("Query should include the DISTINCT keyword, " +
                   "actual query was:\n\n" +
                   serqlQuery,
                   serqlQuery.indexOf(DISTINCT) > -1);

    }

    /**
     *  Build a query with only a single unrecognised parameter.
     *  @since  Version 0.1 2005-07-06
     */
    public final void testGetQueryPredicateNotRecognised() {

        msreq = new MockHttpServletRequest();

        msreq.setParameter(PARAM_BOGUS, VALUE_BOGUS);

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        try {

            builder.getQuery();

            fail(EXPECTED_ISE);
        }
        catch (IllegalStateException ise) { }
    }

    /**
     *  Set the application profile.
     *  @since  Version 0.1 2005-07-06
     */
    public final void testSetApplicationProfileNull() {

        msreq = new MockHttpServletRequest();

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        try {

            builder.setApplicationProfile(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Set the application profile.
     *  @since  Version 0.1 2005-07-06
     */
    public final void testSetApplicationProfileUKeGMS() {

        msreq = new MockHttpServletRequest();

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        ApplicationProfile testProfile = null;

        testProfile = builder.profile;

        assertTrue(testProfile instanceof DublinCoreProfile);

        builder.setApplicationProfile(new UKeGMSProfile());

        testProfile = builder.profile;

        assertTrue(testProfile instanceof UKeGMSProfile);
    }

    /**
     *  Build a title query.
     *  @since  Version 0.1 2005-07-06
     */
    public final void testGetQueryTitleBasic() {

        msreq = new MockHttpServletRequest();

        msreq.setParameter(PARAM_TITLE, TITLE_BASIC);

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        Query query = builder.getQuery();

        String serqlQuery = query.getSeRQLQuery();

        // The order of variables is not significant, nor controlled
        assertTrue("Query should include a select statement, " +
                   "target URI and title variables:\n\n" +
                   SELECT + ", " +
                   TARGET_VAR + ", " +
                   TITLE_VAR + "\n\n" +
                   "Actual query was:\n\n" +
                   serqlQuery,
               ((serqlQuery.indexOf(SELECT) > -1) &&
                ((serqlQuery.indexOf(TARGET_VAR + ", " + TITLE_VAR) > -1) ||
                 (serqlQuery.indexOf(TITLE_VAR + ", " + TARGET_VAR) > -1))));

        assertTrue("Query should include the title path expression:\n\n" +
                   FROM + TITLE_EXPR +
                   "\n\nActual query was:\n\n" +
                   serqlQuery,
                   serqlQuery.indexOf(FROM + TITLE_EXPR) > -1);

        assertTrue("Query should include the DC namespace declaration:\n\n" +
                   Q_NS_DC +
                   "\n\nActual query was:\n\n" +
                   serqlQuery,
                   serqlQuery.indexOf(Q_NS_DC) > -1);

    }

    /**
     *  Build a description query.
     *  @since  Version 0.1 2005-07-06
     */
    public final void testGetQueryDescriptionBasic() {

        msreq = new MockHttpServletRequest();

        msreq.setParameter(PARAM_DESC, DESC_BASIC);

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        Query query = builder.getQuery();

        String serqlQuery = query.getSeRQLQuery();

        // The order of variables is not significant, nor controlled
        assertTrue("Query should include a select statement, " +
                   "target URI and title variables:\n\n" +
                   SELECT + ", " +
                   TARGET_VAR + ", " +
                   DESC_VAR + "\n\n" +
                   "Actual query was:\n\n" +
                   serqlQuery,
               ((serqlQuery.indexOf(SELECT) > -1) &&
                ((serqlQuery.indexOf(TARGET_VAR + ", " + DESC_VAR) > -1) ||
                 (serqlQuery.indexOf(DESC_VAR + ", " + TARGET_VAR) > -1))));

        assertTrue("Query should include the path expression:\n\n" +
                   FROM + DESC_EXPR +
                   "\n\nActual query was:\n\n" +
                   serqlQuery,
                   serqlQuery.indexOf(FROM + DESC_EXPR) > -1);

        assertTrue("Query should include the DC namespace declaration:\n\n" +
                   Q_NS_DC +
                   "\n\nActual query was:\n\n" +
                   serqlQuery,
                   serqlQuery.indexOf(Q_NS_DC) > -1);

    }

    /**
     *  Check parse parameter method.
     *  @since  Version 0.8 2005-06-17
     */
    public final void testParseParameterNullFieldValue() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        SchemaProperty property = profile.getPredicateProperty(PARAM_SUBJ);

        StringBuffer buffer = new StringBuffer();

        try {

            builder.parseParameter(null, property, buffer);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Check parse parameter method.
     *  @since  Version 0.8 2005-06-17
     */
    public final void testParseParameterNullProperty() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        SchemaProperty property = null;

        StringBuffer buffer = new StringBuffer();

        try {

            builder.parseParameter(SUBJ_BASIC, property, buffer);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Check the interpretation of a subject query.
     *  @since  Version 0.2 2005-01-24
     */
    public final void testParseParameterSubjectBasic() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        SchemaProperty property = profile.getPredicateProperty(PARAM_SUBJ);

        StringBuffer buffer = new StringBuffer();

        builder.parseParameter(SUBJ_BASIC, property, buffer);

        Query query = builder.getQuery();

        String serqlQuery = query.getSeRQLQuery();

        // The order of variables is not significant, nor controlled
        assertTrue("Query should include a select statement, " +
                   "target URI and subject variables:\n\n" +
                   SELECT + ", " +
                   TARGET_VAR + ", " +
                   SUBJECT_VAR + "\n\n" +
                   "Actual query was:\n\n" +
                   serqlQuery,
               ((serqlQuery.indexOf(SELECT) > -1) &&
                ((serqlQuery.indexOf(TARGET_VAR + ", " + SUBJECT_VAR) > -1) ||
                 (serqlQuery.indexOf(SUBJECT_VAR + ", " + TARGET_VAR) > -1))));

        assertTrue("Query should include the subject path expression:\n\n" +
                   FROM + SUBJECT_EXPR +
                   "\n\nActual query was:\n\n" +
                   serqlQuery,
                   serqlQuery.indexOf(FROM + SUBJECT_EXPR) > -1);

        assertTrue("Query should include the DC namespace declaration:\n\n" +
                   Q_NS_DC +
                   "\n\nActual query was:\n\n" +
                   serqlQuery,
                   serqlQuery.indexOf(Q_NS_DC) > -1);
    }

    /**
     *  Check the concatenation of a multiple parameters.
     *  @since  Version 0.5 2005-08-02
     */
    public final void testParseParameterMultiple() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        SchemaProperty property = null;

        property = profile.getPredicateProperty(PARAM_SUBJ);

        StringBuffer buffer = new StringBuffer();

        builder.parseParameter(SUBJ_BASIC, property, buffer);


        property = profile.getPredicateProperty(PARAM_TITLE);

        builder.parseParameter(TITLE_BASIC, property, buffer);

        String expected = PARAM_SUBJ +
                          "=" +
                          SUBJ_BASIC +
                          "&amp;" +
                          PARAM_TITLE +
                          "=" +
                          TITLE_BASIC;

        String queryUri = buffer.toString();

        assertTrue("The first parameter should not be preceded by an " +
                   "ampersand, the second parameter should: " +
                   expected + ", actual URI was: " +
                   queryUri + ".",
                   queryUri.indexOf(expected) > -1);
    }

    /**
     *  Check the interpretation of a subject query.
     *  @since  Version 0.2 2005-01-24
     */
    public final void testParseParameterSubjectQuoted() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        SchemaProperty property = profile.getPredicateProperty(PARAM_SUBJ);

        StringBuffer buffer = new StringBuffer();

        builder.parseParameter(SUBJ_QUOTED, property, buffer);

        Query query = builder.getQuery();

        String serqlQuery = query.getSeRQLQuery();

        // The order of variables is not significant, nor controlled
        assertTrue("Query should include a select statement, " +
                   "target URI and subject variables:\n\n" +
                   SELECT + ", " +
                   TARGET_VAR + ", " +
                   SUBJECT_VAR + "\n\n" +
                   "Actual query was:\n\n" +
                   serqlQuery,
               ((serqlQuery.indexOf(SELECT) > -1) &&
                ((serqlQuery.indexOf(TARGET_VAR + ", " + SUBJECT_VAR) > -1) ||
                 (serqlQuery.indexOf(SUBJECT_VAR + ", " + TARGET_VAR) > -1))));

        assertTrue("Query should include the subject path expression:\n\n" +
                   FROM + SUBJECT_EXPR +
                   "\n\nActual query was:\n\n" +
                   serqlQuery,
                   serqlQuery.indexOf(FROM + SUBJECT_EXPR) > -1);

        assertTrue("Query should include the 'WHERE' condition:\n\n" +
                   WHERE_QUOTED +
                   "\n\nActual query was:\n\n" +
                   serqlQuery,
                   serqlQuery.indexOf(WHERE_QUOTED) > -1);

        assertTrue("Query should include the DC namespace declaration:\n\n" +
                   Q_NS_DC +
                   "\n\nActual query was:\n\n" +
                   serqlQuery,
                   serqlQuery.indexOf(Q_NS_DC) > -1);
    }

    /**
     *  Check the interpretation of a subject query.
     *  @since  Version 0.7 2005-09-13
     */
    public final void testGetQueryExtendedDate() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq, true);

        SchemaProperty property = profile.getPredicateProperty(PARAM_SUBJ);

        StringBuffer buffer = new StringBuffer();

        builder.parseParameter(SUBJ_QUOTED, property, buffer);

        property = profile.getPredicateProperty(PARAM_DATE);

        builder.parseParameter(DATE, property, buffer);

        Query query = builder.getQuery();

        String serqlQuery = query.getSeRQLQuery();

        // The order of variables is not significant, nor controlled
        assertTrue("Query should include a select statement, " +
                   "target URI and subject variables:\n\n" +
                   SELECT + ", " +
                   TARGET_VAR + ", " +
                   SUBJECT_VAR + "\n\n" +
                   "Actual query was:\n\n" +
                   serqlQuery,
                   ((serqlQuery.indexOf(SELECT) > -1) &&
                    (serqlQuery.indexOf(TARGET_VAR) > -1) &&
                    (serqlQuery.indexOf(SUBJECT_VAR) > -1)));

        assertTrue("Query should include the date path expression:\n\n" +
                   X_EXPR_DATE +
                   "\n\nActual query was:\n\n" +
                   serqlQuery,
                   serqlQuery.indexOf(X_EXPR_DATE) > -1);

        assertTrue("Query should include the 'WHERE' condition:\n\n" +
                   X_DATE +
                   "\n\nActual query was:\n\n" +
                   serqlQuery,
                   serqlQuery.indexOf(X_DATE) > -1);

        assertTrue("Query should not include the path expression:\n\n" +
                   X_EXPR_SUBJ +
                   "\n\nActual query was:\n\n" +
                   serqlQuery,
                   serqlQuery.indexOf(X_EXPR_SUBJ) == -1);

        assertTrue("Query should not have a 'spare' comma after the second " +
                   "select.",
                   serqlQuery.indexOf("FROM , ") == -1);
    }

    /**
     *  Check the interpretation of a subject query.
     *  @since  Version 0.7 2005-09-13
     */
    public final void testGetQueryExtendedSubject() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq, true);

        SchemaProperty property = profile.getPredicateProperty(PARAM_SUBJ);

        StringBuffer buffer = new StringBuffer();

        builder.parseParameter(SUBJ_QUOTED, property, buffer);

        String serqlQuery = builder.getSeRQLQuery(true);

        // The order of variables is not significant, nor controlled
        assertTrue("Query should include a select statement, " +
                   "target URI and subject variables:\n\n" +
                   SELECT + ", " +
                   TARGET_VAR + ", " +
                   SUBJECT_VAR + "\n\n" +
                   "Actual query was:\n\n" +
                   serqlQuery,
                   ((serqlQuery.indexOf(SELECT) > -1) &&
                    (serqlQuery.indexOf(TARGET_VAR) > -1) &&
                    (serqlQuery.indexOf(SUBJECT_VAR) > -1)));

        assertTrue("Query not include the subject path expression:\n\n" +
                   X_EXPR_SUBJ +
                   "\n\nActual query was:\n\n" +
                   serqlQuery,
                   serqlQuery.indexOf(X_EXPR_SUBJ) == -1);
    }













    /**
     *  Check the interpretation of an identifier query.
     *  @since  Version 0.8 2005-09-15
     */
    public final void testGetQueryExtendedIdentifierUri() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq, true);

        SchemaProperty property = profile.getPredicateProperty(
                                                             PARAM_ID);

        StringBuffer buffer = new StringBuffer();

        builder.parseParameter(URI_ID, property, buffer);

        Query query = builder.getQuery();

        String serqlQuery = query.getSeRQLQuery();

        // The order of variables is not significant, nor controlled
        assertTrue("Query should include a select statement, " +
                   "target URI and subject variables:\n\n" +
                   SELECT + ", " +
                   TARGET_VAR + ", " +
                   ID_VAR + "\n\n" +
                   "Actual query was:\n\n" +
                   serqlQuery,
                   ((serqlQuery.indexOf(SELECT) > -1) &&
                    (serqlQuery.indexOf(TARGET_VAR) > -1) &&
                    (serqlQuery.indexOf(ID_VAR) > -1)));

        assertTrue("Query should include the 'WHERE' condition:\n\n" +
                   X_URI +
                   "\n\nActual query was:\n\n" +
                   serqlQuery,
                   serqlQuery.indexOf(X_URI) > -1);

        assertTrue("Query should include the extended path expression:\n\n" +
                   X_EXPR_ID +
                   "\n\nActual query was:\n\n" +
                   serqlQuery,
                   serqlQuery.indexOf(X_EXPR_ID) > -1);

        assertTrue("Query should include the basic path expression:\n\n" +
                   EXPR_ID +
                   "\n\nActual query was:\n\n" +
                   serqlQuery,
                   serqlQuery.indexOf(EXPR_ID) > -1);
    }

    /**
     *  Check the interpretation of an identifier query.
     *  @since  Version 0.8 2005-09-15
     */
    public final void testGetQueryExtendedIdentifierNonUri() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq, true);

        SchemaProperty property = profile.getPredicateProperty(
                                                             PARAM_ID);

        StringBuffer buffer = new StringBuffer();

        builder.parseParameter(NON_URI_ID, property, buffer);

        Query query = builder.getQuery();

        String serqlQuery = query.getSeRQLQuery();

        // The order of variables is not significant, nor controlled
        assertTrue("Query should include a select statement, " +
                   "target URI and subject variables:\n\n" +
                   SELECT + ", " +
                   TARGET_VAR + ", " +
                   ID_VAR + "\n\n" +
                   "Actual query was:\n\n" +
                   serqlQuery,
                   ((serqlQuery.indexOf(SELECT) > -1) &&
                    (serqlQuery.indexOf(TARGET_VAR) > -1) &&
                    (serqlQuery.indexOf(ID_VAR) > -1)));

        assertTrue("Query should include the 'WHERE' condition:\n\n" +
                   X_NON_URI +
                   "\n\nActual query was:\n\n" +
                   serqlQuery,
                   serqlQuery.indexOf(X_NON_URI) > -1);

        assertTrue("Query should include the extended path expression:\n\n" +
                   X_EXPR_ID +
                   "\n\nActual query was:\n\n" +
                   serqlQuery,
                   serqlQuery.indexOf(X_EXPR_ID) > -1);

        assertTrue("Query should include the basic path expression:\n\n" +
                   EXPR_ID +
                   "\n\nActual query was:\n\n" +
                   serqlQuery,
                   serqlQuery.indexOf(EXPR_ID) > -1);
    }

    /**
     *  Check the interpretation of an identifier query.
     *  @since  Version 0.8 2005-09-15
     */
    public final void testGetQueryIdentifierUri() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq, false);

        SchemaProperty property = profile.getPredicateProperty(
                                                             PARAM_ID);

        StringBuffer buffer = new StringBuffer();

        builder.parseParameter(URI_ID, property, buffer);

        Query query = builder.getQuery();

        String serqlQuery = query.getSeRQLQuery();

        // The order of variables is not significant, nor controlled
        assertTrue("Query should include a select statement, " +
                   "target URI and subject variables:\n\n" +
                   SELECT + ", " +
                   TARGET_VAR + ", " +
                   ID_VAR + "\n\n" +
                   "Actual query was:\n\n" +
                   serqlQuery,
                   ((serqlQuery.indexOf(SELECT) > -1) &&
                    (serqlQuery.indexOf(TARGET_VAR) > -1) &&
                    (serqlQuery.indexOf(ID_VAR) > -1)));

        assertTrue("Query should include the 'WHERE' condition:\n\n" +
                   X_URI +
                   "\n\nActual query was:\n\n" +
                   serqlQuery,
                   serqlQuery.indexOf(X_URI) > -1);

        assertTrue("Query should not include the extended path expression:" +
                   "\n\n" + X_EXPR_ID +
                   "\n\nActual query was:\n\n" +
                   serqlQuery,
                   serqlQuery.indexOf(X_EXPR_ID) == -1);

        assertTrue("Query should include the basic path expression:\n\n" +
                   EXPR_ID +
                   "\n\nActual query was:\n\n" +
                   serqlQuery,
                   serqlQuery.indexOf(EXPR_ID) > -1);
    }

    /**
     *  Check the interpretation of an identifier query.
     *  @since  Version 0.8 2005-09-15
     */
    public final void testGetQueryIdentifierNonUri() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq, false);

        SchemaProperty property = profile.getPredicateProperty(
                                                             PARAM_ID);

        StringBuffer buffer = new StringBuffer();

        builder.parseParameter(NON_URI_ID, property, buffer);

        Query query = builder.getQuery();

        String serqlQuery = query.getSeRQLQuery();

        // The order of variables is not significant, nor controlled
        assertTrue("Query should include a select statement, " +
                   "target URI and subject variables:\n\n" +
                   SELECT + ", " +
                   TARGET_VAR + ", " +
                   ID_VAR + "\n\n" +
                   "Actual query was:\n\n" +
                   serqlQuery,
                   ((serqlQuery.indexOf(SELECT) > -1) &&
                    (serqlQuery.indexOf(TARGET_VAR) > -1) &&
                    (serqlQuery.indexOf(ID_VAR) > -1)));

        assertTrue("Query should include the 'WHERE' condition:\n\n" +
                   X_NON_URI +
                   "\n\nActual query was:\n\n" +
                   serqlQuery,
                   serqlQuery.indexOf(X_NON_URI) > -1);

        assertTrue("Query should not include the extended path expression:" +
                   "\n\n" + X_EXPR_ID +
                   "\n\nActual query was:\n\n" +
                   serqlQuery,
                   serqlQuery.indexOf(X_EXPR_ID) == -1);

        assertTrue("Query should include the basic path expression:\n\n" +
                   EXPR_ID +
                   "\n\nActual query was:\n\n" +
                   serqlQuery,
                   serqlQuery.indexOf(EXPR_ID) > -1);
    }

    /**
     *  Check the interpretation of an empty subject query.
     *  @since  Version 0.2 2005-01-24
     */
    public final void testParseParameterEmpty() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        SchemaProperty property = profile.getPredicateProperty(PARAM_SUBJ);

        StringBuffer buffer = new StringBuffer();

        builder.parseParameter(EMPTY, property, buffer);

        try {

            builder.getQuery();

            fail("Should throw an IllegalStateException, " +
                 "no variables or expressions set.");

        }
        catch (IllegalStateException ise) { }

    }

    /**
     *  Check the interpretation of a null subject.
     *  @since  0.3 2005-01-25
     */
    public final void testParseParameterNullField() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        SchemaProperty property = profile.getPredicateProperty(PARAM_SUBJ);

        StringBuffer buffer = new StringBuffer();

        try {

            builder.parseParameter(null, property, buffer);

            fail(EXPECTED_IAE);

        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Check the interpretation of a null query URI buffer.
     *  @since  0.4 2005-07-29
     */
    public final void testParseParameterNullBuffer() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        SchemaProperty property = profile.getPredicateProperty(PARAM_TITLE);

        StringBuffer buffer = null;

        try {

            builder.parseParameter(TITLE_BASIC, property, buffer);

            fail(EXPECTED_IAE);

        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Check the interpretation of a title query.
     *  @since  Version 0.4 2005-01-26
     */
    public final void testParseParameterTitleBasic() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        SchemaProperty property = profile.getPredicateProperty(PARAM_TITLE);

        StringBuffer buffer = new StringBuffer();

        builder.parseParameter(TITLE_BASIC, property, buffer);

        Query query = builder.getQuery();

        String serqlQuery = query.getSeRQLQuery();

        // The order of variables is not significant, nor controlled
        assertTrue("Query should include a select statement, " +
                   "target URI and title variables:\n\n" +
                   SELECT + ", " +
                   TARGET_VAR + ", " +
                   TITLE_VAR + "\n\n" +
                   "Actual query was:\n\n" +
                   serqlQuery,
               ((serqlQuery.indexOf(SELECT) > -1) &&
                ((serqlQuery.indexOf(TARGET_VAR + ", " + TITLE_VAR) > -1) ||
                 (serqlQuery.indexOf(TITLE_VAR + ", " + TARGET_VAR) > -1))));

        assertTrue("Query should include the title path expression:\n\n" +
                   FROM + TITLE_EXPR +
                   "\n\nActual query was:\n\n" +
                   serqlQuery,
                   serqlQuery.indexOf(FROM + TITLE_EXPR) > -1);

        assertTrue("Query should include the DC namespace declaration:\n\n" +
                   Q_NS_DC +
                   "\n\nActual query was:\n\n" +
                   serqlQuery,
                   serqlQuery.indexOf(Q_NS_DC) > -1);
    }

    /**
     *  Check the interpretation of multiple title terms.
     *  @since  Version 0.4 2005-01-26
     */
    public final void testParseParameterTitleMultiple() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        SchemaProperty property = profile.getPredicateProperty(PARAM_TITLE);

        StringBuffer buffer = new StringBuffer();

        builder.parseParameter(TITLE_MULTI, property, buffer);

        Query query = builder.getQuery();

        String serqlQuery = query.getSeRQLQuery();

        // The order of variables is not significant, nor controlled
        assertTrue("Query should include a select statement, " +
                   "target URI and title variables:\n\n" +
                   SELECT + ", " +
                   TARGET_VAR + ", " +
                   TITLE_VAR + "\n\n" +
                   "Actual query was:\n\n" +
                   serqlQuery,
               ((serqlQuery.indexOf(SELECT) > -1) &&
                ((serqlQuery.indexOf(TARGET_VAR + ", " + TITLE_VAR) > -1) ||
                 (serqlQuery.indexOf(TITLE_VAR + ", " + TARGET_VAR) > -1))));

        assertTrue("Query should include the title path expression:\n\n" +
                   FROM + TITLE_EXPR +
                   "\n\nActual query was:\n\n" +
                   serqlQuery,
                   serqlQuery.indexOf(FROM + TITLE_EXPR) > -1);

        assertTrue("Query should include the DC namespace declaration:\n\n" +
                   Q_NS_DC +
                   "\n\nActual query was:\n\n" +
                   serqlQuery,
                   serqlQuery.indexOf(Q_NS_DC) > -1);
    }


    /**
     *  Check the interpretation of a description query.
     *  @since  Version 0.4 2005-01-26
     */
    public final void testParseParameterDescriptionBasic() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        SchemaProperty property = profile.getPredicateProperty(PARAM_DESC);

        StringBuffer buffer = new StringBuffer();

        builder.parseParameter(DESC_BASIC, property, buffer);

        Query query = builder.getQuery();

        String serqlQuery = query.getSeRQLQuery();

        // The order of variables is not significant, nor controlled
        assertTrue("Query should include a select statement, " +
                   "target URI and title variables:\n\n" +
                   SELECT + ", " +
                   TARGET_VAR + ", " +
                   DESC_VAR + "\n\n" +
                   "Actual query was:\n\n" +
                   serqlQuery,
               ((serqlQuery.indexOf(SELECT) > -1) &&
                ((serqlQuery.indexOf(TARGET_VAR + ", " + DESC_VAR) > -1) ||
                 (serqlQuery.indexOf(DESC_VAR + ", " + TARGET_VAR) > -1))));

        assertTrue("Query should include the title path expression:\n\n" +
                   FROM + DESC_EXPR +
                   "\n\nActual query was:\n\n" +
                   serqlQuery,
                   serqlQuery.indexOf(FROM + DESC_EXPR) > -1);

        assertTrue("Query should include the DC namespace declaration:\n\n" +
                   Q_NS_DC +
                   "\n\nActual query was:\n\n" +
                   serqlQuery,
                   serqlQuery.indexOf(Q_NS_DC) > -1);
    }

    /**
     *  Test non-delimiter strings.
     *  @since  Version 0.1 2005-07-06
     */
    public final void testIsDelimiterFalse() {

        ServletQueryBuilder builder = new ServletQueryBuilder(msreq);

        assertFalse(builder.isDelimiter("a"));
        assertFalse(builder.isDelimiter("to"));
    }

    /**
     *  Test the constructor.
     *  @since  Version 0.1 2005-07-06
     */
    public final void testConstructorNullRequest() {

        try {

            new ServletQueryBuilder(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test the constructor, with bNode extended argument.
     *  @since  Version 0.1 2005-07-06
     */
    public final void testConstructorExtendedNullRequest() {

        try {

            new ServletQueryBuilder(null, true);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }


    /**
     *  Tear down a test scenario as required.
     *  @since  Version 0.1 2005-07-06
    protected void tearDown() {

    }
     */

    /**
     *  Create a test suite for this test case.
     *  @return A test suite.
     *  @since  Version 0.1 2005-07-06
     */
    public static Test suite() {

        return new TestSuite(ServletQueryBuilderTest.class);
    }

    /**
     *  Run this test suite as a stand alone application.
     *  @param  args    Any command line arguments to pass
     *          to the test. None required or expected.
     *  @since  Version 0.1 2005-07-06
     */
    public static void main(final String[] args) {

        junit.textui.TestRunner.run(suite());
    }

}
