package com.mkdoc.sax;

/*
    Created:    2004-12-15
    Issued:     2004-12-15
    Modified:   2005-09-09

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2004 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import org.xml.sax.Locator;
import org.xml.sax.SAXException;

import org.xml.sax.helpers.AttributesImpl;
import org.xml.sax.helpers.LocatorImpl;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  Test suite for <code></code>.
 *  @version    0.2 2005-09-09
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class AbstractContentHandlerTest extends TestCase {

    /**
     *  An out of range attribute index.
     *  @since  Version 0.2 2005-09-09
     */
    private static final int OUT_OF_RANGE = 4032;

    /**
     *  A system ID to mark a locator.
     *  @since  Version 0.1 2004-12-15
     */
    private static final String SYSTEM_ID =
    "http://www.mkdoc.com/uri/AbstractcontHandler";

    /**
     *  The XHTML namespace URI.
     *  @since  Version 0.1 2004-12-15
     */
    private static final String NAMESPACE = "http://www.w3.org/1999/xhtml";

    /**
     *  An XHTML <code>meta</code> element local name.
     *  @since  Version 0.1 2004-12-15
     */
    private static final String META = "meta";

    /**
     *  An XHTML <code>name</code> attribute local name.
     *  @since  Version 0.1 2004-12-15
     */
    private static final String NAME = "name";

    /**
     *  An XHTML <code>name</code> attribute value.
     *  @since  Version 0.1 2004-12-15
     */
    private static final String NAME_VALUE = "DC.Title";

    /**
     *  A generic XHTML <code>description</code> content
     *  value.
     *  @since  Version 0.2 2005-09-09
     */
    private static final String VALUE_DESC =
    "Generic description content";

    /**
     *  A XHTML <code>description</code> with excess white space.
     *  @since  Version 0.2 2005-09-09
     */
    private static final String VALUE_PADDED =
    " Generic description content ";

    /**
     *  A qualified XHTML <code>meta</code> element name.
     *  @since  Version 0.1 2004-12-15
     */
    private static final String QUAL_META = NAMESPACE +
                                            ":" +
                                            META;

    /**
     *  A qualified XHTML <code>name</code> attribute.
     *  @since  Version 0.1 2004-12-15
     */
    private static final String QUAL_NAME = NAMESPACE +
                                            ":" +
                                            NAME;

    /**
     *  A CDATA type string.
     *  @since  Version 0.1 2004-12-15
     */
    private static final String CDATA = "CDATA";

    /**
     *  A namespace prefix.
     *  @since  Version 0.1 2004-12-15
     */
    private static final String PREFIX = "html";

    /**
     *  Element content.
     *  @since  Version 0.1 2004-12-16
     */
    private static final String CHARACTERS = "Element content string";

    /**
     *  Ignorable white space.
     *  @since  Version 0.1 2004-12-16
     */
    private static final String WHITESPACE = "\t \n";

    /**
     *  A mock content handler for general testing.
     *  @since  Version 0.1 2004-12-15
     */
    private MockContentHandler contHandler;

    /**
     *  An attribute set for general testing.
     *  @since  Version 0.1 2004-12-15
     */
    private AttributesImpl attributes;

    /**
     *  Named test instance.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2004-
     */
    public AbstractContentHandlerTest(final String name) {

        super(name);
    }

    /**
     *  Set up a test scenario as required.
     *  @since  Version 0.1 2004-12-15
     */
    protected final void setUp() {

        contHandler = new MockContentHandler();

        attributes = new AttributesImpl();
    }

    /**
     *  Get an attribute value that is out of range.
     *  @since  Version 0.2 2005-09-09
     */
    public final void testGetAttributeValueOutOfRange() {

        attributes.addAttribute(NAMESPACE,
                                NAME,
                                QUAL_NAME,
                                CDATA,
                                VALUE_DESC);

        String value = contHandler.getAttributeValue(attributes, OUT_OF_RANGE);

        assertNull(value);

    }

    /**
     *  Get an attribute value with excess whitespace.
     *  @since  Version 0.2 2005-09-09
     */
    public final void testGetAttributeValueTrim() {

        attributes.addAttribute(NAMESPACE,
                                NAME,
                                QUAL_NAME,
                                CDATA,
                                VALUE_PADDED);

        int index = attributes.getIndex(NAMESPACE, NAME);

        String value = contHandler.getAttributeValue(attributes, index);

        assertTrue(value.equals(VALUE_DESC));
    }

    /**
     *  Assign a <code>Locator</code>.
     *  @since  Version 0.1 2004-12-15
     */
    public final void testSetDocumentLocator() {

        LocatorImpl locator = new LocatorImpl();

        locator.setSystemId(SYSTEM_ID);

        contHandler.setDocumentLocator(locator);

        Locator testLocator = contHandler.locator;

        assertTrue(SYSTEM_ID.equals(testLocator.getSystemId()));

    }

    /**
     *  Assign a null <code>Locator</code>, should throw an
     *  <code>IllegalArgumentException</code>.
     *  @since  Version 0.1 2004-12-16
     */
    public final void testSetDocumentLocatorNull() {

        try {

            contHandler.setDocumentLocator(null);

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Start element with no attributes.
     *  @exception  SAXException    Should not be thrown in
     *          this test, indicates an error.
     *  @since  Version 0.1 2004-12-15
     */
    public final void testStartElementNoAttributes() throws SAXException {

        contHandler.startElement(NAMESPACE,
                                    META,
                                    QUAL_META,
                                    attributes);

        String eventLog = contHandler.getEventLog();

        assertTrue(eventLog.indexOf(META) > -1);
    }

    /**
     *  Start element with no attributes.
     *  @exception  SAXException    Should not be thrown in
     *          this test, indicates an error.
     *  @since  Version 0.1 2004-12-15
     */
    public final void testStartElementWithAttributes() throws SAXException {

        attributes.addAttribute(NAMESPACE,
                                NAME,
                                QUAL_NAME,
                                CDATA,
                                NAME_VALUE);

        contHandler.startElement(NAMESPACE,
                                    META,
                                    QUAL_META,
                                    attributes);

        String eventLog = contHandler.getEventLog();

        assertTrue(eventLog.indexOf(META) > -1);

        assertTrue("Should include an attribute name/value pair, " +
                   "actual event log:\n" + eventLog,
                   eventLog.indexOf(NAME) > -1);

        assertTrue("Should include an attribute name/value pair, " +
                   "actual event log:\n" + eventLog,
                   eventLog.indexOf(NAME_VALUE) > -1);
    }

    /**
     *  Pure coverage test, not implemented in superclass.
     *  @exception  SAXException    Should not be thrown in
     *          this test, indicates an error.
     *  @since  Version 0.1 2004-12-16
     */
    public final void testSuperStartElement() throws SAXException {

        contHandler.superStartElement(NAMESPACE,
                                         META,
                                         QUAL_META,
                                         attributes);

        // No assertion
    }

    /**
     *  Pure coverage test, not implemented in superclass.
     *  @exception  SAXException    Should not be thrown in
     *          this test, indicates an error.
     *  @since  Version 0.1 2004-12-16
     */
    public final void testSuperEndElement() throws SAXException {

        contHandler.superEndElement(NAMESPACE,
                                       META,
                                       QUAL_META);

        // No assertion
    }

    /**
     *  Pure coverage test, not implemented.
     *  @exception  SAXException    Should not be thrown in
     *          this test, indicates an error.
     *  @since  Version 0.1 2004-12-16
     */
    public final void testEndElement() throws SAXException {

        contHandler.endElement(NAMESPACE,
                                  META,
                                  QUAL_META);

        // No assertion
    }

    /**
     *  Pure coverage test, not implemented.
     *  @exception  SAXException    Should not be thrown in
     *          this test, indicates an error.
     *  @since  Version 0.1 2004-12-15
     */
    public final void testStartDocument() throws SAXException {

        contHandler.startDocument();

        // No assertion
    }

    /**
     *  Pure coverage test, not implemented.
     *  @exception  SAXException    Should not be thrown in
     *          this test, indicates an error.
     *  @since  Version 0.1 2004-12-15
     */
    public final void testEndDocument() throws SAXException {

        contHandler.endDocument();

        // No assertion
    }

    /**
     *  Pure coverage test, not implemented.
     *  @exception  SAXException    Should not be thrown in
     *          this test, indicates an error.
     *  @since  Version 0.1 2004-12-15
     */
    public final void testStartPrefixMapping() throws SAXException {

        contHandler.startPrefixMapping(PREFIX,
                                          NAMESPACE);

        // No assertion
    }

    /**
     *  Pure coverage test, not implemented.
     *  @exception  SAXException    Should not be thrown in
     *          this test, indicates an error.
     *  @since  Version 0.1 2004-12-16
     */
    public final void testEndPrefixMapping() throws SAXException {

        contHandler.endPrefixMapping(PREFIX);

        // No assertion
    }

    /**
     *  Pure coverage test, not implemented.
     *  @exception  SAXException    Should not be thrown in
     *          this test, indicates an error.
     *  @since  Version 0.1 2004-12-16
     */
    public final void testCharacters() throws SAXException {

        contHandler.characters(CHARACTERS.toCharArray(),
                                  0,
                                  CHARACTERS.length());

        // No assertion
    }

    /**
     *  Pure coverage test, not implemented.
     *  @exception  SAXException    Should not be thrown in
     *          this test, indicates an error.
     *  @since  Version 0.1 2004-12-16
     */
    public final void testIgnorableWhitespace() throws SAXException {

        contHandler.ignorableWhitespace(WHITESPACE.toCharArray(),
                                           0,
                                           WHITESPACE.length());

        // No assertion
    }

    /**
     *  Pure coverage test, not implemented.
     *  @exception  SAXException    Should not be thrown in
     *          this test, indicates an error.
     *  @since  Version 0.1 2004-12-16
     */
    public final void testProcessingInstruction() throws SAXException {

        contHandler.processingInstruction("TestApplication",
                                             "TestData");

        // No assertion
    }

    /**
     *  Pure coverage test, not implemented.
     *  @exception  SAXException    Should not be thrown in
     *          this test, indicates an error.
     *  @since  Version 0.1 2004-12-16
     */
    public final void testSkippedEntity() throws SAXException {

        contHandler.skippedEntity("[dtd]");

        // No assertion
    }

    /**
     *  Tear down a test scenario as required.
     *  @since  Version 0.1 2004-
    protected void tearDown() {

    }
     */

    /**
     *  Create a test suite for this test case.
     *  @return A test suite.
     *  @since  Version 0.1 2004-12-15
     */
    public static Test suite() {

        return new TestSuite(AbstractContentHandlerTest.class);
    }

    /**
     *  Run this test suite as a stand alone application.
     *  @param  args    Any command line arguments to pass
     *          to the test. None required or expected.
     *  @since  Version 0.1 2004-15
     */
    public static void main(final String[] args) {

        junit.textui.TestRunner.run(suite());
    }

}
