package com.mkdoc.sax;

/*
    Created:    2004-10-25
    Issued:     2004-10-25
    Modified:   2005-01-07

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2004 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import java.io.IOException;

import org.xml.sax.ContentHandler;
import org.xml.sax.DTDHandler;
import org.xml.sax.EntityResolver;
import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXNotRecognizedException;
import org.xml.sax.SAXNotSupportedException;

import org.xml.sax.helpers.DefaultHandler;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  Test suite for <code>AbstractXMLReader</code>.
 *  @version    0.2 2004-10-26
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class AbstractXMLReaderTest extends TestCase {

    /**
     *  The XML namespaces feature identifier.
     *  @since  Version 0.1 2004-10-25
     */
    private static final String NAMESPACES =
    "http://xml.org/sax/features/namespaces";

    /**
     *  The XML namespace prefixes feature identifier.
     *  @since  Version 0.1 2004-10-25
     */
    private static final String PREFIXES =
    "http://xml.org/sax/features/namespace-prefixes";

    /**
     *  An unrecognized feature identifier.
     *  @since  Version 0.1 2004-10-25
     */
    private static final String NOT_RECOGNIZED =
    "http://www.remote.example/sax/not-recognized";

    /**
     *  Named test instance.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2004-10-25
     */
    public AbstractXMLReaderTest(final String name) {

        super(name);
    }

    /**
     *  Set up a test scenario as required.
     *  @since  Version 0.1 2004-10-25
    protected void setUp(){

    }
     */

    /**
     *  Check the XML namespaces feature is recognized.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @exception  IOException Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2004-10-25
     */
    public final void testGetFeatureNamespaces()
        throws SAXException, IOException {

        MockXMLReader xmlReader = new MockXMLReader();

        boolean setting = xmlReader.getFeature(NAMESPACES);

        assertFalse(setting);
    }

    /**
     *  Get feature, null argument.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @exception  IOException Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2004-10-25
     */
    public final void testGetFeatureNameNull()
        throws SAXException, IOException {

        MockXMLReader xmlReader = new MockXMLReader();

        try {

            boolean setting = xmlReader.getFeature(null);

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Set feature, null name.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @exception  IOException Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2004-10-25
     */
    public final void testSetFeatureNameNull()
        throws SAXException, IOException {

        MockXMLReader xmlReader = new MockXMLReader();

        try {

            xmlReader.setFeature(null, true);

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Check the XML namespaces feature is recognized.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @exception  IOException Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2004-10-25
     */
    public final void testSetFeatureNamespaces()
        throws SAXException, IOException {

        MockXMLReader xmlReader = new MockXMLReader();

        xmlReader.setFeature(NAMESPACES, true);

        boolean setting = xmlReader.getFeature(NAMESPACES);

        assertTrue(setting);
    }

    /**
     *  Check the XML namespace prefixes feature is recognized.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @exception  IOException Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2004-10-25
     */
    public final void testGetFeaturePrefixes()
        throws SAXException, IOException {

        MockXMLReader xmlReader = new MockXMLReader();

        boolean setting = xmlReader.getFeature(PREFIXES);

        assertFalse(setting);
    }

    /**
     *  Check the XML namespaces feature is recognized.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @exception  IOException Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2004-10-25
     */
    public final void testSetFeaturePrefixes()
        throws SAXException, IOException {

        MockXMLReader xmlReader = new MockXMLReader();

        xmlReader.setFeature(PREFIXES, true);

        boolean setting = xmlReader.getFeature(PREFIXES);

        assertTrue(setting);
    }

    /**
     *  Check a pre-defined feature is not supported.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @exception  IOException Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2004-10-25
     */
    public final void testGetFeatureNotSupported1()
        throws SAXException, IOException {

        MockXMLReader xmlReader = new MockXMLReader();

        try {

            boolean setting = xmlReader.getFeature(MockXMLReader.TEST1);

            fail("Should throw a SAXNotSupportedException.");
        }
        catch (SAXNotSupportedException snre) { }

    }

    /**
     *  Check a pre-defined feature is not supported.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @exception  IOException Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2004-10-25
     */
    public final void testGetFeatureNotSupported2()
        throws SAXException, IOException {

        MockXMLReader xmlReader = new MockXMLReader();

        try {

            boolean setting = xmlReader.getFeature(MockXMLReader.TEST2);

            fail("Should throw a SAXNotSupportedException.");
        }
        catch (SAXNotSupportedException snre) { }

    }

    /**
     *  Check a pre-defined feature is not supported.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @exception  IOException Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2004-10-25
     */
    public final void testSetFeatureNotSupported1()
        throws SAXException, IOException {

        MockXMLReader xmlReader = new MockXMLReader();

        try {

            xmlReader.setFeature(MockXMLReader.TEST1, false);

            fail("Should throw a SAXNotSupportedException.");
        }
        catch (SAXNotSupportedException snre) { }

    }

    /**
     *  Check a pre-defined feature is not supported.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @exception  IOException Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2004-10-25
     */
    public final void testSetFeatureNotSupported2()
        throws SAXException, IOException {

        MockXMLReader xmlReader = new MockXMLReader();

        try {

            xmlReader.setFeature(MockXMLReader.TEST2, true);

            fail("Should throw a SAXNotSupportedException.");
        }
        catch (SAXNotSupportedException snre) { }

    }

    /**
     *  Check an unknown feature is not recognized.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @exception  IOException Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2004-10-25
     */
    public final void testGetFeatureNotRecognized()
        throws SAXException, IOException {

        MockXMLReader xmlReader = new MockXMLReader();

        try {

            boolean setting = xmlReader.getFeature(NOT_RECOGNIZED);

            fail("Should throw a SAXNotRecognizedException.");
        }
        catch (SAXNotRecognizedException snre) { }

    }

    /**
     *  Check an unknown feature is not recognized.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @exception  IOException Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2004-10-25
     */
    public final void testSetFeatureNotRecognized()
        throws SAXException, IOException {

        MockXMLReader xmlReader = new MockXMLReader();

        try {

            xmlReader.setFeature(NOT_RECOGNIZED, true);

            fail("Should throw a SAXNotRecognizedException.");
        }
        catch (SAXNotRecognizedException snre) { }

    }

    /**
     *  Get property, null argument.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @exception  IOException Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2004-10-25
     */
    public final void testGetPropertyNameNull()
        throws SAXException, IOException {

        MockXMLReader xmlReader = new MockXMLReader();

        try {

            Object value = xmlReader.getProperty(null);

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Set property, null name.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @exception  IOException Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2004-10-25
     */
    public final void testSetPropertyNameNull()
        throws SAXException, IOException {

        MockXMLReader xmlReader = new MockXMLReader();

        try {

            String value = new String("Test value");

            xmlReader.setProperty(null, value);

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Set property, null value.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @exception  IOException Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2004-10-25
     */
    public final void testSetPropertyValueNull()
        throws SAXException, IOException {

        MockXMLReader xmlReader = new MockXMLReader();

        try {

            xmlReader.setProperty(MockXMLReader.TEST4, null);

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Check unknown properties are not recognized.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @exception  IOException Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2004-10-25
     */
    public final void testGetPropertyUnrecognized()
        throws SAXException, IOException {

        MockXMLReader xmlReader = new MockXMLReader();

        try {

            Object value = xmlReader.getProperty(NOT_RECOGNIZED);

            fail("Should throw a SAXNotRecognizedException.");

        }
        catch (SAXNotRecognizedException snre) { }

    }

    /**
     *  Check unknown properties are not recognized.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @exception  IOException Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2004-10-25
     */
    public final void testSetPropertyUnrecognized()
        throws SAXException, IOException {

        MockXMLReader xmlReader = new MockXMLReader();

        try {

            String value = new String("Property not recognized.");

            xmlReader.setProperty(NOT_RECOGNIZED, value);


            fail("Should throw a SAXNotRecognizedException.");
        }
        catch (SAXNotRecognizedException snre) { }

    }

    /**
     *  Check un-supported properties throw an exception.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @exception  IOException Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2004-10-25
     */
    public final void testGetPropertyUnsupported()
        throws SAXException, IOException {

        MockXMLReader xmlReader = new MockXMLReader();

        try {

            Object value = xmlReader.getProperty(MockXMLReader.TEST3);

            fail("Should throw a SAXNotSupportedException.");

        }
        catch (SAXNotSupportedException snre) { }

    }

    /**
     *  Check un-supported properties throw an exception.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @exception  IOException Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2004-10-25
     */
    public final void testSetPropertyUnsupported()
        throws SAXException, IOException {

        MockXMLReader xmlReader = new MockXMLReader();

        try {

            String value = new String("Property not supported.");

            xmlReader.setProperty(MockXMLReader.TEST3, value);


            fail("Should throw a SAXNotSupportedException.");
        }
        catch (SAXNotSupportedException snre) { }

    }

    /**
     *  Check un-supported properties throw an exception.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @exception  IOException Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2004-10-25
     */
    public final void testGetPropertySupported()
        throws SAXException, IOException {

        MockXMLReader xmlReader = new MockXMLReader();

        String value = (String) xmlReader.getProperty(MockXMLReader.TEST4);

        assertTrue(value.equals("Property 2 value"));
    }

    /**
     *  Check un-supported properties throw an exception.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @exception  IOException Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2004-10-25
     */
    public final void testSetPropertySupported()
        throws SAXException, IOException {

        MockXMLReader xmlReader = new MockXMLReader();

        String value = new String("Property not supported.");

        xmlReader.setProperty(MockXMLReader.TEST4, value);

        String value2 = (String) xmlReader.getProperty(MockXMLReader.TEST4);

        assertTrue(value.equals(value2));

    }

    /**
     *  Set entity resolver, null argument.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2004-10-25
     */
    public final void testSetEntityResolverNull() throws SAXException {

        MockXMLReader xmlReader = new MockXMLReader();

        try {

            xmlReader.setEntityResolver(null);

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Set DTD handler, null argument.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2004-10-25
     */
    public final void testSetDTDHandlerNull() throws SAXException {

        MockXMLReader xmlReader = new MockXMLReader();

        try {

            xmlReader.setDTDHandler(null);

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Set content handler, null argument.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2004-10-25
     */
    public final void testSetContentHandlerNull() throws SAXException {

        MockXMLReader xmlReader = new MockXMLReader();

        try {

            xmlReader.setContentHandler(null);

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Set error handler, null argument.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2004-10-25
     */
    public final void testSetErrorHandlerNull() throws SAXException {

        MockXMLReader xmlReader = new MockXMLReader();

        try {

            xmlReader.setErrorHandler(null);

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Check set/get entity resolver.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2004-10-25
     */
    public final void testSetEntityResolver() throws SAXException {

        MockXMLReader xmlReader = new MockXMLReader();

        DefaultHandler handler = new DefaultHandler();

        xmlReader.setEntityResolver((EntityResolver) handler);

        EntityResolver resolver = xmlReader.getEntityResolver();

        assertTrue(resolver.equals(handler));

    }

    /**
     *  Check set/get DTD handler.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2004-10-25
     */
    public final void testSetDTDHandler() throws SAXException {

        MockXMLReader xmlReader = new MockXMLReader();

        DefaultHandler handler = new DefaultHandler();

        xmlReader.setDTDHandler((DTDHandler) handler);

        DTDHandler dtdHandler = xmlReader.getDTDHandler();

        assertTrue(dtdHandler.equals(handler));

    }

    /**
     *  Check set/get content handler.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2004-10-25
     */
    public final void testSetContentHandler() throws SAXException {

        MockXMLReader xmlReader = new MockXMLReader();

        DefaultHandler handler = new DefaultHandler();

        xmlReader.setContentHandler((ContentHandler) handler);

        ContentHandler contentHandler = xmlReader.getContentHandler();

        assertTrue(contentHandler.equals(handler));

    }

    /**
     *  Check set/get DTD handler.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2004-10-25
     */
    public final void testSetErrorHandler() throws SAXException {

        MockXMLReader xmlReader = new MockXMLReader();

        DefaultHandler handler = new DefaultHandler();

        xmlReader.setErrorHandler((ErrorHandler) handler);

        ErrorHandler errorHandler = xmlReader.getErrorHandler();

        assertTrue(errorHandler.equals(handler));

    }

    /**
     *  Check parse throws SAXException.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @exception  IOException Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2004-10-25
     */
    public final void testParseInputSourceThrowsSAXException()
        throws SAXException, IOException {

        MockXMLReader xmlReader = new MockXMLReader();

        xmlReader.setException("SAXException");

        InputSource inputSource = new InputSource();

        try {

            xmlReader.parse(inputSource);

            fail("Should throw a SAXException.");
        }
        catch (SAXException se) { }

    }

    /**
     *  Check parse throws IllegalArugmentException.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @exception  IOException Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2004-10-25
     */
    public final void testParseSystemIdNull()
        throws SAXException, IOException {

        MockXMLReader xmlReader = new MockXMLReader();

        String systemId = null;

        try {

            xmlReader.parse(systemId);

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Check parse throws SAXException.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @exception  IOException Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2004-10-25
     */
    public final void testParseSystemIdThrowsSAXException()
        throws SAXException, IOException {

        MockXMLReader xmlReader = new MockXMLReader();

        xmlReader.setException("SAXException");

        String systemId = "//arbitrary/path";

        try {

            xmlReader.parse(systemId);

            fail("Should throw a SAXException.");
        }
        catch (SAXException se) { }

    }


    /**
     *  Check parse throws IOException.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2004-10-25
     */
    public final void testParseInputSourceThrowsIOException()
        throws SAXException {

        MockXMLReader xmlReader = new MockXMLReader();

        xmlReader.setException("IOException");

        InputSource inputSource = new InputSource();

        try {

            xmlReader.parse(inputSource);

            fail("Should throw a IOException.");
        }
        catch (IOException se) { }

    }

    /**
     *  Check parse throws IOException.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2004-10-25
     */
    public final void testParseSystemIdThrowsIOException()
        throws SAXException {

        MockXMLReader xmlReader = new MockXMLReader();

        xmlReader.setException("IOException");

        String systemId = "//arbitrary/path";

        try {

            xmlReader.parse(systemId);

            fail("Should throw a IOException.");
        }
        catch (IOException se) { }

    }

    /**
     *  Check parse completes.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @exception  IOException Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2004-10-25
     */
    public final void testParseInputSourceSuccess()
        throws SAXException, IOException {

        MockXMLReader xmlReader = new MockXMLReader();

        InputSource inputSource = new InputSource();

        xmlReader.parse(inputSource);

        // No assertion, mock class does nothing.
    }

    /**
     *  Check parse completes.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @exception  IOException Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2004-10-25
     */
    public final void testParseSystemIdSuccess()
        throws SAXException, IOException {

        MockXMLReader xmlReader = new MockXMLReader();

        String systemId = "//arbitrary/path";

        xmlReader.parse(systemId);

        // No assertion, mock class does nothing.
    }

    /**
     *  Check null exception setting for MockXMLReader.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @exception  IOException Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2004-10-25
     */
    public final void testSetExceptionNull()
        throws SAXException, IOException {

        MockXMLReader xmlReader = new MockXMLReader();

        try {

            xmlReader.setException(null);

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Check an un-recognized exception setting for MockXMLReader.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @exception  IOException Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2004-10-25
     */
    public final void testSetExceptionNotMatched()
        throws SAXException, IOException {

        MockXMLReader xmlReader = new MockXMLReader();

        xmlReader.setException("NotRecognizedException");

        String systemId = "//arbitrary/path";

        xmlReader.parse(systemId);

        // Should parse without exception
    }

    /**
     *  Tear down a test scenario as required.
     *  @since  Version 0.1 2004-10-25
    protected void tearDown(){

    }
     */

    /**
     *  Create a test suite with a coverage decorator.
     *  @return A test coverage decorator.
     *  @since  Version 0.1 2004-10-25
     */
    public static Test suite() {

        return new TestSuite(AbstractXMLReaderTest.class);
    }

/*
    public static Test suite() {

        return new CoverageDecorator(AbstractXMLReaderTest.class,
                                     new Class[] {AbstractXMLReader.class,
                                                  MockXMLReader.class });
    }
*/
    /**
     *  Run this test suite.
     *  @param  args    Any command line arguments to pass
     *          to the test. None required or expected.
     *  @since  Version 0.1 2004-10-25
     */
    public static void main(final String[] args) {

        junit.textui.TestRunner.run(suite());
    }

}
