package com.mkdoc.sax;

/*
    Created:    2004-12-16
    Issued:     2004-12-16
    Modified:   2005-10-21

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2004 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import com.mkdoc.io.MockOutputStream;

import com.mkdoc.schema.ApplicationProfile;
import com.mkdoc.schema.Schema;
import com.mkdoc.schema.UKeGMS;
import com.mkdoc.schema.UKeGMSProfile;

import com.mkdoc.store.MockVirtualFileStoreManager;

import java.io.IOException;
import java.io.PrintWriter;

import java.net.URL;

import org.xml.sax.SAXException;

import org.xml.sax.helpers.AttributesImpl;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  Test suite for <code>MetaTripleWriter</code>.
 *  @version    1.5 2005-10-21
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class MetaTripleWriterTest extends TestCase {

    /**
     *  An &quot;actual output&quot; statement for test failures.
     *  @since  Version 1.5 2005-10-21
     */
    protected static final String ACTUALLY =
    "Output actually equals:\n";

    /**
     *  A &quot;should not ...&quot; statement for test failures.
     *  @since  Version 1.5 2005-10-21
     */
    protected static final String SHOULD_NOT =
    "Should not give any output, actually: '";

    /**
     *  An expected <code>SAXException</code> message.
     *  @since  Version 0.1 2004-12-16
     */
    private static final String EXPECTED_SE =
    "Should throw a SAXException.";

    /**
     *  An initial test URI.
     *  @since  Version 0.1 2004-12-16
     */
    protected static final String URI_1 =
    "http://www.remote.example/test/page.html";

    /**
     *  A second test URI.
     *  @since  Version 0.1 2004-12-16
     */
    protected static final String URI_2 =
    "http://www.remote.example/second/page.html";

    /**
     *  The XHTML namespace URI.
     *  @since  Version 0.2 2004-12-17
     */
    private static final String NAMESPACE = "http://www.w3.org/1999/xhtml";

    /**
     *  An XHTML <code>head</code> element local name.
     *  @since  Version 0.2 2004-12-17
     */
    private static final String HEAD = "head";

    /**
     *  An XHTML <code>meta</code> element local name.
     *  @since  Version 0.2 2004-12-17
     */
    private static final String META = "meta";

    /**
     *  An XHTML <code>name</code> attribute local name.
     *  @since  Version 0.2 2004-12-17
     */
    private static final String NAME = "name";

    /**
     *  A qualified XHTML <code>name</code> attribute.
     *  @since  Version 0.2 2004-12-17
     */
    private static final String QUAL_NAME = NAMESPACE +
                                            ":" +
                                            NAME;

    /**
     *  An XHTML <code>scheme</code> attribute local name.
     *  @since  Version 0.4 2004-12-23
     */
    private static final String SCHEME = "scheme";

    /**
     *  A qualified XHTML <code>scheme</code> attribute.
     *  @since  Version 0.4 2004-12-23
     */
    private static final String QUAL_SCHEME = NAMESPACE +
                                              ":" +
                                              SCHEME;

    /**
     *  An XHTML <code>content</code> attribute local name.
     *  @since  Version 0.2 2004-12-17
     */
    private static final String CONTENT = "content";

    /**
     *  A qualified XHTML <code>name</code> attribute.
     *  @since  Version 0.2 2004-12-17
     */
    private static final String QUAL_CONTENT = NAMESPACE +
                                               ":" +
                                               CONTENT;

    /**
     *  A generic XHTML <code>description</code> name value.
     *  @since  Version 0.2 2004-12-17
     */
    private static final String NAME_DESC = "description";

    /**
     *  A generic XHTML <code>description</code> content
     *  value.
     *  @since  Version 0.2 2004-12-17
     */
    private static final String VALUE_DESC =
    "Generic description content";

    /**
     *  A DC title predicate.
     *  @since  Version 0.4 2004-12-23
     */
    private static final String DC_DATE = "DC.Date";

    /**
     *  The DC W3C-DTF encoding name.
     *  @since  Version 0.4 2004-12-23
     */
    private static final String W3CDTF = "W3CDTF";

    /**
     *  A W3C-DTF date value.
     *  @since  Version 0.4 2004-12-23
     */
    private static final String DATE_W3CDTF = "2004-12-23";

    /**
     *  A W3C-DTF date value with surplus whitespace.
     *  @since  Version 0.4 2004-12-23
     */
    private static final String DATE_PADDED = " 2004-12-23 ";


    /**
     *  A triple set for the generic description.
     *  @since  Version 0.2 2004-12-17
     */
    private static final String NEW_LINE = System.getProperty("line.separator");

    /**
     *  A triple set for the Dublin Core date.
     *  @since  Version 0.6 2005-01-07
     */
    private static final String TRIPLE_DC = " <" + URI_1 + "> " +
                    "<http://purl.org/dc/elements/1.1/date> " +
                    "\"" +
                    DATE_W3CDTF +
                    "\" ." +
                    NEW_LINE;

    /**
     *  A triple set with bNode for DC Date.
     *  @since  Version 0.4 2004-12-23
     */
    private static final String TRIPLE_BNODE = " <" + URI_1 + "> " +
                           "<http://purl.org/dc/elements/1.1/date> " +
                           "_:mkdoc";

    /**
     *  A triple set for a bNode with RDF type.
     *  @since  Version 0.4 2004-12-23
     */
    private static final String BNODE_TYPE = // Variable bNode omitted
                 "<http://www.w3.org/1999/02/22-rdf-syntax-ns#type> " +
                 "<http://purl.org/dc/terms/W3CDTF> ." +
                 NEW_LINE;

    /**
     *  A triple set for a bNode with RDF type.
     *  @since  Version 0.4 2004-12-23
     */
    private static final String BNODE_VALUE = // Variable bNode omitted
                  "<http://www.w3.org/1999/02/22-rdf-syntax-ns#value> " +
                  "\"" + DATE_W3CDTF + "\" ." +
                  NEW_LINE;

    /**
     *  A qualified XHTML <code>meta</code> element name.
     *  @since  Version 0.2 2004-12-17
     */
    private static final String QUAL_META = NAMESPACE +
                                            ":" +
                                            META;

    /**
     *  A qualified XHTML <code>head</code> element name.
     *  @since  Version 0.2 2004-12-17
     */
    private static final String QUAL_HEAD = NAMESPACE +
                                            ":" +
                                            HEAD;

    /**
     *  A CDATA type string.
     *  @since  Version 0.2 2004-12-17
     */
    private static final String CDATA = "CDATA";

    /**
     *  An expected <code>IllegalArgumentException</code>.
     *  @since  Version 0.9 2005-02-15
     */
    private static final String EXPECT_IAE =
    "Should throw an IllegalArgumentException.";

    /**
     *  A test subject URI.
     *  @since  Version 0.9 2005-02-15
     */
    private static final String URI_SUBJ_1 =
    "http://www.mkdoc.org/uri/test/MetaTripleWriter#one";

    /**
     *  An empty attribute set for testing.
     *  @since  Version Version 0.2 2004-12-17
     */
    private AttributesImpl attributes;

    /**
     *  A general test content handler.
     *  @since  Version 0.1 2004-12-16
     */
    protected MetaTripleWriter tripleWriter;

    /**
     *  An instance with default construction for testing.
     *  @since  Version 0.9 2005-02-15
     */
    protected MetaTripleWriter basicWriter;

    /**
     *  An output stream for the test content handler.
     *  @since  Version 0.1 2004-12-16
     */
    protected MockOutputStream outStream;

    /**
     *  A mock subject manager to check output.
     *  @since  Version 1.1 2005-04-29
     */
    protected MockVirtualFileStoreManager manager;

    /**
     *  Named test instance.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2004-12-16
     */
    public MetaTripleWriterTest(final String name) {

        super(name);
    }

    /**
     *  Set up a test scenario as required.
     *  @exception  IOException Should not be thrown by this test.
     *  @since  Version 0.1 2004-
     */
    protected final void setUp() throws IOException {

        outStream = new MockOutputStream();

        manager = new MockVirtualFileStoreManager(outStream);

        tripleWriter = new MetaTripleWriter(manager,
                                            new URL(URI_1));

        attributes = new AttributesImpl();

        basicWriter = new MetaTripleWriter();
    }

    /**
     *  Test constructor with custom application profile.
     *  @since  Version 1.3 2005-05-25
     */
    public final void testConstructorNullApplicationProfile() {

        ApplicationProfile profile = null;

        try {

            new MetaTripleWriter(profile);

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test constructor with custom application profile.
     *  @since  Version 1.3 2005-05-25
     */
    public final void testConstructorCustomApplicationProfile() {

        Schema egms = new UKeGMS();

        ApplicationProfile profile = new UKeGMSProfile();

        MetaTripleWriter customWriter = new MetaTripleWriter(profile);

        String predicate = "egms.Mandate";

        String expanded = customWriter.expandPredicate(predicate);

        String uri = egms.getUri() + "mandate";

        assertTrue("Should expand the predicate to the URI " +
                   uri + ", actually returned '" +
                   expanded + "'.",
                   expanded.equals(uri));
    }

    /**
     *  Check whether an element is accepted.
     *  @since  Version 0.9 2005-02-15
     */
    public final void testAcceptElementNullUri() {

        try {

            tripleWriter.acceptElement(null,
                                       META);

            fail(EXPECT_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Check whether an element is accepted.
     *  @since  Version 0.9 2005-02-15
     */
    public final void testAcceptElementNullLocalName() {

        try {

            tripleWriter.acceptElement(NAMESPACE,
                                       null);

            fail(EXPECT_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Check whether an element is accepted.
     *  @since  Version 0.9 2005-02-15
     */
    public final void testAcceptElementFalse() {


        assertFalse(tripleWriter.acceptElement("http://example.org/1999/xhtml",
                                               META));
    }

    /**
     *  Check whether an element is accepted.
     *  @since  Version 0.9 2005-02-15
     */
    public final void testAcceptElementTrue() {


        assertTrue(tripleWriter.acceptElement(NAMESPACE,
                                              META));
    }

    /**
     *  Add triples with null argument.
     *  @since  Version 0.9 2005-02-15
     */
    public final void testPrintTriplesNullSourceUri() {

        PrintWriter writer =  new PrintWriter(outStream);

        try {

            tripleWriter.printTriples(null,
                                      attributes,
                                      writer);

            fail(EXPECT_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Add triples with null argument.
     *  @since  Version 0.9 2005-02-15
     */
    public final void testPrintTriplesNullAttributes() {

        PrintWriter writer =  new PrintWriter(outStream);

        try {

            tripleWriter.printTriples(URI_SUBJ_1,
                                      null,
                                      writer);

            fail(EXPECT_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Add triples with null argument.
     *  @since  Version 0.9 2005-02-15
     */
    public final void testPrintTriplesNullPrintWriter() {

        try {

            tripleWriter.printTriples(URI_SUBJ_1,
                                      attributes,
                                      null);

            fail(EXPECT_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Start a <code>meta</code> element with a default instance.
     *  @since  Version 0.9 2005-02-15
     */
    public final void testStartElementDefaultInstance() {

        try {

            basicWriter.startElement(NAMESPACE,
                                     META,
                                     QUAL_META,
                                     attributes);

            fail(EXPECTED_SE);
        }
        catch (SAXException se) { }

    }

    /**
     *  Start a <code>meta</code> element with a default instance.
     *  @since  Version 0.9 2005-02-15
     */
    public final void testStartElementDefaultInstanceWithSubject() {

        basicWriter.setSubject(URI_SUBJ_1);

        try {

            basicWriter.startElement(NAMESPACE,
                                     META,
                                     QUAL_META,
                                     attributes);

            fail(EXPECTED_SE);
        }
        catch (SAXException se) { }

    }

    /**
     *  XHTML <code>meta</code> basic attributes.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2004-12-17
     */
    public final void testStartElementGeneric() throws SAXException {

        attributes.addAttribute(NAMESPACE,
                                NAME,
                                QUAL_NAME,
                                CDATA,
                                NAME_DESC);

        attributes.addAttribute(NAMESPACE,
                                CONTENT,
                                QUAL_CONTENT,
                                CDATA,
                                VALUE_DESC);

        tripleWriter.startDocument();

        outStream.reset();

        tripleWriter.startElement(NAMESPACE,
                                  META,
                                  QUAL_META,
                                  attributes);

        String start = outStream.getOutput();

        assertTrue("Should be no triple output when predicate " +
                   "is not expanded to a URI. '" +
                   "Actual output: '" + start + "'.",
                   start.equals(""));
    }

    /**
     *  XHTML <code>meta</code> with schema prefix.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @since  Version 0.6 2005-01-07
     */
    public final void testStartElementSchema() throws SAXException {

        attributes.addAttribute(NAMESPACE,
                                NAME,
                                QUAL_NAME,
                                CDATA,
                                DC_DATE);

        attributes.addAttribute(NAMESPACE,
                                CONTENT,
                                QUAL_CONTENT,
                                CDATA,
                                DATE_W3CDTF);

        tripleWriter.startDocument();

        outStream.reset();

        tripleWriter.startElement(NAMESPACE,
                                  META,
                                  QUAL_META,
                                  attributes);

        String start = outStream.getOutput();

        assertTrue("The triple output should equal: '" +
                   TRIPLE_DC + "'\n" +
                   "actually equals:                '" +
                   start + "'.",
                   start.equals(TRIPLE_DC));
    }

    /**
     *  XHTML <code>meta</code> with surplus white space around the date value.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @since  Version 0.6 2005-01-07
     */
    public final void testStartElementSchemaTrim() throws SAXException {

        attributes.addAttribute(NAMESPACE,
                                NAME,
                                QUAL_NAME,
                                CDATA,
                                DC_DATE);

        attributes.addAttribute(NAMESPACE,
                                CONTENT,
                                QUAL_CONTENT,
                                CDATA,
                                DATE_PADDED);

        tripleWriter.startDocument();

        outStream.reset();

        tripleWriter.startElement(NAMESPACE,
                                  META,
                                  QUAL_META,
                                  attributes);

        String start = outStream.getOutput();

        assertTrue("The triple output should equal: '" +
                   TRIPLE_DC + "'\n" +
                   "actually equals:                '" +
                   start + "'.",
                   start.equals(TRIPLE_DC));
    }



    /**
     *  XHTML <code>meta</code> with <code>scheme</code>
     *  attribute.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2004-12-17
     */
    public final void testStartElementWithScheme() throws SAXException {

        attributes.addAttribute(NAMESPACE,
                                NAME,
                                QUAL_NAME,
                                CDATA,
                                DC_DATE);

        attributes.addAttribute(NAMESPACE,
                                SCHEME,
                                QUAL_SCHEME,
                                CDATA,
                                W3CDTF);

        attributes.addAttribute(NAMESPACE,
                                CONTENT,
                                QUAL_CONTENT,
                                CDATA,
                                DATE_W3CDTF);

        tripleWriter.startDocument();

        outStream.reset();

        tripleWriter.startElement(NAMESPACE,
                                  META,
                                  QUAL_META,
                                  attributes);

        String start = outStream.getOutput();

        assertTrue("The triple output should include the bNode:\n" +
                   TRIPLE_BNODE + "\n" +
                   ACTUALLY +
                   start,
                   (start.indexOf(TRIPLE_BNODE) > -1));

        assertTrue("The triple output should include the bNode type:\n" +
                   BNODE_TYPE + "\n" +
                   ACTUALLY +
                   start,
                   (start.indexOf(BNODE_TYPE) > -1));

        assertTrue("The triple output should include the bNode value:\n" +
                   BNODE_VALUE + "\n" +
                   "actually equals:\n" +
                   start,
                   (start.indexOf(BNODE_VALUE) > -1));

    }

    /**
     *  XHTML <code>meta</code> with surplus white space on the date value.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2004-12-17
     */
    public final void testStartElementWithSchemeTrim() throws SAXException {

        attributes.addAttribute(NAMESPACE,
                                NAME,
                                QUAL_NAME,
                                CDATA,
                                DC_DATE);

        attributes.addAttribute(NAMESPACE,
                                SCHEME,
                                QUAL_SCHEME,
                                CDATA,
                                W3CDTF);

        attributes.addAttribute(NAMESPACE,
                                CONTENT,
                                QUAL_CONTENT,
                                CDATA,
                                DATE_PADDED);

        tripleWriter.startDocument();

        outStream.reset();

        tripleWriter.startElement(NAMESPACE,
                                  META,
                                  QUAL_META,
                                  attributes);

        String start = outStream.getOutput();

        assertTrue("The triple output should include the bNode:\n" +
                   TRIPLE_BNODE + "\n" +
                   ACTUALLY +
                   start,
                   (start.indexOf(TRIPLE_BNODE) > -1));

        assertTrue("The triple output should include the bNode type:\n" +
                   BNODE_TYPE + "\n" +
                   ACTUALLY +
                   start,
                   (start.indexOf(BNODE_TYPE) > -1));

        assertTrue("The triple output should include the bNode value:\n" +
                   BNODE_VALUE + "\n" +
                   "actually equals:\n" +
                   start,
                   (start.indexOf(BNODE_VALUE) > -1));

    }


    /**
     *  XHTML <code>meta</code> no <code>name</code>
     *  attribute, should not generate a triple.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2004-12-17
     */
    public final void testStartElementNoNameAttr() throws SAXException {

        attributes.addAttribute(NAMESPACE,
                                CONTENT,
                                QUAL_CONTENT,
                                CDATA,
                                DATE_W3CDTF);

        tripleWriter.startDocument();

        outStream.reset();

        tripleWriter.startElement(NAMESPACE,
                                  META,
                                  QUAL_META,
                                  attributes);

        String output = outStream.getOutput();

        assertTrue(SHOULD_NOT + output + "'.",
                   output.equals(""));

    }

    /**
     *  XHTML <code>meta</code> no <code>content</code>
     *  attribute, should not generate a triple.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2004-12-17
     */
    public final void testStartElementNoContentAttr() throws SAXException {

        attributes.addAttribute(NAMESPACE,
                                NAME,
                                QUAL_NAME,
                                CDATA,
                                DC_DATE);

        tripleWriter.startDocument();

        outStream.reset();

        tripleWriter.startElement(NAMESPACE,
                                  META,
                                  QUAL_META,
                                  attributes);

        String output = outStream.getOutput();

        assertTrue(SHOULD_NOT + output + "'.",
                   output.equals(""));
    }

    /**
     *  XHTML <code>head</code> elements should throw a
     *  <code>SAXException</code>.
     *  @exception  SAXException Should not be thrown by this test.
     *  @since  Version 0.1 2004-12-17
     */
    public final void testStartElementHeadElement() throws SAXException {

        tripleWriter.startDocument();

        outStream.reset();

        try {

            tripleWriter.startElement(NAMESPACE,
                                      HEAD,
                                      QUAL_HEAD,
                                      attributes);

            fail(EXPECTED_SE);
        }
        catch (SAXException se) { }

    }

    /**
     *  Should output an initial comment line that includes
     *  the subject URI.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2004-12-16
     */
    public final void testStartDocument() throws SAXException {

        tripleWriter.startDocument();

        String start = outStream.getOutput();

        assertTrue("Output should start with a comment and include the " +
                   "subject URI. Actual output: '" + start + "'.",
                   (start.indexOf("#") == 0) &&
                   (start.indexOf(URI_1) > -1));

    }

    /**
     *  Should throw a SAX exception from an IOException.
     *  @exception  IOException     Should not be thrown by this test.
     *  @exception  SAXException    Should not be thrown by this test.
     *  @since  Version 1.2 2005-05-10
     */
    public final void testStartDocumentThrowsIOException()
        throws IOException, SAXException {

        MockVirtualFileStoreManager badManager = null;

        badManager = new MockVirtualFileStoreManager();

        badManager.setFailOnGetOutput(true);

        tripleWriter = new MetaTripleWriter(badManager,
                                            new URL(URI_1));

        try {

            tripleWriter.startDocument();

            fail(EXPECTED_SE);
        }
        catch (SAXException se) { }
    }

    /**
     *  Should output a new line and end.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2004-12-16
     */
    public final void testEndDocument() throws SAXException {

        tripleWriter.startDocument();

        tripleWriter.endDocument();

        String newline = System.getProperty("line.separator");

        String end = outStream.getOutput();

        String veryEnd = end.substring(end.length() - newline.length(),
                                       end.length());

        assertTrue("Output should end with a new line." +
                   "Very end was '" + end + "'.",
                   veryEnd.equals(newline));
    }

    /**
     *  End without starting should do nothing, pure coverage test.
     *  @exception  SAXException    Should not be thrown by this test.
     *  @since  Version 1.2 2005-05-10
     */
    public final void testEndDocumentNotStarted() throws SAXException {

        tripleWriter.endDocument();

        // No assertion, no side effect
    }

    /**
     *  Set null subject should throw an
     *  <code>IllegalArgumentException</code>.
     *  @since  Version 0.1 2004-12-16
     */
    public final void testSetSubjectNull() {

        try {

            tripleWriter.setSubject(null);

            fail(EXPECT_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Should adopt the new URI reference.
     *  @since  Version 0.1 2004-12-16
     */
    public final void testSetGetSubject() {

        assertTrue(URI_1.equals(tripleWriter.getSubject()));

        tripleWriter.setSubject(URI_2);

        assertTrue(URI_2.equals(tripleWriter.getSubject()));
    }

    /**
     *  Constructor with null subject manager should throw an
     *  <code>IllegalArgumentException</code>.
     *  @exception  IOException Should not be thrown by this test.
     *  @since  Version 0.1 2004-12-16
     */
    public final void testConstructorNullSubjectManager() throws IOException {

        try {

            tripleWriter = new MetaTripleWriter(null,
                                                new URL(URI_1));

            fail(EXPECT_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Constructor with null URI should throw an
     *  <code>IllegalArgumentException</code>.
     *  @since  Version 0.1 2004-12-16
     */
    public final void testConstructorNullUri() {

        try {

            tripleWriter = new MetaTripleWriter(manager,
                                                null);

            fail(EXPECT_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }


    /**
     *  Tear down a test scenario as required.
     *  @since  Version 0.1 2004-
    protected void tearDown() {

    }
     */

    /**
     *  Create a test suite for this test case.
     *  @return A test suite.
     *  @since  Version 0.1 2004-12-16
     */
    public static Test suite() {

        return new TestSuite(MetaTripleWriterTest.class);
    }

    /**
     *  Run this test suite as a stand alone application.
     *  @param  args    Any command line arguments to pass
     *          to the test. None required or expected.
     *  @since  Version 0.1 2004-12-16
     */
    public static void main(final String[] args) {

        junit.textui.TestRunner.run(suite());
    }

}
