package com.mkdoc.sax;

/*
    Created:    2005-02-16
    Issued:     2005-02-16
    Modified:   2005-11-01

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import org.openrdf.model.Graph;
import org.openrdf.model.URI;
import org.openrdf.model.Value;
import org.openrdf.model.ValueFactory;

import org.openrdf.model.impl.GraphImpl;

import com.mkdoc.sesame.MockSesameRepository;
import com.mkdoc.sesame.MockRdfSource;
import com.mkdoc.sesame.MockLocalService;

import com.mkdoc.store.MockStoreManager;

import java.io.IOException;

import java.net.URL;

import org.xml.sax.SAXException;

import org.xml.sax.helpers.AttributesImpl;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  Test suite for <code>XhtmlStoreWriter</code>.
 *  @version    0.4 2005-11-01
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class XhtmlStoreWriterTest extends TestCase {

    /**
     *  An expected graph content message.
     *  @since  Version 0.4 2005-11-01
     */
    public static final String GRAPH_SHOULD =
    "Graph should contain a statement with subject '";

    /**
     *  An expected literal content message.
     *  @since  Version 0.4 2005-11-01
     */
    public static final String AND_LITERAL =
    "', and literal value '";

    /**
     *  An actual graph content prefix.
     *  @since  Version 0.4 2005-11-01
     */
    public static final String ACTUAL_GRAPH =
    "Actual graph =\n";

    /**
     *  An end quote string.
     *  @since  Version 0.4 2005-11-01
     */
    public static final String ENDQUOTE = "'. ";

    /**
     *  An expected bNode content message.
     *  @since  Version 0.4 2005-11-01
     */
    public static final String EXPECT_BNODE =
    "Graph should contain a bNode statement with predicate '";

    /**
     *  A repository identifier.
     *  @since  Version 0.2 2005-03-29
     */
    protected static final String REPO_ID = "Test repository";

    /**
     *  The XHTML namespace URI.
     *  @since  Version 0.1 2005-02-16
     */
    private static final String NAMESPACE = "http://www.w3.org/1999/xhtml";

    /**
     *  A test subject URI.
     *  @since  Version 0.1 2005-02-16
     */
    private static final String URI_SUBJ_1 =
    "http://www.mkdoc.org/uri/test/XhtmlStoreWriter#one";

    /**
     *  A test subject URI.
     *  @since  Version 0.1 2005-02-16
     */
    private static final String URI_SUBJ_2 =
    "http://www.mkdoc.org/uri/test/XhtmlStoreWriter#two";

    /**
     *  The DC rights URI.
     *  @since  Version 0.1 2005-02-16
     */
    private static final String URI_RIGHTS =
    "http://purl.org/dc/elements/1.1/rights";

    /**
     *  The DC qualified URI type.
     *  @since  Version 0.1 2005-02-16
     */
    private static final String URI_DCTERMS = "http://purl.org/dc/terms/URI";

    /**
     *  The DC date URI.
     *  @since  Version 0.1 2005-02-16
     */
    private static final String URI_DC_DATE =
    "http://purl.org/dc/elements/1.1/date";

    /**
     *  The DC terms W3C DTF URI.
     *  @since  Version 0.1 2005-02-16
     */
    private static final String URI_W3CDTF =
    "http://purl.org/dc/terms/W3CDTF";

    /**
     *  The RDF type URI.
     *  @since  Version 0.1 2005-02-16
     */
    private static final String RDF_TYPE =
    "http://www.w3.org/1999/02/22-rdf-syntax-ns#type";

    /**
     *  The RDF value URI.
     *  @since  Version 0.1 2005-02-16
     */
    private static final String RDF_VALUE =
    "http://www.w3.org/1999/02/22-rdf-syntax-ns#value";

    /**
     *  An XHTML <code>meta</code> element local name.
     *  @since  Version 0.1 2005-02-16
     */
    private static final String META = "meta";

    /**
     *  An XHTML <code>name</code> attribute local name.
     *  @since  Version 0.1 2005-02-16
     */
    private static final String NAME = "name";

    /**
     *  A qualified XHTML <code>name</code> attribute.
     *  @since  Version 0.1 2005-02-16
     */
    private static final String QUAL_NAME = NAMESPACE +
                                            ":" +
                                            NAME;

    /**
     *  An XHTML <code>scheme</code> attribute local name.
     *  @since  Version 0.1 2005-02-16
     */
    private static final String SCHEME = "scheme";

    /**
     *  A qualified XHTML <code>scheme</code> attribute.
     *  @since  Version 0.1 2005-02-16
     */
    private static final String QUAL_SCHEME = NAMESPACE +
                                              ":" +
                                              SCHEME;

    /**
     *  An XHTML <code>content</code> attribute local name.
     *  @since  Version 0.1 2005-02-16
     */
    private static final String CONTENT = "content";

    /**
     *  A qualified XHTML <code>name</code> attribute.
     *  @since  Version 0.1 2005-02-16
     */
    private static final String QUAL_CONTENT = NAMESPACE +
                                               ":" +
                                               CONTENT;

    /**
     *  A generic XHTML <code>description</code> name value.
     *  @since  Version 0.1 2005-02-16
     */
    private static final String NAME_DESC = "description";

    /**
     *  A generic XHTML <code>description</code> content
     *  value.
     *  @since  Version 0.1 2005-02-16
     */
    private static final String VALUE_DESC =
    "Generic description content";

    /**
     *  A DC title predicate.
     *  @since  Version 0.1 2005-02-16
     */
    private static final String DC_DATE = "DC.Date";

    /**
     *  The DC W3C-DTF encoding name.
     *  @since  Version 0.1 2005-02-16
     */
    private static final String W3CDTF = "W3CDTF";

    /**
     *  A W3C-DTF date value.
     *  @since  Version 0.1 2005-02-16
     */
    private static final String DATE_W3CDTF = "2005-01-13";

    /**
     *  A qualified XHTML <code>meta</code> element name.
     *  @since  Version 0.1 2005-02-16
     */
    private static final String QUAL_META = NAMESPACE +
                                            ":" +
                                            META;

    /**
     *  An XHTML <code>head</code> element local name.
     *  @since  Version 0.1 2005-02-16
     */
    private static final String HEAD = "head";

    /**
     *  An XHTML <code>link</code> element local name.
     *  @since  Version 0.1 2005-02-16
     */
    private static final String LINK = "link";

    /**
     *  An XHTML <code>rel</code> attribute local name.
     *  @since  Version 0.1 2005-02-16
     */
    private static final String REL = "rel";

    /**
     *  An XHTML <code>rev</code> attribute local name.
     *  @since  Version 0.1 2005-02-16
     */
    private static final String REV = "rev";

    /**
     *  A qualified XHTML <code>rel</code> attribute.
     *  @since  Version 0.1 2005-02-16
     */
    private static final String QUAL_REL = NAMESPACE +
                                            ":" +
                                            REL;

    /**
     *  A qualified XHTML <code>rev</code> attribute.
     *  @since  Version 0.1 2005-02-16
     */
    private static final String QUAL_REV = NAMESPACE +
                                           ":" +
                                           REV;

    /**
     *  An XHTML <code>href</code> attribute local name.
     *  @since  Version 0.1 2005-02-16
     */
    private static final String HREF = "href";

    /**
     *  A qualified XHTML <code>rel</code> attribute.
     *  @since  Version 0.1 2005-02-16
     */
    private static final String QUAL_HREF = NAMESPACE +
                                               ":" +
                                               HREF;

    /**
     *  A generic XHTML <code>copyright</code> rel value.
     *  @since  Version 0.1 2005-02-16
     */
    private static final String REL_GENERIC = "copyright";

    /**
     *  A generic XHTML <code>copyright</code> href value.
     *  @since  Version 0.1 2005-02-16
     */
    private static final String HREF_COPY =
    "http://test.mksearch.mkdoc.org/test/copyright";

    /**
     *  A DC rights predicate.
     *  @since  Version 0.1 2005-02-16
     */
    private static final String DC_RIGHTS = "DC.Rights";

    /**
     *  A DC rights object URI value.
     *  @since  Version 0.1 2005-02-16
     */
    private static final String HREF_RIGHTS =
    "http://test.mksearch.mkdoc.org/test/rights";

    /**
     *  A qualified XHTML <code>link</code> element name.
     *  @since  Version 0.1 2005-02-16
     */
    private static final String QUAL_LINK = NAMESPACE +
                                            ":" +
                                            LINK;

    /**
     *  A qualified XHTML <code>head</code> element name.
     *  @since  Version 0.1 2005-02-16
     */
    private static final String QUAL_HEAD = NAMESPACE +
                                            ":" +
                                            HEAD;

    /**
     *  A CDATA type string.
     *  @since  Version 0.1 2005-02-16
     */
    private static final String CDATA = "CDATA";

    /**
     *  An expected <code>IllegalArgumentException</code>.
     *  @since  Version 0.1 2005-02-16
     */
    private static final String EXPECT_IAE =
    "Should throw an IllegalArgumentException.";

    /**
     *  An expected <code>SAXException</code>.
     *  @since  Version 0.1 2005-02-16
     */
    private static final String EXPECT_SE =
    "Should throw a SAXException.";

    /**
     *  An empty attribute set for testing.
     *  @since  Version Version 0.1 2005-02-16
     */
    private AttributesImpl attributes;

    /**
     *  Instance to test.
     *  @since  Version 0.1 2005-02-16
     */
    protected XhtmlStoreWriter storeWriter;

    /**
     *  A <code>MockSesameRepository</code>.
     *  @since  Version 0.1 2005-02-16
     */
    protected MockSesameRepository repository;

    /**
     *  A <code>MockStoreManager</code>.
     *  @since  Version 0.1 2005-02-16
     */
    protected MockStoreManager manager;

    /**
     *  A value factory for generating statements.
     *  @since  Version 0.1 2005-02-16
     */
    protected ValueFactory factory;

    /**
     *  A graph implementation.
     *  @since  Version 0.1 2005-02-16
     */
    protected Graph graph;

    /**
     *  Named test instance.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2005-02-16
     */
    public XhtmlStoreWriterTest(final String name) {

        super(name);
    }

    /**
     *  Set up a test scenario as required.
     *  @exception  IOException Should not be thrown.
     *  @since  Version 0.1 2005-02-16
     */
    protected final void setUp() throws IOException {

        graph = new GraphImpl();

        factory = graph.getValueFactory();

        repository = new MockSesameRepository(REPO_ID,
                                              new MockRdfSource(),
                                              new MockLocalService());

        manager = new MockStoreManager();

        storeWriter = new XhtmlStoreWriter(manager, new URL(URI_SUBJ_1));

        attributes = new AttributesImpl();

    }

    /**
     *  Start an unrecognized element should throw
     *  <code>SAXException</code>.
     *  @since  Version 0.1 2005-02-16
     */
    public final void testStartElementUnrecognized() {

        try {

            storeWriter.startElement(NAMESPACE,
                                     HEAD,
                                     QUAL_HEAD,
                                     attributes);

            fail(EXPECT_SE);
        }
        catch (SAXException se) { }

    }

    /**
     *  XHTML <code>link</code> no <code>href</code>
     *  attribute, should not generate a statement.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2005-02-16
     */
    public final void testStartElementLinkNoHrefAttr() throws SAXException {

        attributes.addAttribute(NAMESPACE,
                                REL,
                                QUAL_REL,
                                CDATA,
                                DC_RIGHTS);

        storeWriter.startElement(NAMESPACE,
                                 LINK,
                                 QUAL_LINK,
                                 attributes);

        URI subject = factory.createURI(URI_SUBJ_1);

        storeWriter.endDocument();

        graph = manager.getGraph();

        assertFalse(graph.contains(subject, null, null));
    }

    /**
     *  XHTML <code>link</code> no <code>rel</code>
     *  attribute, should not generate a statement.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2005-02-16
     */
    public final void testStartElementLinkNoRelAttr() throws SAXException {

        attributes.addAttribute(NAMESPACE,
                                HREF,
                                QUAL_HREF,
                                CDATA,
                                HREF_RIGHTS);

        storeWriter.startElement(NAMESPACE,
                                 LINK,
                                 QUAL_LINK,
                                 attributes);

        URI subject = factory.createURI(URI_SUBJ_1);

        storeWriter.endDocument();

        graph = manager.getGraph();

        assertFalse(graph.contains(subject, null, null));

    }

    /**
     *  XHTML <code>link</code> basic attributes.
     *  @exception  SAXException    If there is a problem
     *          parsing the content, indicates an error.
     *  @since  Version 0.1 2005-02-16
     */
    public final void testStartElementLinkGeneric() throws SAXException {

        attributes.addAttribute(NAMESPACE,
                                REL,
                                QUAL_REL,
                                CDATA,
                                REL_GENERIC);

        attributes.addAttribute(NAMESPACE,
                                HREF,
                                QUAL_HREF,
                                CDATA,
                                HREF_COPY);

        storeWriter.startElement(NAMESPACE,
                                 LINK,
                                 QUAL_LINK,
                                 attributes);

        storeWriter.endDocument();

        graph = manager.getGraph();

        URI subject = factory.createURI(URI_SUBJ_1);

        assertFalse(graph.contains(subject, null, null));
    }

    /**
     *  XHTML <code>link</code> with schema prefix -- forward relation.
     *  @exception  SAXException    If there is a problem
     *          parsing the content, indicates an error.
     *  @since  Version 0.1 2005-02-16
     */
    public final void testStartElementLinkSchemaRel() throws SAXException {

        attributes.addAttribute(NAMESPACE,
                                REL,
                                QUAL_REL,
                                CDATA,
                                DC_RIGHTS);

        attributes.addAttribute(NAMESPACE,
                                HREF,
                                QUAL_HREF,
                                CDATA,
                                HREF_RIGHTS);

        storeWriter.startElement(NAMESPACE,
                                 LINK,
                                 QUAL_LINK,
                                 attributes);

        storeWriter.endDocument();

        graph = manager.getGraph();

        String graphString = manager.getGraphString();


        URI subject = factory.createURI(URI_SUBJ_1);

        URI predicate = factory.createURI(URI_RIGHTS);

        assertTrue(GRAPH_SHOULD +
                   subject + "' and predicate '" + predicate + ENDQUOTE +
                   ACTUAL_GRAPH + graphString,
                   graph.contains(subject, predicate, null));

        predicate = factory.createURI(RDF_TYPE);

        Value object = factory.createURI(URI_DCTERMS);

        assertTrue(EXPECT_BNODE +
                   predicate + "', and object URI '" + object + ENDQUOTE +
                   ACTUAL_GRAPH + graphString,
                   graph.contains(null, predicate, object));

        predicate = factory.createURI(RDF_VALUE);

        object = factory.createURI(HREF_RIGHTS);

        assertTrue(EXPECT_BNODE +
                   predicate + "', and URI object '" + object + ENDQUOTE +
                   ACTUAL_GRAPH + graphString,
                   graph.contains(null, predicate, object));
    }


    /**
     *  XHTML <code>link</code> with schema prefix -- reverse relation.
     *  @exception  SAXException    If there is a problem
     *          parsing the content, indicates an error.
     *  @since  Version 0.1 2005-02-16
     */
    public final void testStartElementLinkSchemaRev() throws SAXException {

        attributes.addAttribute(NAMESPACE,
                                REV,
                                QUAL_REV,
                                CDATA,
                                DC_RIGHTS);

        attributes.addAttribute(NAMESPACE,
                                HREF,
                                QUAL_HREF,
                                CDATA,
                                HREF_RIGHTS);

        storeWriter.startElement(NAMESPACE,
                                 LINK,
                                 QUAL_LINK,
                                 attributes);

        storeWriter.endDocument();

        graph = manager.getGraph();

        String graphString = manager.getGraphString();


        URI subject = factory.createURI(HREF_RIGHTS);

        URI predicate = factory.createURI(URI_RIGHTS);

        assertTrue(GRAPH_SHOULD +
                   subject + "' and predicate '" + predicate + ENDQUOTE +
                   ACTUAL_GRAPH + graphString,
                   graph.contains(subject, predicate, null));

        predicate = factory.createURI(RDF_TYPE);

        Value object = factory.createURI(URI_DCTERMS);

        assertTrue(EXPECT_BNODE +
                   predicate + "', and object URI '" + object + ENDQUOTE +
                   ACTUAL_GRAPH + graphString,
                   graph.contains(null, predicate, object));

        predicate = factory.createURI(RDF_VALUE);

        object = factory.createURI(URI_SUBJ_1);

        assertTrue(EXPECT_BNODE +
                   predicate + "', and URI object '" + object + ENDQUOTE +
                   ACTUAL_GRAPH + graphString,
                   graph.contains(null, predicate, object));
    }

    /**
     *  XHTML <code>meta</code> no <code>content</code>
     *  attribute, should not generate a statement.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2005-02-16
     */
    public final void testStartElementMetaNoContentAttr() throws SAXException {

        attributes.addAttribute(NAMESPACE,
                                NAME,
                                QUAL_NAME,
                                CDATA,
                                DC_DATE);

        storeWriter.startElement(NAMESPACE,
                                 META,
                                 QUAL_META,
                                 attributes);

        URI subject = factory.createURI(URI_SUBJ_1);

        storeWriter.endDocument();

        graph = manager.getGraph();

        assertFalse(graph.contains(subject, null, null));
    }

    /**
     *  XHTML <code>meta</code> no <code>name</code>
     *  attribute, should not generate a statement.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2005-02-16
     */
    public final void testStartElementMetaNoNameAttr() throws SAXException {

        attributes.addAttribute(NAMESPACE,
                                CONTENT,
                                QUAL_CONTENT,
                                CDATA,
                                DATE_W3CDTF);

        storeWriter.startElement(NAMESPACE,
                                 META,
                                 QUAL_META,
                                 attributes);

        URI subject = factory.createURI(URI_SUBJ_1);

        storeWriter.endDocument();

        graph = manager.getGraph();

        assertFalse(graph.contains(subject, null, null));

    }

    /**
     *  XHTML <code>meta</code> basic attributes.
     *  @exception  SAXException    If there is a problem
     *          parsing the content, indicates an error.
     *  @since  Version 0.1 2005-02-16
     */
    public final void testStartElementMetaGeneric() throws SAXException {

        attributes.addAttribute(NAMESPACE,
                                NAME,
                                QUAL_NAME,
                                CDATA,
                                NAME_DESC);

        attributes.addAttribute(NAMESPACE,
                                CONTENT,
                                QUAL_CONTENT,
                                CDATA,
                                VALUE_DESC);

        storeWriter.startElement(NAMESPACE,
                                 META,
                                 QUAL_META,
                                 attributes);

        storeWriter.endDocument();

        graph = manager.getGraph();

        URI subject = factory.createURI(URI_SUBJ_1);

        assertFalse(graph.contains(subject, null, null));
    }

    /**
     *  XHTML <code>meta</code> with schema prefix.
     *  @exception  SAXException    If there is a problem
     *          parsing the content, indicates an error.
     *  @since  Version 0.1 2005-02-16
     */
    public final void testStartElementMetaSchema() throws SAXException {

        attributes.addAttribute(NAMESPACE,
                                NAME,
                                QUAL_NAME,
                                CDATA,
                                DC_DATE);

        attributes.addAttribute(NAMESPACE,
                                CONTENT,
                                QUAL_CONTENT,
                                CDATA,
                                DATE_W3CDTF);

        storeWriter.startElement(NAMESPACE,
                                 META,
                                 QUAL_META,
                                 attributes);

        storeWriter.endDocument();

        graph = manager.getGraph();

        String graphString = manager.getGraphString();

        URI subject = factory.createURI(URI_SUBJ_1);

        URI predicate = factory.createURI(URI_DC_DATE);

        Value object = factory.createLiteral(DATE_W3CDTF);

        assertTrue(GRAPH_SHOULD +
                   subject + "', predicate '" +
                   predicate + AND_LITERAL +
                   object + ENDQUOTE +
                   ACTUAL_GRAPH + graphString,
                   graph.contains(subject, predicate, object));
    }

    /**
     *  XHTML <code>meta</code> with <code>scheme</code>
     *  attribute.
     *  @exception  SAXException    If there is a problem
     *          parsing the content, indicates an error.
     *  @since  Version 0.1 2005-02-16
     */
    public final void testStartElementMetaWithScheme() throws SAXException {

        attributes.addAttribute(NAMESPACE,
                                NAME,
                                QUAL_NAME,
                                CDATA,
                                DC_DATE);

        attributes.addAttribute(NAMESPACE,
                                SCHEME,
                                QUAL_SCHEME,
                                CDATA,
                                W3CDTF);

        attributes.addAttribute(NAMESPACE,
                                CONTENT,
                                QUAL_CONTENT,
                                CDATA,
                                DATE_W3CDTF);

        storeWriter.startElement(NAMESPACE,
                                 META,
                                 QUAL_META,
                                 attributes);

        storeWriter.endDocument();

        graph = manager.getGraph();

        String graphString = manager.getGraphString();


        URI subject = factory.createURI(URI_SUBJ_1);

        URI predicate = factory.createURI(URI_DC_DATE);

        assertTrue(GRAPH_SHOULD +
                   subject + "' and predicate '" + predicate + ENDQUOTE +
                   ACTUAL_GRAPH + graphString,
                   graph.contains(subject, predicate, null));

        predicate = factory.createURI(RDF_TYPE);

        Value object = factory.createURI(URI_W3CDTF);

        assertTrue(EXPECT_BNODE +
                   predicate + "', and object URI '" + object + ENDQUOTE +
                   ACTUAL_GRAPH + graphString,
                   graph.contains(null, predicate, object));

        predicate = factory.createURI(RDF_VALUE);

        object = factory.createLiteral(DATE_W3CDTF);

        assertTrue(EXPECT_BNODE +
                   predicate + AND_LITERAL + object + ENDQUOTE +
                   ACTUAL_GRAPH + graphString,
                   graph.contains(null, predicate, object));
    }

    /**
     *  Set a null URI subject should throw an
     *  <code>IllegalArgumentException</code>.
     *  @since  Version 0.1 2005-02-16
     */
    public final void testSetSubjectUri() {

        try {

            storeWriter.setSubjectUri(null);

            fail(EXPECT_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Set and get a subject URI.
     *  @since  Version 0.1 2005-02-16
     */
    public final void testSetGetSubjectUri() {

        URI uri = storeWriter.getSubjectUri();

        assertTrue(URI_SUBJ_1.equals(uri.getURI()));

        uri = factory.createURI(URI_SUBJ_2);

        storeWriter.setSubjectUri(uri);

        uri = storeWriter.getSubjectUri();

        assertTrue(URI_SUBJ_2.equals(uri.getURI()));
    }

    /**
     *  End document adds a graph to the manager.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2005-02-16
     */
    public final void testEndDocument() throws SAXException {

        graph = manager.getGraph();

        assertNull(graph);

        storeWriter.endDocument();

        graph = manager.getGraph();

        assertNotNull(graph);
    }

    /**
     *  Repository throws an exception adding the graph.
     *  @since  Version 0.1 2005-02-16
     */
/*
    public final void testEndDocumentRepositoryException() {

        repository.setException("AccessDeniedException");

        try {

            storeWriter.endDocument();

            fail(EXPECT_SE);
        }
        catch (SAXException saxe) { }

    }
*/

    /**
     *  Constructor with null manager should throw an
     *  <code>IllegalArgumentException</code>.
     *  @exception  IOException Should not be thrown by this test.
     *  @since  Version 0.1 2005-02-16
     */
    public final void testConstructorNullRepository() throws IOException {

        try {

            new XhtmlStoreWriter(null,
                                 new URL(URI_SUBJ_1));

            fail(EXPECT_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Constructor with null subject URI should throw an
     *  <code>IllegalArgumentException</code>.
     *  @since  Version 0.1 2005-02-16
     */
    public final void testConstructorNullUri() {

        try {

            new XhtmlStoreWriter(manager,
                                 null);

            fail(EXPECT_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Tear down a test scenario as required.
     *  @since  Version 0.1 2005-
    protected void tearDown() {

    }
     */

    /**
     *  Create a test suite for this test case.
     *  @return A test suite.
     *  @since  Version 0.1 2005-02-16
     */
    public static Test suite() {

        return new TestSuite(XhtmlStoreWriterTest.class);
    }

    /**
     *  Run this test suite as a stand alone application.
     *  @param  args    Any command line arguments to pass
     *          to the test. None required or expected.
     *  @since  Version 0.1 2005-02-16
     */
    public static void main(final String[] args) {

        junit.textui.TestRunner.run(suite());
    }

}
