package com.mkdoc.schema;

/*
    Created:    2005-01-06
    Issued:     2005-01-06
    Modified:   2005-06-28

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import java.util.Collection;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  Test suite for <code>AbstractSchema</code>.
 *  @version    1.0 2005-06-28
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class AbstractSchemaTest extends TestCase {

    /**
     *  An expected <code>IllegalArgumentException</code> message.
     *  @since  Version 0.4 2005-03-11
     */
    public static final String EXPECTED_IAE =
    "Should throw an IllegalArgumentException.";

    /**
     *  An array of predicates or schemes to test.
     *  @since  Version 0.1 2005-01-06
     */
    protected static final String[] ELEMENTS = new String[] {"test",
                                                             "scruitinize",
                                                             "examine",
                                                             "check"};

    /**
     *  A matched predicate.
     *  @since  Version 0.1 2005-01-06
     */
    protected static final String P_MATCHED = ELEMENTS[2];


    /**
     *  A matched scheme.
     *  @since  Version 1.0 2005-06-28
     */
    protected static final String S_MATCHED = ELEMENTS[3];

    /**
     *  An un-matched predicate.
     *  @since  Version 0.1 2005-01-06
     */
    protected static final String UN_MATCHED = "unmatched";

    /**
     *  A fallback value.
     *  @since  Version 0.1 2005-01-06
     */
    protected static final String FALLBACK = "fallback";

    /**
     *  A mock schema to test.
     *  @since  Version 0.1 2005-01-06
     */
    protected MockSchema mockSchema;

    /**
     *  Named test instance.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2005-01-06
     */
    public AbstractSchemaTest(final String name) {

        super(name);
    }

    /**
     *  Set up a test scenario as required.
     *  @since  Version 0.1 2005-
     */
    protected final void setUp() {

         mockSchema = new MockSchema();
    }

    /**
     *  Get named predicate property.
     *  @since  Version 0.9 2005-06-17
     */
    public final void testGetNamedPredicatePropertyUnmatched() {

        assertNull(mockSchema.getNamedPredicateProperty(UN_MATCHED));
    }

    /**
     *  Get named predicate property.
     *  @since  Version 0.9 2005-06-17
     */
    public final void testGetNamedPredicatePropertyMatched() {

        assertNotNull(mockSchema.getNamedPredicateProperty(P_MATCHED));
    }

    /**
     *  Get the schema name.
     *  @since  Version 0.9 2005-06-17
     */
    public final void testGetName() {

        String expected = "Mock Schema";

        assertTrue(expected.equals(mockSchema.getName()));
    }

    /**
     *  Test provides qualifiers methods.
     *  @since  Version 0.4 2005-03-11
     */
    public final void testProvidesQualifiers() {

        assertFalse(mockSchema.providesQualifiers());

        assertFalse(mockSchema.providesQualifiersFor(DublinCoreElements.BASE));
    }

    /**
     *  Get a set of predicates
     *  @since  Version 0.8 2005-06-10
     */
    public final void testGetPredicates() {

        Collection predicates = mockSchema.getPredicates();

        assertTrue(predicates.size() == 1);

        SchemaProperty property = null;

        property = mockSchema.getNamedPredicateProperty("examine");

        assertTrue(predicates.contains(property));
    }

    /**
     *  Test add schema extension.
     *  @since  Version 0.4 2005-03-10
     */

/* No longer used

    public final void testAddSchemaExtensionNull() {

        try {

            mockSchema.addSchemaExtension(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }
*/

    /**
     *  Test add schema extension.
     *  @since  Version 0.4 2005-03-10
     */
/* No longer used

    public final void testAddSchemaExtension() {

        assertTrue(mockSchema.extensions.size() == 0);

        mockSchema.addSchemaExtension(new UKeGMS());

        assertTrue(mockSchema.extensions.size() == 1);
    }
*/

    /**
     *  Test expand an extension predicate, null predicate.
     *  @since  Version 0.5 2005-05-24
     */
/* No longer used

    public final void testExpandExtensionPredicateNullPredicate() {

        try {

            mockSchema.expandExtensionPredicate(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }
*/

    /**
     *  Test expand an extension schema.
     *  @since  Version 0.4 2005-03-10
     */
/* No longer used

    public final void testExpandExtensionPredicateNoExtensions() {

        String predicate = "eGMS.mandate";

        String expanded = mockSchema.expandExtensionPredicate(predicate);

        assertTrue(expanded.equals(predicate));
    }
*/

    /**
     *  Test expand an extension schema.
     *  @since  Version 0.4 2005-03-10
     */
/* No longer used

    public final void testExpandExtensionPredicateUKeGMSMatched() {

        Schema schema = new UKeGMS();

        mockSchema.addSchemaExtension(schema);

        String predicate = "eGMS.mandate";

        String expanded = mockSchema.expandExtensionPredicate(predicate);

        assertTrue("Should expand the predicate to " +
                   schema.getUri() + predicate + ", " +
                   "actual expanded value was '" +
                   expanded + "'.",
                   expanded.equals(schema.getUri() + "mandate"));
    }
*/

    /**
     *  Test expand an extension schema.
     *  @since  Version 0.4 2005-03-10
     */
/* No longer used

    public final void testExpandExtensionPredicateUKeGMSNotMatched() {

        Schema schema = new UKeGMS();

        mockSchema.addSchemaExtension(schema);

        String predicate = "eGMS.bogus";

        String expanded = mockSchema.expandExtensionPredicate(predicate);

        assertTrue("Should not expand the predicate, " +
                   "actual expanded value was '" +
                   expanded + "'.",
                   expanded.equals(predicate));
    }
*/

    /**
     *  Test expand an extension schema, null schema.
     *  @since  Version 0.5 2005-05-24
     */
/* No longer used

    public final void testExpandExtensionSchemeNullSchema() {

        try {

            mockSchema.expandExtensionScheme(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }
*/

    /**
     *  Test expand an extension schema.
     *  @since  Version 0.4 2005-03-10
     */
/* No longer used

    public final void testExpandExtensionSchemeNoExtensions() {

        String predicate = "egmsaes";

        String expanded = mockSchema.expandExtensionScheme(predicate);

        assertTrue(expanded.equals(predicate));
    }
*/

    /**
     *  Test expand an extension schema.
     *  @since  Version 0.4 2005-03-10
     */
/* No longer used

    public final void testExpandExtensionSchemeUKeGMSMatched() {

        Schema schema = new UKeGMS();

        mockSchema.addSchemaExtension(schema);

        String predicate = "egmsaes";

        String expanded = mockSchema.expandExtensionScheme(predicate);

        assertTrue("Should expand the scheme to " +
                   schema.getUri() + predicate + ", " +
                   "actual expanded value was '" +
                   expanded + "'.",
                   expanded.equals(schema.getUri() + predicate));
    }
*/

    /**
     *  Test expand an extension schema.
     *  @since  Version 0.4 2005-03-11
     */
/* No longer used

    public final void testExpandExtensionSchemeUKeGMSNotMatched() {

        Schema schema = new UKeGMS();

        mockSchema.addSchemaExtension(schema);

        String predicate = "bogus";

        String expanded = mockSchema.expandExtensionScheme(predicate);

        assertTrue("Should not expand the scheme, " +
                   "actual expanded value was '" +
                   expanded + "'.",
                   expanded.equals(predicate));
    }
*/

    /**
     *  Test getting the mock schema prefix.
     *  @since  Version 0.2 2005-03-01
     */
    public final void testGetDefaultPrefix() {

        assertTrue("mock".equals(mockSchema.getDefaultPrefix()));
    }

    /**
     *  Test the prefixes method.
     *  @since  Version 0.7 2005-05-27
     */
    public final void testCopyPrefixes() {

        String[] prefixes = mockSchema.getAllPrefixes();

        String[] test = mockSchema.copyPrefixes(prefixes);

        assertTrue(test[0].equals(prefixes[0]));
    }

    /**
     *  Test the prefixes method.
     *  @since  Version 0.7 2005-05-27
     */
    public final void testCopyPrefixesNullArray() {

        try {

            mockSchema.copyPrefixes(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test the prefixes method.
     *  @since  Version 0.7 2005-05-27
     */
    public final void testCopyPrefixesZeroLength() {

        try {

            mockSchema.copyPrefixes(new String[0]);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Null argument should throw an
     *  <code>IllegalArgumentException</code>.
     *  @since  Version 0.1 2005-01-06
     */
/* No longer used

    public final void testExpandNullName() {

        try {

            mockSchema.expand(null,
                              ELEMENTS,
                              P_MATCHED);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }
*/

    /**
     *  Null argument should throw an
     *  <code>IllegalArgumentException</code>.
     *  @since  Version 0.1 2005-01-06
     */
/* No longer used

    public final void testExpandNullElements() {

        try {

            mockSchema.expand(P_MATCHED,
                              null,
                              P_MATCHED);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }
*/

    /**
     *  Matched predicate should return a URI value.
     *  @since  Version 0.1 2005-01-06
     */
/* No longer used

    public final void testExpandMatched() {

        String expanded = mockSchema.expand(P_MATCHED,
                                            ELEMENTS,
                                            P_MATCHED);

        assertTrue(expanded.equals(mockSchema.getUri() +
                                   P_MATCHED));
    }
*/

    /**
     *  Predicate without prefix should not be expanded.
     *  @since  Version 0.1 2005-01-06
     */
    public final void testExpandPredicateNoPrefix() {

        String expanded = mockSchema.expandPredicate(P_MATCHED,
                                                     FALLBACK);

        assertTrue("Predicate without prefix should not be expanded. " +
                   "Actual expanded value = '" +
                   expanded + "'.",
                   expanded.equals(FALLBACK));
    }

    /**
     *  Matched predicate should return a URI value.
     *  @since  Version 0.1 2005-01-06
     */
    public final void testExpandPredicateNoFallbackMatched() {

        String expanded = mockSchema.expandPredicate("mock." + P_MATCHED);

        assertTrue("Expanded predicate should equal '" +
                   mockSchema.getUri() + P_MATCHED + "', " +
                   "Actual expanded value = '" +
                   expanded + "'.",
                   expanded.equals(mockSchema.getUri() +
                                   P_MATCHED));
    }

    /**
     *  An un-matched predicate should return the original value.
     *  @since  Version 0.1 2005-01-06
     */
    public final void testExpandPredicateNoFallbackUnmatched() {

        String expanded = mockSchema.expandPredicate("mock." + UN_MATCHED);

        assertTrue("Should not expanded the predicate '" +
                   "mock." + UN_MATCHED + "', " +
                   "Actual expanded value = '" +
                   expanded + "'.",
                   expanded.equals("mock." + UN_MATCHED));
    }

    /**
     *  Scheme without prefix should not be expanded.
     *  @since  Version 0.1 2005-01-06
     */
    public final void testExpandSchemeNoPrefix() {

        String expanded = mockSchema.expandScheme(P_MATCHED);

        assertTrue("Should not expand a predicate without prefix, '" +
                   P_MATCHED + "'. " +
                   "Actual expanded value = '" +
                   expanded + "'.",
                   expanded.equals(P_MATCHED));
    }

    /**
     *  Un-matched predicate should return the fallback.
     *  @since  Version 0.1 2005-01-06
     */
/*
    public final void testExpandUnmatched() {

        String expanded = mockSchema.expand(UN_MATCHED,
                                            ELEMENTS,
                                            FALLBACK);

        assertTrue(expanded.equals(FALLBACK));
    }
*/

    /**
     *  Un-matched predicate should return the fallback.
     *  @since  Version 0.1 2005-01-06
     */
    public final void testExpandPredicateUnmatched() {

        String expanded = mockSchema.expandPredicate("mock." + UN_MATCHED,
                                                     FALLBACK);

        assertTrue(expanded.equals(FALLBACK));
    }

    /**
     *  Scheme without prefix should not be expanded.
     *  @since  Version 0.6 2005-05-25
     */
    public final void testExpandSchemeWithFallback() {

        String expanded = mockSchema.expandScheme("mock." + S_MATCHED,
                                                  FALLBACK);

        assertTrue("Should expand a predicate with prefix, '" +
                   "mock." + S_MATCHED + "'. " +
                   "Actual expanded value = '" +
                   expanded + "'.",
                   expanded.equals(mockSchema.getUri() + S_MATCHED));
    }

    /**
     *  Un-matched scheme should return the original.
     *  @since  Version 0.1 2005-01-06
     */
    public final void testExpandSchemeUnmatched() {

        String expanded = mockSchema.expandScheme("mock." + UN_MATCHED);

        assertTrue(expanded.equals("mock." + UN_MATCHED));
    }

    /**
     *  Un-matched scheme should return the fallback.
     *  @since  Version 0.6 2005-05-25
     */
    public final void testExpandSchemeUnmatchedFallback() {

        String expanded = mockSchema.expandScheme("mock." + UN_MATCHED,
                                                  FALLBACK);

        assertTrue(expanded.equals(FALLBACK));
    }


    /**
     *  Un-matched predicate should return the original.
     *  @since  Version 0.1 2005-01-06
     */
/*
    public final void testExpandElementsUnmatched() {

        String expanded = mockSchema.expand(UN_MATCHED, ELEMENTS);

        assertTrue(expanded.equals(UN_MATCHED));
    }
*/

    /**
     *  Null argument should throw an
     *  <code>IllegalArgumentException</code>.
     *  @since  Version 0.1 2005-01-06
     */
/*
    public final void testExpandNullFallback() {

        try {

            mockSchema.expand(P_MATCHED,
                              ELEMENTS,
                              null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }
*/

    /**
     *  URI should equal the protected field value.
     *  @since  Version 0.1 2005-01-06
     */
    public final void testGetUri() {

        assertTrue(mockSchema.URI.equals(mockSchema.getUri()));
    }

    /**
     *  Tear down a test scenario as required.
     *  @since  Version 0.1 2005-
    protected void tearDown() {

    }
     */

    /**
     *  Create a test suite for this test case.
     *  @return A test suite.
     *  @since  Version 0.1 2005-01-06
     */
    public static Test suite() {

        return new TestSuite(AbstractSchemaTest.class);
    }

    /**
     *  Run this test suite as a stand alone application.
     *  @param  args    Any command line arguments to pass
     *          to the test. None required or expected.
     *  @since  Version 0.1 2005-01-06
     */
    public static void main(final String[] args) {

        junit.textui.TestRunner.run(suite());
    }

}
