package com.mkdoc.schema;

/*
    Created:    2005-05-27
    Issued:     2005-05-27
    Modified:   2005-07-25

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import java.util.Map;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  Test suite for <code>DublinCoreProfile</code>.
 *  <p>
 *    Skeleton suite to adopt former <code>DublinCoreElements</code> tests for
 *    DC qualified terms which will be handled separately.
 *  </p>
 *  @version    0.6 2005-07-25
 *  @author Philip Shaw <phil@mkdoc.com>
 */
public class DublinCoreProfileTest extends TestCase {

    /**
     *  Dublin Core encoded scheme.
     *  @since  Version 0.1 2005-05-27
     */
    private static final String SCHEME = "DCTERMS.W3CDTF";

    /**
     *  Old form of Dublin Core qualified predicate.
     *  @since  Version 0.1 2005-05-27
     */
    private static final String OLD_FORM = "DC.Date.modified";

    /**
     *  Dublin Core qualified audience predicate.
     *  @since  Version 0.1 2005-05-27
     */
    public static final String NAME_AUDNCE = "DCTERMS.audience";

    /**
     *  An e-GMS mandate predicate.
     *  @since  Version 0.1 2005-05-27
     */
    public static final String MANDATE = "eGMS.Mandate";

    /**
     *  An expected <code>IllegalArgumentException</code> message.
     *  @since  Version 0.1 2005-05-27
     */
    public static final String EXPECTED_IAE =
    "Should throw an IllegalArgumentException.";

    /**
     *  A generic description predicate.
     *  @since  Version 0.1 2005-05-27
     */
    private static final String GENERIC = "description";

    /**
     *  A bogus Dublin Core encoded predicate.
     *  @since  Version 0.1 2005-05-27
     */
    private static final String BOGUS = "DC.bogus";

    /**
     *  A predicate with unknown schema prefix.
     *  @since  Version 0.1 2005-05-27
     */
    private static final String UNKNOWN = "MYSTERY.TwilightZone";

    /**
     *  An encoded Title predicate.
     *  @since  Version 0.1 2005-05-27
     */
    private static final String TITLE = "DC.Title";

    /**
     *  An encoded Creator predicate.
     *  @since  Version 0.1 2005-05-27
     */
    private static final String CREATOR = "DC.Creator";

    /**
     *  An encoded Subject predicate.
     *  @since  Version 0.1 2005-05-27
     */
    private static final String SUBJECT = "DC.Subject";

    /**
     *  An encoded Description predicate.
     *  @since  Version 0.1 2005-05-27
     */
    private static final String DESCRIPTION = "DC.Description";

    /**
     *  An encoded Publisher predicate.
     *  @since  Version 0.1 2005-05-27
     */
    private static final String PUBLISHER = "DC.Publisher";

    /**
     *  An encoded Contributor predicate.
     *  @since  Version 0.1 2005-05-27
     */
    private static final String CONTRIBUTOR = "DC.Contributor";

    /**
     *  An encoded Date predicate.
     *  @since  Version 0.1 2005-05-27
     */
    private static final String DATE = "DC.Date";

    /**
     *  An encoded Type predicate.
     *  @since  Version 0.1 2005-05-27
     */
    private static final String TYPE = "DC.Type";

    /**
     *  An encoded Format predicate.
     *  @since  Version 0.1 2005-05-27
     */
    private static final String FORMAT = "DC.Format";

    /**
     *  An encoded Identifier predicate.
     *  @since  Version 0.1 2005-05-27
     */
    private static final String IDENTIFIER = "DC.Identifier";

    /**
     *  An encoded Source predicate.
     *  @since  Version 0.1 2005-05-27
     */
    private static final String SOURCE = "DC.Source";

    /**
     *  An encoded Language predicate.
     *  @since  Version 0.1 2005-05-27
     */
    private static final String LANGUAGE = "DC.Language";

    /**
     *  An encoded Relation predicate.
     *  @since  Version 0.1 2005-05-27
     */
    private static final String RELATION = "DC.Relation";

    /**
     *  An encoded Coverage predicate.
     *  @since  Version 0.1 2005-05-27
     */
    private static final String COVERAGE = "DC.Coverage";

    /**
     *  An encoded Rights predicate.
     *  @since  Version 0.1 2005-05-27
     */
    private static final String RIGHTS = "DC.Rights";

    /**
     *  An expected <code>IllegalArgumentException</code> message.
     *  @since  Version 0.4 2005-03-11
     */
    public static final String EXPECT_IAE =
    "Should throw an IllegalArgumentException.";

    /**
     *  A test fallback value.
     *  @since  Version 0.2 2005-01-24
     */
    public static final String FALLBACK = "testFallback";

    /**
     *  A modified scheme value in the original DC in XHTML format.
     *  @since  Version 0.2 2005-01-24
     */
    public static final String W3CDTF_OLD = "W3CDTF";

    /**
     *  Dublin Core encoded Alternative predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_ALTIVE = "DCTERMS.alternative";

    /**
     *  Dublin Core encoded Table Of Contents predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_TABCNT = "DCTERMS.tableOfContents";

    /**
     *  Dublin Core encoded Abstract predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_ABSRCT = "DCTERMS.abstract";

    /**
     *  Dublin Core encoded Created predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_CREATD = "DCTERMS.created";

    /**
     *  Dublin Core encoded Valid predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_VALID = "DCTERMS.valid";

    /**
     *  Dublin Core encoded Available predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_AVAIBL = "DCTERMS.available";

    /**
     *  Dublin Core encoded Issued predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_ISSUED = "DCTERMS.issued";

    /**
     *  Dublin Core encoded Modified predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_MODFID = "DCTERMS.modified";

    /**
     *  Dublin Core encoded Modified predicate, old form.
     *  @since  Version 0.4 2005-03-11
     */
    public static final String MODIFIED_OLD = "DC.Date.modified";

    /**
     *  Dublin Core encoded Extent predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_EXTENT = "DCTERMS.extent";

    /**
     *  Dublin Core encoded Medium predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_MEDIUM = "DCTERMS.medium";

    /**
     *  Dublin Core encoded Is Version Of predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_VERNOF = "DCTERMS.isVersionOf";

    /**
     *  Dublin Core encoded Has Version predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_HASVER = "DCTERMS.hasVersion";

    /**
     *  Dublin Core encoded Is Replaced By predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_REPLBY = "DCTERMS.isReplacedBy";

    /**
     *  Dublin Core encoded Replaces predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_REPLCS = "DCTERMS.replaces";

    /**
     *  Dublin Core encoded Is Required By predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_REQDBY = "DCTERMS.isRequiredBy";

    /**
     *  Dublin Core encoded Requires predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_REQRES = "DCTERMS.requires";

    /**
     *  Dublin Core encoded Is Part Of predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_PARTOF = "DCTERMS.isPartOf";

    /**
     *  Dublin Core encoded Has Part predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_HASPRT = "DCTERMS.hasPart";

    /**
     *  Dublin Core encoded Is Referenced By predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_REFDBY = "DCTERMS.isReferencedBy";

    /**
     *  Dublin Core encoded References predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_REFRCS = "DCTERMS.references";

    /**
     *  Dublin Core encoded Is Format Of predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_FMATOF = "DCTERMS.isFormatOf";

    /**
     *  Dublin Core encoded Has Format predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_HASFMT = "DCTERMS.hasFormat";

    /**
     *  Dublin Core encoded Conforms To predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_CONFTO = "DCTERMS.conformsTo";

    /**
     *  Dublin Core encoded Spatial predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_SPTIAL = "DCTERMS.spatial";

    /**
     *  Dublin Core encoded Temporal predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_TEMPRL = "DCTERMS.temporal";

    /**
     *  Dublin Core encoded Mediator predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_MEDITR = "DCTERMS.mediator";

    /**
     *  Dublin Core encoded Date Accepted predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_ACCPTD = "DCTERMS.dateAccepted";

    /**
     *  Dublin Core encoded Date Copyrighted predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_CPYRGT = "DCTERMS.dateCopyrighted";

    /**
     *  Dublin Core encoded Date Submitted predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_SUBMTD = "DCTERMS.dateSubmitted";

    /**
     *  Dublin Core encoded Education Level predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_EDULVL = "DCTERMS.educationLevel";

    /**
     *  Dublin Core encoded Access Rights predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_ACCRGT = "DCTERMS.accessRights";

    /**
     *  Dublin Core encoded Bibliographic Citation predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_BIBCIT =
    "DCTERMS.bibliographicCitation";

    /**
     *  Dublin Core encoded License predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_LICNSE = "DCTERMS.license";

    /**
     *  Dublin Core encoded Rights Holder predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_RGTHLD = "DCTERMS.rightsHolder";

    /**
     *  Dublin Core encoded Provenance predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_PRVNCE = "DCTERMS.provenance";

    /**
     *  Dublin Core encoded NAME_LCSH predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_LCSH = "DCTERMS.LCSH";

    /**
     *  Dublin Core encoded MeSH predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_MESH = "DCTERMS.MESH";

    /**
     *  Dublin Core encoded DDC predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String DDC = "DCTERMS.DDC";

    /**
     *  Dublin Core encoded NAME_LCC predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_LCC = "DCTERMS.LCC";

    /**
     *  Dublin Core encoded UDC predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String UDC = "DCTERMS.UDC";

    /**
     *  Dublin Core encoded DCMI Type predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_DCTYPE = "DCTERMS.DCMIType";

    /**
     *  Dublin Core encoded NAME_IMT predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_IMT = "DCTERMS.IMT";

    /**
     *  Dublin Core encoded ISO639-2 predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_IS3692 = "DCTERMS.ISO639-2";

    /**
     *  Dublin Core encoded RFC 1766 predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_RC1766 = "DCTERMS.RFC1766";

    /**
     *  Dublin Core encoded URI predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String URI = "DCTERMS.URI";

    /**
     *  Dublin Core encoded DCMI Point predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_POINT = "DCTERMS.Point";

    /**
     *  Dublin Core encoded NAME_IS3166 predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_IS3166 = "DCTERMS.ISO3166";

    /**
     *  Dublin Core encoded DCMI Box predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_BOX = "DCTERMS.Box";

    /**
     *  Dublin Core encoded NAME_TGN predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_TGN = "DCTERMS.TGN";

    /**
     *  Dublin Core encoded DCMI Period predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_PERIOD = "DCTERMS.Period";

    /**
     *  Dublin Core encoded W3C-DTF predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String W3CDTF = "DCTERMS.W3CDTF";

    /**
     *  Dublin Core encoded RFC 3066 predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public static final String NAME_RC3066 = "DCTERMS.RFC3066";

    /**
     *  An instance to test.
     *  @since  Version 0.1 2005-05-27
     */
    protected final DublinCoreProfile profile = new DublinCoreProfile();

    /**
     *  Named test instance.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2005-05-27
     */
    public DublinCoreProfileTest(final String name) {

        super(name);
    }

    /**
     *  Set up a test scenario as required.
     *  @since  Version 0.1 2005-
    protected void setUp() {

    }
     */

    /**
     *  Get a predicate property by name.
     *  @since  Version 0.5 2005-06-17
     */
    public final void testGetPredicatePropertyNull() {

        try {

            profile.getPredicateProperty(null);

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Get a predicate property by name.
     *  @since  Version 0.5 2005-06-17
     */
    public final void testGetPredicatePropertyGeneric() {

        assertNull(profile.getPredicateProperty(GENERIC));
    }

    /**
     *  Get a predicate property by name.
     *  @since  Version 0.5 2005-06-17
     */
    public final void testGetPredicatePropertyBogus() {

        assertNull(profile.getPredicateProperty(BOGUS));
    }

    /**
     *  Get a predicate property by name.
     *  @since  Version 0.5 2005-06-17
     */
    public final void testGetPredicatePropertyUnknown() {

        assertNull(profile.getPredicateProperty(UNKNOWN));
    }

    /**
     *  Get a predicate property by name.
     *  @since  Version 0.5 2005-06-17
     */
    public final void testGetPredicatePropertySuccess() {

        SchemaProperty property = profile.getPredicateProperty(TITLE);

        String expected = "dc.title";

        assertTrue(expected.equals(property.getParameterName()));
    }

    /**
     *  Get the primary search property.
     *  @since  Version 0.5 2005-06-17
     */
    public final void testGetPrimarySearchProperty() {

        SchemaProperty property = profile.getPrimarySearchProperty();

        assertNotNull(property);

        String expected = "dc.subject";

        assertTrue(expected.equals(property.getParameterName()));
    }

    /**
     *  Get the title property.
     *  @since  Version 0.6 2005-07-25
     */
    public final void testGetTitleProperty() {

        SchemaProperty property = profile.getTitleProperty();

        assertNotNull(property);

        String expected = "dc.title";

        assertTrue(expected.equals(property.getParameterName()));
    }

    /**
     *  Get the title property.
     *  @since  Version 0.6 2005-07-25
     */
    public final void testGetSummaryProperty() {

        SchemaProperty property = profile.getSummaryProperty();

        assertNotNull(property);

        String expected = "dc.description";

        assertTrue(expected.equals(property.getParameterName()));
    }


    /**
     *  Get the schema map.
     *  @since  Version 0.3 2005-06-10
     */
    public final void testGetSchemaMap() {

        Map schemas = profile.getSchemaMap();

        assertTrue(schemas.size() == 2);

        assertTrue(schemas.containsKey("dc"));
        assertTrue(schemas.containsKey("dcterms"));
    }

    /**
     *  Test the expand predicate template method.
     *  @since  Version 0.2 2005-05-31
     */
    public final void testExpandPredicateProfileNull() {

        try {

            profile.expandPredicateProfile(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Test the expand scheme template method.
     *  @since  Version 0.2 2005-05-31
     */
    public final void testExpandSchemeProfileNull() {

        try {

            profile.expandSchemeProfile(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Test expand scheme, old form.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandScheme() {

        String expanded = profile.expandScheme(SCHEME);

        assertTrue("Predicate should not be expanded to, '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_W3CDTF + "', " +
                   "actually returned: '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_W3CDTF));
    }

    /**
     *  Test expand an old format Dublin Core qualified predicate.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandDCOldForm() {

        String expanded = profile.expandPredicate(OLD_FORM);

        assertTrue("Predicate should be expanded to, '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_MODFID + "', " +
                   "actually returned: '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_MODFID));
    }

    /**
     *  Test expand a new format Dublin Core qualified predicate.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandDCNewForm() {

        String expanded = profile.expandPredicate(NAME_AUDNCE);

        assertTrue("Predicate should be expanded to, '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_AUDNCE + "', " +
                   "actually returned: '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_AUDNCE));
    }

    /**
     *  Test expand an audience predicate, lower case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandPredicateAudienceLowerCase() {

        String expanded = profile.expandPredicate(NAME_AUDNCE.toLowerCase());

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_AUDNCE + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_AUDNCE));
    }

    /**
     *  Test expand an audience predicate, upper case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandPredicateAudienceUpperCase() {

        String expanded = profile.expandPredicate(NAME_AUDNCE.toUpperCase());

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_AUDNCE + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_AUDNCE));
    }

    /**
     *  Test expand null predicate.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandDualNullPredicate() {

        try {

            profile.expandPredicate(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Test expand scheme, null scheme.
     *  @since  Version 0.2 2004-12-23
     */
    public final void testExpandSchemeNull() {

        try {

            profile.expandScheme(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Test expand generic predicate.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandGenericPredicate() {


        String expanded = profile.expandPredicate(GENERIC);

        assertTrue("Predicate should not be expanded, " +
                   "actually returned: '" + expanded + "'.",
                   expanded.equals(GENERIC));
    }

    /**
     *  Test expand a bogus Dublin Core predicate.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandDCBogusPredicate() {


        String expanded = profile.expandPredicate(BOGUS);

        assertTrue("Predicate should not be expanded, " +
                   "actually returned: '" + expanded + "'.",
                   expanded.equals(BOGUS));
    }

    /**
     *  Test expand a predicate with unknown schema prefix.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandPredicateSchemaUnknown() {


        String expanded = profile.expandPredicate(UNKNOWN);

        assertTrue("Predicate should not be expanded, " +
                   "actually returned: '" + expanded + "'.",
                   expanded.equals(UNKNOWN));
    }

    /**
     *  Test expand a scheme with unknown schema prefix.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandSchemeSchemaUnknown() {


        String expanded = profile.expandScheme(UNKNOWN);

        assertTrue("Scheme should not be expanded, " +
                   "actually returned: '" + expanded + "'.",
                   expanded.equals(UNKNOWN));
    }

    /**
     *  Test add a schema, null.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testAddSchemaNull() {

        try {

            profile.addSchema(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test expand an e-GMS predicate.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandUnmatchedPredicate() {


        String expanded = profile.expandPredicate(MANDATE);

        assertTrue("Predicate should not be expanded, " +
                   "actually returned: '" + expanded + "'.",
                   expanded.equals(MANDATE));
    }

    /**
     *  Test expand Title, mixed case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandTitleMixedCase() {


        String expanded = profile.expandPredicate(TITLE);

        assertTrue("Encoded predicate should expand to " +
                   DublinCoreElements.BASE + DublinCoreElements.NAME_TITLE +
                   ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(DublinCoreElements.BASE +
                                   DublinCoreElements.NAME_TITLE));
    }

    /**
     *  Test expand Title, lower case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandTitleLowerCase() {


        String expanded = profile.expandPredicate(TITLE.toLowerCase());

        assertTrue("Encoded predicate should expand to " +
                   DublinCoreElements.BASE + DublinCoreElements.NAME_TITLE +
                   ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(DublinCoreElements.BASE +
                                   DublinCoreElements.NAME_TITLE));
    }

    /**
     *  Test expand Title, upper case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandTitleUpperCase() {


        String expanded = profile.expandPredicate(TITLE.toUpperCase());

        assertTrue("Encoded predicate should expand to " +
                   DublinCoreElements.BASE + DublinCoreElements.NAME_TITLE +
                   ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(DublinCoreElements.BASE +
                                   DublinCoreElements.NAME_TITLE));
    }

    /**
     *  Test expand Creator, mixed case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandCreatorMixedCase() {


        String expanded = profile.expandPredicate(CREATOR);

        assertTrue("Encoded predicate should expand to " +
                   DublinCoreElements.BASE + DublinCoreElements.NAME_CREATR +
                   ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(DublinCoreElements.BASE +
                                   DublinCoreElements.NAME_CREATR));
    }

    /**
     *  Test expand Creator, lower case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandCreatorLowerCase() {


        String expanded = profile.expandPredicate(CREATOR.toLowerCase());

        assertTrue("Encoded predicate should expand to " +
                   DublinCoreElements.BASE + DublinCoreElements.NAME_CREATR +
                   ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(DublinCoreElements.BASE +
                                   DublinCoreElements.NAME_CREATR));
    }

    /**
     *  Test expand Creator, upper case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandCreatorUpperCase() {


        String expanded = profile.expandPredicate(CREATOR.toUpperCase());

        assertTrue("Encoded predicate should expand to " +
                   DublinCoreElements.BASE + DublinCoreElements.NAME_CREATR +
                   ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(DublinCoreElements.BASE +
                                   DublinCoreElements.NAME_CREATR));
    }

    /**
     *  Test expand Subject, mixed case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandSubjectMixedCase() {


        String expanded = profile.expandPredicate(SUBJECT);

        assertTrue("Encoded predicate should expand to " +
                   DublinCoreElements.BASE + DublinCoreElements.NAME_SUBJCT +
                   ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(DublinCoreElements.BASE +
                                   DublinCoreElements.NAME_SUBJCT));
    }

    /**
     *  Test expand Subject, lower case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandSubjectLowerCase() {


        String expanded = profile.expandPredicate(SUBJECT.toLowerCase());

        assertTrue("Encoded predicate should expand to " +
                   DublinCoreElements.BASE + DublinCoreElements.NAME_SUBJCT +
                   ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(DublinCoreElements.BASE +
                                   DublinCoreElements.NAME_SUBJCT));
    }

    /**
     *  Test expand Subject, upper case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandSubjectUpperCase() {


        String expanded = profile.expandPredicate(SUBJECT.toUpperCase());

        assertTrue("Encoded predicate should expand to " +
                   DublinCoreElements.BASE + DublinCoreElements.NAME_SUBJCT +
                   ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(DublinCoreElements.BASE +
                                   DublinCoreElements.NAME_SUBJCT));
    }

    /**
     *  Test expand Description, mixed case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandDescriptionMixedCase() {


        String expanded = profile.expandPredicate(DESCRIPTION);

        assertTrue("Encoded predicate should expand to " +
                   DublinCoreElements.BASE + DublinCoreElements.NAME_DESCRN +
                   ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(DublinCoreElements.BASE +
                                   DublinCoreElements.NAME_DESCRN));
    }

    /**
     *  Test expand Description, lower case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandDescriptionLowerCase() {


        String expanded = profile.expandPredicate(DESCRIPTION.toLowerCase());

        assertTrue("Encoded predicate should expand to " +
                   DublinCoreElements.BASE + DublinCoreElements.NAME_DESCRN +
                   ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(DublinCoreElements.BASE +
                                   DublinCoreElements.NAME_DESCRN));
    }

    /**
     *  Test expand Description, upper case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandDescriptionUpperCase() {


        String expanded = profile.expandPredicate(DESCRIPTION.toUpperCase());

        assertTrue("Encoded predicate should expand to " +
                   DublinCoreElements.BASE + DublinCoreElements.NAME_DESCRN +
                   ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(DublinCoreElements.BASE +
                                   DublinCoreElements.NAME_DESCRN));
    }

    /**
     *  Test expand Publisher, mixed case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandPublisherMixedCase() {


        String expanded = profile.expandPredicate(PUBLISHER);

        assertTrue("Encoded predicate should expand to " +
                   DublinCoreElements.BASE + DublinCoreElements.NAME_PUBSHR +
                   ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(DublinCoreElements.BASE +
                                   DublinCoreElements.NAME_PUBSHR));
    }

    /**
     *  Test expand Publisher, lower case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandPublisherLowerCase() {


        String expanded = profile.expandPredicate(PUBLISHER.toLowerCase());

        assertTrue("Encoded predicate should expand to " +
                   DublinCoreElements.BASE + DublinCoreElements.NAME_PUBSHR +
                   ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(DublinCoreElements.BASE +
                                   DublinCoreElements.NAME_PUBSHR));
    }

    /**
     *  Test expand Publisher, upper case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandPublisherUpperCase() {


        String expanded = profile.expandPredicate(PUBLISHER.toUpperCase());

        assertTrue("Encoded predicate should expand to " +
                   DublinCoreElements.BASE + DublinCoreElements.NAME_PUBSHR +
                   ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(DublinCoreElements.BASE +
                                   DublinCoreElements.NAME_PUBSHR));
    }

    /**
     *  Test expand Contributor, mixed case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandContributorMixedCase() {


        String expanded = profile.expandPredicate(CONTRIBUTOR);

        assertTrue("Encoded predicate should expand to " +
                   DublinCoreElements.BASE + DublinCoreElements.NAME_CONTRB +
                   ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(DublinCoreElements.BASE +
                                   DublinCoreElements.NAME_CONTRB));
    }

    /**
     *  Test expand Contributor, lower case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandContributorLowerCase() {


        String expanded = profile.expandPredicate(CONTRIBUTOR.toLowerCase());

        assertTrue("Encoded predicate should expand to " +
                   DublinCoreElements.BASE + DublinCoreElements.NAME_CONTRB +
                   ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(DublinCoreElements.BASE +
                                   DublinCoreElements.NAME_CONTRB));
    }

    /**
     *  Test expand Contributor, upper case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandContributorUpperCase() {


        String expanded = profile.expandPredicate(CONTRIBUTOR.toUpperCase());

        assertTrue("Encoded predicate should expand to " +
                   DublinCoreElements.BASE + DublinCoreElements.NAME_CONTRB +
                   ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(DublinCoreElements.BASE +
                                   DublinCoreElements.NAME_CONTRB));
    }

    /**
     *  Test expand Date, mixed case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandDateMixedCase() {


        String expanded = profile.expandPredicate(DATE);

        assertTrue("Encoded predicate should expand to " +
                   DublinCoreElements.BASE + DublinCoreElements.NAME_DATE +
                   ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(DublinCoreElements.BASE +
                                   DublinCoreElements.NAME_DATE));
    }

    /**
     *  Test expand Date, lower case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandDateLowerCase() {


        String expanded = profile.expandPredicate(DATE.toLowerCase());

        assertTrue("Encoded predicate should expand to " +
                   DublinCoreElements.BASE + DublinCoreElements.NAME_DATE +
                   ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(DublinCoreElements.BASE +
                                   DublinCoreElements.NAME_DATE));
    }

    /**
     *  Test expand Date, upper case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandDateUpperCase() {


        String expanded = profile.expandPredicate(DATE.toUpperCase());

        assertTrue("Encoded predicate should expand to " +
                   DublinCoreElements.BASE + DublinCoreElements.NAME_DATE +
                   ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(DublinCoreElements.BASE +
                                   DublinCoreElements.NAME_DATE));
    }

    /**
     *  Test expand Type, mixed case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandTypeMixedCase() {


        String expanded = profile.expandPredicate(TYPE);

        assertTrue("Encoded predicate should expand to " +
                   DublinCoreElements.BASE + DublinCoreElements.NAME_TYPE +
                   ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(DublinCoreElements.BASE +
                                   DublinCoreElements.NAME_TYPE));
    }

    /**
     *  Test expand Type, lower case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandTypeLowerCase() {


        String expanded = profile.expandPredicate(TYPE.toLowerCase());

        assertTrue("Encoded predicate should expand to " +
                   DublinCoreElements.BASE + DublinCoreElements.NAME_TYPE +
                   ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(DublinCoreElements.BASE +
                                   DublinCoreElements.NAME_TYPE));
    }

    /**
     *  Test expand Type, upper case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandTypeUpperCase() {


        String expanded = profile.expandPredicate(TYPE.toUpperCase());

        assertTrue("Encoded predicate should expand to " +
                   DublinCoreElements.BASE + DublinCoreElements.NAME_TYPE +
                   ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(DublinCoreElements.BASE +
                                   DublinCoreElements.NAME_TYPE));
    }

    /**
     *  Test expand Format, mixed case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandFormatMixedCase() {


        String expanded = profile.expandPredicate(FORMAT);

        assertTrue("Encoded predicate should expand to " +
                   DublinCoreElements.BASE + DublinCoreElements.NAME_FORMAT +
                   ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(DublinCoreElements.BASE +
                                   DublinCoreElements.NAME_FORMAT));
    }

    /**
     *  Test expand Format, lower case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandFormatLowerCase() {


        String expanded = profile.expandPredicate(FORMAT.toLowerCase());

        assertTrue("Encoded predicate should expand to " +
                   DublinCoreElements.BASE + DublinCoreElements.NAME_FORMAT +
                   ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(DublinCoreElements.BASE +
                                   DublinCoreElements.NAME_FORMAT));
    }

    /**
     *  Test expand Format, upper case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandFormatUpperCase() {


        String expanded = profile.expandPredicate(FORMAT.toUpperCase());

        assertTrue("Encoded predicate should expand to " +
                   DublinCoreElements.BASE + DublinCoreElements.NAME_FORMAT +
                   ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(DublinCoreElements.BASE +
                                   DublinCoreElements.NAME_FORMAT));
    }

    /**
     *  Test expand Identifier, mixed case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandIdentifierMixedCase() {


        String expanded = profile.expandPredicate(IDENTIFIER);

        assertTrue("Encoded predicate should expand to " +
                   DublinCoreElements.BASE + DublinCoreElements.NAME_IDNTFR +
                   ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(DublinCoreElements.BASE +
                                   DublinCoreElements.NAME_IDNTFR));
    }

    /**
     *  Test expand Identifier, lower case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandIdentifierLowerCase() {


        String expanded = profile.expandPredicate(IDENTIFIER.toLowerCase());

        assertTrue("Encoded predicate should expand to " +
                   DublinCoreElements.BASE + DublinCoreElements.NAME_IDNTFR +
                   ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(DublinCoreElements.BASE +
                                   DublinCoreElements.NAME_IDNTFR));
    }

    /**
     *  Test expand Identifier, upper case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandIdentifierUpperCase() {


        String expanded = profile.expandPredicate(IDENTIFIER.toUpperCase());

        assertTrue("Encoded predicate should expand to " +
                   DublinCoreElements.BASE + DublinCoreElements.NAME_IDNTFR +
                   ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(DublinCoreElements.BASE +
                                   DublinCoreElements.NAME_IDNTFR));
    }

    /**
     *  Test expand Source, mixed case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandSourceMixedCase() {


        String expanded = profile.expandPredicate(SOURCE);

        assertTrue("Encoded predicate should expand to " +
                   DublinCoreElements.BASE + DublinCoreElements.NAME_SOURCE +
                   ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(DublinCoreElements.BASE +
                                   DublinCoreElements.NAME_SOURCE));
    }

    /**
     *  Test expand Source, lower case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandSourceLowerCase() {


        String expanded = profile.expandPredicate(SOURCE.toLowerCase());

        assertTrue("Encoded predicate should expand to " +
                   DublinCoreElements.BASE + DublinCoreElements.NAME_SOURCE +
                   ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(DublinCoreElements.BASE +
                                   DublinCoreElements.NAME_SOURCE));
    }

    /**
     *  Test expand Source, upper case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandSourceUpperCase() {


        String expanded = profile.expandPredicate(SOURCE.toUpperCase());

        assertTrue("Encoded predicate should expand to " +
                   DublinCoreElements.BASE + DublinCoreElements.NAME_SOURCE +
                   ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(DublinCoreElements.BASE +
                                   DublinCoreElements.NAME_SOURCE));
    }

    /**
     *  Test expand Language, mixed case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandLanguageMixedCase() {


        String expanded = profile.expandPredicate(LANGUAGE);

        assertTrue("Encoded predicate should expand to " +
                   DublinCoreElements.BASE + DublinCoreElements.NAME_LNGAGE +
                   ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(DublinCoreElements.BASE +
                                   DublinCoreElements.NAME_LNGAGE));
    }

    /**
     *  Test expand Language, lower case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandLanguageLowerCase() {


        String expanded = profile.expandPredicate(LANGUAGE.toLowerCase());

        assertTrue("Encoded predicate should expand to " +
                   DublinCoreElements.BASE + DublinCoreElements.NAME_LNGAGE +
                   ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(DublinCoreElements.BASE +
                                   DublinCoreElements.NAME_LNGAGE));
    }

    /**
     *  Test expand Language, upper case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandLanguageUpperCase() {


        String expanded = profile.expandPredicate(LANGUAGE.toUpperCase());

        assertTrue("Encoded predicate should expand to " +
                   DublinCoreElements.BASE + DublinCoreElements.NAME_LNGAGE +
                   ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(DublinCoreElements.BASE +
                                   DublinCoreElements.NAME_LNGAGE));
    }

    /**
     *  Test expand Relation, mixed case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandRelationMixedCase() {


        String expanded = profile.expandPredicate(RELATION);

        assertTrue("Encoded predicate should expand to " +
                   DublinCoreElements.BASE + DublinCoreElements.NAME_RELATN +
                   ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(DublinCoreElements.BASE +
                                   DublinCoreElements.NAME_RELATN));
    }

    /**
     *  Test expand Relation, lower case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandRelationLowerCase() {


        String expanded = profile.expandPredicate(RELATION.toLowerCase());

        assertTrue("Encoded predicate should expand to " +
                   DublinCoreElements.BASE + DublinCoreElements.NAME_RELATN +
                   ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(DublinCoreElements.BASE +
                                   DublinCoreElements.NAME_RELATN));
    }

    /**
     *  Test expand Relation, upper case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandRelationUpperCase() {


        String expanded = profile.expandPredicate(RELATION.toUpperCase());

        assertTrue("Encoded predicate should expand to " +
                   DublinCoreElements.BASE + DublinCoreElements.NAME_RELATN +
                   ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(DublinCoreElements.BASE +
                                   DublinCoreElements.NAME_RELATN));
    }

    /**
     *  Test expand Coverage, mixed case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandCoverageMixedCase() {


        String expanded = profile.expandPredicate(COVERAGE);

        assertTrue("Encoded predicate should expand to " +
                   DublinCoreElements.BASE + DublinCoreElements.NAME_COVRGE +
                   ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(DublinCoreElements.BASE +
                                   DublinCoreElements.NAME_COVRGE));
    }

    /**
     *  Test expand Coverage, lower case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandCoverageLowerCase() {


        String expanded = profile.expandPredicate(COVERAGE.toLowerCase());

        assertTrue("Encoded predicate should expand to " +
                   DublinCoreElements.BASE + DublinCoreElements.NAME_COVRGE +
                   ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(DublinCoreElements.BASE +
                                   DublinCoreElements.NAME_COVRGE));
    }

    /**
     *  Test expand Coverage, upper case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandCoverageUpperCase() {


        String expanded = profile.expandPredicate(COVERAGE.toUpperCase());

        assertTrue("Encoded predicate should expand to " +
                   DublinCoreElements.BASE + DublinCoreElements.NAME_COVRGE +
                   ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(DublinCoreElements.BASE +
                                   DublinCoreElements.NAME_COVRGE));
    }

    /**
     *  Test expand Rights, mixed case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandRightsMixedCase() {


        String expanded = profile.expandPredicate(RIGHTS);

        assertTrue("Encoded predicate should expand to " +
                   DublinCoreElements.BASE + DublinCoreElements.NAME_RIGHTS +
                   ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(DublinCoreElements.BASE +
                                   DublinCoreElements.NAME_RIGHTS));
    }

    /**
     *  Test expand Rights, lower case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandRightsLowerCase() {


        String expanded = profile.expandPredicate(RIGHTS.toLowerCase());

        assertTrue("Encoded predicate should expand to " +
                   DublinCoreElements.BASE + DublinCoreElements.NAME_RIGHTS +
                   ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(DublinCoreElements.BASE +
                                   DublinCoreElements.NAME_RIGHTS));
    }

    /**
     *  Test expand Rights, upper case.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testExpandRightsUpperCase() {


        String expanded = profile.expandPredicate(RIGHTS.toUpperCase());

        assertTrue("Encoded predicate should expand to " +
                   DublinCoreElements.BASE + DublinCoreElements.NAME_RIGHTS +
                   ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(DublinCoreElements.BASE +
                                   DublinCoreElements.NAME_RIGHTS));
    }

    /**
     *  Test expand a generic scheme.
     *  @since  Version 0.2 2005-01-24
     */
    public final void testExpandSchemeGeneric() {

        String expanded = profile.expandScheme(GENERIC);

        assertTrue("Generic scheme should not be expanded, '" +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(GENERIC));

    }

    /**
     *  Test expand a scheme using the original DC in XHTML format.
     *  @since  Version 0.2 2005-01-24
     */
    public final void testExpandSchemeOriginal() {

        String expanded = profile.expandScheme(W3CDTF_OLD);

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_W3CDTF + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_W3CDTF));
    }

    /**
     *  Test expand an audience predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateAudience() {

        String expanded = profile.expandPredicate(NAME_AUDNCE);

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_AUDNCE + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_AUDNCE));
    }

    /**
     *  Test expand an alternative predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateAlternative() {

        String expanded = profile.expandPredicate(NAME_ALTIVE);

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_ALTIVE + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_ALTIVE));
    }

    /**
     *  Test expand an alternative predicate, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateAlternativeLowerCase() {

        String expanded = profile.expandPredicate(NAME_ALTIVE.toLowerCase());

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_ALTIVE + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_ALTIVE));
    }

    /**
     *  Test expand an alternative predicate, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateAlternativeUpperCase() {

        String expanded = profile.expandPredicate(NAME_ALTIVE.toUpperCase());

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_ALTIVE + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_ALTIVE));
    }

    /**
     *  Test expand a table of contents predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateTableOfContents() {

        String expanded = profile.expandPredicate(NAME_TABCNT);

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_TABCNT + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_TABCNT));
    }

    /**
     *  Test expand a table of contents predicate, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateTableOfContentsLowerCase() {

        String expanded = profile.expandPredicate(
                                    NAME_TABCNT.toLowerCase());

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_TABCNT + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_TABCNT));
    }

    /**
     *  Test expand a table of contents predicate, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateTableOfContentsUpperCase() {

        String expanded = profile.expandPredicate(
                                    NAME_TABCNT.toUpperCase());

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_TABCNT + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_TABCNT));
    }

    /**
     *  Test expand an abstract predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateAbstract() {

        String expanded = profile.expandPredicate(NAME_ABSRCT);

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_ABSRCT + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_ABSRCT));
    }

    /**
     *  Test expand an abstract predicate, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateAbstractLowerCase() {

        String expanded = profile.expandPredicate(NAME_ABSRCT.toLowerCase());

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_ABSRCT + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_ABSRCT));
    }

    /**
     *  Test expand an abstract predicate, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateAbstractUpperCase() {

        String expanded = profile.expandPredicate(NAME_ABSRCT.toUpperCase());

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_ABSRCT + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_ABSRCT));
    }

    /**
     *  Test expand an created predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateCreated() {

        String expanded = profile.expandPredicate(NAME_CREATD);

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_CREATD + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_CREATD));
    }

    /**
     *  Test expand an created predicate, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateCreatedLowerCase() {

        String expanded = profile.expandPredicate(NAME_CREATD.toLowerCase());

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_CREATD + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_CREATD));
    }

    /**
     *  Test expand an created predicate, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateCreatedUpperCase() {

        String expanded = profile.expandPredicate(NAME_CREATD.toUpperCase());

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_CREATD + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_CREATD));
    }

    /**
     *  Test expand an valid predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateValid() {

        String expanded = profile.expandPredicate(NAME_VALID);

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_VALID + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_VALID));
    }

    /**
     *  Test expand an valid predicate, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateValidLowerCase() {

        String expanded = profile.expandPredicate(NAME_VALID.toLowerCase());

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_VALID + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_VALID));
    }

    /**
     *  Test expand an valid predicate, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateValidUpperCase() {

        String expanded = profile.expandPredicate(NAME_VALID.toUpperCase());

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_VALID + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_VALID));
    }

    /**
     *  Test expand an available predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateAvailable() {

        String expanded = profile.expandPredicate(NAME_AVAIBL);

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_AVAIBL + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_AVAIBL));
    }

    /**
     *  Test expand an available predicate, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateAvailableLowerCase() {

        String expanded = profile.expandPredicate(NAME_AVAIBL.toLowerCase());

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_AVAIBL + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_AVAIBL));
    }

    /**
     *  Test expand an available predicate, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateAvailableUpperCase() {

        String expanded = profile.expandPredicate(NAME_AVAIBL.toUpperCase());

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_AVAIBL + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_AVAIBL));
    }

    /**
     *  Test expand an issued predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateIssued() {

        String expanded = profile.expandPredicate(NAME_ISSUED);

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_ISSUED + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_ISSUED));
    }

    /**
     *  Test expand an issued predicate, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateIssuedLowerCase() {

        String expanded = profile.expandPredicate(NAME_ISSUED.toLowerCase());

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_ISSUED + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_ISSUED));
    }

    /**
     *  Test expand an issued predicate, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateIssuedUpperCase() {

        String expanded = profile.expandPredicate(NAME_ISSUED.toUpperCase());

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_ISSUED + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_ISSUED));
    }

    /**
     *  Test expand a modified date.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateModified() {

        String expanded = profile.expandPredicate(NAME_MODFID);

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_MODFID + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_MODFID));
    }

    /**
     *  Test expand a modified date, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateModifiedLowerCase() {

        String expanded = profile.expandPredicate(NAME_MODFID.toLowerCase());

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_MODFID + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_MODFID));
    }

    /**
     *  Test expand a modified date, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateModifiedUpperCase() {

        String expanded = profile.expandPredicate(NAME_MODFID.toUpperCase());

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_MODFID + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_MODFID));
    }


    /**
     *  Test expand a modified date, old form.
     *  @since  Version 0.4 2005-03-11
     */
    public final void testExpandPredicateModifiedOldForm() {

        String expanded = profile.expandPredicate(MODIFIED_OLD);

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_MODFID + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_MODFID));
    }

    /**
     *  Test expand a modified date, lower case, old form.
     *  @since  Version 0.4 2005-03-11
     */
    public final void testExpandPredicateModifiedLowerCaseOldForm() {

        String expanded = profile.expandPredicate(MODIFIED_OLD.toLowerCase());

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_MODFID + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_MODFID));
    }

    /**
     *  Test expand a modified date, upper case, old form.
     *  @since  Version 0.4 2005-03-11
     */
    public final void testExpandPredicateModifiedUpperCaseOldForm() {

        String expanded = profile.expandPredicate(MODIFIED_OLD.toUpperCase());

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_MODFID + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_MODFID));
    }

    /**
     *  Test expand an extent predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateExtent() {

        String expanded = profile.expandPredicate(NAME_EXTENT);

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_EXTENT + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_EXTENT));
    }

    /**
     *  Test expand an extent predicate, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateExtentLowerCase() {

        String expanded = profile.expandPredicate(NAME_EXTENT.toLowerCase());

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_EXTENT + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_EXTENT));
    }

    /**
     *  Test expand an extent predicate, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateExtentUpperCase() {

        String expanded = profile.expandPredicate(NAME_EXTENT.toUpperCase());

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_EXTENT + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_EXTENT));
    }

    /**
     *  Test expand a medium predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateMedium() {

        String expanded = profile.expandPredicate(NAME_MEDIUM);

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_MEDIUM + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_MEDIUM));
    }

    /**
     *  Test expand a medium predicate, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateMediumLowerCase() {

        String expanded = profile.expandPredicate(NAME_MEDIUM.toLowerCase());

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_MEDIUM + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_MEDIUM));
    }

    /**
     *  Test expand a medium predicate, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateMediumUpperCase() {

        String expanded = profile.expandPredicate(NAME_MEDIUM.toUpperCase());

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_MEDIUM + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_MEDIUM));
    }

    /**
     *  Test expand an is version of predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateIsVersionOf() {

        String expanded = profile.expandPredicate(NAME_VERNOF);

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_VERNOF + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_VERNOF));
    }

    /**
     *  Test expand an is version of predicate, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateIsVersionOfLowerCase() {

        String expanded = profile.expandPredicate(NAME_VERNOF.toLowerCase());

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_VERNOF + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_VERNOF));
    }

    /**
     *  Test expand an is version of predicate, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateIsVersionOfUpperCase() {

        String expanded = profile.expandPredicate(NAME_VERNOF.toUpperCase());

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_VERNOF + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_VERNOF));
    }

    /**
     *  Test expand a has version predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateHasVersion() {

        String expanded = profile.expandPredicate(NAME_HASVER);

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_HASVER + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_HASVER));
    }

    /**
     *  Test expand a has version predicate, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateHasVersionLowerCase() {

        String expanded = profile.expandPredicate(NAME_HASVER.toLowerCase());

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_HASVER + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_HASVER));
    }

    /**
     *  Test expand a has version predicate, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateHasVersionUpperCase() {

        String expanded = profile.expandPredicate(NAME_HASVER.toUpperCase());

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_HASVER + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_HASVER));
    }

    /**
     *  Test expand an is replaced by predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateIsReplacedBy() {

        String expanded = profile.expandPredicate(NAME_REPLBY);

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_REPLBY + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_REPLBY));
    }

    /**
     *  Test expand an is replaced by predicate, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateIsReplacedByLowerCase() {

        String expanded = profile.expandPredicate(NAME_REPLBY.toLowerCase());

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_REPLBY + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_REPLBY));
    }

    /**
     *  Test expand an is replaced by predicate, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateIsReplacedByUpperCase() {

        String expanded = profile.expandPredicate(NAME_REPLBY.toUpperCase());

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_REPLBY + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_REPLBY));
    }


    /**
     *  Test expand a replaces predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateReplaces() {

        String expanded = profile.expandPredicate(NAME_REPLCS);

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_REPLCS + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_REPLCS));
    }

    /**
     *  Test expand a replaces predicate, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateReplacesLowerCase() {

        String expanded = profile.expandPredicate(NAME_REPLCS.toLowerCase());

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_REPLCS + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_REPLCS));
    }

    /**
     *  Test expand a replaces predicate, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateReplacesUpperCase() {

        String expanded = profile.expandPredicate(NAME_REPLCS.toUpperCase());

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_REPLCS + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_REPLCS));
    }

    /**
     *  Test expand an is required by predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateIsRequiredBy() {

        String expanded = profile.expandPredicate(NAME_REQDBY);

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_REQDBY + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_REQDBY));
    }

    /**
     *  Test expand an is required by predicate, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateIsRequiredByLowerCase() {

        String expanded = profile.expandPredicate(NAME_REQDBY.toLowerCase());

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_REQDBY + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_REQDBY));
    }

    /**
     *  Test expand an is required by predicate, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateIsRequiredByUpperCase() {

        String expanded = profile.expandPredicate(NAME_REQDBY.toUpperCase());

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_REQDBY + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_REQDBY));
    }

    /**
     *  Test expand a requires predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateRequires() {

        String expanded = profile.expandPredicate(NAME_REQRES);

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_REQRES + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_REQRES));
    }

    /**
     *  Test expand a requires predicate, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateRequiresLowerCase() {

        String expanded = profile.expandPredicate(NAME_REQRES.toLowerCase());

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_REQRES + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_REQRES));
    }

    /**
     *  Test expand a requires predicate, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateRequiresUpperCase() {

        String expanded = profile.expandPredicate(NAME_REQRES.toUpperCase());

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_REQRES + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_REQRES));
    }

    /**
     *  Test expand an is part of predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateIsPartOf() {

        String expanded = profile.expandPredicate(NAME_PARTOF);

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_PARTOF + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_PARTOF));
    }

    /**
     *  Test expand an is part of predicate, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateIsPartOfLowerCase() {

        String expanded = profile.expandPredicate(NAME_PARTOF.toLowerCase());

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_PARTOF + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_PARTOF));
    }

    /**
     *  Test expand an is part of predicate, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateIsPartOfUpperCase() {

        String expanded = profile.expandPredicate(NAME_PARTOF.toUpperCase());

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_PARTOF + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_PARTOF));
    }

    /**
     *  Test expand a has part predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateHasPart() {

        String expanded = profile.expandPredicate(NAME_HASPRT);

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_HASPRT + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_HASPRT));
    }

    /**
     *  Test expand a has part predicate, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateHasPartLowerCase() {

        String expanded = profile.expandPredicate(NAME_HASPRT.toLowerCase());

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_HASPRT + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_HASPRT));
    }

    /**
     *  Test expand a has part predicate, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateHasPartUpperCase() {

        String expanded = profile.expandPredicate(NAME_HASPRT.toUpperCase());

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_HASPRT + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_HASPRT));
    }

    /**
     *  Test expand an is referenced by predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateIsReferencedBy() {

        String expanded = profile.expandPredicate(NAME_REFDBY);

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_REFDBY + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_REFDBY));
    }

    /**
     *  Test expand an is referenced by predicate, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateIsReferencedByLowerCase() {

        String expanded = profile.expandPredicate(NAME_REFDBY.toLowerCase());

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_REFDBY + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_REFDBY));
    }

    /**
     *  Test expand an is referenced by predicate, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateIsReferencedByUpperCase() {

        String expanded = profile.expandPredicate(NAME_REFDBY.toUpperCase());

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_REFDBY + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_REFDBY));
    }

    /**
     *  Test expand a references predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateReferences() {

        String expanded = profile.expandPredicate(NAME_REFRCS);

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_REFRCS + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_REFRCS));
    }

    /**
     *  Test expand a references predicate, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateReferencesLowerCase() {

        String expanded = profile.expandPredicate(NAME_REFRCS.toLowerCase());

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_REFRCS + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_REFRCS));
    }

    /**
     *  Test expand a references predicate, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateReferencesUpperCase() {

        String expanded = profile.expandPredicate(NAME_REFRCS.toUpperCase());

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_REFRCS + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_REFRCS));
    }

    /**
     *  Test expand an is format of predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateIsFormatOf() {

        String expanded = profile.expandPredicate(NAME_FMATOF);

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_FMATOF + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_FMATOF));
    }

    /**
     *  Test expand an is format of predicate, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateIsFormatOfLowerCase() {

        String expanded = profile.expandPredicate(NAME_FMATOF.toLowerCase());

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_FMATOF + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_FMATOF));
    }

    /**
     *  Test expand an is format of predicate, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateIsFormatOfUpperCase() {

        String expanded = profile.expandPredicate(NAME_FMATOF.toUpperCase());

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_FMATOF + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_FMATOF));
    }

    /**
     *  Test expand a has format predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateHasFormat() {

        String expanded = profile.expandPredicate(NAME_HASFMT);

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_HASFMT + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_HASFMT));
    }

    /**
     *  Test expand a has format predicate, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateHasFormatLowerCase() {

        String expanded = profile.expandPredicate(NAME_HASFMT.toLowerCase());

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_HASFMT + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_HASFMT));
    }

    /**
     *  Test expand a has format predicate, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateHasFormatUpperCase() {

        String expanded = profile.expandPredicate(NAME_HASFMT.toUpperCase());

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_HASFMT + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_HASFMT));
    }

    /**
     *  Test expand a conforms to predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateConformsTo() {

        String expanded = profile.expandPredicate(NAME_CONFTO);

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_CONFTO + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_CONFTO));
    }

    /**
     *  Test expand a conforms to predicate, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateConformsToLowerCase() {

        String expanded = profile.expandPredicate(NAME_CONFTO.toLowerCase());

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_CONFTO + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_CONFTO));
    }

    /**
     *  Test expand a conforms to predicate, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateConformsToUpperCase() {

        String expanded = profile.expandPredicate(NAME_CONFTO.toUpperCase());

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_CONFTO + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_CONFTO));
    }

    /**
     *  Test expand a spatial predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateSpatial() {

        String expanded = profile.expandPredicate(NAME_SPTIAL);

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_SPTIAL + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_SPTIAL));
    }

    /**
     *  Test expand a spatial predicate, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateSpatialLowerCase() {

        String expanded = profile.expandPredicate(NAME_SPTIAL.toLowerCase());

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_SPTIAL + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_SPTIAL));
    }

    /**
     *  Test expand a spatial predicate, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateSpatialUpperCase() {

        String expanded = profile.expandPredicate(NAME_SPTIAL.toUpperCase());

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_SPTIAL + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_SPTIAL));
    }

    /**
     *  Test expand a temporal predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateTemporal() {

        String expanded = profile.expandPredicate(NAME_TEMPRL);

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_TEMPRL + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_TEMPRL));
    }

    /**
     *  Test expand a temporal predicate, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateTemporalLowerCase() {

        String expanded = profile.expandPredicate(NAME_TEMPRL.toLowerCase());

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_TEMPRL + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_TEMPRL));
    }

    /**
     *  Test expand a temporal predicate, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateTemporalUpperCase() {

        String expanded = profile.expandPredicate(NAME_TEMPRL.toUpperCase());

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_TEMPRL + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_TEMPRL));
    }

    /**
     *  Test expand a mediator predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateMediator() {

        String expanded = profile.expandPredicate(NAME_MEDITR);

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_MEDITR + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_MEDITR));
    }

    /**
     *  Test expand a mediator predicate, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateMediatorLowerCase() {

        String expanded = profile.expandPredicate(NAME_MEDITR.toLowerCase());

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_MEDITR + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_MEDITR));
    }

    /**
     *  Test expand a mediator predicate, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateMediatorUpperCase() {

        String expanded = profile.expandPredicate(NAME_MEDITR.toUpperCase());

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_MEDITR + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                                   DublinCoreTerms.NAME_MEDITR));
    }

    /**
     *  Test expand a date accepted predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateDateAccepted() {

        String expanded = profile.expandPredicate(NAME_ACCPTD);

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_ACCPTD + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_ACCPTD));
    }

    /**
     *  Test expand a date accepted predicate, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateDateAcceptedLowerCase() {

        String expanded = profile.expandPredicate(NAME_ACCPTD.toLowerCase());

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_ACCPTD + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_ACCPTD));
    }

    /**
     *  Test expand a date accepted predicate, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateDateAcceptedUpperCase() {

        String expanded = profile.expandPredicate(NAME_ACCPTD.toUpperCase());

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_ACCPTD + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_ACCPTD));
    }

    /**
     *  Test expand a date copyrighted predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateDateCopyrighted() {

        String expanded = profile.expandPredicate(NAME_CPYRGT);

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_CPYRGT + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_CPYRGT));
    }

    /**
     *  Test expand a date copyrighted predicate, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateDateCopyrightedLowerCase() {

        String expanded = profile.expandPredicate(
                                    NAME_CPYRGT.toLowerCase());

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_CPYRGT + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_CPYRGT));
    }

    /**
     *  Test expand a date copyrighted predicate, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateDateCopyrightedUpperCase() {

        String expanded = profile.expandPredicate(
                                    NAME_CPYRGT.toUpperCase());

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_CPYRGT + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_CPYRGT));
    }

    /**
     *  Test expand a date submitted predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateDateSubmitted() {

        String expanded = profile.expandPredicate(NAME_SUBMTD);

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_SUBMTD + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_SUBMTD));
    }

    /**
     *  Test expand a date submitted predicate, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateDateSubmittedLowerCase() {

        String expanded = profile.expandPredicate(NAME_SUBMTD.toLowerCase());

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_SUBMTD + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_SUBMTD));
    }

    /**
     *  Test expand a date submitted predicate, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateDateSubmittedUpperCase() {

        String expanded = profile.expandPredicate(NAME_SUBMTD.toUpperCase());

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_SUBMTD + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_SUBMTD));
    }

    /**
     *  Test expand an education level predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateEducationLevel() {

        String expanded = profile.expandPredicate(NAME_EDULVL);

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_EDULVL + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_EDULVL));
    }

    /**
     *  Test expand an education level predicate, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateEducationLevelLowerCase() {

        String expanded = profile.expandPredicate(NAME_EDULVL.toLowerCase());

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_EDULVL + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_EDULVL));
    }

    /**
     *  Test expand an education level predicate, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateEducationLevelUpperCase() {

        String expanded = profile.expandPredicate(NAME_EDULVL.toUpperCase());

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_EDULVL + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_EDULVL));
    }

    /**
     *  Test expand an access rights predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateAccessRights() {

        String expanded = profile.expandPredicate(NAME_ACCRGT);

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_ACCRGT + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_ACCRGT));
    }

    /**
     *  Test expand an access rights predicate, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateAccessRightsLowerCase() {

        String expanded = profile.expandPredicate(NAME_ACCRGT.toLowerCase());

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_ACCRGT + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_ACCRGT));
    }

    /**
     *  Test expand an access rights predicate, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateAccessRightsUpperCase() {

        String expanded = profile.expandPredicate(NAME_ACCRGT.toUpperCase());

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_ACCRGT + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_ACCRGT));
    }

    /**
     *  Test expand a bibliographic citation predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateBibliographicCitation() {

        String expanded = profile.expandPredicate(NAME_BIBCIT);

        assertTrue("Encoded predicate should be expanded to '" +
       DublinCoreTerms.BASE + DublinCoreTerms.NAME_BIBCIT + "'. " +
       "Actually returned '" + expanded + "'.",
       expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_BIBCIT));
    }

    /**
     *  Test expand a bibliographic citation predicate, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateBibliographicCitationLowerCase() {

        String expanded = profile.expandPredicate(
                                    NAME_BIBCIT.toLowerCase());

        assertTrue("Encoded predicate should be expanded to '" +
       DublinCoreTerms.BASE + DublinCoreTerms.NAME_BIBCIT + "'. " +
       "Actually returned '" + expanded + "'.",
       expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_BIBCIT));
    }

    /**
     *  Test expand a bibliographic citation predicate, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateBibliographicCitationUpperCase() {

        String expanded = profile.expandPredicate(
                                    NAME_BIBCIT.toUpperCase());

        assertTrue("Encoded predicate should be expanded to '" +
       DublinCoreTerms.BASE + DublinCoreTerms.NAME_BIBCIT + "'. " +
       "Actually returned '" + expanded + "'.",
       expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_BIBCIT));
    }

    /**
     *  Test expand a license predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateLicense() {

        String expanded = profile.expandPredicate(NAME_LICNSE);

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_LICNSE + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_LICNSE));
    }

    /**
     *  Test expand a license predicate, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateLicenseLowerCase() {

        String expanded = profile.expandPredicate(NAME_LICNSE.toLowerCase());

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_LICNSE + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_LICNSE));
    }

    /**
     *  Test expand a license predicate, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateLicenseUpperCase() {

        String expanded = profile.expandPredicate(NAME_LICNSE.toUpperCase());

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_LICNSE + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_LICNSE));
    }

    /**
     *  Test expand a rights holder predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateRightsHolder() {

        String expanded = profile.expandPredicate(NAME_RGTHLD);

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_RGTHLD + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_RGTHLD));
    }

    /**
     *  Test expand a rights holder predicate, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateRightsHolderLowerCase() {

        String expanded = profile.expandPredicate(NAME_RGTHLD.toLowerCase());

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_RGTHLD + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_RGTHLD));
    }

    /**
     *  Test expand a rights holder predicate, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateRightsHolderUpperCase() {

        String expanded = profile.expandPredicate(NAME_RGTHLD.toUpperCase());

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_RGTHLD + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_RGTHLD));
    }

    /**
     *  Test expand a provenance predicate.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateProvenance() {

        String expanded = profile.expandPredicate(NAME_PRVNCE);

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_PRVNCE + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_PRVNCE));
    }

    /**
     *  Test expand a provenance predicate, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateProvenanceLowerCase() {

        String expanded = profile.expandPredicate(NAME_PRVNCE.toLowerCase());

        assertTrue("Encoded predicate should be expanded to '" +
               DublinCoreTerms.BASE + DublinCoreTerms.NAME_PRVNCE + "'. " +
               "Actually returned '" + expanded + "'.",
               expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_PRVNCE));
    }

    /**
     *  Test expand a provenance predicate, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandPredicateProvenanceUpperCase() {

        String expanded = profile.expandPredicate(NAME_PRVNCE.toUpperCase());

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_PRVNCE + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_PRVNCE));
    }

    /**
     *  Test expand an NAME_LCSH scheme.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeLCSH() {

        String expanded = profile.expandScheme(NAME_LCSH);

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_LCSH + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_LCSH));
    }

    /**
     *  Test expand an NAME_LCSH scheme, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeLCSHLowerCase() {

        String expanded = profile.expandScheme(NAME_LCSH.toLowerCase());

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_LCSH + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_LCSH));
    }

    /**
     *  Test expand an NAME_LCSH scheme, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeLCSHUpperCase() {

        String expanded = profile.expandScheme(NAME_LCSH.toUpperCase());

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_LCSH + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_LCSH));
    }

    /**
     *  Test expand a NAME_MESH scheme.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeMESH() {

        String expanded = profile.expandScheme(NAME_MESH);

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_MESH + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_MESH));
    }

    /**
     *  Test expand a NAME_MESH scheme, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeMESHLowerCase() {

        String expanded = profile.expandScheme(NAME_MESH.toLowerCase());

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_MESH + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_MESH));
    }

    /**
     *  Test expand a NAME_MESH scheme, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeMESHUpperCase() {

        String expanded = profile.expandScheme(NAME_MESH.toUpperCase());

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_MESH + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_MESH));
    }

    /**
     *  Test expand a DDC scheme.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeDDC() {

        String expanded = profile.expandScheme(DDC);

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_DDC + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_DDC));
    }

    /**
     *  Test expand a DDC scheme, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeDDCLowerCase() {

        String expanded = profile.expandScheme(DDC.toLowerCase());

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_DDC + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_DDC));
    }

    /**
     *  Test expand a DDC scheme, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeDDCUpperCase() {

        String expanded = profile.expandScheme(DDC.toUpperCase());

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_DDC + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_DDC));
    }

    /**
     *  Test expand an NAME_LCC scheme.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeLCC() {

        String expanded = profile.expandScheme(NAME_LCC);

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_LCC + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_LCC));
    }

    /**
     *  Test expand an NAME_LCC scheme, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeLCCLowerCase() {

        String expanded = profile.expandScheme(NAME_LCC.toLowerCase());

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_LCC + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_LCC));
    }

    /**
     *  Test expand an NAME_LCC scheme, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeLCCUpperCase() {

        String expanded = profile.expandScheme(NAME_LCC.toUpperCase());

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_LCC + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_LCC));
    }

    /**
     *  Test expand a UDC scheme.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeUDC() {

        String expanded = profile.expandScheme(UDC);

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_UDC + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_UDC));
    }

    /**
     *  Test expand a UDC scheme, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeUDCLowerCase() {

        String expanded = profile.expandScheme(UDC.toLowerCase());

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_UDC + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_UDC));
    }

    /**
     *  Test expand a UDC scheme, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeUDCUpperCase() {

        String expanded = profile.expandScheme(UDC.toUpperCase());

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_UDC + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_UDC));
    }

    /**
     *  Test expand a DCMI Type scheme.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeDCMIType() {

        String expanded = profile.expandScheme(NAME_DCTYPE);

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_DCTYPE + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_DCTYPE));
    }

    /**
     *  Test expand a DCMI Type scheme, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeDCMITypeLowerCase() {

        String expanded = profile.expandScheme(NAME_DCTYPE.toLowerCase());

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_DCTYPE + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_DCTYPE));
    }

    /**
     *  Test expand a DCMI Type scheme, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeDCMITypeUpperCase() {

        String expanded = profile.expandScheme(NAME_DCTYPE.toUpperCase());

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_DCTYPE + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_DCTYPE));
    }

    /**
     *  Test expand an NAME_IMT scheme.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeIMT() {

        String expanded = profile.expandScheme(NAME_IMT);

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_IMT + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_IMT));
    }

    /**
     *  Test expand an NAME_IMT scheme, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeIMTLowerCase() {

        String expanded = profile.expandScheme(NAME_IMT.toLowerCase());

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_IMT + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_IMT));
    }

    /**
     *  Test expand an NAME_IMT scheme, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeIMTUpperCase() {

        String expanded = profile.expandScheme(NAME_IMT.toUpperCase());

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_IMT + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_IMT));
    }

    /**
     *  Test expand an ISO639-2 scheme.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeIS3692() {

        String expanded = profile.expandScheme(NAME_IS3692);

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_IS3692 + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_IS3692));
    }

    /**
     *  Test expand an ISO639-2 scheme, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeIS3692LowerCase() {

        String expanded = profile.expandScheme(NAME_IS3692.toLowerCase());

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_IS3692 + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_IS3692));
    }

    /**
     *  Test expand an ISO639-2 scheme, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeIS3692UpperCase() {

        String expanded = profile.expandScheme(NAME_IS3692.toUpperCase());

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_IS3692 + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_IS3692));
    }

    /**
     *  Test expand an RFC 1766 scheme.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeRC1766() {

        String expanded = profile.expandScheme(NAME_RC1766);

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_RC1766 + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_RC1766));
    }

    /**
     *  Test expand an RFC 1766 scheme, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeRC1766LowerCase() {

        String expanded = profile.expandScheme(NAME_RC1766.toLowerCase());

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_RC1766 + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_RC1766));
    }

    /**
     *  Test expand an RFC 1766 scheme, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeRC1766UpperCase() {

        String expanded = profile.expandScheme(NAME_RC1766.toUpperCase());

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_RC1766 + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_RC1766));
    }

    /**
     *  Test expand a URI scheme.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeURI() {

        String expanded = profile.expandScheme(URI);

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_URI + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_URI));
    }

    /**
     *  Test expand a URI scheme, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeURILowerCase() {

        String expanded = profile.expandScheme(URI.toLowerCase());

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_URI + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_URI));
    }

    /**
     *  Test expand a URI scheme, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeURIUpperCase() {

        String expanded = profile.expandScheme(URI.toUpperCase());

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_URI + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_URI));
    }


    /**
     *  Test expand a DCMI Point scheme.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemePoint() {

        String expanded = profile.expandScheme(NAME_POINT);

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_POINT + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_POINT));
    }

    /**
     *  Test expand a DCMI Point scheme, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemePointLowerCase() {

        String expanded = profile.expandScheme(NAME_POINT.toLowerCase());

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_POINT + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_POINT));
    }

    /**
     *  Test expand a DCMI Point scheme, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemePointUpperCase() {

        String expanded = profile.expandScheme(NAME_POINT.toUpperCase());

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_POINT + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_POINT));
    }

    /**
     *  Test expand an NAME_IS3166 scheme.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeIS3166() {

        String expanded = profile.expandScheme(NAME_IS3166);

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_IS3166 + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_IS3166));
    }

    /**
     *  Test expand an NAME_IS3166 scheme, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeIS3166LowerCase() {

        String expanded = profile.expandScheme(NAME_IS3166.toLowerCase());

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_IS3166 + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_IS3166));
    }

    /**
     *  Test expand an NAME_IS3166 scheme, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeIS3166UpperCase() {

        String expanded = profile.expandScheme(NAME_IS3166.toUpperCase());

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_IS3166 + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_IS3166));
    }

    /**
     *  Test expand a DCMI Box scheme.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeDCMIBox() {

        String expanded = profile.expandScheme(NAME_BOX);

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_BOX + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_BOX));
    }

    /**
     *  Test expand a DCMI Box scheme, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeDCMIBoxLowerCase() {

        String expanded = profile.expandScheme(NAME_BOX.toLowerCase());

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_BOX + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_BOX));
    }

    /**
     *  Test expand a DCMI Box scheme, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeDCMIBoxUpperCase() {

        String expanded = profile.expandScheme(NAME_BOX.toUpperCase());

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_BOX + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_BOX));
    }

    /**
     *  Test expand a NAME_TGN scheme.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeTGN() {

        String expanded = profile.expandScheme(NAME_TGN);

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_TGN + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_TGN));
    }

    /**
     *  Test expand a NAME_TGN scheme, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeTGNLowerCase() {

        String expanded = profile.expandScheme(NAME_TGN.toLowerCase());

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_TGN + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_TGN));
    }

    /**
     *  Test expand a NAME_TGN scheme, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeTGNUpperCase() {

        String expanded = profile.expandScheme(NAME_TGN.toUpperCase());

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_TGN + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_TGN));
    }

    /**
     *  Test expand a DCMI Period scheme.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeDCMIPeriod() {

        String expanded = profile.expandScheme(NAME_PERIOD);

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_PERIOD + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_PERIOD));
    }

    /**
     *  Test expand a DCMI Period scheme, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeDCMIPeriodLowerCase() {

        String expanded = profile.expandScheme(NAME_PERIOD.toLowerCase());

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_PERIOD + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_PERIOD));
    }

    /**
     *  Test expand a DCMI Period scheme, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeDCMIPeriodUpperCase() {

        String expanded = profile.expandScheme(NAME_PERIOD.toUpperCase());

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_PERIOD + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_PERIOD));
    }

    /**
     *  Test expand a W3C-DTF scheme.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeW3CDTF() {

        String expanded = profile.expandScheme(W3CDTF);

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_W3CDTF + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_W3CDTF));
    }

    /**
     *  Test expand a W3C-DTF scheme, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeW3CDTFLowerCase() {

        String expanded = profile.expandScheme(W3CDTF.toLowerCase());

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_W3CDTF + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_W3CDTF));
    }

    /**
     *  Test expand a W3C-DTF scheme, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeW3CDTFUpperCase() {

        String expanded = profile.expandScheme(W3CDTF.toUpperCase());

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_W3CDTF + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_W3CDTF));
    }

    /**
     *  Test expand an NAME_RC3066 scheme.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeRC3066() {

        String expanded = profile.expandScheme(NAME_RC3066);

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_RC3066 + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_RC3066));
    }

    /**
     *  Test expand an NAME_RC3066 scheme, lower case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeRC3066LowerCase() {

        String expanded = profile.expandScheme(NAME_RC3066.toLowerCase());

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_RC3066 + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_RC3066));
    }

    /**
     *  Test expand an NAME_RC3066 scheme, upper case.
     *  @since  Version 0.1 2004-12-23
     */
    public final void testExpandSchemeRC3066UpperCase() {

        String expanded = profile.expandScheme(NAME_RC3066.toUpperCase());

        assertTrue("Encoded scheme should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_RC3066 + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DublinCoreTerms.BASE +
                               DublinCoreTerms.NAME_RC3066));
    }

    /**
     *  Test has primary schema.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testHasPrimarySchemaTrue() {

        assertTrue(profile.hasPrimarySchema("dc"));
    }

    /**
     *  Test has primary schema.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testHasPrimarySchemaFalse() {

        assertFalse(profile.hasPrimarySchema("maybe-not"));
    }

    /**
     *  Test has primary schema.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testHasPrimarySchemaNullPrefix() {

        try {

            profile.hasPrimarySchema(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }


    /**
     *  Test get primary schema.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testGetPrimarySchemaTrue() {

        assertNotNull(profile.getPrimarySchema("dc"));
    }

    /**
     *  Test get primary schema.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testGetPrimarySchemaFalse() {

        assertNull(profile.getPrimarySchema("maybe-not"));
    }

    /**
     *  Test get primary schema.
     *  @since  Version 0.1 2005-05-27
     */
    public final void testGetPrimarySchemaNullPrefix() {

        try {

            profile.getPrimarySchema(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Tear down a test scenario as required.
     *  @since  Version 0.1 2005-
    protected void tearDown() {

    }
     */

    /**
     *  Create a test suite for this test case.
     *  @return A test suite.
     *  @since  Version 0.1 2005-05-27
     */
    public static Test suite() {

        return new TestSuite(DublinCoreProfileTest.class);
    }

    /**
     *  Run this test suite as a stand alone application.
     *  @param  args    Any command line arguments to pass
     *          to the test. None required or expected.
     *  @since  Version 0.1 2005-05-27
     */
    public static void main(final String[] args) {

        junit.textui.TestRunner.run(suite());
    }

}
