package com.mkdoc.servlet;

/*
    Created:    2005-01-20
    Issued:     2005-01-20
    Modified:   2005-07-26

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import com.mkdoc.util.ApplicationContext;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  Test suite for <code></code>.
 *  @version    0.4 2005-07-26
 *  @author Philip Shaw <phil@mkdoc.com>
 */
public class AbstractMKDocServletTest extends TestCase {

    /**
     *  An expected <code>IllegalArgumentException</code>.
     *  @since  Version 0.3 2005-07-25
     */
    public static final String EXPECTED_IAE =
    "Should throw an IllegalArgumentException.";

    /**
     *  A parameter name to check.
     *  @since  Version 0.1 2005-01-20
     */
    protected static final String NAME_1 = "param1";

    /**
     *  A parameter value to check.
     *  @since  Version 0.1 2005-01-20
     */
    protected static final String VALUE_1 = "value1";

    /**
     *  A parameter value to check.
     *  @since  Version 0.1 2005-01-20
     */
    protected static final String VALUE_2 = "value2";

    /**
     *  A literal markup string.
     *  @since  Version 0.2 2005-05-18
     */
    protected static final String MARKUP =
    "<script type=\"text/javascript\" src=\"http://malicious.example/" +
    "bad-script\"></script> &evil;";

    /**
     *  A filtered markup string.
     *  @since  Version 0.2 2005-05-18
     */
    protected static final String FILTERED =
    "&lt;script type=&quot;text/javascript&quot; src=&quot;" +
    "http://malicious.example/bad-script&quot;&gt;&lt;/script&gt; &amp;evil;";

    /**
     *  Named test instance.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2005-01-20
     */
    public AbstractMKDocServletTest(final String name) {

        super(name);
    }

    /**
     *  Set up a test scenario as required.
     *  @since  Version 0.1 2005-
    protected void setUp() {

    }
     */

    /**
     *  Get the application context.
     *  @since  Version 0.3 2005-07-22
     */
    public final void testGetApplicationContextNull() {

        try {

            AbstractMKDocServlet.getApplicationContext(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Get the application context.
     *  @since  Version 0.3 2005-07-22
     */
    public final void testGetApplicationContext() {

        MockServletContext context = new MockServletContext();

        String value = "success";

        context.setInitParameter("test", value);

        ApplicationContext appContext = null;

        appContext = AbstractMKDocServlet.getApplicationContext(context);

        assertTrue((appContext != null) &&
                   (appContext instanceof ServletApplicationContext));

        String testValue = appContext.getStringParameter("test");

        assertTrue(value.equals(testValue));
    }

    /**
     *  Filter potentially malicious markup.
     *  @since  Version 0.2 2005-05-18
     */
    public final void testFilter() {

        String out = AbstractMKDocServlet.filter(MARKUP);

        assertTrue(out.equals(FILTERED));
    }

    /**
     *  Get first parameter with null request should throw an
     *  <code>IllegalArgumentException</code>.
     *  @since  Version 0.1 2005-01-20
     */
    public final void testGetFirstParamNullRequest() {

        try {

            AbstractMKDocServlet.getFirstParam(null, NAME_1);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Get first parameter with null name should throw an
     *  <code>IllegalArgumentException</code>.
     *  @since  Version 0.1 2005-01-20
     */
    public final void testGetFirstParamNullName() {

        // Not configured
        MockHttpServletRequest msreq = new MockHttpServletRequest();

        try {

            AbstractMKDocServlet.getFirstParam(msreq, null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  With one parameter should return the only value.
     *  @since  Version 0.1 2005-01-20
     */
    public final void testGetFirstParamOnePresent() {

        // Not configured
        MockHttpServletRequest msreq = new MockHttpServletRequest();

        String value = AbstractMKDocServlet.getFirstParam(msreq, NAME_1);

        assertNull(value);

    }

    /**
     *  With an empty string value should return null.
     *  @since  Version 0.1 2005-01-20
     */
    public final void testGetFirstParamEmptyValue() {

        MockHttpServletRequest msreq = new MockHttpServletRequest();

        msreq.setParameter(NAME_1, "");

        String value = AbstractMKDocServlet.getFirstParam(msreq, NAME_1);

        assertNull("Value should be null, actually '" + value + "'.",
                   value);

    }

    /**
     *  With two parameters should return only the first value.
     *  @since  Version 0.1 2005-01-20
     */
    public final void testGetFirstParamTwoPresent() {

        MockHttpServletRequest msreq = new MockHttpServletRequest();

        msreq.setParameter(NAME_1, VALUE_1);
        msreq.setParameter(NAME_1, VALUE_2);

        String value = AbstractMKDocServlet.getFirstParam(msreq, NAME_1);

        assertTrue(value.equals(VALUE_1));

    }

    /**
     *  Tear down a test scenario as required.
     *  @since  Version 0.1 2005-
    protected void tearDown() {

    }
     */

    /**
     *  Create a test suite for this test case.
     *  @return A test suite.
     *  @since  Version 0.1 2005-01-20
     */
    public static Test suite() {

        return new TestSuite(AbstractMKDocServletTest.class);
    }

    /**
     *  Run this test suite as a stand alone application.
     *  @param  args    Any command line arguments to pass
     *          to the test. None required or expected.
     *  @since  Version 0.1 2005-01-20
     */
    public static void main(final String[] args) {

        junit.textui.TestRunner.run(suite());
    }

}
