package com.mkdoc.servlet;

/*
    Created:    2005-01-25
    Issued:     2005-01-25
    Modified:   2005-01-26

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import java.io.IOException;

import javax.servlet.ServletOutputStream;

/**
 *  A mock <code>ServletOutputStream</code> for testing.
 *  @version    0.2 2005-01-26
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class MockServletOutputStream extends ServletOutputStream {

    /**
     *  Whether to fail on write.
     *  @since  Version 0.2 2005-01-26
     */
    private boolean failOnWrite = false;

    /**
     *  The fail on write exception message.
     *  @since  Version 0.2 2005-01-26
     */
    private static final String FAIL_MESSAGE =
    "MockServletOutputStream throws IOException.";

    /**
     *  Store all output for subsequent checking.
     *  @since  Version 0.1 2005-01-25
     */
    private StringBuffer out = new StringBuffer();

    /**
     *  Write an int.
     *  @param  value   The int value to write.
     *  @exception  IOException If there is a problem writing the output.
     *  @since  Version 0.1 2005-01-25
     */
    public final void write(final int value) throws IOException {

        out.append(value);
    }

    /**
     *  Write a segment of a byte array.
     *  @param  byteArray   The array from which to write bytes.
     *  @param  offset  The offset index from which to start writing.
     *  @param  length  The number of bytes to write from the array.
     *  @exception  IOException If there is a problem writing the output.
     *  @since  Version 0.1 2005-01-25
     */
    public final void write(final byte[] byteArray,
                            final int offset,
                            final int length) throws IOException {

        if (failOnWrite) {

            throw new IOException(FAIL_MESSAGE);
        }

        if ((offset + length) <= byteArray.length) {

            for (int i = offset; i <= length; i++) {

                out.append(byteArray[i]);
            }
        }
        else {

            throw new IOException(
                        "Incompatible index values, " +
                        "array length = " + byteArray.length + " " +
                        "write offset = " + offset + " " +
                        "write length = " + length + ".");
        }
    }

    /**
     *  Write a whole byte array.
     *  @param  byteArray   The array from which to write all bytes.
     *  @exception  IOException If there is a problem writing the output.
     *  @since  Version 0.1 2005-01-25
     */
    public final void write(final byte[] byteArray) throws IOException {

        if (failOnWrite) {

            throw new IOException(FAIL_MESSAGE);
        }

        write(byteArray, 0, byteArray.length);
    }

    /**
     *  Print a string.
     *  @param  value   The string value to write.
     *  @exception  IOException If there is a problem writing the output.
     *  @since  Version 0.1 2005-01-25
     */
    public final void print(final String value) throws IOException {

        if (failOnWrite) {

            throw new IOException(FAIL_MESSAGE);
        }

        out.append(value);
    }

    /**
     *  Print a boolean.
     *  @param  value   The boolean value to write.
     *  @exception  IOException If there is a problem writing the output.
     *  @since  Version 0.1 2005-01-25
     */
    public final void print(final boolean value) throws IOException {

        if (failOnWrite) {

            throw new IOException(FAIL_MESSAGE);
        }

        out.append(value);
    }

    /**
     *  Print a char.
     *  @param  value   The char value to write.
     *  @exception  IOException If there is a problem writing the output.
     *  @since  Version 0.1 2005-01-25
     */
    public final void print(final char value) throws IOException {

        if (failOnWrite) {

            throw new IOException(FAIL_MESSAGE);
        }

        out.append(value);
    }


    /**
     *  Print an int.
     *  @param  value   The int value to write.
     *  @exception  IOException If there is a problem writing the output.
     *  @since  Version 0.1 2005-01-25
     */
    public final void print(final int value) throws IOException {

        if (failOnWrite) {

            throw new IOException(FAIL_MESSAGE);
        }

        out.append(value);
    }

    /**
     *  Print a long.
     *  @param  value   The long value to write.
     *  @exception  IOException If there is a problem writing the output.
     *  @since  Version 0.1 2005-01-25
     */
    public final void print(final long value) throws IOException {

        if (failOnWrite) {

            throw new IOException(FAIL_MESSAGE);
        }

        out.append(value);
    }

    /**
     *  Print a float.
     *  @param  value   The float value to write.
     *  @exception  IOException If there is a problem writing the output.
     *  @since  Version 0.1 2005-01-25
     */
    public final void print(final float value) throws IOException {

        if (failOnWrite) {

            throw new IOException(FAIL_MESSAGE);
        }

        out.append(value);
    }

    /**
     *  Print a double.
     *  @param  value   The double value to write.
     *  @exception  IOException If there is a problem writing the output.
     *  @since  Version 0.1 2005-01-25
     */
    public final void print(final double value) throws IOException {

        if (failOnWrite) {

            throw new IOException(FAIL_MESSAGE);
        }

        out.append(value);
    }

    /**
     *  Print a new line.
     *  @exception  IOException If there is a problem writing the output.
     *  @since  Version 0.1 2005-01-25
     */
    public final void println() throws IOException {

        if (failOnWrite) {

            throw new IOException(FAIL_MESSAGE);
        }

        out.append("\r\n");
    }


    /**
     *  Print a string followed by a new line.
     *  @param  value   The string value to write.
     *  @exception  IOException If there is a problem writing the output.
     *  @since  Version 0.1 2005-01-25
     */
    public final void println(final String value) throws IOException {

        print(value);
        println();
    }

    /**
     *  Print a boolean followed by a new line.
     *  @param  value   The boolean value to write.
     *  @exception  IOException If there is a problem writing the output.
     *  @since  Version 0.1 2005-01-25
     */
    public final void println(final boolean value) throws IOException {

        print(value);
        println();
    }

    /**
     *  Print a char followed by a new line.
     *  @param  value   The char value to write.
     *  @exception  IOException If there is a problem writing the output.
     *  @since  Version 0.1 2005-01-25
     */
    public final void println(final char value) throws IOException {

        print(value);
        println();
    }

    /**
     *  Print an int followed by a new line.
     *  @param  value   The int value to write.
     *  @exception  IOException If there is a problem writing the output.
     *  @since  Version 0.1 2005-01-25
     */
    public final void println(final int value) throws IOException {

        print(value);
        println();
    }

    /**
     *  Print a long followed by a new line.
     *  @param  value   The long value to write.
     *  @exception  IOException If there is a problem writing the output.
     *  @since  Version 0.1 2005-01-25
     */
    public final void println(final long value) throws IOException {

        print(value);
        println();
    }

    /**
     *  Print a float followed by a new line.
     *  @param  value   The float value to write.
     *  @exception  IOException If there is a problem writing the output.
     *  @since  Version 0.1 2005-01-25
     */
    public final void println(final float value) throws IOException {

        print(value);
        println();
    }

    /**
     *  Print a double followed by a new line.
     *  @param  value   The double value to write.
     *  @exception  IOException If there is a problem writing the output.
     *  @since  Version 0.1 2005-01-25
     */
    public final void println(final double value) throws IOException {

        print(value);
        println();
    }

    /**
     *  Get the servlet output as a string.
     *  @return All output written to the stream.
     *  @since  Version 0.1 2005-01-25
     */
    public final String getOutput() {

        return out.toString();
    }

    /**
     *  Set the fail on write trigger.
     *  @param  failOnWrite Whether the output stream should fail with write
     *          methods.
     *  @since  Version 0.1 2005-01-25
     */
    public final void setFailOnWrite(final boolean failOnWrite) {

        this.failOnWrite = failOnWrite;
    }

}
