package com.mkdoc.servlet;

/*
    Created:    2005-01-21
    Issued:     2005-01-21
    Modified:   2005-07-25

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import java.io.IOException;
import java.io.PrintWriter;

import java.util.Locale;

import javax.servlet.ServletOutputStream;
import javax.servlet.ServletResponse;

/**
 *  A mock servelet response for testing.
 *  <p>
 *    Most fields are initially <code>null</code> and must be configured using
 *    the relevant mutators if their accessors are expected to be called. Some
 *    accessors issue fixed values. Most mutators do not have validation, so
 *    <code>null</code> values can generally be assigned.
 *  </p>
 *  @version    0.3 2005-07-22
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class MockServletResponse implements ServletResponse {

    /**
     *  The default content type.
     *  @since  Version 0.1 2005-01-21
     */
    public static final String DEFAULT_TYPE = "text/plain";

    /**
     *  The default buffer size.
     *  @since  Version 0.1 2005-01-21
     */
    public static final int DEFAULT_SIZE = 32;

    /**
     *  Common service properties.
     *  @since  Version 0.1 2005-01-21
     */
    protected MockServiceProperties properties = new MockServiceProperties();

    /**
     *  The buffer size.
     *  @since  Version 0.1 2005-01-21
     */
    protected int bufferSize = DEFAULT_SIZE;

    /**
     *  The character encoding.
     *  @since  Version 0.1 2005-01-21
     */
    protected String encoding;

    /**
     *  The content type.
     *  @since  Version 0.1 2005-01-21
     */
    protected String contentType = DEFAULT_TYPE;

    /**
     *  A servlet output stream.
     *  @since  Version 0.1 2005-01-21
     */
    protected ServletOutputStream outputStream = new MockServletOutputStream();

    /**
     *  Whether the response is committed.
     *  @since  Version 0.1 2005-01-21
     */
    protected boolean committed;

    /**
     *  If an output has already been issued for the response.
     *  @since  Version 0.1 2005-01-21
     */
    protected boolean written;

    /**
     *  The content length.
     *  @since  Version 0.1 2005-01-21
     */
    protected int length;

    /**
     *  The locale.
     *  @since  Version 0.1 2005-01-21
     */
    protected Locale locale;

    /**
     *  The response URL path.
     *  @since  Version 0.1 2005-01-21
     */
    protected String path;

    /**
     *  Set the content length.
     *  @param  length  The content length to set.
     *  @since  Version 0.1 2005-01-21
     */
    public final void setContentLength(final int length) {

        this.length = length;
    }

    /**
     *  Get the content length.
     *  @return The content length.
     *  @since  Version 0.1 2005-01-21
     */
    public final int getContentLength() {

        return length;
    }

    /**
     *  Set the content type.
     *  @param  contentType The content type to set.
     *  @since  Version 0.1 2005-01-21
     */
    public final void setContentType(final String contentType) {

        this.contentType = contentType;
    }

    /**
     *  Get the content type.
     *  @return The content type.
     *  @since  Version 0.1 2005-01-21
     */
    public final String getContentType() {

        return contentType;
    }

    /**
     *  Set the output stream for the response.
     *  @param  outputStream    The output stream for the response.
     *  @since  Version 0.1 2005-01-21
     */
    public final void setOutputStream(final ServletOutputStream outputStream) {

        this.outputStream = outputStream;
    }

    /**
     *  Get the servlet output stream.
     *  @exception  IOException If no output stream has been configured.
     *  @return The servlet output stream.
     *  @since  Version 0.1 2005-01-21
     */
    public final ServletOutputStream getOutputStream() throws IOException {

        if (written) {

            throw new IllegalStateException("Output has already been issued.");
        }
        else {

            written = true;

            return outputStream;
        }
    }

    /**
     *  Get a print writer for the servlet output.
     *  @return A print writer wrapping the servlet output stream.
     *  @exception  IOException If there is a problem obtaining the print
     *              writer.
     *  @since  Version 0.1 2005-01-21
     */
    public final PrintWriter getWriter() throws IOException {

        return new PrintWriter(getOutputStream(), true);
    }

    /**
     *  Set the character encoding.
     *  @param  encoding    The character encoding to set.
     *  @since  Version 0.1 2005-01-21
     */
    public final void setCharacterEncoding(final String encoding) {

        this.encoding = encoding;
    }

    /**
     *  Get the character encoding
     *  @return The character encoding.
     *  @since  Version 0.1 2005-01-21
     */
    public final String getCharacterEncoding() {

        return encoding;
    }

    /**
     *  Set the buffer size.
     *  @param  bufferSize  The new buffer size in bytes.
     *  @since  Version 0.1 2005-01-21
     */
    public final void setBufferSize(final int bufferSize) {

        this.bufferSize = bufferSize;
    }

    /**
     *  Get the buffer size.
     *  @return The current buffer size.
     *  @since  Version 0.1 2005-01-21
     */
    public final int getBufferSize() {

        return bufferSize;
    }

    /**
     *  Reset the output buffer.
     *  @since  Version 0.1 2005-01-21
     */
    public final void reset() {

        if (committed) {

            throw new IllegalStateException("Output is already committed.");
        }

        // Do nothing
    }

    /**
     *  Check whether the response has been committed.
     *  @return <code>true</code> if the servlet output stream has been
     *          committed.
     *  @since  Version 0.1 2005-01-21
     */
    public final boolean isCommitted() {

        return committed;
    }

    /**
     *  Set whether the response has been committed.
     *  @param  committed   Whether the response has been committed.
     *  @since  Version 0.1 2005-01-21
     */
    public final void setCommitted(final boolean committed) {

        this.committed = committed;
    }

    /**
     *  Flush the buffer.
     *  @exception  IOException Not thrown by this implementation.
     *  @since  Version 0.1 2005-01-21
     */
    public void flushBuffer() throws IOException {

        // Empty
    }

    /**
     *  Flush the buffer.
     *  @since  Version 0.1 2005-01-21
     */
    public void resetBuffer() {

        // Empty
    }

    /**
     *  Set the locale.
     *  @param  locale  The locale of the response.
     *  @since  Version 0.1 2005-01-21
     */
    public final void setLocale(final Locale locale) {

        this.locale = locale;
    }

    /**
     *  Get the locale.
     *  @return The locale of the response.
     *  @since  Version 0.1 2005-01-21
     */
    public final Locale getLocale() {

        return locale;
    }

    /**
     *  Set the URL path of the response.
     *  <p>
     *    May be used by mock <code>RequestDispatcher</code> types when
     *    forwarding a response.
     *  </p>
     *  @param  path    The target URL path of forwarded responses and error
     *          responses.
     *  @since  Version 0.1 2005-01-21
     */
    public final void setPath(final String path) {

        this.path = path;
    }

    /**
     *  Get the URL path of the response.
     *  <p>
     *    Used by test classes to  and subsequently checked
     *  </p>
     *  @return The effective URL path of forwarded responses and error
     *          responses.
     *  @since  Version 0.1 2005-01-21
     */
    public final String getPath() {

        return path;
    }

    /**
     *  Get the servlet output.
     *  @return All output written to the servlet output stream.
     *  @since  Version 0.3 2005-07-22
     */
    public final String getOutput() {

        try {

            MockServletOutputStream mockStream = null;

            mockStream = (MockServletOutputStream) outputStream;

            return mockStream.getOutput();
        }
        catch (ClassCastException cce) {

            return "Not a mock servlet output type.";
        }
    }

}
