package com.mkdoc.sesame;

/*
    Created:    2005-02-02
    Issued:     2005-02-02
    Modified:   2005-02-02

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import org.openrdf.model.Value;

import org.openrdf.model.impl.LiteralImpl;
import org.openrdf.model.impl.URIImpl;

import org.openrdf.sesame.query.QueryResultsTable;

import com.mkdoc.servlet.MockServletOutputStream;

import java.io.IOException;

import java.util.List;
import java.util.Vector;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  Test suite for <code>XhtmlResultsRenderer</code>.
 *  @version    0.1 2005-02-02
 *  @author Philip Shaw <phil@mkdoc.com>
 */
public class XhtmlResultsRendererTest extends TestCase {

    /**
     *  The expected content type.
     *  @since  Version 0.1 2005-02-02
     */
    public static final String CONTENT_TYPE = "text/html";

    /**
     *  Start of an <code>a</code> element open tag.
     *  @since  Version 0.1 2005-02-02
     */
    public static final String A_START_TAG = "<a href=\"";

    /**
     *  End of an open tag with close attribute quote.
     *  @since  Version 0.1 2005-02-02
     */
    public static final String END_ATT_TAG = "\">";

    /**
     *  A close <code>a</code> element tag.
     *  @since  Version 0.1 2005-02-02
     */
    public static final String A_CLOSE_TAG = "</a>";

    /**
     *  Start <code>dt</code> element.
     *  @since  Version 0.1 2005-02-02
     */
    public static final String DT_START = "<dt>";

    /**
     *  End <code>dt</code> element.
     *  @since  Version 0.1 2005-02-02
     */
    public static final String DT_CLOSE = "</dt>";

    /**
     *  The new line separator.
     *  @since  Version 0.1 2005-02-02
     */
    public static final String NEWLINE = System.getProperty("line.separator");

    /**
     *  The first results column name.
     *  @since  Version 0.1 2005-02-02
     */
    public static final String COL_1 = "First";

    /**
     *  The second results column name.
     *  @since  Version 0.1 2005-02-02
     */
    public static final String COL_2 = "Second";

    /**
     *  The results column names.
     *  @since  Version 0.1 2005-02-02
     */
    public static final String[] COL_NAMES = new String[] {COL_1,
                                                           COL_2};
    /**
     *  A tab character.
     *  @since  Version 0.1 2005-02-02
     */
    public static final String TAB = "\t";

    /**
     *  A literal value.
     *  @since  Version 0.1 2005-02-02
     */
    public static final String LITERAL_A1 =
    "http://test.mksearch.mkdoc.org/#A1";

    /**
     *  Result table value.
     *  @since  Version 0.1 2005-02-02
     */
    public static final Value VALUE_A1 = new URIImpl(LITERAL_A1);

    /**
     *  A literal value.
     *  @since  Version 0.1 2005-02-02
     */
    public static final String LITERAL_B1 = "B1";

    /**
     *  Result table value.
     *  @since  Version 0.1 2005-02-02
     */
    public static final Value VALUE_B1 = new LiteralImpl(LITERAL_B1);

    /**
     *  A literal value.
     *  @since  Version 0.1 2005-02-02
     */
    public static final String LITERAL_A2 =
    "http://test.mksearch.mkdoc.org/#A2";

    /**
     *  Result table value.
     *  @since  Version 0.1 2005-02-02
     */
    public static final Value VALUE_A2 = new URIImpl(LITERAL_A2);

    /**
     *  A literal value.
     *  @since  Version 0.1 2005-02-02
     */
    public static final String LITERAL_B2 = "B2";

    /**
     *  Result table value.
     *  @since  Version 0.1 2005-02-02
     */
    public static final Value VALUE_B2 = new LiteralImpl(LITERAL_B2);

    /**
     *  A literal value.
     *  @since  Version 0.1 2005-02-02
     */
    public static final String LITERAL_A3 =
    "http://test.mksearch.mkdoc.org/#A3";

    /**
     *  Result table value.
     *  @since  Version 0.1 2005-02-02
     */
    public static final Value VALUE_A3 = new URIImpl(LITERAL_A3);

    /**
     *  A null output string.
     *  @since  Version 0.1 2005-02-02
     */
    public static final String NULL = "<em>null</em>";

    /**
     *  The first row values.
     *  @since  Version 0.1 2005-02-02
     */
    protected List row1;

    /**
     *  The second row values.
     *  @since  Version 0.1 2005-02-02
     */
    protected List row2;

    /**
     *  The third row values.
     *  @since  Version 0.1 2005-02-02
     */
    protected List row3;

    /**
     *  A renderer to test.
     *  @since  Version 0.1 2005-02-02
     */
    protected XhtmlResultsRenderer renderer = new XhtmlResultsRenderer();

    /**
     *  A results table to test.
     *  @since  Version 0.1 2005-02-02
     */
    protected QueryResultsTable table;

    /**
     *  An empty results table to test.
     *  @since  Version 0.1 2005-02-02
     */
    protected QueryResultsTable tableEmpty;

    /**
     *  An output stream to test.
     *  @since  Version 0.1 2005-02-02
     */
    protected MockServletOutputStream outputStream;

    /**
     *  Named test instance.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2005-02-02
     */
    public XhtmlResultsRendererTest(final String name) {

        super(name);
    }

    /**
     *  Set up a test scenario as required.
     *  @since  Version 0.1 2005-02-02
     */
    protected final void setUp() {

        table = new QueryResultsTable(COL_NAMES.length,
                                      COL_NAMES);

        row1 = new Vector();

        row1.add(VALUE_A1);
        row1.add(VALUE_B1);

        table.addRow(row1);

        row2 = new Vector();

        row2.add(null);
        row2.add(VALUE_B2);

        table.addRow(row2);

        row3 = new Vector();

        row3.add(VALUE_A3);
        row3.add(null);

        table.addRow(row3);

        outputStream = new MockServletOutputStream();

        tableEmpty = new QueryResultsTable(COL_NAMES.length,
                                           COL_NAMES);

    }

    /**
     *  Get the content type.
     *  @since  Version 0.1 2005-02-02
     */
    public final void testGetContentType() {

        assertTrue(CONTENT_TYPE.equals(renderer.getContentType()));
    }

    /**
     *  Render results, no rows.
     *  @exception  IOException Should not be thrown by this test.
     *  @since  Version 0.1 2005-02-02
     */
    public final void testRenderResultsNoRows() throws IOException {

        renderer.renderResults(outputStream, tableEmpty);

        String results = outputStream.getOutput();

        assertTrue(results.indexOf("No results") > -1);
    }

    /**
     *  Render results, complete.
     *  @exception  IOException Should not be thrown by this test.
     *  @since  Version 0.1 2005-02-02
     */
    public final void testRenderResults() throws IOException {

        renderer.renderResults(outputStream, table);

        String results = outputStream.getOutput();

        assertTrue("Results output should include a link for row 1:\n\n" +
                   A_START_TAG +
                   LITERAL_A1 +
                   END_ATT_TAG +
                   LITERAL_A1 +
                   A_CLOSE_TAG + "\n\n" +
                   "Actual results were:\n\n" +
                   results,
                   results.indexOf(A_START_TAG +
                                   LITERAL_A1 +
                                   END_ATT_TAG +
                                   LITERAL_A1 +
                                   A_CLOSE_TAG) > -1);

        assertTrue("Results output should include a null for row 2:\n\n" +
                   DT_START +
                   NEWLINE +
                   NULL +
                   NEWLINE +
                   DT_CLOSE + "\n\n" +
                   "Actual results were:\n\n" +
                   results,
                   results.indexOf(DT_START +
                                   NEWLINE +
                                   NULL +
                                   NEWLINE +
                                   DT_CLOSE) > -1);

        assertTrue("Results output should include a link for row 3:\n\n" +
                   A_START_TAG +
                   LITERAL_A3 +
                   END_ATT_TAG +
                   LITERAL_A3 +
                   A_CLOSE_TAG + "\n\n" +
                   "Actual results were:\n\n" +
                   results,
                   results.indexOf(A_START_TAG +
                                   LITERAL_A3 +
                                   END_ATT_TAG +
                                   LITERAL_A3 +
                                   A_CLOSE_TAG) > -1);

    }

    /**
     *  Render results to null output stream.
     *  @exception  IOException Should not be thrown by this test.
     *  @since  Version 0.1 2005-02-02
     */
    public final void testRenderResultsNullOuputStream() throws IOException {

        try {

            renderer.renderResults(null, table);

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Render results to servlet output, null results table.
     *  @exception  IOException Should not be thrown by this test.
     *  @since  Version 0.1 2005-02-02
     */
    public final void testRenderResultsNullResultsTable() throws IOException {


        try {

            renderer.renderResults(outputStream, null);

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Render results to string, null results table.
     *  @exception  IOException Should not be thrown by this test.
     *  @since  Version 0.1 2005-02-02
     */
    public final void testRenderResultsToStringNullResultsTable()
        throws IOException {

        try {

            renderer.renderResults(null);

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Render results to string, complete.
     *  @exception  IOException Should not be thrown by this test.
     *  @since  Version 0.1 2005-02-02
     */
    public final void testRenderResultsToString() throws IOException {

        String results = renderer.renderResults(table);

        assertTrue("Results output should include a link for row 1:\n\n" +
                   A_START_TAG +
                   LITERAL_A1 +
                   END_ATT_TAG +
                   LITERAL_A1 +
                   A_CLOSE_TAG + "\n\n" +
                   "Actual results were:\n\n" +
                   results,
                   results.indexOf(A_START_TAG +
                                   LITERAL_A1 +
                                   END_ATT_TAG +
                                   LITERAL_A1 +
                                   A_CLOSE_TAG) > -1);

        assertTrue("Results output should include a null for row 2:\n\n" +
                   DT_START +
                   "\n" +
                   NULL +
                   "\n" +
                   DT_CLOSE + "\n\n" +
                   "Actual results were:\n\n" +
                   results,
                   results.indexOf(DT_START +
                                   "\n" +
                                   NULL +
                                   "\n" +
                                   DT_CLOSE) > -1);

        assertTrue("Results output should include a link for row 3:\n\n" +
                   A_START_TAG +
                   LITERAL_A3 +
                   END_ATT_TAG +
                   LITERAL_A3 +
                   A_CLOSE_TAG + "\n\n" +
                   "Actual results were:\n\n" +
                   results,
                   results.indexOf(A_START_TAG +
                                   LITERAL_A3 +
                                   END_ATT_TAG +
                                   LITERAL_A3 +
                                   A_CLOSE_TAG) > -1);

    }

    /**
     *  Render results, no rows.
     *  @exception  IOException Should not be thrown by this test.
     *  @since  Version 0.1 2005-02-02
     */
    public final void testRenderResultsToStringNoRows() throws IOException {

        String results = renderer.renderResults(tableEmpty);

        assertTrue(results.indexOf("No results") > -1);
    }

    /**
     *  Tear down a test scenario as required.
     *  @since  Version 0.1 2005-
    protected void tearDown() {

    }
     */

    /**
     *  Create a test suite for this test case.
     *  @return A test suite.
     *  @since  Version 0.1 2005-02-02
     */
    public static Test suite() {

        return new TestSuite(XhtmlResultsRendererTest.class);
    }

    /**
     *  Run this test suite as a stand alone application.
     *  @param  args    Any command line arguments to pass
     *          to the test. None required or expected.
     *  @since  Version 0.1 2005-02-02
     */
    public static void main(final String[] args) {

        junit.textui.TestRunner.run(suite());
    }

}
