package com.mkdoc.store;

/*
    Created:    2005-10-31
    Issued:     2005-10-31
    Modified:   2005-11-01

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import com.mkdoc.util.MockApplicationContext;
import com.mkdoc.util.ApplicationContextException;

import java.io.File;
import java.io.IOException;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  Test suite for <code>DatabaseStoreManager</code>.
 *  @version    0.2 2005-11-01
 *  @author Philip Shaw <phil@mkdoc.com>
 */
public class DatabaseStoreManagerTest extends TestCase {

    /**
     *  An expected ApplicationContextException message.
     *  @since  Version 0.1 2005-10-31
     */
    public static final String EXPECTED_ACE =
    "Should throw an ApplicationContextException.";

    /**
     *  An expected <code>IllegalArgumentException</code> message.
     *  @since  Version 0.2 2005-11-01
     */
    private static final String EXPECTED_IAE =
    "Should throw an IllegalArgumentException.";

    /**
     *  The project source directory.
     *  @since  Version 0.1 2005-10-31
     */
    protected static final String HOME = System.getProperty("mkHome");

    /**
     *  The default JSpider output directory.
     *  @since  Version 0.1 2005-10-31
     */
    protected static final String OUTPUT = System.getProperty("outputDir");

    /**
     *  The default JSpider output path.
     *  @since  Version 0.1 2005-10-31
     */
    protected static final String PATH_OUT = HOME +
                                             File.separator +
                                             OUTPUT;

    /**
     *  The database vendor.
     *  @since  Version 0.1 2005-10-31
     */
    protected static final String DB_VENDOR = System.getProperty("dbVendor");

    /**
     *  The postgresql database vendor.
     *  @since  Version 0.1 2005-10-31
     */
    protected static final String PGSQL_VENDOR = "pgsql";

    /**
     *  The mysql database vendor.
     *  @since  Version 0.1 2005-10-31
     */
    protected static final String MYSQL_VENDOR = "mysql";

    /**
     *  An unrecognised database vendor.
     *  @since  Version 0.1 2005-10-31
     */
    protected static final String BAD_VENDOR = "nosql";

    /**
     *  The database name.
     *  @since  Version 0.1 2005-10-31
     */
    protected static final String DB_NAME = System.getProperty("dbName");

    /**
     *  A bad database name.
     *  @since  Version 0.1 2005-10-31
     */
    protected static final String BAD_NAME = "dbNotExist";

    /**
     *  The database user name.
     *  @since  Version 0.1 2005-10-31
     */
    protected static final String DB_USER = System.getProperty("dbUser");

    /**
     *  A bad database user name.
     *  @since  Version 0.1 2005-10-31
     */
    protected static final String BAD_USER = "HoodedClaw";

    /**
     *  The database user password.
     *  @since  Version 0.1 2005-10-31
     */
    protected static final String DB_PASS = System.getProperty("dbPass");


    /**
     *  A file reference to the test output directory.
     *  @since  Version 0.1 2005-10-27
     */
    protected static final File DIR_OUT = new File(PATH_OUT);

    /**
     *  An application context for testing.
     *  @since  Version 0.1 2005-10-31
     */
    protected MockApplicationContext context;

    /**
     *  A subject manager to test.
     *  @since  Version 0.1 2005-10-31
     */
    protected DatabaseStoreManager manager;

    /**
     *  Named test instance.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2005-10-31
     */
    public DatabaseStoreManagerTest(final String name) {

        super(name);
    }

    /**
     *  Set up a test scenario as required.
     *  @since  Version 0.1 2005-10-31
     */
    protected final void setUp() {

        context = new MockApplicationContext(DIR_OUT);

        manager = new DatabaseStoreManager();
    }

    /**
     *  Test initialize.
     *  @exception  IOException Should not be thrown by this test.
     *  @since  Version 0.2 2005-11-01
     */
    public final void testInitialize() throws IOException {

        assertFalse(manager.isActive());

        assertNull(manager.repository);

        context = new MockApplicationContext(new File(PATH_OUT));

        context.setParameter(DatabaseStoreManager.DB_VENDOR,
                             DB_VENDOR);

        context.setParameter(DatabaseStoreManager.DB_NAME,
                             DB_NAME);

        context.setParameter(DatabaseStoreManager.DB_USER,
                             DB_USER);

        context.setParameter(DatabaseStoreManager.DB_PASS,
                             DB_PASS);

        manager.initialize(context);

        assertTrue(manager.isActive());

        assertNotNull(manager.repository);
    }

    /**
     *  Initialize and check the store file path.
     *  @exception  IOException Should not be thrown by this test.
     *  @since  Version 0.2 2005-11-01
     */
    public final void testInitializeNullPropertySet() throws IOException {

        try {

            manager.initialize(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Test shutdown.
     *  @since  Version 0.2 2005-11-01
     */
    public final void testShutdownIsInitialized() {

        context.setParameter(DatabaseStoreManager.DB_VENDOR,
                             DB_VENDOR);

        context.setParameter(DatabaseStoreManager.DB_NAME,
                             DB_NAME);

        context.setParameter(DatabaseStoreManager.DB_USER,
                             DB_USER);

        context.setParameter(DatabaseStoreManager.DB_PASS,
                             DB_PASS);

        manager.initialize(context);

        assertTrue(manager.isActive());

        manager.shutdown();

        assertFalse(manager.isActive());
    }

    /**
     *  Test shutdown not set up, should silently ignore.
     *  @since  Version 0.2 2005-11-01
     */
    public final void testShutdownNotInitialized() {

        assertFalse(manager.isActive());

        assertNull(manager.repository);

        manager.shutdown();

        assertFalse(manager.isActive());
    }

    /**
     *  Test configure repository.
     *  @since  Version 0.1 2005-10-31
     */
    public final void testConfigureRepositoryNullContext() {

        try {

            manager.configureRepository(null);

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test configure repository.
     *  @since  Version 0.1 2005-10-31
     */
    public final void testConfigureRepositorySuccess() {

        context.setParameter(DatabaseStoreManager.DB_VENDOR,
                             DB_VENDOR);

        context.setParameter(DatabaseStoreManager.DB_NAME,
                             DB_NAME);

        context.setParameter(DatabaseStoreManager.DB_USER,
                             DB_USER);

        context.setParameter(DatabaseStoreManager.DB_PASS,
                             DB_PASS);

        manager.configureRepository(context);
    }

    /**
     *  Test configure repository.
     *  @since  Version 0.1 2005-10-31
     */
    public final void testConfigureRepositoryOtherVendor() {

        if (DB_VENDOR.equals(DatabaseStoreManager.PGSQL)) {

            context.setParameter(DatabaseStoreManager.DB_VENDOR,
                                 DatabaseStoreManager.MYSQL);
        }
        else {

            context.setParameter(DatabaseStoreManager.DB_VENDOR,
                                 DatabaseStoreManager.PGSQL);
        }

        context.setParameter(DatabaseStoreManager.DB_NAME,
                             DB_NAME);

        context.setParameter(DatabaseStoreManager.DB_USER,
                             DB_USER);

        context.setParameter(DatabaseStoreManager.DB_PASS,
                             DB_PASS);

        try {

            manager.configureRepository(context);

            fail(EXPECTED_ACE);
        }
        catch (ApplicationContextException ace) { }

    }


    /**
     *  Test configure repository.
     *  @since  Version 0.1 2005-10-31
     */
    public final void testConfigureRepositoryBadVendor() {

        context.setParameter(DatabaseStoreManager.DB_VENDOR,
                             BAD_VENDOR);

        context.setParameter(DatabaseStoreManager.DB_NAME,
                             DB_NAME);

        context.setParameter(DatabaseStoreManager.DB_USER,
                             DB_USER);

        context.setParameter(DatabaseStoreManager.DB_PASS,
                             DB_PASS);

        try {

            manager.configureRepository(context);

            fail(EXPECTED_ACE);
        }
        catch (ApplicationContextException ace) { }
    }

    /**
     *  Test configure repository.
     *  @since  Version 0.1 2005-10-31
     */
    public final void testConfigureRepositoryBadUser() {

        context.setParameter(DatabaseStoreManager.DB_VENDOR,
                             DB_VENDOR);

        context.setParameter(DatabaseStoreManager.DB_NAME,
                             DB_NAME);

        context.setParameter(DatabaseStoreManager.DB_USER,
                             BAD_USER);

        context.setParameter(DatabaseStoreManager.DB_PASS,
                             DB_PASS);

        try {

            manager.configureRepository(context);

            fail(EXPECTED_ACE);
        }
        catch (ApplicationContextException ace) { }
    }

    /**
     *  Test configure repository, null vendor.
     *  @since  Version 0.1 2005-10-31
     */
    public final void testConfigureRepositoryNullVendor() {

        context.setParameter(DatabaseStoreManager.DB_NAME,
                             DB_NAME);

        context.setParameter(DatabaseStoreManager.DB_USER,
                             DB_USER);

        context.setParameter(DatabaseStoreManager.DB_PASS,
                             DB_PASS);

        try {

            manager.configureRepository(context);

            fail(EXPECTED_ACE);
        }
        catch (ApplicationContextException ace) { }
    }


    /**
     *  Test configure repository.
     *  @since  Version 0.1 2005-10-31
     */
    public final void testConfigureRepositoryNullDbName() {

        context.setParameter(DatabaseStoreManager.DB_VENDOR,
                             DB_VENDOR);

        context.setParameter(DatabaseStoreManager.DB_USER,
                             DB_USER);

        context.setParameter(DatabaseStoreManager.DB_PASS,
                             DB_PASS);

        try {

            manager.configureRepository(context);

            fail(EXPECTED_ACE);
        }
        catch (ApplicationContextException ace) { }
    }

    /**
     *  Tear down a test scenario as required.
     *  @since  Version 0.1 2005-10-31
    protected void tearDown() {

    }
     */

    /**
     *  Create a test suite for this test case.
     *  @return A test suite.
     *  @since  Version 0.1 2005-10-31
     */
    public static Test suite() {

        return new TestSuite(DatabaseStoreManagerTest.class);
    }

    /**
     *  Run this test suite as a stand alone application.
     *  @param  args    Any command line arguments to pass
     *          to the test. None required or expected.
     *  @since  Version 0.1 2005-10-31
     */
    public static void main(final String[] args) {

        junit.textui.TestRunner.run(suite());
    }

}
