package com.mkdoc.store;

/*
    Created:    2005-05-17
    Issued:     2005-05-17
    Modified:   2005-11-01

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import org.openrdf.model.Graph;
import org.openrdf.model.Resource;
import org.openrdf.model.Statement;
import org.openrdf.model.URI;
import org.openrdf.model.Value;

import org.openrdf.sesame.sail.StatementIterator;

import com.mkdoc.query.Query;
import com.mkdoc.query.QueryResult;
import com.mkdoc.query.NullQueryResult;

import com.mkdoc.util.ApplicationContext;

import java.io.OutputStream;
import java.io.InputStream;

import java.net.URL;

/**
 *  A type that implements all manager interfaces.
 *  @version    0.2 2005-11-01
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class MockStoreManager implements StoreManager {

    /**
     *  An output stream reference.
     *  @since  Version 0.2 2005-11-01
     */
    protected OutputStream outStream;

    /**
     *  The most recently added graph.
     *  @since  Version 0.2 2005-11-01
     */
    protected Graph graph;

    /**
     *  The most recently added subject URI.
     *  @since  Version 0.2 2005-11-01
     */
    protected URI uri;

    /**
     *  Whether the manager is initialized.
     *  @since  Version 0.2 2005-11-01
     */
    protected boolean initialized;

    /**
     *  Whether the manager is shutdown.
     *  @since  Version 0.2 2005-11-01
     */
    protected boolean active;

    /**
     *  Whether to fail on get output.
     *  @since  Version 0.2 2005-11-01
     */
    protected boolean failOnOutput;

    /**
     *  Whether to fail on adding a graph.
     *  @since  Version 0.2 2005-11-01
     */
    protected boolean failOnAdd;

    /**
     *  A default constructor, no output stream.
     *  @since  Version 0.2 2005-11-01
     */
    public MockStoreManager() {

        super();
    }

    /**
     *  A constructor with output stream.
     *  @param  outStream   An output stream to which RDF statements will be
     *          written.
     *  @since  Version 0.2 2005-11-01
     */
    public MockStoreManager(final OutputStream outStream) {

        super();

        this.outStream = outStream;
    }

    /**
     *  Set whether to fail on get output.
     *  @param  setting Whether to fail on get output.
     *  @since  Version 0.2 2005-11-01
     */
    public final void setFailOnGetOutput(final boolean setting) {

        failOnOutput = setting;
    }

    /**
     *  Set whether to fail on get output.
     *  @param  setting Whether to fail on get output.
     *  @since  Version 0.2 2005-11-01
     */
    public final void setFailOnAdd(final boolean setting) {

        failOnAdd = setting;
    }

    /**
     *  Assign an output stream.
     *  @param  outStream   An output stream to which RDF statements will be
     *          written.
     *  @since  Version 0.2 2005-11-01
     */
    public final void setOutputStream(final OutputStream outStream) {

        this.outStream = outStream;
    }

    /**
     *  Get the most recent graph added to the manager.
     *  @return The most recent graph added to the manager.
     *  @since  Version 0.2 2005-11-01
     */
    public final Graph getGraph() {

        return graph;
    }

    /**
     *  Get the most recent subject URI added to the manager.
     *  @return The most recent subject URI added to the manager.
     *  @since  Version 0.2 2005-11-01
     */
    public final URI getSubjectUri() {

        return uri;
    }

    /**
     *  Get the most recently added graph.
     *  @return A line per statement list of the most
     *          recently added <code>Graph</code>.
     *  @since  Version 0.2 2005-11-01
     */
    public final String getGraphString() {

        StatementIterator iterator = graph.getStatements();

        StringBuffer buffer = new StringBuffer();

        Statement statement = null;

        Resource subject = null;

        URI predicate = null;

        Value object = null;

        while (iterator.hasNext()) {

            statement = iterator.next();

            subject = statement.getSubject();

            predicate = statement.getPredicate();

            object = statement.getObject();

            buffer.append("Subject: ");
            buffer.append(subject);
            buffer.append(", Predicate: ");
            buffer.append(predicate);
            buffer.append(", Object: ");
            buffer.append(object);
            buffer.append(".\n");
        }

        return buffer.toString();
    }

    /**
     *  Add statements about a document by subject URI.
     *  @param  subjectUri  The URI of the subject document to add.
     *  @param  graph   The RDF graph of statements about this document.
     *  @exception  StoreException  If an error occurs with the triple store.
     *  @since  Version 0.2 2005-11-01
     */
    public final void addDocumentStatements(final URI subjectUri,
                                            final Graph graph)
        throws StoreException {

        if (failOnAdd) {

            throw new StoreException(
                        "MockStoreManager throws StoreException.");
        }

        this.graph = graph;
        uri = subjectUri;
    }

    /**
     *  Issue an output stream to which RDF statements can be passed directly.
     *  <p>
     *    <em>Temporary measure to adapt plugin <code>writeFile</code> methods.
     *  </p>
     *  @param  subject The subject of the RDF statements.
     *  @return An output stream to write RDF statements to.
     *  @exception  StoreException  If an error occurs with the triple store.
     *  @since  Version 0.2 2005-11-01
     */
    public final OutputStream getOutputStream(final URL subject)
        throws StoreException {

        if (failOnOutput) {

            throw new StoreException(
                        "MockStoreManager throws StoreException.");
        }

        return outStream;
    }

    /**
     *  Initialize the manager.
     *  @param  context The application context for the manager.
     *  @since  Version 0.2 2005-11-01
     */
    public final void initialize(final ApplicationContext context) {

        initialized = true;

        active = true;
    }

    /**
     *  Check whether the manager is initialized.
     *  @return <code>true</code> if the <code>initialize</code> method has
     *          been called.
     *  @since  Version 0.2 2005-11-01
     */
    public final boolean isInitialized() {

        return initialized;
    }

    /**
     *  Shut down the manager.
     *  @since  Version 0.2 2005-11-01
     */
    public final void shutdown() {

        active = false;

        initialized = false;
    }

    /**
     *  Check whether the manager is shutdown.
     *  @return <code>true</code> if the <code>shutdown</code> method has
     *          been called.
     *  @since  Version 0.2 2005-11-01
     */
    public final boolean isShutdown() {

        return !active;
    }

    /**
     *  Add an RDF data stream to the store.
     *  @param  inStream    The data stream to add to the store.
     *  @param  baseUri The URL base of the source.
     *  @exception  StoreException  If there is a problem importing the data to
     *          the repository.
     *  @since  Version 0.2 2005-11-01
     */
    public final void addXmlRdfStream(final InputStream inStream,
                                      final URL baseUri)
        throws StoreException {

        // Empty;
    }

    /**
     *  Execute a query against the data store.
     *  @param  query   The query to apply to the store.
     *  @return A result set that fulfils the submitted query.
     *  @since  Version 0.1 2005-07-19
     */
    public final QueryResult executeQuery(final Query query) {

        return new NullQueryResult();
    }

}
