package com.mkdoc.store;

/*
    Created:    2005-10-21
    Issued:     2005-10-21
    Modified:   2005-11-02

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import com.mkdoc.util.ApplicationContext;

import java.io.OutputStream;

import java.net.URL;

/**
 *  Mock file system manager for testing.
 *  @version    0.2 2005-11-02
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class MockVirtualFileStoreManager implements FileStoreManager {

    /**
     *  The extension for storage files.
     *  @return An type-specific file extension.
     *  @since  Version 0.1 2005-10-21
     */
    private static final String EXTENSION = ".tmp";

    /**
     *  An output stream reference.
     *  @since  Version 0.1 2005-10-21
     */
    protected OutputStream outStream;

    /**
     *  Whether the manager is initialized.
     *  @since  Version 0.1 2005-10-21
     */
    protected boolean initialized;

    /**
     *  Whether the manager is shutdown.
     *  @since  Version 0.1 2005-10-21
     */
    protected boolean shut;

    /**
     *  Whether to fail on get output.
     *  @since  Version 0.1 2005-10-21
     */
    protected boolean failOnOutput;

    /**
     *  Whether to fail on adding a graph.
     *  @since  Version 0.1 2005-10-21
     */
    protected boolean failOnAdd;

    /**
     *  A default constructor, no output stream.
     *  @since  Version 0.1 2005-10-21
     */
    public MockVirtualFileStoreManager() {

        super();
    }

    /**
     *  A constructor with output stream.
     *  @param  outStream   An output stream to which RDF statements will be
     *          written.
     *  @since  Version 0.1 2005-10-21
     */
    public MockVirtualFileStoreManager(final OutputStream outStream) {

        super();

        this.outStream = outStream;
    }

    /**
     *  Set whether to fail on get output.
     *  @param  setting Whether to fail on get output.
     *  @since  Version 0.1 2005-10-21
     */
    public final void setFailOnGetOutput(final boolean setting) {

        failOnOutput = setting;
    }

    /**
     *  Set whether to fail on get output.
     *  @param  setting Whether to fail on get output.
     *  @since  Version 0.1 2005-10-21
     */
    public final void setFailOnAdd(final boolean setting) {

        failOnAdd = setting;
    }

    /**
     *  Assign an output stream.
     *  @param  outStream   An output stream to which RDF statements will be
     *          written.
     *  @since  Version 0.1 2005-10-21
     */
    public final void setOutputStream(final OutputStream outStream) {

        this.outStream = outStream;
    }

    /**
     *  Issue an output stream to which RDF statements can be passed directly.
     *  <p>
     *    <em>Temporary measure to adapt plugin <code>writeFile</code> methods.
     *  </p>
     *  @param  subject The subject of the RDF statements.
     *  @return An output stream to write RDF statements to.
     *  @exception  StoreException  If an error occurs with the triple store.
     *  @since  Version 0.1 2005-04-29
     */
    public final OutputStream getOutputStream(final URL subject)
        throws StoreException {

        if (failOnOutput || shut) {

            throw new StoreException(
                        "MockSubjectManager throws StoreException.");
        }

        return outStream;
    }

    /**
     *  Initialize the manager.
     *  @param  context The application context for the manager.
     *  @since  Version 0.1 2005-03-31
     */
    public final void initialize(final ApplicationContext context) {

        initialized = true;

        shut = false;
    }

    /**
     *  Check whether the manager is initialized.
     *  @return <code>true</code> if the <code>initialize</code> method has
     *          been called.
     *  @since  Version 0.2 2005-10-21
     */
    public final boolean isInitialized() {

        return initialized;
    }

    /**
     *  Shut down the manager.
     *  @since  Version 0.1 2005-03-31
     */
    public final void shutdown() {

        shut = true;

        initialized = false;
    }

    /**
     *  Check whether the manager is shutdown.
     *  @return <code>true</code> if the <code>shutdown</code> method has
     *          been called.
     *  @since  Version 0.2 2005-10-21
     */
    public final boolean isShutdown() {

        return shut;
    }

    /**
     *  Get the extension for storage files.
     *  @return An type-specific file extension.
     *  @since  Version 0.1 2005-10-21
     */
    protected final String getExtension() {

        return EXTENSION;
    }

}
