package com.mkdoc.tag;

/*
    Created:    2005-06-28
    Issued:     2005-06-28
    Modified:   2005-10-14

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import com.mkdoc.servlet.MockServletRequest;
import com.mkdoc.servlet.MockServletResponse;
import com.mkdoc.servlet.MockHttpServletResponse;
import com.mkdoc.servlet.MockHttpServletRequest;

import com.mkdoc.jsp.MockJspWriter;
import com.mkdoc.jsp.MockPageContext;

import com.mkdoc.schema.ApplicationProfile;
import com.mkdoc.schema.DublinCoreProfile;
import com.mkdoc.schema.DublinCoreTerms;
import com.mkdoc.schema.Schema;
import com.mkdoc.schema.SchemaProperty;

import java.io.IOException;

import java.util.List;
import java.util.Vector;

import javax.servlet.jsp.JspException;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  Test suite for <code>QueryBuilderTag</code>.
 *  @version    0.4 2005-10-14
 *  @author Philip Shaw <phil@mkdoc.com>
 */
public class QueryBuilderTagTest extends TestCase {

    /**
     *  The query handler path.
     *  @since  Version 0.1 2005-07-04
     */
    public static final String QUERY_PATH = "/QueryHandler";

    /**
     *  A test page.
     *  @since  Version 0.1 0.3 2005-09-13
     */
    public static final String TEST_PAGE = "/TestPage.jsp";

    /**
     *  The builder page path.
     *  @since  Version 0.1 2005-07-04
     */
    public static final String BUILDER_PATH = "/Builder.jsp";

    /**
     *  The DC Title predicate.
     *  @since  Version 0.2 2005-07-25
     */
    public static final String DC_TITLE = "dc.title";

    /**
     *  A test parameter name.
     *  @since  Version 0.1 2005-07-04
     */
    public static final String PARAM_1 = "test";

    /**
     *  An expected <code>IllegalArgumentException</code> message.
     *  @since  Version 0.1 2005-06-28
     */
    public static final String EXPECTED_IAE =
    "Should throw an IllegalArgumentException.";

    /**
     *  A DC Created predicate.
     *  @since  Version 0.1 2005-07-04
     */
    public static final String DC_CREATOR = "dc.creator";

    /**
     *  A query builder tag to test.
     *  @since  Version 0.1 2005-06-29
     */
    protected QueryBuilderTag tag;

    /**
     *  A schema implementation for testing.
     *  @since  Version 0.1 2005-07-04
     */
    protected Schema schema = new DublinCoreTerms();

    /**
     *  A profile implementation for testing.
     *  @since  Version 0.1 2005-07-04
     */
    protected ApplicationProfile dcProfile = new DublinCoreProfile();

    /**
     *  A page context.
     *  @since  Version 0.1 2005-06-29
     */
    protected MockPageContext pageContext;

    /**
     *  Named test instance.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2005-06-28
     */
    public QueryBuilderTagTest(final String name) {

        super(name);
    }

    /**
     *  Set up a test scenario as required.
     *  @since  Version 0.1 2005-06-28
     */
    protected final void setUp() {

        tag = new QueryBuilderTag();

        pageContext = new MockPageContext();

        tag.setPageContext(pageContext);
    }

    /**
     *  Concatenate servlet request parameter values.
     *  @since  Version 0.1 2005-06-29
     */
    public final void testGetParameterValues() {

        MockHttpServletRequest msreq = new MockHttpServletRequest();

        String paramName = PARAM_1;

        msreq.setParameter(paramName, "One");

        msreq.setParameter(paramName, "Two");

        msreq.setParameter(paramName, "Three");

        String values = tag.getParameterValues(msreq, paramName);

        String expected = "One Two Three";

        assertTrue(expected.equals(values));

    }

    /**
     *  Concatenate servlet request parameter values.
     *  @since  Version 0.1 2005-06-29
     */
    public final void testGetParameterValuesOneValue() {

        MockHttpServletRequest msreq = new MockHttpServletRequest();

        String paramName = PARAM_1;

        msreq.setParameter(paramName, "One");

        String values = tag.getParameterValues(msreq, paramName);

        String expected = "One";

        assertTrue(expected.equals(values));

    }


    /**
     *  Concatenate servlet request parameter values.
     *  @since  Version 0.1 2005-06-29
     */
    public final void testGetParameterValuesNoValues() {

        MockHttpServletRequest msreq = new MockHttpServletRequest();

        String paramName = PARAM_1;

        String values = tag.getParameterValues(msreq, paramName);

        String expected = "";

        assertTrue(expected.equals(values));

    }

    /**
     *  Concatenate servlet request parameter values.
     *  @since  Version 0.1 2005-06-29
     */
    public final void testGetParameterValuesEmtpyValues() {

        MockHttpServletRequest msreq = new MockHttpServletRequest();

        String paramName = PARAM_1;

        msreq.setParameter(paramName, "");

        msreq.setParameter(paramName, "");

        msreq.setParameter(paramName, "");

        String values = tag.getParameterValues(msreq, paramName);

        String expected = "";

        assertTrue("Should return the empty string, actually returned '" +
                   values + "'.",
                   expected.equals(values));

    }


    /**
     *  Concatenate servlet request parameter values.
     *  @since  Version 0.1 2005-06-29
     */
    public final void testGetParameterValuesNullRequest() {

        MockHttpServletRequest msreq = null;

        try {

            tag.getParameterValues(msreq, PARAM_1);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Concatenate servlet request parameter values.
     *  @since  Version 0.1 2005-06-29
     */
    public final void testGetParameterValuesNullName() {

        MockHttpServletRequest msreq = new MockHttpServletRequest();

        try {

            tag.getParameterValues(msreq, null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Concatenate servlet request parameter values.
     *  @since  Version 0.1 2005-06-29
     */
    public final void testSetQueryUrlNull() {

        try {

            tag.setQueryUrl(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Concatenate servlet request parameter values.
     *  @since  Version 0.1 2005-06-29
     */
    public final void testSetGetQueryUrlGenericServletTypes() {

        String action = TEST_PAGE;

        MockServletResponse msres = new MockServletResponse();

        pageContext.setResponse(msres);

        MockServletRequest msreq = new MockServletRequest();

        pageContext.setRequest(msreq);

        tag.setQueryUrl(action);

        assertTrue(action.equals(tag.getQueryUrl()));
    }

    /**
     *  Concatenate servlet request parameter values.
     *  @since  Version 0.1 2005-06-29
     */
    public final void testSetGetQueryUrlHttpServletResponse() {

        String action = TEST_PAGE;

        MockHttpServletResponse msres = new MockHttpServletResponse();

        pageContext.setResponse(msres);

        MockHttpServletRequest msreq = new MockHttpServletRequest();

        pageContext.setRequest(msreq);

        tag.setQueryUrl(action);

        assertTrue(action.equals(tag.getQueryUrl()));
    }

    /**
     *  Concatenate servlet request parameter values.
     *  @since  Version 0.1 2005-06-29
     */
    public final void testSetGetQueryUrlWithContextPath() {

        String action = TEST_PAGE;

        MockHttpServletResponse msres = new MockHttpServletResponse();

        pageContext.setResponse(msres);

        MockHttpServletRequest msreq = new MockHttpServletRequest();

        String contextPath = "/test";

        msreq.setContextPath(contextPath);

        pageContext.setRequest(msreq);

        tag.setQueryUrl(action);

        String expected = contextPath + action;

        assertTrue(expected.equals(tag.getQueryUrl()));
    }

    /**
     *  Concatenate servlet request parameter values.
     *  @since  Version 0.1 2005-06-29
     */
    public final void testSetGetQueryUrlHttpServletResponseWithRewriting() {

        String sessionId = "RememberMeThisWay";

        MockHttpServletResponse msres = new MockHttpServletResponse(sessionId,
                                                                    true);

        pageContext.setResponse(msres);

        MockHttpServletRequest msreq = new MockHttpServletRequest();

        pageContext.setRequest(msreq);

        String action = TEST_PAGE;

        tag.setQueryUrl(action);

        String url = tag.getQueryUrl();

        assertTrue(url.indexOf(sessionId) > -1);
    }

    /**
     *  Concatenate servlet request parameter values.
     *  @since  Version 0.3 2005-09-12
     */
    public final void testSetBuilderUrlNull() {

        try {

            tag.setBuilderUrl(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Concatenate servlet request parameter values.
     *  @since  Version 0.3 2005-09-12
     */
    public final void testSetGetBuilderUrlGenericServletTypes() {

        String action = TEST_PAGE;

        MockServletResponse msres = new MockServletResponse();

        pageContext.setResponse(msres);

        MockServletRequest msreq = new MockServletRequest();

        pageContext.setRequest(msreq);

        tag.setBuilderUrl(action);

        assertTrue(action.equals(tag.getBuilderUrl()));
    }

    /**
     *  Concatenate servlet request parameter values.
     *  @since  Version 0.3 2005-09-12
     */
    public final void testSetGetBuilderUrlHttpServletResponse() {

        String action = TEST_PAGE;

        MockHttpServletResponse msres = new MockHttpServletResponse();

        pageContext.setResponse(msres);

        MockHttpServletRequest msreq = new MockHttpServletRequest();

        pageContext.setRequest(msreq);

        tag.setBuilderUrl(action);

        assertTrue(action.equals(tag.getBuilderUrl()));
    }

    /**
     *  Concatenate servlet request parameter values.
     *  @since  Version 0.3 2005-09-12
     */
    public final void testSetGetBuilderUrlWithContextPath() {

        String action = TEST_PAGE;

        MockHttpServletResponse msres = new MockHttpServletResponse();

        pageContext.setResponse(msres);

        MockHttpServletRequest msreq = new MockHttpServletRequest();

        String contextPath = "/test";

        msreq.setContextPath(contextPath);

        pageContext.setRequest(msreq);

        tag.setBuilderUrl(action);

        String expected = contextPath + action;

        assertTrue(expected.equals(tag.getBuilderUrl()));
    }



    /**
     *  Concatenate servlet request parameter values.
     *  @since  Version 0.3 2005-09-12
     */
    public final void testSetGetBuilderUrlHttpServletResponseWithRewriting() {

        String sessionId = "RememberMeThisWay";

        MockHttpServletResponse msres = new MockHttpServletResponse(sessionId,
                                                                    true);

        pageContext.setResponse(msres);

        MockHttpServletRequest msreq = new MockHttpServletRequest();

        pageContext.setRequest(msreq);

        String action = TEST_PAGE;

        tag.setBuilderUrl(action);

        String url = tag.getBuilderUrl();

        assertTrue(url.indexOf(sessionId) > -1);
    }

    /**
     *  Output the start tag.
     *  @exception  JspException    Should not be thrown by this test.
     *  @since  Version 0.1 2005-06-29
     */
    public final void testDoStartTagResponseRewriteBuilderAction()
        throws JspException {

        MockHttpServletRequest msreq = new MockHttpServletRequest();

        pageContext.setRequest(msreq);

        String sessionId = "RememberMeThisWay";

        MockHttpServletResponse msres = new MockHttpServletResponse(sessionId,
                                                                    true);

        pageContext.setResponse(msres);

        MockJspWriter jspWriter = new MockJspWriter();

        pageContext.setOut(jspWriter);


        tag.setBuilderUrl(BUILDER_PATH);

        tag.doStartTag();

        String out = jspWriter.getOut();

        String expected = " action=\"" +
                          BUILDER_PATH +
                          ";jsessionid=" +
                          sessionId +
                          "\"";

        assertTrue(out.indexOf(expected) > -1);

    }

    /**
     *  Output the start tag.
     *  @exception  JspException    Should not be thrown by this test.
     *  @since  Version 0.1 2005-06-29
     */
    public final void testDoStartTagResponseNoRewriting()
        throws JspException {

        MockHttpServletRequest msreq = new MockHttpServletRequest();

        pageContext.setRequest(msreq);

        MockHttpServletResponse msres = new MockHttpServletResponse();

        pageContext.setResponse(msres);

        MockJspWriter jspWriter = new MockJspWriter();

        pageContext.setOut(jspWriter);

        tag.setBuilderUrl(BUILDER_PATH);

        tag.doStartTag();

        String out = jspWriter.getOut();

        String expected = " action=\"" +
                          BUILDER_PATH +
                          "\"";

        assertTrue(out.indexOf(expected) > -1);

    }

    /**
     *  Output the start tag.
     *  @since  Version 0.1 2005-06-29
     */
    public final void testDoStartTagGenericServlet() {

        MockServletRequest msreq = new MockServletRequest();

        pageContext.setRequest(msreq);

        MockServletResponse msres = new MockServletResponse();

        pageContext.setResponse(msres);

        MockJspWriter jspWriter = new MockJspWriter();

        pageContext.setOut(jspWriter);

        try {

            tag.doStartTag();

            fail("Should throw a JspException.");
        }
        catch (JspException je) { }

    }

    /**
     *  Output the start tag.
     *  @since  Version 0.1 2005-06-29
     */
    public final void testDoStartTagThrowsIOException() {

        MockHttpServletRequest msreq = new MockHttpServletRequest();

        pageContext.setRequest(msreq);

        MockHttpServletResponse msres = new MockHttpServletResponse();

        pageContext.setResponse(msres);

        MockJspWriter jspWriter = new MockJspWriter();

        jspWriter.setFailOnPrint(true);

        pageContext.setOut(jspWriter);

        tag.setBuilderUrl(BUILDER_PATH);

        try {

            tag.doStartTag();

            fail("Should throw a JspException.");
        }
        catch (JspException je) { }

    }

    /**
     *  Print the query form.
     *  @exception  IOException    Should not be thrown by this test.
     *  @since  Version 0.1 2005-06-29
     */
    public final void testPrintQueryFormNullRequest()
        throws IOException {

        try {

            tag.printQueryForm(null, new MockJspWriter());

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Print the query form.
     *  @exception  IOException    Should not be thrown by this test.
     *  @since  Version 0.1 2005-06-29
     */
    public final void testPrintQueryFormNullJspWriter()
        throws IOException {

        try {

            tag.printQueryForm(new MockHttpServletRequest(), null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Print the builder form.
     *  @exception  IOException    Should not be thrown by this test.
     *  @since  Version 0.1 2005-07-04
     */
    public final void testPrintBuilderFormNullRequest()
        throws IOException {

        try {

            tag.printBuilderForm(null,
                                 new MockHttpServletResponse(),
                                 new MockJspWriter());

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Print the builder form.
     *  @exception  IOException    Should not be thrown by this test.
     *  @since  Version 0.1 2005-07-04
     */
    public final void testPrintBuilderFormNullResponse()
        throws IOException {

        try {

            tag.printBuilderForm(new MockHttpServletRequest(),
                                 null,
                                 new MockJspWriter());

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Test addition of selector options.
     *  @exception  IOException Should not be thrown by this test.
     *  @since  Version 0.1 2005-07-04
     */
    public final void testPrintBuilderFormNoneSelected()
        throws IOException {

        MockHttpServletRequest msreq = new MockHttpServletRequest();

        pageContext.setRequest(msreq);

        MockHttpServletResponse msres = new MockHttpServletResponse();

        pageContext.setResponse(msres);

        MockJspWriter out = new MockJspWriter();

        pageContext.setOut(out);

        tag.setBuilderUrl(BUILDER_PATH);

        tag.printBuilderForm(msreq, msres, out);

        String output = out.getOut();

        assertTrue(output.indexOf(" label=\"Dublin Core Elements\"") > -1);
        assertTrue(output.indexOf(" label=\"Dublin Core Qualifiers\"") > -1);

        assertTrue(output.indexOf("value=\"dcterms.audience\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.alternative\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.tableOfContents\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.abstract\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.created\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.valid\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.available\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.issued\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.modified\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.extent\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.medium\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.isVersionOf\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.hasVersion\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.isReplacedBy\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.replaces\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.isRequiredBy\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.requires\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.isPartOf\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.hasPart\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.isReferencedBy\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.references\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.isFormatOf\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.hasFormat\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.conformsTo\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.spatial\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.temporal\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.mediator\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.dateAccepted\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.dateCopyrighted\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.dateSubmitted\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.educationLevel\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.accessRights\"") > -1);
        assertTrue(output.indexOf(
                              "value=\"dcterms.bibliographicCitation\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.license\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.rightsHolder\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.provenance\"") > -1);

    }

    /**
     *  Test addition of selector options.
     *  @exception  IOException Should not be thrown by this test.
     *  @since  Version 0.1 2005-07-04
     */
    public final void testAddSelectorOptionsOneSelected()
        throws IOException {

        MockHttpServletRequest msreq = new MockHttpServletRequest();

        msreq.setParameter("dcterms.audience", "");

        pageContext.setRequest(msreq);

        MockHttpServletResponse msres = new MockHttpServletResponse();

        pageContext.setResponse(msres);

        MockJspWriter out = new MockJspWriter();

        pageContext.setOut(out);

        tag.setBuilderUrl(BUILDER_PATH);

        tag.printBuilderForm(msreq, msres, out);

        String output = out.getOut();

        assertTrue(output.indexOf(" label=\"Dublin Core Elements\"") > -1);
        assertTrue(output.indexOf(" label=\"Dublin Core Qualifiers\"") > -1);

        assertTrue(output.indexOf("value=\"dcterms.audience\"") == -1);
        assertTrue(output.indexOf("value=\"dcterms.alternative\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.tableOfContents\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.abstract\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.created\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.valid\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.available\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.issued\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.modified\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.extent\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.medium\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.isVersionOf\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.hasVersion\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.isReplacedBy\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.replaces\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.isRequiredBy\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.requires\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.isPartOf\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.hasPart\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.isReferencedBy\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.references\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.isFormatOf\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.hasFormat\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.conformsTo\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.spatial\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.temporal\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.mediator\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.dateAccepted\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.dateCopyrighted\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.dateSubmitted\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.educationLevel\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.accessRights\"") > -1);
        assertTrue(output.indexOf(
                              "value=\"dcterms.bibliographicCitation\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.license\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.rightsHolder\"") > -1);
        assertTrue(output.indexOf("value=\"dcterms.provenance\"") > -1);

    }

    /**
     *  Print the primary form field.
     *  @exception  IOException    Should not be thrown by this test.
     *  @since  Version 0.1 2005-07-04
     */
    public final void testPrintPrimaryFieldNullRequest()
        throws IOException {

        MockHttpServletRequest msreq = null;

        MockJspWriter out = new MockJspWriter();

        List selected = new Vector();

        try {

            tag.printPrimaryField(msreq,
                                  out,
                                  selected,
                                  true);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Print the primary form field.
     *  @exception  IOException    Should not be thrown by this test.
     *  @since  Version 0.1 2005-07-04
     */
    public final void testPrintPrimaryFieldNullJspWriter()
        throws IOException {

        MockHttpServletRequest msreq = new MockHttpServletRequest();

        MockJspWriter out = null;

        List selected = new Vector();

        try {

            tag.printPrimaryField(msreq,
                                  out,
                                  selected,
                                  true);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Print the primary form field.
     *  @exception  IOException    Should not be thrown by this test.
     *  @since  Version 0.1 2005-07-04
     */
    public final void testPrintPrimaryFieldCheckSelected()
        throws IOException {

        MockHttpServletRequest msreq = new MockHttpServletRequest();

        MockJspWriter out = new MockJspWriter();

        List selected = new Vector();

        tag.printPrimaryField(msreq,
                              out,
                              selected,
                              true);

        SchemaProperty property = dcProfile.getPrimarySearchProperty();

        assertTrue("Selected list should contain the primary search property.",
                   selected.contains(property));

    }


    /**
     *  Print the secondary form fields.
     *  @exception  IOException    Should not be thrown by this test.
     *  @since  Version 0.1 2005-07-04
     */
    public final void testPrintSecondaryFieldsNotRecognised()
        throws IOException {

        MockHttpServletRequest msreq = new MockHttpServletRequest();

        msreq.setParameter(DC_TITLE, "Test document");
        msreq.setParameter("bogus.notExist", "test value");

        MockJspWriter out = new MockJspWriter();

        List selected = new Vector();

        tag.printSecondaryFields(msreq,
                                 out,
                                 selected,
                                 true);

        String output = out.getOut();

        assertTrue(output.indexOf("name=\"dc.title\"") > -1);
        assertTrue(output.indexOf("name=\"bogus.notExist\"") == -1);

        SchemaProperty property = dcProfile.getPredicateProperty(DC_TITLE);

        assertTrue("The selected list should contain the DC title property.",
                   selected.contains(property));
    }

    /**
     *  Print the secondary form fields.
     *  @exception  IOException    Should not be thrown by this test.
     *  @since  Version 0.1 2005-07-04
     */
    public final void testPrintSecondaryFieldsOneSelected()
        throws IOException {

        MockHttpServletRequest msreq = new MockHttpServletRequest();

        msreq.setParameter(DC_TITLE, "Test document");
        msreq.setParameter(DC_CREATOR, "Babbage, Charles");

        MockJspWriter out = new MockJspWriter();

        List selected = new Vector();

        SchemaProperty property = dcProfile.getPredicateProperty(DC_TITLE);

        assertNotNull("Title property should not be null.",
                      property);

        selected.add(property);

        tag.printSecondaryFields(msreq,
                                 out,
                                 selected,
                                 true);

        String output = out.getOut();

        assertTrue("Secondary form fields should not include a " +
                   "DC Title field, actual output was:\n" +
                   output,
                   output.indexOf("name=\"dc.title\"") == -1);

        assertTrue("Secondary form fields should include a " +
                   "DC Creator field, actual output was:\n" +
                   output,
                   output.indexOf("name=\"dc.creator\"") > -1);
    }

    /**
     *  Print the primary form field.
     *  @exception  IOException    Should not be thrown by this test.
     *  @since  Version 0.1 2005-07-04
     */
    public final void testPrintPrimaryFieldNullList()
        throws IOException {

        MockHttpServletRequest msreq = new MockHttpServletRequest();

        MockJspWriter out = new MockJspWriter();

        List selected = null;

        try {

            tag.printPrimaryField(msreq,
                                  out,
                                  selected,
                                  true);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Print the secondary form fields.
     *  @exception  IOException    Should not be thrown by this test.
     *  @since  Version 0.1 2005-07-04
     */
    public final void testPrintSecondaryFieldsNullRequest()
        throws IOException {

        MockHttpServletRequest msreq = null;

        MockJspWriter out = new MockJspWriter();

        List selected = new Vector();

        try {

            tag.printSecondaryFields(msreq,
                                     out,
                                     selected,
                                     true);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Print the secondary form fields.
     *  @exception  IOException    Should not be thrown by this test.
     *  @since  Version 0.1 2005-07-04
     */
    public final void testPrintSecondaryFieldsNullJspWriter()
        throws IOException {

        MockHttpServletRequest msreq = new MockHttpServletRequest();

        MockJspWriter out = null;

        List selected = new Vector();

        try {

            tag.printSecondaryFields(msreq,
                                     out,
                                     selected,
                                     true);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Print the secondary form fields.
     *  @exception  IOException    Should not be thrown by this test.
     *  @since  Version 0.1 2005-07-04
     */
    public final void testPrintSecondaryFieldsNullList()
        throws IOException {

        MockHttpServletRequest msreq = new MockHttpServletRequest();

        MockJspWriter out = new MockJspWriter();

        List selected = null;

        try {

            tag.printSecondaryFields(msreq,
                                     out,
                                     selected,
                                     true);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Print any added form fields.
     *  @exception  IOException    Should not be thrown by this test.
     *  @since  Version 0.1 2005-07-04
     */
    public final void testPrintAddedFieldNullRequest()
        throws IOException {

        MockHttpServletRequest msreq = null;

        MockJspWriter out = new MockJspWriter();

        List selected = new Vector();

        try {

            tag.printAddedField(msreq,
                                out,
                                selected,
                                true);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Print any added form fields.
     *  @exception  IOException    Should not be thrown by this test.
     *  @since  Version 0.1 2005-07-04
     */
    public final void testPrintAddedFieldNullJspWriter()
        throws IOException {

        MockHttpServletRequest msreq = new MockHttpServletRequest();

        MockJspWriter out = null;

        List selected = new Vector();

        try {

            tag.printAddedField(msreq,
                                out,
                                selected,
                                true);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Print any added form fields.
     *  @exception  IOException    Should not be thrown by this test.
     *  @since  Version 0.1 2005-07-04
     */
    public final void testPrintAddedFieldNullList()
        throws IOException {

        MockHttpServletRequest msreq = new MockHttpServletRequest();

        MockJspWriter out = new MockJspWriter();

        List selected = null;

        try {

            tag.printAddedField(msreq,
                                out,
                                selected,
                                true);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Print any added form fields.
     *  @exception  IOException    Should not be thrown by this test.
     *  @since  Version 0.1 2005-07-04
     */
    public final void testPrintAddedFieldWithAdded()
        throws IOException {

        MockHttpServletRequest msreq = new MockHttpServletRequest();

        msreq.setParameter("add-field", DC_CREATOR);

        MockJspWriter out = new MockJspWriter();

        List selected = new Vector();

        tag.printAddedField(msreq,
                            out,
                            selected,
                            true);

        String output = out.getOut();

        assertTrue(output.indexOf("name=\"dc.creator\"") > -1);

        SchemaProperty property = dcProfile.getPredicateProperty("dc.creator");

        assertTrue("The selected list should contain the DC creator property.",
                   selected.contains(property));

    }

    /**
     *  Print any added form fields.
     *  @exception  IOException    Should not be thrown by this test.
     *  @since  Version 0.1 2005-07-04
     */
    public final void testPrintAddedFieldNotRecognised()
        throws IOException {

        MockHttpServletRequest msreq = new MockHttpServletRequest();

        msreq.setParameter("add-field", "bogus.notExist");

        MockJspWriter out = new MockJspWriter();

        List selected = new Vector();

        tag.printAddedField(msreq,
                            out,
                            selected,
                            true);

        String output = out.getOut();

        assertTrue(output.indexOf("name=\"bogus.notExist\"") == -1);

    }

    /**
     *  Print any added form fields.
     *  @exception  IOException    Should not be thrown by this test.
     *  @since  Version 0.1 2005-07-04
     */
    public final void testPrintAddedFieldAlreadyAdded()
        throws IOException {

        MockHttpServletRequest msreq = new MockHttpServletRequest();

        msreq.setParameter("add-field", DC_CREATOR);

        MockJspWriter out = new MockJspWriter();

        List selected = new Vector();

        SchemaProperty property = dcProfile.getPredicateProperty(DC_CREATOR);

        selected.add(property);

        tag.printAddedField(msreq,
                            out,
                            selected,
                            true);

        String output = out.getOut();

        assertTrue(output.indexOf("name=\"bogus.notExist\"") == -1);

    }

    /**
     *  Print the builder form.
     *  @exception  IOException    Should not be thrown by this test.
     *  @since  Version 0.1 2005-07-04
     */
    public final void testPrintBuilderFormNullJspWriter()
        throws IOException {

        try {

            tag.printBuilderForm(new MockHttpServletRequest(),
                                 new MockHttpServletResponse(),
                                 null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Tear down a test scenario as required.
     *  @since  Version 0.1 2005-06-28
    protected void tearDown() {

    }
     */

    /**
     *  Create a test suite for this test case.
     *  @return A test suite.
     *  @since  Version 0.1 2005-06-28
     */
    public static Test suite() {

        return new TestSuite(QueryBuilderTagTest.class);
    }

    /**
     *  Run this test suite as a stand alone application.
     *  @param  args    Any command line arguments to pass
     *          to the test. None required or expected.
     *  @since  Version 0.1 2005-06-28
     */
    public static void main(final String[] args) {

        junit.textui.TestRunner.run(suite());
    }

}
